Package: ggvis
Title: Interactive Grammar of Graphics
Version: 0.4.10
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Winston", "Chang", , "winston@posit.co", role = "aut"),
    person(, "Posit", role = "cph"),
    person(, "jQuery Foundation", role = "cph",
           comment = "jQuery library and jQuery UI library"),
    person(, "jQuery contributors", role = c("ctb", "cph"),
           comment = "jQuery library; authors listed in inst/www/lib/jquery/AUTHORS.txt"),
    person(, "jQuery UI contributors", role = c("ctb", "cph"),
           comment = "jQuery UI library; authors listed in inst/www/lib/jquery-ui/AUTHORS.txt"),
    person("Mike", "Bostock", role = c("ctb", "cph"),
           comment = "D3 library"),
    person(, "D3 contributors", role = "ctb",
           comment = "D3 library; authors listed at https://github.com/d3/d3/graphs/contributors"),
    person(, "Trifacta Inc.", role = "cph",
           comment = "Vega library"),
    person(, "Vega contributors", role = "ctb",
           comment = "Vega library; authors listed at https://github.com/trifacta/vega/graphs/contributors"),
    person("Sebastián", "Décima", role = c("ctb", "cph"),
           comment = "javascript-detect-element-resize library")
  )
Description: An implementation of an interactive grammar of graphics,
    taking the best parts of 'ggplot2', combining them with the reactive
    framework of 'shiny' and drawing web graphics using 'vega'.
License: GPL-2 | file LICENSE
URL: https://ggvis.rstudio.com/
Depends: R (>= 3.0)
Imports: assertthat, dplyr (>= 0.5.0), htmltools (>= 0.2.4), jsonlite
        (>= 0.9.11), magrittr, methods, rlang, shiny (>= 0.11.1)
Suggests: knitr (>= 1.6), lubridate, MASS, mgcv, rmarkdown, testthat
        (>= 0.8.1)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-10 14:25:27 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Winston Chang [aut],
  Posit [cph],
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/lib/jquery/AUTHORS.txt),
  jQuery UI contributors [ctb, cph] (jQuery UI library; authors listed in
    inst/www/lib/jquery-ui/AUTHORS.txt),
  Mike Bostock [ctb, cph] (D3 library),
  D3 contributors [ctb] (D3 library; authors listed at
    https://github.com/d3/d3/graphs/contributors),
  Trifacta Inc. [cph] (Vega library),
  Vega contributors [ctb] (Vega library; authors listed at
    https://github.com/trifacta/vega/graphs/contributors),
  Sebastián Décima [ctb, cph] (javascript-detect-element-resize library)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-02-10 14:50:02 UTC
Built: R 4.6.0; ; 2026-04-28 22:50:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",ggvis_props)
S3method(apply_props,data.frame)
S3method(apply_props,grouped_df)
S3method(apply_scale_defaults,ggvis_scale)
S3method(apply_scale_defaults,scale_datetime)
S3method(apply_scale_defaults,scale_nominal)
S3method(apply_scale_defaults,scale_numeric)
S3method(apply_scale_defaults,scale_ordinal)
S3method(arrange,ggvis)
S3method(arrange,reactive)
S3method(as.character,prop_band)
S3method(as.character,prop_constant)
S3method(as.character,prop_reactive)
S3method(as.character,prop_variable)
S3method(as.data.frame,singular)
S3method(as.vega,axis_props)
S3method(as.vega,data.frame)
S3method(as.vega,ggvis)
S3method(as.vega,ggvis_axis)
S3method(as.vega,ggvis_legend)
S3method(as.vega,ggvis_props)
S3method(as.vega,ggvis_scale)
S3method(as.vega,grouped_df)
S3method(as.vega,legend_props)
S3method(as.vega,mark)
S3method(as.vega,mark_group)
S3method(as.vega,padding)
S3method(as.vega,scaled_value)
S3method(as_char,call)
S3method(as_char,default)
S3method(as_char,name)
S3method(bin_params,POSIXct)
S3method(bin_params,default)
S3method(bin_vector,Date)
S3method(bin_vector,POSIXct)
S3method(bin_vector,default)
S3method(bin_vector,numeric)
S3method(compute_align,data.frame)
S3method(compute_align,ggvis)
S3method(compute_bin,data.frame)
S3method(compute_bin,ggvis)
S3method(compute_bin,grouped_df)
S3method(compute_boxplot,data.frame)
S3method(compute_boxplot,ggvis)
S3method(compute_boxplot,grouped_df)
S3method(compute_boxplot_outliers,data.frame)
S3method(compute_boxplot_outliers,ggvis)
S3method(compute_boxplot_outliers,grouped_df)
S3method(compute_count,data.frame)
S3method(compute_count,ggvis)
S3method(compute_count,grouped_df)
S3method(compute_density,data.frame)
S3method(compute_density,ggvis)
S3method(compute_density,grouped_df)
S3method(compute_model_prediction,data.frame)
S3method(compute_model_prediction,ggvis)
S3method(compute_model_prediction,grouped_df)
S3method(compute_stack,data.frame)
S3method(compute_stack,ggvis)
S3method(compute_stack,grouped_df)
S3method(compute_tabulate,data.frame)
S3method(compute_tabulate,ggvis)
S3method(compute_tabulate,grouped_df)
S3method(constant_vars,data.frame)
S3method(constant_vars,grouped_df)
S3method(data_range,character)
S3method(data_range,default)
S3method(data_range,factor)
S3method(distinct,ggvis)
S3method(distinct,reactive)
S3method(empty,default)
S3method(eval_vector,data.frame)
S3method(explain,ggvis)
S3method(extract_reactives,default)
S3method(extract_reactives,ggvis_props)
S3method(extract_reactives,prop_reactive)
S3method(format,ggvis_axis)
S3method(format,ggvis_legend)
S3method(format,ggvis_props)
S3method(format,ggvis_scale)
S3method(format,mark)
S3method(format,prop)
S3method(format_vec_csv,Date)
S3method(format_vec_csv,POSIXt)
S3method(format_vec_csv,character)
S3method(format_vec_csv,factor)
S3method(format_vec_csv,numeric)
S3method(format_vec_d3json,Date)
S3method(format_vec_d3json,POSIXt)
S3method(format_vec_d3json,character)
S3method(format_vec_d3json,factor)
S3method(format_vec_d3json,numeric)
S3method(formula,prop)
S3method(fullseq,Date)
S3method(fullseq,POSIXt)
S3method(fullseq,numeric)
S3method(group_by,ggvis)
S3method(group_by,reactive)
S3method(groups,ggvis)
S3method(groups,reactive)
S3method(max_rows,data.frame)
S3method(max_rows,grouped_df)
S3method(mutate,ggvis)
S3method(mutate,reactive)
S3method(new_prop,band)
S3method(new_prop,call)
S3method(new_prop,default)
S3method(new_prop,formula)
S3method(new_prop,name)
S3method(new_prop,prop)
S3method(new_prop,reactive)
S3method(pred_grid,lm)
S3method(pred_grid,loess)
S3method(preserve_constants,data.frame)
S3method(preserve_constants,grouped_df)
S3method(print,ggvis)
S3method(print,ggvis_axis)
S3method(print,ggvis_legend)
S3method(print,ggvis_props)
S3method(print,ggvis_scale)
S3method(print,mark)
S3method(print,prop)
S3method(print,singular)
S3method(print,template)
S3method(prop_domain,prop_band)
S3method(prop_domain,prop_constant)
S3method(prop_domain,prop_reactive)
S3method(prop_domain,prop_variable)
S3method(prop_label,prop_band)
S3method(prop_label,prop_constant)
S3method(prop_label,prop_reactive)
S3method(prop_label,prop_variable)
S3method(prop_type,data.frame)
S3method(prop_value,prop_band)
S3method(prop_value,prop_constant)
S3method(prop_value,prop_reactive)
S3method(prop_value,prop_variable)
S3method(prop_vega,prop_band)
S3method(prop_vega,prop_constant)
S3method(prop_vega,prop_reactive)
S3method(prop_vega,prop_variable)
S3method(remove_missing,data.frame)
S3method(remove_missing,default)
S3method(remove_missing,split_df)
S3method(rename,ggvis)
S3method(rename,reactive)
S3method(rep,singular)
S3method(scale_countable,default)
S3method(scale_countable,scale_datetime)
S3method(scale_countable,scale_logical)
S3method(scale_countable,scale_nominal)
S3method(scale_countable,scale_numeric)
S3method(scale_countable,scale_ordinal)
S3method(select,ggvis)
S3method(select,reactive)
S3method(slice,ggvis)
S3method(slice,reactive)
S3method(summarise,ggvis)
S3method(summarise,reactive)
S3method(to_csv,data.frame)
S3method(transmute,ggvis)
S3method(transmute,reactive)
S3method(ungroup,ggvis)
S3method(ungroup,reactive)
S3method(value,default)
S3method(value,reactive)
S3method(vector_type,"NULL")
S3method(vector_type,Date)
S3method(vector_type,POSIXt)
S3method(vector_type,character)
S3method(vector_type,default)
S3method(vector_type,factor)
S3method(vector_type,integer)
S3method(vector_type,logical)
S3method(vector_type,numeric)
S3method(vector_type,ordered)
S3method(vector_type,singular)
S3method(vega_data_parser,"NULL")
S3method(vega_data_parser,Date)
S3method(vega_data_parser,POSIXt)
S3method(vega_data_parser,character)
S3method(vega_data_parser,default)
S3method(vega_data_parser,factor)
S3method(vega_data_parser,integer)
S3method(vega_data_parser,logical)
S3method(vega_data_parser,numeric)
S3method(vega_data_parser,ordered)
export("%>%")
export(add_axis)
export(add_data)
export(add_guide_axis)
export(add_guide_legend)
export(add_legend)
export(add_props)
export(add_relative_scales)
export(add_scale)
export(add_tooltip)
export(auto_group)
export(axis_props)
export(band)
export(bind_shiny)
export(bind_shiny_ui)
export(compute_align)
export(compute_bin)
export(compute_boxplot)
export(compute_count)
export(compute_density)
export(compute_model_prediction)
export(compute_smooth)
export(compute_stack)
export(compute_tabulate)
export(create_broker)
export(create_input)
export(default_options)
export(emit_arcs)
export(emit_images)
export(emit_paths)
export(emit_points)
export(emit_rects)
export(emit_ribbons)
export(emit_text)
export(explain)
export(export_png)
export(export_svg)
export(fullseq)
export(get_data)
export(ggvis)
export(ggvisControlOutput)
export(ggvisOutput)
export(ggvis_message)
export(ggvis_scale)
export(group_by)
export(handle_brush)
export(handle_click)
export(handle_hover)
export(handle_resize)
export(hide_axis)
export(hide_legend)
export(hide_tooltip)
export(input_checkbox)
export(input_checkboxgroup)
export(input_numeric)
export(input_radiobuttons)
export(input_select)
export(input_slider)
export(input_text)
export(is.axis_props)
export(is.broker)
export(is.dynamic)
export(is.ggvis)
export(is.ggvis_props)
export(is.ggvis_scale)
export(is.legend_props)
export(is.mark)
export(is.prop)
export(is.prop_constant)
export(is.prop_reactive)
export(is.prop_variable)
export(is.scaled_value)
export(layer_arcs)
export(layer_bars)
export(layer_boxplots)
export(layer_densities)
export(layer_f)
export(layer_freqpolys)
export(layer_guess)
export(layer_histograms)
export(layer_images)
export(layer_lines)
export(layer_model_predictions)
export(layer_paths)
export(layer_points)
export(layer_rects)
export(layer_ribbons)
export(layer_smooths)
export(layer_text)
export(left_right)
export(legend_props)
export(linked_brush)
export(mainTopPanel)
export(mark)
export(padding)
export(plot_height)
export(plot_width)
export(prop)
export(propname_to_scale)
export(props)
export(resolution)
export(save_spec)
export(scale_datetime)
export(scale_logical)
export(scale_nominal)
export(scale_numeric)
export(scale_ordinal)
export(scale_singular)
export(scaled_value)
export(set_default_scale)
export(set_dscale)
export(set_options)
export(show_spec)
export(show_tooltip)
export(sidebarBottomPage)
export(sidebarBottomPanel)
export(singular)
export(up_down)
export(vector_type)
export(view_dynamic)
export(view_static)
export(waggle)
export(zero_range)
import(assertthat)
importFrom(dplyr,arrange)
importFrom(dplyr,distinct)
importFrom(dplyr,explain)
importFrom(dplyr,group_by)
importFrom(dplyr,groups)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(magrittr,"%>%")
importFrom(shiny,reactive)

# ggvis

## Status
![](https://img.shields.io/badge/lifecycle-dormant-blue.svg)

ggvis is currently dormant. We fundamentally believe in the ideas that underlie ggvis: reactive programming is the right foundation for interactive visualisation. However, we are not currently working on ggvis because we do not see it as the most pressing issue for the R community as you can only use interactive graphics once you've successfuly tackled the rest of the data analysis process.

We hope to come back to ggvis in the future; in the meantime you might want to try out [plotly](https://plotly.com/ggplot2/getting-started/) or creating inteactive graphics [with shiny](https://posit.co/blog/shiny-0-12-interactive-plots-with-ggplot2/).

## Introduction

The goal of ggvis is to make it easy to describe interactive web graphics in
R. It combines:

* a grammar of graphics from [ggplot2](https://github.com/tidyverse/ggplot2),

* reactive programming from [shiny](https://github.com/rstudio/shiny), and

* data transformation pipelines from [dplyr](https://github.com/tidyverse/dplyr).

ggvis graphics are rendered with [vega](https://github.com/trifacta/vega), so you can generate both raster graphics with HTML5 canvas and vector graphics with
[svg](https://en.wikipedia.org/wiki/Scalable_Vector_Graphics). ggvis is less flexible than raw [d3](https://d3js.org/) or vega, but is much more succinct and is tailored to the needs of exploratory data analysis.

If you find a bug, please file a minimal reproducible example at https://github.com/rstudio/ggvis/issues. If you're not sure if something is a bug, you'd like to discuss new features or have any other questions about ggvis, please join us on the mailing list: https://groups.google.com/group/ggvis.

## Installation

Install the latest release version from CRAN with:

```R
install.packages("ggvis")
```

Install the latest development version with:

```R
# install.packages("devtools")
devtools::install_github("rstudio/ggvis")
```

## Getting started

You construct a visualisation by piping pieces together with `%>%`. The pipeline starts with a data set, flows into `ggvis()` to specify default visual properties, then layers on some visual elements:

```R
mtcars %>% ggvis(~mpg, ~wt) %>% layer_points()
```

The vignettes, available from https://ggvis.rstudio.com/, provide many more details. Start with the introduction, then work your way through the more advanced topics. Also check out the
various demos in the `demo/` directory. See the basics in `demo/scatterplot.r`
then check out the the coolest demos, `demo/interactive.r` and `demo/tourr.r`.

This directory contains example Shiny apps. If you are in the top-level ggvis directory, these can be run with:

```
shiny::runApp('demo/apps/basic')
```

If you've installed `ggvis`, you can run the demo with 

```
shiny::runApp(system.file("demo/apps/basic", package = "ggvis"))
```

#' Bin data along a continuous variable
#'
#' @param x Dataset-like object to bin. Built-in methods for data frames,
#'   grouped data frames and ggvis visualisations.
#' @param x_var,w_var Names of x and weight variables. The x variable must be
#'   continuous.
#' @param width The width of the bins. The default is \code{NULL}, which yields
#'   30 bins that cover the range of the data. You should always override this
#'   value, exploring multiple widths to find the best to illustrate the stories
#'   in your data.
#' @param center The center of one of the bins.  Note that if center is above or
#'   below the range of the data, things will be shifted by an appropriate
#'   number of \code{width}s. To center on integers, for example, use
#'   \code{width=1} and \code{center=0}, even if \code{0} is outside the range
#'   of the data.  At most one of \code{center} and \code{boundary} may be
#'   specified.
#' @param boundary A boundary between two bins. As with \code{center}, things
#'   are shifted when \code{boundary} is outside the range of the data. For
#'   example, to center on integers, use \code{width = 1} and \code{boundary =
#'   0.5}, even if \code{1} is outside the range of the data.  At most one of
#'   \code{center} and \code{boundary} may be specified.
#' @param closed One of \code{"right"} or \code{"left"} indicating whether right
#'   or left edges of bins are included in the bin.
#' @param pad If \code{TRUE}, adds empty bins at either end of x. This ensures
#'   frequency polygons touch 0. Defaults to \code{FALSE}.
#' @param binwidth Deprecated; use \code{width} instead.
#' @seealso \code{\link{compute_count}} For counting cases at specific locations
#'   of a continuous variable. This is useful when the variable is continuous
#'   but the data is granular.
#' @export
#' @return A data frame with columns:
#'  \item{count_}{the number of points}
#'  \item{x_}{mid-point of bin}
#'  \item{xmin_}{left boundary of bin}
#'  \item{xmax_}{right boundary of bin}
#'  \item{width_}{width of bin}
#' @examples
#' mtcars %>% compute_bin(~mpg)
#' mtcars %>% compute_bin(~mpg, width = 10)
#' mtcars %>% group_by(cyl) %>% compute_bin(~mpg, width = 10)
#'
#' # It doesn't matter whether you transform inside or outside of a vis
#' mtcars %>% compute_bin(~mpg) %>% ggvis(~x_, ~count_) %>% layer_paths()
#' mtcars %>% ggvis(~ x_, ~ count_) %>% compute_bin(~mpg) %>% layer_paths()
#'
#' # Missing values get own bin
#' mtcars2 <- mtcars
#' mtcars2$mpg[sample(32, 5)] <- NA
#' mtcars2 %>% compute_bin(~mpg, width = 10)
#'
#' # But are currently silently dropped in histograms
#' mtcars2 %>% ggvis() %>% layer_histograms(~mpg)
compute_bin <- function(x, x_var, w_var = NULL, width = NULL,
                        center = NULL, boundary = NULL,
                        closed = c("right", "left"), pad = FALSE,
                        binwidth) {
  UseMethod("compute_bin")
}

#' @export
compute_bin.data.frame <- function(x, x_var, w_var = NULL, width = NULL,
                                   center = NULL, boundary = NULL,
                                   closed = c("right", "left"), pad = FALSE,
                                   binwidth) {

  if (!missing(binwidth)) {
    width <- binwidth
    deprecated("binwidth", "width", version = "0.3.0")
  }

  closed <- match.arg(closed)
  assert_that(is.formula(x_var))

  x_val <- eval_vector(x, x_var)

  # Special case zero-row input
  if (length(x_val) == 0) return(bin_out())

  params <- bin_params(range2(x_val, na.rm = TRUE), width = width,
                       center = center, boundary = boundary, closed = closed)

  if (is.null(w_var)) {
    w_val <- NULL
  } else {
    w_val <- eval_vector(x, w_var)
  }

  bin_vector(x_val, weight = w_val, width = params$width,
             origin = params$origin, closed = params$closed, pad = pad)
}

#' @export
compute_bin.grouped_df <- function(x, x_var, w_var = NULL, width = NULL,
                                   center = NULL, boundary = NULL,
                                   closed = c("right", "left"), pad = FALSE,
                                   binwidth) {

  if (!missing(binwidth)) {
    width <- binwidth
    deprecated("binwidth", "width", version = "0.3.0")
  }

  closed <- match.arg(closed)
  x_val <- eval_vector(x, x_var)

  # We want to use the same boundary and width across groups, so calculate
  # bin params here.
  params <- bin_params(range2(x_val), width = width, center = center,
                       boundary = boundary, closed = closed)

  dplyr::do(x, compute_bin(.,
    x_var,
    w_var = w_var,
    width = params$width,
    center = NULL,
    boundary = params$origin,
    closed = params$closed,
    pad = pad
  ))
}

#' @export
compute_bin.ggvis <- function(x, x_var, w_var = NULL, width = NULL,
                              center = NULL, boundary = NULL,
                              closed = c("right", "left"), pad = FALSE,
                              binwidth) {
  if (!missing(binwidth)) {
    width <- binwidth
    deprecated("binwidth", "width", version = "0.3.0")
  }

  closed <- match.arg(closed)
  args <- list(x_var = x_var, w_var = w_var, width = width,
               center = center, boundary = boundary, closed = closed, pad = pad)

  register_computation(x, args, "bin", function(data, args) {
    output <- do_call(compute_bin, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

# Compute parameters -----------------------------------------------------------

bin_params <- function(x_range, width = NULL, center = NULL, boundary = NULL,
                       closed = c("right", "left")) {
  UseMethod("bin_params")
}

#' @export
bin_params.default <- function(x_range, width = NULL, center = NULL,
                               boundary = NULL, closed = c("right", "left")) {
  closed <- match.arg(closed)

  if (length(x_range) == 0) {
    return(list(width = width, origin = NULL, closed = closed))
  }

  stopifnot(length(x_range) == 2)
  if (!is.null(boundary) && !is.null(center)) {
    stop("Only one of 'boundary' and 'center' may be specified.")
  }

  if (is.null(width)) {
    # Find a nice-looking value for width
    bounds <- pretty(x_range, 30)
    width <- bounds[2] - bounds[1]
    notify_guess(width, paste0("range / ", length(bounds)-1))
  }

  if (is.null(boundary)) {
    if (is.null(center)) {
      # If neither edge nor center given, compute both using tile layer's
      # algorithm. This puts min and max of data in outer half of their bins.
      boundary <- width / 2

    } else {
      # If center given but not boundary, compute boundary.
      boundary <- center - width / 2
    }
  }

  # Inputs could be Dates or POSIXct, so make sure these are all numeric
  x_range <- as.numeric(x_range)
  width <- as.numeric(width)
  boundary <- as.numeric(boundary)

  origin <- find_origin(x_range, width, boundary)

  list(width = width, origin = origin, closed = closed)
}

#' @export
bin_params.POSIXct <- function(x_range, width = NULL, center = NULL,
                               boundary = NULL, closed = c("right", "left")) {
  if (length(x_range) == 0) {
    return(list(width = width, origin = NULL, closed = closed))
  }

  if (is.null(width)) {
    bounds <- pretty(x_range, 30)
    width <- bounds[2] - bounds[1]
    notify_guess(width, paste0("range / ", length(bounds)-1))
  }

  # Period object from lubridate package - need lubridate::as.difftime to find
  # the correct generic, instead of base::as.difftime.
  if (methods::is(width, "Period")) {
    width <- as.numeric(lubridate::as.difftime(width, units = "secs"))
  }

  bin_params(
    as.numeric(x_range),
    as.numeric(width, units = "secs"),
    as_numeric(center),
    as_numeric(boundary),
    closed
  )
}

# Find the left side of left-most bin
find_origin <- function(x_range, width, boundary) {
  shift <- floor((x_range[1] - boundary) / width)
  boundary + shift * width
}

# Bin individual vector --------------------------------------------------------

#' Bin vectors
#'
#' A generic and several implementations for binning vectors.
#'
#' @param x A vector to bin
#' @param weight If specified, an integer vector of the same length as \code{x}
#'   representing the number of occurances of each value in \code{x}
#' @param width The width of a bin
#' @param origin The left-most value for bins.
#' @param closed One of \code{"right"} or \code{"left"} indicating whether
#'   right or left edges of bins are included in the bin.
#' @param pad A logical indicating whether the bins should be padded to include
#'   an empty bin on each side.
#' @param ... additional arguments passed through to methods.
#' @keywords internal
bin_vector <- function(x, weight = NULL, ...) {
  UseMethod("bin_vector")
}

#' @export
#' @rdname bin_vector
bin_vector.numeric <- function(x, weight = NULL, ..., width = 1,
                               origin = 0, closed = c("right", "left"),
                               pad = FALSE) {
  closed <- match.arg(closed)

  if (all(is.na(x))) {
    return(bin_out(length(x), NA, NA, xmin = NA, xmax = NA))
  }

  stopifnot(is.numeric(width) && length(width) == 1)
  stopifnot(is.numeric(origin) && length(origin) == 1)

  if (is.null(weight)) {
    weight <- rep(1, length(x))
  } else {
    weight[is.na(weight)] <- 0
  }

  min_x <- origin
  # Small correction factor so that we don't get an extra bin when, for
  # example, origin=0, max(x)=20, width=10.
  max_x <- max(x, na.rm = TRUE) + (1 - 1e-08) * width
  breaks <- seq(min_x, max_x, width)
  fuzzybreaks <- adjust_breaks(breaks, closed = closed)

  bins <- cut(x, fuzzybreaks, include.lowest = TRUE, right = (closed == "right"))

  left <- breaks[-length(breaks)]
  right <- breaks[-1]
  x <- (left + right) / 2
  bin_widths <- diff(breaks)

  count <- as.numeric(tapply(weight, bins, sum, na.rm = TRUE))
  count[is.na(count)] <- 0

  if (pad) {
    count <- c(0, count, 0)
    bin_widths <- c(width, bin_widths, width)
    x <- c(x[1] - width, x, x[length(x)] + width)
  }

  # Add row for missings
  if (any(is.na(bins))) {
    count <- c(count, sum(is.na(bins)))
    left <- c(left, NA)
    right <- c(right, NA)
    x <- c(x, NA)
    bin_widths <- c(bin_widths, NA)
  }

  bin_out(count, x, bin_widths)
}

#' @export
bin_vector.POSIXct <- function(x, weight = NULL, ..., width = 1,
                               origin = NULL, closed = c("right", "left"),
                               pad = FALSE) {

  # Convert times to raw numbers (seconds since UNIX epoch)
  if (methods::is(width, "Period")) {
    width <- as.numeric(lubridate::as.difftime(width, units = "secs"))
  }

  results <- bin_vector(
    as.numeric(x),
    weight = weight,
    width = width,
    origin = if (is.null(origin)) NULL else as.numeric(origin),
    closed = closed,
    pad = pad
  )

  # Convert some columns from numeric back to POSIXct objects
  tz <- attr(x, "tzone", TRUE)
  time_cols <- c("x_", "xmin_", "xmax_")
  results[time_cols] <- lapply(results[time_cols], function(col) {
    as.POSIXct(col, origin = "1970-01-01", tz = tz)
  })

  results
}

#' @export
bin_vector.Date <- function(x, weight = NULL, ..., width = 1,
                            origin = NULL, closed = c("right", "left"),
                            pad = FALSE) {

  results <- bin_vector(
    as.numeric(x),
    weight = weight,
    width = width,
    origin = if (is.null(origin)) NULL else as.numeric(origin),
    closed = closed,
    pad = pad
  )

  # Convert some columns from numeric back to Date objects
  time_cols <- c("x_", "xmin_", "xmax_")
  results[time_cols] <- lapply(results[time_cols], function(col) {
    class(col) <- "Date"
    col
  })

  results
}


#' @export
bin_vector.default <- function(x, weight = NULL, ...) {
  stop("Don't know how to bin vector of type ", class(x))
}

bin_out <- function(count = integer(0), x = numeric(0), width = numeric(0),
                    xmin = x - width / 2, xmax = x + width / 2) {
  data.frame(
    count_ = count,
    x_ = x,
    xmin_ = xmin,
    xmax_ = xmax,
    width_ = width,
    stringsAsFactors = FALSE
  )
}

# Adapt break fuzziness from base::hist - this protects from floating
# point rounding errors
adjust_breaks <- function(breaks, closed = "left") {
  closed <- match.arg(closed, c("right", "left"))

  diddle <- 1e-08 * stats::median(diff(breaks))
  if (closed == "right") {
    fuzz <- c(-diddle, rep.int(diddle, length(breaks) - 1))
  } else {
    fuzz <- c(rep.int(-diddle, length(breaks) - 1), diddle)
  }
  sort(breaks) + fuzz
}

context("compute_bin")

comp_bin <- function(...) {
  suppressMessages(compute_bin(...))
}

test_that("compute_bin preserves dates and times", {
  dates <- data.frame(val = as.Date("2013-06-01") + 0:100)
  NYtimes <- data.frame(
    val = as.POSIXct('2001-06-01 21:00', tz = 'America/New_York') + 0:10 * 100
  )
  UTCtimes <- data.frame(
    val = as.POSIXct('2001-06-01 21:00', tz = 'UTC') + seq(0, 1000, by = 10)
  )

  res <- comp_bin(dates, ~val, width = 30)
  expect_true(inherits(res$x_, "Date"))
  expect_true(inherits(res$xmin_, "Date"))
  expect_true(inherits(res$xmax_, "Date"))
  expect_equal(sum(res$count_), length(dates$val))

  res <- comp_bin(NYtimes, ~val, width = 120)
  expect_true(inherits(res$x_, "POSIXct"))
  expect_true(inherits(res$xmin_, "POSIXct"))
  expect_true(inherits(res$xmax_, "POSIXct"))
  expect_equal(sum(res$count_), length(NYtimes$val))
  expect_identical(attr(NYtimes$val, "tzone"), attr(res$x_, "tzone"))

  res <- comp_bin(UTCtimes, ~val, width = 120)
  expect_equal(sum(res$count_), length(UTCtimes$val))
  expect_identical(attr(UTCtimes$val, "tzone"), attr(res$x_, "tzone"))
})

test_that("width in lubridate::Period", {
  UTCtimes <- data.frame(
    val = as.POSIXct('2001-06-01 21:00', tz = 'UTC') + seq(0, 1000, by = 10)
  )

  # width specified as a Period from lubridate
  expect_identical(
    comp_bin(UTCtimes, ~val, width = lubridate::ms("1 42")),
    comp_bin(UTCtimes, ~val, width = 102)
  )
})

test_that("Closed left or right", {
  dat <- data.frame(x = c(0, 10))

  res <- comp_bin(dat, ~x, width = 10, pad = FALSE)
  expect_identical(res$count_, c(1, 1))
  res <- comp_bin(dat, ~x, width = 10, boundary = 5, pad = FALSE)
  expect_identical(res$count_, c(1, 1))
  res <- comp_bin(dat, ~x, width = 10, boundary = 0, pad = FALSE)
  expect_identical(res$count_, 2)
  res <- comp_bin(dat, ~x, width = 5, boundary = 0, pad = FALSE)
  expect_identical(res$count_, c(1, 1))

  res <- comp_bin(dat, ~x, width = 10, pad = FALSE, closed = "left")
  expect_identical(res$count_, c(1, 1))
  res <- comp_bin(dat, ~x, width = 10, boundary = 5, pad = FALSE, closed = "left")
  expect_identical(res$count_, c(1, 1))
  res <- comp_bin(dat, ~x, width = 10, boundary = 0, pad = FALSE, closed = "left")
  expect_identical(res$count_, c(2))
  res <- comp_bin(dat, ~x, width = 5, boundary = 0, pad = FALSE, closed = "left")
  expect_identical(res$count_, c(1, 1))
})


test_that("Setting boundary and center", {
  # numeric
  dat <- data.frame(x = c(0, 30))

  # Error if both boundary and center are specified
  expect_error(comp_bin(dat, ~x, width = 10, bondary = 5, center = 0, pad = FALSE))

  res <- comp_bin(dat, ~x, width = 10, boundary = 0, pad = FALSE)
  expect_identical(res$count_, c(1, 0, 1))
  expect_identical(res$xmin_[1], 0)
  expect_identical(res$xmax_[3], 30)

  res <- comp_bin(dat, ~x, width = 10, center = 0, pad = FALSE)
  expect_identical(res$count_, c(1, 0, 0, 1))
  expect_identical(res$xmin_[1], dat$x[1] - 5)
  expect_identical(res$xmax_[4], dat$x[2] + 5)


  # Date
  dat <- data.frame(x = as.Date("2013-06-01") + c(0, 30))

  res <- comp_bin(dat, ~x, width = 10, boundary = as.Date("2013-06-01"), pad = FALSE)
  expect_identical(res$count_, c(1, 0, 1))
  expect_identical(res$xmin_[1], dat$x[1])
  expect_identical(res$xmax_[3], dat$x[2])

  res <- comp_bin(dat, ~x, width = 10, center = as.Date("2013-06-01"), pad = FALSE)
  expect_identical(res$count_, c(1, 0, 0, 1))
  expect_identical(res$xmin_[1], dat$x[1] - 5)
  expect_identical(res$xmax_[4], dat$x[2] + 5)


  # POSIXct
  dat <- data.frame(
    x = as.POSIXct('2001-06-01 21:00', tz = 'America/New_York') + c(0, 30000)
  )

  res <- comp_bin(dat, ~x, width = 10000, boundary = dat$x[1], pad = FALSE)
  expect_identical(res$count_, c(1, 0, 1))
  expect_identical(res$xmin_[1], dat$x[1])
  expect_identical(res$xmax_[3], dat$x[2])

  res <- comp_bin(dat, ~x, width = 10000, center = dat$x[1], pad = FALSE)
  expect_identical(res$count_, c(1, 0, 0, 1))
  expect_identical(res$xmin_[1], dat$x[1] - 5000)
  expect_identical(res$xmax_[4], dat$x[2] + 5000)
})


test_that("Automatic width", {
  dat <- data.frame(
    num = c(0, 25.0),
    num2 = c(0, 50.0),
    int = c(1L, 25L),
    int2 = c(1L, 50L),
    date = as.Date("2013-06-01") + c(0, 100),
    posixct = as.POSIXct('2001-06-01 21:00', tz = 'UTC') + c(0, 1000)
  )

  # numeric
  res <- comp_bin(dat, ~num)
  # It generates approx 30 bins, at round numbers, so should have width 1
  expect_identical(res$width_, rep(1, length(res$width_)))
  res <- comp_bin(dat, ~num2)
  expect_identical(res$width_, rep(2, length(res$width_)))

  # integer
  res <- comp_bin(dat, ~int)
  expect_true(all(res$width_ == 1L))
  res <- comp_bin(dat, ~int2)
  expect_true(all(res$width_ == 2L))

  # Date
  res <- comp_bin(dat, ~date)
  # There was a change in behavior of `pretty` in 3.3.0 which results in
  # different output from earlier versions.
  r_version <- as.package_version(paste0(R.Version()$major, ".", R.Version()$minor))
  if (r_version >= "3.3.0")
    expect_identical(res$width_, rep(7, length(res$width_)))
  else
    expect_identical(res$width_, rep(2, length(res$width_)))

  # POSIXct
  res <- comp_bin(dat, ~posixct)
  expect_identical(res$width_, rep(30, length(res$width_)))
})


test_that("Boundaries across groups should be aligned", {
  dat <- data.frame(x = c(0:2, 0:2+0.7), g=c('a','a','a', 'b','b','b'))

  res <- dat %>% group_by(g) %>% compute_bin(~x, width = 1, pad = FALSE)
  expect_true(length(unique(res$x_ %% 1)) == 1)
  expect_identical(dplyr::groups(res), list(quote(g)))
  expect_identical(res$count_, rep(1, 6))
})


test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_bin(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c("count_", "x_", "xmin_", "xmax_", "width_")
  ))

  # Grouped
  res <- mtcars[0,] %>% group_by(cyl) %>% compute_bin(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c("cyl", "count_", "x_", "xmin_", "xmax_", "width_")
  ))
})

test_that("weights are added", {
  df <- data.frame(x = 1:10, y = 1:10)

  binned <- df %>% compute_bin(~x, ~y, width = 1, pad = FALSE)
  expect_equal(binned$count_, df$y)
})


# Bin vector -------------------------------------------------------------------

test_that("NAs get own bin", {
  x <- c(1:10, NA, NA, NA, NA)
  binned <- bin_vector(x, width = 100)
  expect_equal(binned$count_, c(10, 4))
  expect_equal(binned$x_, c(50, NA))
})

test_that("only NA, one row of output", {
  x <- as.numeric(c(NA, NA, NA, NA))
  binned <- bin_vector(x)
  expect_equal(binned$count_, 4)
  expect_equal(binned$x_, NA)
})

#' Coerce an ggvis object to a vega list.
#'
#' This generic function powers the coercion of ggvis objects to vega
#' compatible data structures.
#'
#' @param x an object to convert to vega
#' @return a list. When converted to JSON, will be the type of structure
#'   that vega expects.
#' @keywords internal
as.vega <- function(x, ...) {
  UseMethod("as.vega", x)
}

#' @method as.vega ggvis
#' @export
#' @rdname as.vega
#' @param session a session object from shiny
#' @param dynamic whether to generate dynamic or static spec
as.vega.ggvis <- function(x, session = NULL, dynamic = FALSE, ...) {
  # Any changes to default should happen at top-level
  x$cur_vis <- NULL

  if (length(x$marks) == 0) {
    x <- layer_guess(x)
  }

  data_props <- combine_data_props(x$marks)
  data_ids <- names(data_props)
  data_table <- x$data[data_ids]

  # Collapse each list of scale objects into one scale object.
  x <- collapse_scales(x)
  scale_data_table <- scale_domain_data(x)

  # Wrap each of the reactive data objects in another reactive which returns
  # only the columns that are actually used, and adds any calculated columns
  # that are used in the props.
  data_table <- active_props(data_table, data_props)

  # From an environment containing data_table objects, get static data for the
  # specified ids.
  static_datasets <- function(data_table, ids) {
    datasets <- lapply(ids, function(id) {
      data <- shiny::isolate(data_table[[id]]())
      as.vega(data, id)
    })
    unlist(datasets, recursive = FALSE)
  }

  datasets <- static_datasets(data_table, data_ids)
  scale_datasets <- static_datasets(scale_data_table, names(scale_data_table))
  check_scales_complete(x)

  # Each of these operations results in a more completely specified (and still
  # valid) ggvis object
  x <- add_missing_axes(x)
  x <- apply_axes_defaults(x)
  x <- add_missing_legends(x)
  x <- fortify_legends(x)
  x <- apply_legends_defaults(x)
  x <- add_default_options(x)

  spec <- list(
    data = c(datasets, scale_datasets),
    scales = lapply(unname(x$scales), as.vega),
    marks = lapply(x$marks, as.vega),
    legends = compact(lapply(x$legends, as.vega)),
    axes = compact(lapply(x$axes, as.vega)),
    padding = as.vega(x$options$padding),
    ggvis_opts = x$options,
    handlers = if (dynamic) x$handlers
  )

  structure(
    spec,
    data_table = data_table,
    scale_data_table = scale_data_table,
    controls = x$controls,
    connectors = x$connectors
  )
}

gather_scales <- function(x) {
  groups <- Filter(is.mark_group, x$marks)
  c(x$scales, unlist(pluck(groups, "scales"), recursive = FALSE))
}

#' @export
as.vega.mark_group <- function(x, ...) {
  this_scales <- vpluck(x$scales, "name", character(1))

  list(
    type = "group",
    properties = as.vega(x$props),
    from = list(data = data_id(x$data)),
    marks = lapply(x$marks, as.vega, in_group = TRUE),
    scales = lapply(unname(x$scales), as.vega),
    legends = lapply(x$legends, as.vega),
    axes = lapply(x$axes, as.vega)
  )
}


# Given a ggvis mark object, output a vega mark object
#' @export
as.vega.mark <- function(x, in_group = FALSE, ...) {
  data_id <- data_id(x$data)

  # Pull out key from props, if present
  key <- x$props$key
  x$props$key <- NULL

  # Add the custom ggvis properties set for storing ggvis-specific information
  # in the Vega spec.
  properties <- as.vega(x$props)
  properties$ggvis <- list()
  properties$ggvis$data <- list(value = data_id)

  group_vars <- dplyr::groups(shiny::isolate(x$data()))
  if (!in_group && length(group_vars)) {
    # FIXME: probably should go away and just use subvis

    # String representation of groups
    group_vars <- vapply(group_vars, deparse, character(1))

    m <- list(
      type = "group",
      from = list(data = data_id),
      marks = list(
        list(
          type = x$type,
          properties = properties
        )
      )
    )

  } else {
    m <- list(
      type = x$type,
      properties = properties
    )
    if (!in_group) {
      # If mark inside group, inherits data from parent.
      m$from <- list(data = data_id)
    }
  }

  if (!is.null(key)) {
    m$key <- paste0("data.", safe_vega_var(prop_label(key)))
  }
  m
}

#' @export
as.vega.ggvis_props <- function(x, default_scales = NULL, ...) {
  x <- prop_event_sets(x)

  # Given a list of property sets (enter, update, etc.), return appropriate
  # vega property set.
  vega_prop_set <- function(x) {
    if (empty(x)) return(NULL)

    props <- trim_prop_event(names(x))
    default_scales <- default_scales %||% propname_to_scale(props)
    Map(prop_vega, x, default_scales)
  }

  lapply(x, vega_prop_set)
}

#' @export
as.vega.ggvis_axis <- function(x, ...) {
  if (isTRUE(x$hide)) return(NULL)

  if (empty(x$properties)) {
    x$properties <- NULL
  } else {
    x$properties <- as.vega(x$properties)
  }

  unclass(x)
}
#' @export
as.vega.ggvis_legend <- as.vega.ggvis_axis

#' @export
as.vega.data.frame <- function(x, name, ...) {
  # Figure out correct vega parsers for non-string columns
  parsers <- drop_nulls(lapply(x, vega_data_parser))

  list(list(
    name = name,
    format = list(
      type = "csv",
      parse = parsers
    ),
    values = to_csv(x)
  ))
}

#' @export
as.vega.grouped_df <- function(x, name, ...) {
  # Create a flat data set and add a transform-facet data set which uses the
  # flat data as a source.
  group_vars <- vapply(dplyr::groups(x), deparse, character(1))
  res <- as.vega(dplyr::ungroup(x), paste0(name, "_flat"), ...)

  res[[length(res) + 1]] <- list(
    name = name,
    source = paste0(name, "_flat"),
    transform = list(list(
      type = "treefacet",
      keys = as.list(paste0("data.", safe_vega_var(group_vars)))
    ))
  )

  res
}

#' Stack overlapping data.
#'
#' @return A data frame with columns:
#'  \item{stack_upr_}{the lower y coordinate for a stack bar}
#'  \item{stack_lwr_}{the upper y coordinate for a stack bar}
#' @export
#' @param x A data object
#' @param stack_var A string specifying the stacking variable.
#' @param group_var A string specifying the grouping variable.
#' @examples
#' mtcars %>% cbind(count = 1) %>% compute_stack(~count, ~cyl)
#'
#' # Shouldn't use or affect existing grouping
#' mtcars %>% cbind(count = 1) %>% group_by(am) %>% compute_stack(~count, ~cyl)
#'
#' # If given a ggvis object, will use x variable for stacking by default
#' mtcars %>% ggvis(x = ~cyl, y = ~wt) %>%
#'   compute_stack(stack_var = ~wt, group_var = ~cyl) %>%
#'   layer_rects(x = ~cyl - 0.5, x2 = ~cyl + 0.5, y = ~stack_upr_,
#'     y2 = ~stack_lwr_)
#'
#' # Collapse across hair & eye colour data across sex
#' hec <- as.data.frame(xtabs(Freq ~ Hair + Eye, HairEyeColor))
#' hec %>% compute_stack(~Freq, ~Hair)
#'
#' # Without stacking - bars overlap
#' hec %>% ggvis(~Hair, ~Freq, fill = ~Eye, fillOpacity := 0.5) %>%
#'   layer_rects(y2 = 0, width = band())
#'
#' # With stacking
#' hec %>% ggvis(x = ~Hair, y = ~Freq, fill = ~Eye, fillOpacity := 0.5) %>%
#'   compute_stack(~Freq, ~Hair) %>%
#'   layer_rects(y = ~stack_lwr_, y2 = ~stack_upr_, width = band())
#'
#' # layer_bars stacks automatically:
#' hec %>% ggvis(~Hair, ~Freq, fill = ~Eye, fillOpacity := 0.5) %>%
#'   group_by(Eye) %>%
#'   layer_bars(width = 1)
compute_stack <- function(x, stack_var = NULL, group_var = NULL) {
  UseMethod("compute_stack")
}

#' @export
compute_stack.grouped_df <- function(x, stack_var = NULL, group_var = NULL) {
  assert_that(is.formula(stack_var), is.formula(group_var))

  # Save original groups, and restore after stacking
  old_groups <- dplyr::groups(x)

  x <- dplyr::ungroup(x)
  x <- compute_stack(x, stack_var, group_var)
  x <- dplyr::group_by(x, !!!old_groups)

  x
}

#' @export
compute_stack.data.frame <- function(x, stack_var = NULL, group_var = NULL) {
  assert_that(is.formula(stack_var), is.formula(group_var))

  # Round grouping variable to 8 significant digits
  x <- dplyr::group_by(x, group__ = round_fp(!!group_var[[2]]))

  # Quiet R CMD check note
  stack_upr_ <- NULL

  # FIXME: mutate evaluates in this function's environment, which isn't right.
  x <- dplyr::mutate(x,
    stack_upr_ = cumsum(!!stack_var[[2]]),
    stack_lwr_ = dplyr::lag(stack_upr_, default = 0)
  )

  dplyr::ungroup(x)
}

round_fp <- function(x) {
  if (!is.numeric(x)) return(x)
  signif(x, 8)
}

#' @export
compute_stack.ggvis <- function(x, stack_var = NULL, group_var = NULL) {
  args <- list(stack_var = stack_var, group_var = group_var)

  register_computation(x, args, "stack", function(data, args) {
    output <- do_call(compute_stack, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

context("compute_stack")

test_that("compute_stack works as expected", {
  dat <- data.frame(
    g1 = c("a", "a", "a", "b", "b", "c", "c", "c"),
    g2 = c(1, 2, 3, 1, 3, 1, 2, 3),
    val = 1:8
  )

  # Basic stacking: stack y value at each x
  stacked <- dat %>% compute_stack(~val, ~g1) %>% as.data.frame()
  expected <- data.frame(
    stack_lwr_ = c(0,1,3,0,4,0,6,13),
    stack_upr_ = c(1,3,6,4,9,6,13,21)
  )

  expect_equal(stacked[c("stack_lwr_", "stack_upr_")], expected)

  # With grouping
  stacked <- dat %>% group_by(g2) %>% compute_stack(~val, ~g1) %>%
    as.data.frame()
  expected <- expected[order(dat$g2), ] %>% as.data.frame()
  expected <- expected[order(dat$g2), ] %>% as.data.frame()
  expect_equal(stacked[c("stack_lwr_", "stack_upr_")], expected)
})


test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_stack(~wt, ~cyl)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c(names(mtcars), "group__", "stack_upr_", "stack_lwr_"))
  )

  # Grouped
  res <- mtcars %>% group_by(cyl) %>% dplyr::filter(FALSE) %>% compute_stack(~wt, ~cyl)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c(names(mtcars), "group__", "stack_upr_", "stack_lwr_"))
  )
})

library(ggvis)

mtcars %>%
  ggvis(x = ~wt) %>%
  layer_histograms(width = 1) %>%
  save_spec("layer/histogram.json")

mtcars %>%
  ggvis(x = ~wt, stroke = ~cyl) %>%
  group_by(cyl) %>%
  layer_freqpolys(width = 1) %>%
  save_spec("layer/freqpoly-grouped.json")

mtcars %>%
  ggvis(x = ~wt, y = ~mpg) %>%
  layer_smooths() %>%
  save_spec("layer/smooth.json")

mtcars %>%
  ggvis(x = ~wt, y = ~mpg) %>%
  group_by(cyl) %>%
  layer_smooths() %>%
  save_spec("layer/smooth-grouped.json")

#' Calculate boxplot values
#'
#' @param x Dataset-like object to compute boxplot values. There are built-in
#'   methods for data frames, grouped data frames, and ggvis visualisations.
#' @param var Name of variable for which to compute boxplot values. The variable
#'   must be continuous.
#' @param coef The maximum length of the whiskers as multiple of the
#'   inter-quartile range. Default value is 1.5.
#' @seealso \code{\link{layer_boxplots}}
#' @return A data frame with columns:
#'   \item{min_}{Lower whisker = smallest observation greater than or equal to lower hinge - 1.5 * IQR}
#'   \item{lower_}{Lower hinge (25th percentile)}
#'   \item{median_}{Median (50th percentile)}
#'   \item{upper_}{Upper hinge (75th percentile)}
#'   \item{max_}{Upper whisker = largest observation less than or equal to upper hinge + 1.5 * IQR}
#'   \item{outliers_}{A vector of values that are outside of the min and max}
#' @examples
#' mtcars %>% compute_boxplot(~mpg)
#' mtcars %>% group_by(cyl) %>% compute_boxplot(~mpg)
#' @export
compute_boxplot <- function(x, var = NULL, coef = 1.5) {
  UseMethod("compute_boxplot")
}

#' @export
compute_boxplot.grouped_df <- function(x, var = NULL, coef = 1.5) {
  dplyr::do(x, compute_boxplot(., var, coef))
}

#' @export
compute_boxplot.data.frame <- function(x, var = NULL, coef = 1.5) {
  vals <- eval_vector(x, var)

  # Special case zero-row input
  if (length(vals) == 0) {
    n0 <- numeric(0)
    res <- data.frame(
      min_ = n0, lower_ = n0, median_ = n0, upper_ = n0, max_ = n0
    )
    res$outliers_ <- list()
    return(res)
  }

  qs <- c(0, 0.25, 0.5, 0.75, 1)

  stats <- as.numeric(stats::quantile(vals, qs))
  names(stats) <- c("min_", "lower_", "median_", "upper_", "max_")

  iqr <- diff(stats[c(2, 4)])

  outliers <- vals < (stats[2] - coef * iqr) | vals > (stats[4] + coef * iqr)
  if (any(outliers)) {
    stats[c(1, 5)] <- range(c(stats[2:4], vals[!outliers]), na.rm = TRUE)
  }

  res <- as.data.frame(as.list(stats))
  res$outliers_ <- list(vals[outliers])
  res
}

#' @export
compute_boxplot.ggvis <- function(x, var = NULL, coef = 1.5) {
  args <- list(var = var, coef = coef)

  register_computation(x, args, "boxplot", function(data, args) {
    output <- do_call(compute_boxplot, quote(data), .args = args)
    preserve_constants(data, output)
  })
}



compute_boxplot_outliers <- function(x) UseMethod("compute_boxplot_outliers")

#' @export
compute_boxplot_outliers.grouped_df <- function(x) {
  dplyr::do(x, compute_boxplot_outliers(.))
}

#' @export
compute_boxplot_outliers.data.frame <- function(x) {
  data.frame(value_ = unlist(x$outliers_))
}

#' @export
compute_boxplot_outliers.ggvis <- function(x) {
  register_computation(x, args = NULL, "boxplot_outliers", function(data, args) {
    output <- compute_boxplot_outliers(data)
    preserve_constants(data, output)
  })
}

context("compute_boxplot")

test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_boxplot(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c("min_", "lower_", "median_", "upper_", "max_", "outliers_")
  ))
  expect_identical(res$outliers_, list())

  # Grouped
  res <- mtcars %>% group_by(cyl) %>% dplyr::filter(FALSE) %>% compute_boxplot(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c("cyl", "min_", "lower_", "median_", "upper_", "max_", "outliers_")
  ))
  expect_identical(res$outliers_, list())
})

# Convert a data object to a D3-structured data object.
df_to_d3json <- function(x) {
  rows <- nrow(x)
  colnames <- stats::setNames(names(x), names(x))

  x <- lapply(x, format_vec_d3json)

  lapply(seq_len(rows), function(i) {
    lapply(colnames, function(colname) {
      .subset2(.subset2(x, colname), i)
    })
  })
}

reactive_id <- function(x) {
  attr(x, "reactive_id", TRUE)
}

`reactive_id<-` <- function(x, value) {
  attr(x, "reactive_id") <- value
  x
}


# Pull reactives out of various types of objects
extract_reactives <- function(x) UseMethod("extract_reactives")
#' @export
extract_reactives.ggvis_props <- function(x) {
  compact(lapply(x, extract_reactives))
}
#' @export
extract_reactives.prop_reactive <- function(x) x$value
#' @export
extract_reactives.default <- function(x) NULL

# Get the value of a reactive or non-reactive object.
value <- function(x) UseMethod("value")
#' @export
value.default <- function(x) x
#' @export
value.reactive <- function(x) x()

values <- function(x) lapply(x, value)

#' Divide data into groups.
#'
#' @param x a visualisation
#' @param ... variables to group by.
#' @param add By default, when \code{add = FALSE}, \code{group_by} will
#'   override existing groups. To instead add to the existing groups,
#'   use \code{add = TRUE}
#' @importFrom dplyr group_by
#' @name group_by
#' @export
NULL


#' Dplyr verbs for ggvis.
#'
#' Applying a dplyr verb to a ggvis object creates a reactive transformation:
#' whenever the underlying data changes the transformation will be recomputed.
#'
#' @section Non-standard evaluation:
#' Both dplyr and shiny do non-standard evaluation, so to help each package
#' figure out when it should evaluate its code, reactive components in
#' these functions must be wrapped in \code{eval()}.
#'
#' @name dplyr-ggvis
#' @keywords internal
#' @examples
#' library(dplyr)
#' base <- mtcars %>% ggvis(~mpg, ~cyl) %>% layer_points()
#' base %>% group_by(cyl) %>% summarise(mpg = mean(mpg)) %>%
#'   layer_points(fill := "red", size := 100)
#'
#' base %>% filter(mpg > 25) %>% layer_points(fill := "red")
#'
#' base %>% mutate(cyl = jitter(cyl)) %>% layer_points(fill := "red")
#'
#' \dontrun{
#' # Dynamically restrict range using filter
#' mtcars %>% ggvis(~disp, ~mpg) %>%
#'    filter(cyl > eval(input_slider(0, 10))) %>%
#'    layer_points()
#'
#' # Dynamically compute box-cox transformation with mutate
#' bc <- function(x, lambda) {
#'   if (abs(lambda) < 1e-6) log(x) else (x ^ lambda - 1) / lambda
#' }
#' bc_slider <- input_slider(-2, 2, 1, step = 0.1)
#' mtcars %>%
#'  ggvis(~disp, ~mpg) %>%
#'  mutate(disp = bc(disp, eval(bc_slider))) %>%
#'  layer_points()
#' }
NULL

# Methods for ggvis objects ----------------------------------------------------

#' @importFrom dplyr groups
#' @export
#' @rdname dplyr-ggvis
groups.ggvis <- function(x) {
  shiny::isolate(dplyr::groups(x$cur_data()))
}

#' @importFrom dplyr group_by
#' @export
#' @rdname dplyr-ggvis
group_by.ggvis <- function(.data, ..., .add = FALSE) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "group_by", function(data, args) {
    dplyr::group_by(data, !!!lapply(pieces$lazy, add_args, args), .add = .add)
  })
}

#' @importFrom dplyr ungroup
#' @export
#' @rdname dplyr-ggvis
ungroup.ggvis <- function(x, ...) {
  register_computation(x, list(), "ungroup", function(data, args) {
    dplyr::ungroup(data)
  })
}

#' @importFrom dplyr summarise
#' @rdname dplyr-ggvis
#' @export
summarise.ggvis <- function (.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "summarise", function(data, args) {
    summarise(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr mutate
#' @rdname dplyr-ggvis
#' @export
mutate.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "mutate", function(data, args) {
    dplyr::mutate(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr arrange
#' @rdname dplyr-ggvis
#' @export
arrange.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "arrange", function(data, args) {
    dplyr::arrange(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr select
#' @rdname dplyr-ggvis
#' @export
select.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "select", function(data, args) {
    dplyr::select(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @rdname dplyr-ggvis
# exported on load
filter.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "filter", function(data, args) {
    dplyr::filter(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr distinct
#' @rdname dplyr-ggvis
#' @export
distinct.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "distinct", function(data, args) {
    dplyr::distinct(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr slice
#' @rdname dplyr-ggvis
#' @export
slice.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "slice", function(data, args) {
    dplyr::slice(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr rename
#' @rdname dplyr-ggvis
#' @export
rename.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "rename", function(data, args) {
    dplyr::rename(data, !!!lapply(pieces$lazy, add_args, args))
  })
}

#' @importFrom dplyr transmute
#' @rdname dplyr-ggvis
#' @export
transmute.ggvis <- function(.data, ...) {
  dots <- rlang::enquos(...)
  pieces <- extract_lazy_inputs(dots)

  register_computation(.data, pieces$inputs, "transmute", function(data, args) {
    dplyr::transmute(data, !!!lapply(pieces$lazy, add_args, args))
  })
}


#' Extract reactive inputs from a lazy dots.
#'
#' This works by replacing each reactive expression (which must be wrapped
#' in \code{eval}), with a reference to \code{args$xyz}. Then this function
#' returns both the modified lazy dots, and a list of reactives that need
#' to be created.
#'
#' @noRd
#' @examples
#' # extract_inputs() works with language objects ------------------------------
#'
#' # Simple expressions are returned as is
#' extract_inputs(quote(1))
#' extract_inputs(quote(x))
#' extract_inputs(quote(x + y))
#'
#' # If the call contains eval, then the subexpression is evalutes, the
#' # original is replaced with a reference to args, and inputs gains
#' # a reactive broker
#' extract_inputs(quote(eval(input_slider(0, 100))))
#'
#' slider <- input_slider(0, 100)
#' extract_inputs(quote(eval(slider) + eval(slider)))
#'
#' # extract_lazy_inputs() works with lazy objects -----------------------------
#' library(lazyeval)
#' extract_lazy_inputs(lazy(x + y))
#' extract_lazy_inputs(lazy(x + eval(input_slider(0, 100))))
#'
#' s1 <- input_slider(0, 100)
#' s2 <- input_slider(0, 200)
#' extract_lazy_inputs(lazy_dots(x + eval(s1), eval(s2), eval(s1) / eval(s2)))
extract_lazy_inputs <- function(x) {
  if (inherits(x, "quosures")) {
    pieces <- lapply(x, extract_lazy_inputs)

    lazy <- rlang::new_quosures(pluck(pieces, "lazy"))
    inputs <- unlist(unname(pluck(pieces, "inputs")), recursive = FALSE)
    inputs <- inputs[!duplicated(names(inputs))]

    list(
      lazy = lazy,
      inputs = inputs
    )
  } else if (inherits(x, "quosure")) {
    new <- extract_inputs(rlang::quo_get_expr(x), rlang::quo_get_env(x))

    x$expr <- new$expr
    list(lazy = x, inputs = new$inputs)
  } else {
    stop("Unknown input type: ", paste0(class(x), collapse = "/"),
      call. = FALSE)
  }
}

extract_inputs <- function(expr, env = parent.frame()) {
  if (is.name(expr) || is.atomic(expr)) {
    # Base case
    list(expr = expr, inputs = list())
  } else if (is.call(expr) && identical(expr[[1]], quote(eval))) {
    # If it's a call to eval, it's an input and should be evaluated
    stopifnot(length(expr) == 2)

    input <- eval(expr[[2]], env)
    stopifnot(is.broker(input))
    nm <- names(attr(input, "broker", TRUE)$controls)

    list(
      expr = substitute(args$nm, list(nm = as.name(nm))),
      inputs = stats::setNames(list(input), nm)
    )
  } else if (is.call(expr)) {
    # Recursive over arguments and join back together again
    args_out <- lapply(expr[-1], extract_inputs, env = env)

    args <- pluck(args_out, "expr")
    inputs <- unlist(pluck(args_out, "inputs"), recursive = FALSE)
    inputs <- inputs[!duplicated(names(inputs))]

    list(
      expr = as.call(c(expr[[1]], args)),
      inputs = inputs
    )
  } else {
    stop("Don't know how to deal with input of type: ", class(expr)[[1]],
      call. = FALSE)
  }
}

# Given a lazy object, modify it so it's environment also gets to
# access the args list.
add_args <- function(x, args) {
  e <- new.env(parent = rlang::quo_get_env(x))
  e$args <- args

  rlang::new_quosure(x$expr, e)
}



# Methods for reactive data frames --------------------------------------------

#' @rdname dplyr-ggvis
#' @export
groups.reactive <- function(x) reactive(dplyr::groups(x()))
#' @rdname dplyr-ggvis
#' @export
ungroup.reactive <- function(x, ...) reactive(dplyr::ungroup(x()))
#' @rdname dplyr-ggvis
#' @export
group_by.reactive <- function(.data, ..., add = FALSE) {
  reactive(dplyr::group_by(.data(), ..., add = add))
}
#' @rdname dplyr-ggvis
#' @export
summarise.reactive <- function(.data, ...) {
  reactive(dplyr::summarise(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
mutate.reactive <- function(.data, ...) {
  reactive(dplyr::mutate(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
arrange.reactive <- function(.data, ...) {
  reactive(dplyr::arrange(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
select.reactive <- function(.data, ...) {
  reactive(dplyr::select(.data(), ...))
}
#' @rdname dplyr-ggvis
# exported onLoad
filter.reactive <- function(.data, ...) {
  reactive(dplyr::filter(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
distinct.reactive <- function(.data, ...) {
  reactive(dplyr::distinct(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
slice.reactive <- function(.data, ...) {
  reactive(dplyr::slice(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
rename.reactive <- function(.data, ...) {
  reactive(dplyr::rename(.data(), ...))
}
#' @rdname dplyr-ggvis
#' @export
transmute.reactive <- function(.data, ...) {
  reactive(dplyr::transmute(.data(), ...))
}

#' singular.
#'
#' Use singular when you want constant x or y position.
#'
#' @export
#' @examples
#' mtcars %>% ggvis("", ~mpg) %>%
#'   layer_points() %>%
#'   scale_nominal("x") %>%
#'   add_axis("x", title = "", tick_size_major = 0)
#'
#' # OR
#' mtcars %>% ggvis("", ~mpg) %>%
#'   layer_points() %>%
#'   scale_singular("x")
#'
#' # OR, even simpler
#' mtcars %>% ggvis(singular(), ~mpg) %>% layer_points()
#'
#' # In the other direction:
#' mtcars %>% ggvis(~mpg, singular()) %>% layer_points()
#' @export
singular <- function() {
  structure("", class = "singular")
}

#' @export
rep.singular <- function(x, ...) {
  structure(NextMethod(), class = "singular")
}
#' @export
print.singular <- function(x, ...) cat("<singular>\n")

#' @export
as.data.frame.singular <- function(x, ...) {
  df <- list(x)
  attr(df, "row.names") <- .set_row_names(length(x))
  class(df) <- "data.frame"

  df
}

#' @export
vector_type.singular <- function(x) "singular"

#' @rdname singular
#' @export
#' @inheritParams scale_nominal
scale_singular <- function(vis, property, name = property, label = name,
                           points = TRUE, domain = NULL, override = NULL) {
  # Some of the arguments are ignored; they're there to provide a consistent
  # interface with other scales
  vis <- scale_nominal(vis, domain = "", property = property, name = name,
                           label = "", points = points, override = override)
  vis <- add_axis(vis, property, tick_size_major = 0)
  vis
}

ggvisLayout <- function(plot_id, has_controls = TRUE, spec = NULL,
                        shiny = TRUE) {
  plot_div <- ggvisOutputElements(plot_id, spec = spec, shiny = shiny)

  if (!has_controls) {
    plot_div
  } else {
    shiny::bootstrapPage(
      sidebarBottomPage(
        sidebarBottomPanel(ggvisControlOutput("ggvis_controls", plot_id)),
        mainTopPanel(plot_div)
      )
    )
  }
}

# This is the user-facing wrapper for ggvisOutputElements. When the user calls
# it, it will always be used with Shiny.
#' @rdname shiny-ggvis
#' @param plot_id unique identifier to use for the div containing the ggvis plot.
#' @export
ggvisOutput <- function(plot_id = rand_id("plot_id")) {
  ggvisOutputElements(plot_id, spec = NULL, shiny = TRUE)
}

#' Create HTML elements for ggvis output
#'
#' This is an internal-facing function similar to ggvisOutput, but with more
#' options.
#'
#' @param plot_id Unique identifier to use for the div containing the ggvis plot.
#' @param spec Plot specification, used internally.
#' @param shiny Should this include headers for Shiny? For dynamic and
#'   interactive plots, this should be TRUE; otherwise FALSE.
#' @keywords internal
ggvisOutputElements <- function(plot_id = rand_id("plot_id"), spec = NULL,
                                shiny = TRUE) {

  validate_plot_id(plot_id)

  htmltools::attachDependencies(
    htmltools::tagList(
      ggvisPlot(plot_id),
      ggvisSpec(plot_id, spec)
    ),
    c(
      ggvis_dependencies(),
      if (shiny) list(shiny_dependency())
    )
  )
}

ggvisPlot <- function(plot_id) {
  htmltools::div(id = paste0(plot_id, "-container"), class = "ggvis-output-container",
    # Div containing the plot
    htmltools::div(id = plot_id, class = "ggvis-output"),
    htmltools::div(class = "plot-gear-icon",
      ggvisControlGroup(plot_id)
    )
  )
}

ggvisSpec <- function(plot_id, spec = NULL) {
  if (is.null(spec)) return()
  json <- jsonlite::toJSON(spec, pretty = TRUE, auto_unbox = TRUE, force = TRUE,
                           null = "null")

  htmltools::tags$script(type = "text/javascript", paste0('\n',
    'var ', plot_id, '_spec = ', json, ';\n',
    'ggvis.getPlot("', plot_id, '").parseSpec(', plot_id, '_spec);\n'
  ))
}

# Controls drop down
ggvisControlGroup <- function(plot_id) {
  # The <a> tags need the onclick so that they work properly in Shiny Doc iframes
  htmltools::tags$nav(class = "ggvis-control",
    htmltools::tags$a(class = "ggvis-dropdown-toggle", title = "Controls",
                      onclick = "return false;"),
    htmltools::tags$ul(class = "ggvis-dropdown",
      htmltools::tags$li(
        "Renderer: ",
        htmltools::tags$a(
          id = paste0(plot_id, "_renderer_svg"),
          class = "ggvis-renderer-button",
          onclick = "return false;",
          `data-plot-id` = plot_id,
          `data-renderer` = "svg",
          "SVG"
        ),
        " | ",
        htmltools::tags$a(
          id = paste0(plot_id, "_renderer_canvas"),
          class = "ggvis-renderer-button",
          onclick = "return false;",
          `data-plot-id` = plot_id,
          `data-renderer` = "canvas",
          "Canvas"
        )
      ),
      htmltools::tags$li(htmltools::tags$a(
        id = paste0(plot_id, "_download"),
        class = "ggvis-download",
        `data-plot-id` = plot_id,
        "Download"
      ))
    )
  )
}


#' Create a page with a sidebar
#'
#' This creates a page with a sidebar, where the sidebar moves to the bottom
#' when the width goes below a particular value.
#'
#' @param sidebarPanel The \code{\link{sidebarBottomPanel}} containing input
#'   controls.
#' @param mainPanel The \code{\link{mainTopPanel}} containing the main content.
#' @param shiny_headers Should Shiny headers be embedded in the page? This
#'   should be TRUE for interactive/dynamic pages, FALSE for static pages.
#' @param ... Additional tags.
#' @export
#' @examples
#' sidebarBottomPage(sidebarBottomPanel(), mainTopPanel())
sidebarBottomPage <- function(sidebarPanel, mainPanel, shiny_headers = TRUE) {
  content <- htmltools::div(
    class = "container-fluid",
    htmltools::div(class = "row-fluid",
      mainPanel,
      sidebarPanel
    )
  )

  if (shiny_headers) {
    shiny::bootstrapPage(content)
  } else {
    content
  }
}

#' @export
#' @rdname sidebarBottomPage
sidebarBottomPanel <- function(...) {
  htmltools::div(class = "col-sm-4 sidebar-bottom",
    htmltools::tags$form(class = "well well-small",
      ...
    )
  )
}

#' @rdname sidebarBottomPage
#' @export
mainTopPanel <- function(...) {
  htmltools::div(class = "col-sm-8 main-top",
    ...
  )
}

#' Create a ggvis control output element in UI
#'
#' This is effectively the same as \code{\link[shiny]{uiOutput}}, except that
#' on the client side it may call some plot resizing functions after new
#' controls are drawn.
#'
#' \code{ggvisControlOutput} is intended to be used with
#' \code{\link{bind_shiny}} on the server side.
#'
#' @param outputId The output variable to read the value from.
#' @param plotId An optional plot ID or vector of plot IDs. The plots will
#'   have their .onControlOutput functions called after the controls are drawn.
#' @examples
#' ggvisControlOutput("plot1")
#' @export
ggvisControlOutput <- function(outputId, plotId = NULL) {
  if (is.null(plotId)) {
    htmltools::div(id = outputId, class = "ggvis-control-output")

  } else {
    htmltools::div(
      id = outputId,
      class = "ggvis-control-output",
      `data-plot-id` = paste(plotId, collapse = " ")
    )
  }
}

library(ggvis)

shinyServer(function(input, output, session) {
  mtcars <- cbind(mtcars, id = seq_len(nrow(mtcars)))

  # Create a linked brush object
  lb <- linked_brush(keys = mtcars$id, "red")

  # Just the brushed points
  selected <- lb$selected
  mtcars_selected <- reactive({
    if (!any(selected())) return(mtcars)
    mtcars[selected(), ]
  })

  mtcars %>%
    ggvis(~wt, ~mpg) %>%
    layer_points(fill := lb$fill, fill.brush := "red") %>%
    lb$input() %>%
    add_data(mtcars_selected) %>%
    layer_model_predictions(model = "lm")%>%
    bind_shiny("plot1")

  output$brush_data <- renderPrint({
    cat("Number of points selected: ", nrow(mtcars_selected()), "\n\n")
    print(summary(mtcars_selected()))
  })
})

#' Set options for a ggvis plot
#'
#' @param vis Visualisation to modify
#' @param width,height Width and height of plot, in pixels. Default is 600x400.
#'   \code{width} or \code{height} can also be \code{"auto"}, in which case the
#'   plot will size to fit in the containing div. This is useful only in a Shiny
#'   app or custom HTML output. Note that \code{height="auto"} should only be
#'   used when the plot is placed within a div that has a fixed height; if not,
#'   automatic height will not work, due to the way that web browsers do
#'   vertical layout.
#' @param keep_aspect Should the aspect ratio be preserved? The default value is
#'   \code{FALSE}, or the value of \code{getOption("ggvis.keep_aspect")}, if it
#'   is set.
#' @param resizable If TRUE, allow the user to resize the plot. The default
#'   value is \code{TRUE}, or the value of \code{getOption("ggvis.resizable")},
#'   if it is set. Not compatible when \code{width} or \code{height} is
#'   \code{"auto"}.
#' @param padding A padding object specifying padding on the top, right, left,
#'   and bottom. See \code{\link{padding}}.
#' @param duration Duration of transitions, in milliseconds.
#' @param renderer The renderer to use in the browser. Can be \code{"canvas"} or
#'   \code{"svg"} (the default).
#' @param hover_duration The amount of time for hover transitions, in
#'   milliseconds.
#'
#' @seealso \code{\link{getOption}} and \code{\link{options}}, for getting and
#'   setting global options.
#' @seealso \code{\link{default_options}} to see the default options.
#' @examples
#' mtcars %>%
#'   ggvis(~wt, ~mpg) %>%
#'   layer_points() %>%
#'   set_options(width = 300, height = 200, padding = padding(10, 10, 10, 10))
#'
#' # Display the default options
#' str(default_options())
#'
#' @export
set_options <- function(vis, width = NULL, height = NULL, keep_aspect = NULL,
                 resizable = NULL, padding = NULL, duration = NULL,
                 renderer = NULL, hover_duration = NULL) {

  options <- compact(list(
    width = shiny::validateCssUnit(width),
    height = shiny::validateCssUnit(height),
    keep_aspect = keep_aspect,
    resizable = resizable,
    padding = padding,
    duration = duration,
    renderer = renderer,
    hover_duration = hover_duration
  ))

  add_options(vis, options)
}

#' Default options
#'
#' This returns an object containing the default options for ggvis.
#'
#' @export
#' @keywords internal
default_options <- function() {
  list(
    width = 600,
    height = 400,
    keep_aspect = getOption("ggvis.keep_aspect", FALSE),
    resizable = getOption("ggvis.resizable", TRUE),
    padding = padding(),
    duration = 250,
    renderer = getOption("ggvis.renderer", "svg"),
    hover_duration = 0
  )
}

# Given a ggvis_opts object, merge it into default options
add_default_options <- function(vis) {
  add_options(vis, default_options(), replace = FALSE)
}

#' Define padding.
#'
#' @param top,right,bottom,left Amount of padding on each border. Can either
#'   be a single number, "auto", or "strict"
#' @export
#' @examples
#' p <- mtcars %>% ggvis(~wt, ~mpg) %>% layer_points()
#' p %>% set_options(padding = padding())
#' p %>% set_options(padding = padding(10, 10, 10, 10))
padding <- function(top = NULL, right = NULL, bottom = NULL, left = NULL) {
  assert_that(is_padding(top), is_padding(right), is_padding(bottom),
    is_padding(left))

  structure(
    compact(list(
      top = top,
      right = right,
      bottom = bottom,
      left = left)
    ),
    class = "padding"
  )
}

#' @export
as.vega.padding <- function(x, ...) {
  if (length(x) == 0) {
    NULL
  } else {
    unclass(x)
  }
}

is_padding <- function(x) {
  if (is.null(x)) return(TRUE)

  length(x) == 1 && (is.numeric(x) || (x %in% c("auto", "strict")))
}
on_failure(is_padding) <- function(call, env) {
  paste0(deparse2(call$x), " is not a single number, 'auto', or 'strict'.")
}

.onAttach <- function(...) {
  if (!interactive() || stats::runif(1) > 0.1) return()
  packageStartupMessage("The ggvis API is currently rapidly evolving. ",
    "We strongly recommend that you do not rely on this for production, but ",
    "feel free to explore. If you encounter a clear bug, please file a ",
    "minimal reproducible example at https://github.com/rstudio/ggvis/issues. ",
    "For questions and other discussion, please use ",
    "https://groups.google.com/group/ggvis.")
}

.onLoad <- function(libname, pkgname) {
  register_s3_method("dplyr", "filter", "ggvis")
  register_s3_method("dplyr", "filter", "reactive")

  # ggvis provides methods for knitr::knit_print, but knitr isn't a Depends or
  # Imports of ggvis, only a Suggests. This code snippet manually registers
  # our method(s) with S3 once both ggvis and knitr are loaded.
  register_s3_method("knitr", "knit_print", "ggvis")
}

register_s3_method <- function(pkg, generic, class, fun = NULL) {
  stopifnot(is.character(pkg), length(pkg) == 1)
  stopifnot(is.character(generic), length(generic) == 1)
  stopifnot(is.character(class), length(class) == 1)

  if (is.null(fun)) {
    fun <- get(paste0(generic, ".", class), envir = parent.frame())
  } else {
    stopifnot(is.function(fun))
  }

  if (pkg %in% loadedNamespaces()) {
    registerS3method(generic, class, fun, envir = asNamespace(pkg))
  }

  # Always register hook in case package is later unloaded & reloaded
  setHook(
    packageEvent(pkg, "onLoad"),
    function(...) {
      registerS3method(generic, class, fun, envir = asNamespace(pkg))
    }
  )
}

#' Align positions using length.
#'
#' This compute function is often used in conjunction with
#' \code{\link{compute_count}}, when used on data with a continuous x variable.
#' By default, the computed width will be equal to the resolution of the data,
#' or, in other words the smallest difference between two values in the data.
#'
#' An absolute width for each x can be specified by using the \code{width}
#' argument. If \code{width} is NULL (the default), it will use the resolution
#' of the data as the width.
#'
#' @param x Dataset-like object to align. Built-in methods for data frames,
#'   grouped data frames and ggvis visualisations.
#' @param var Name of variable to compute width of.
#' @param length An absolute length to use. If \code{NULL} (the default), the
#'   width will be equivalent to the resolution of the data.
#' @param dir Direction, i.e. \code{"x"} or \code{"y"}. Used to generate
#'   variable names in output.
#' @param align Where does the existing variable fall on the new bins?
#'   0 = left edge, 0.5 = center, 1 = right edge.
#' @seealso \code{\link{compute_bin}} For counting cases within ranges of
#'   a continuous variable.
#' @seealso \code{\link{compute_count}} For counting cases at specific values
#'   of a variable.
#' @export
#' @return The original data frame, with additional columns:
#'  \item{'dir'min_}{left boundary of bin}
#'  \item{'dir'max_}{right boundary of bin}
#'  \item{'dir'len_}{width of bin}
#'
#' @examples
#' mtcars %>% compute_count(~disp) %>% compute_align(~x_)
#' mtcars %>% compute_count(~mpg) %>% compute_align(~x_)
#'
#' # Use a specific width
#' pressure %>% compute_count(~temperature) %>% compute_align(~x_)
#' pressure %>% compute_count(~temperature) %>% compute_align(~x_, length = 5)
#'
#' # It doesn't matter whether you transform inside or outside of a vis
#' mtcars %>% compute_count(~cyl, ~wt) %>%
#'   compute_align(~x_, length = .5) %>%
#'   ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) %>%
#'   layer_rects()
#'
#' mtcars %>%
#'   ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) %>%
#'   compute_count(~cyl, ~wt) %>%
#'   compute_align(~x_) %>%
#'   layer_rects()
#'
#' # Varying align
#' mtcars %>%
#'   ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) %>%
#'   compute_count(~cyl, ~wt) %>%
#'   compute_align(~x_, length = 0.5, align = input_slider(0, 1)) %>%
#'   layer_rects()
compute_align <- function(x, var, length = NULL, align = 0.5, dir = "x") {
  UseMethod("compute_align")
}

#' @export
compute_align.data.frame <- function(x, var, length = NULL, align = 0.5,
                                     dir = "x") {
  assert_that(is.formula(var))

  val <- eval_vector(x, var)
  if (vector_countable(val)) {
    stop("compute_align requires continuous data.")
  }

  if (is.null(length)) {
    length <- resolution(val)
  }

  x[paste0(dir, "min_")] <- val - length * align
  x[paste0(dir, "max_")] <- val + length * (1 - align)
  x[paste0(dir, "len_")] <- length

  x
}

#' @export
compute_align.ggvis <- function(x, var, length = NULL, align = 0.5, dir = "x") {
  args <- list(var = var, length = length, align = align, dir = dir)

  register_computation(x, args, "align", function(data, args) {
    output <- do_call(compute_align, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

#' Vega marks.
#'
#' These functions create mark objects, corresponding to vega marks. Marks
#' are leaves in the plot tree, and control the details of the final rendering.
#' Marks are equivalent to the basic geoms in ggplot2 (e.g. point, line,
#' polygon), where ggvis layers correspond to combinations of geoms and
#' statistical transforms.
#'
#' Note that by supplying a fill property to \code{mark_path} will produce
#' a filled property. \code{mark_point} is an alias to \code{mark_symbol}.
#'
#' @template properties
#' @param vis Visualisation to modify
#' @name marks
#' @param props,... A \code{\link{props}} object, named according to the
#'   properties listed below.
#' @param data An optional dataset, if you want to override the usual data
#'   inheritance for this mark.
NULL

#' @rdname marks
#' @export
emit_points <- function(vis, props) {
  add_mark(vis, "symbol", props)
}
#' @rdname marks
#' @export
layer_points <- function(vis, ..., data = NULL) {
  add_mark(vis, "symbol", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

#' @rdname marks
#' @export
emit_images <- function(vis, props) {
  add_mark(vis, "image", props)
}
#' @rdname marks
#' @export
layer_images <- function(vis, ..., data = NULL) {
  add_mark(vis, "image", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

#' @rdname marks
#' @export
emit_arcs <- function(vis, props) {
  add_mark(vis, "arc", props)
}
#' @rdname marks
#' @export
layer_arcs <- function(vis, ..., data = NULL) {
  add_mark(vis, "arc", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}


#' @rdname marks
#' @export
emit_ribbons <- function(vis, props) {
  add_mark(vis, "area", props)
}
#' @rdname marks
#' @export
layer_ribbons <- function(vis, ..., data = NULL) {
  add_mark(vis, "area", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

#' @rdname marks
#' @export
emit_paths <- function(vis, props) {
  add_mark(vis, "line", props)
}
#' @rdname marks
#' @export
layer_paths <- function(vis, ..., data = NULL) {
  add_mark(vis, "line", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

#' @rdname marks
#' @export
emit_rects <- function(vis, props) {
  add_mark(vis, "rect", props)
}
#' @rdname marks
#' @export
layer_rects <- function(vis, ..., data = NULL) {
  add_mark(vis, "rect", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

#' @rdname marks
#' @export
emit_text <- function(vis, props) {
  add_mark(vis, "text", props)
}
#' @rdname marks
#' @export
layer_text <- function(vis, ..., data = NULL) {
  add_mark(vis, "text", props(..., env = parent.frame()), data,
    deparse2(substitute(data)))
}

common_valid_props <- c("x", "y", "stroke", "strokeOpacity", "fill",
  "fillOpacity", "opacity", "strokeWidth", "strokeDash", "key")

valid_props <- list(
  arc = c(common_valid_props, "innerRadius", "outerRadius", "startAngle",
    "endAngle"),
  area = c(common_valid_props, "y2", "height", "interpolate", "tension"),
  image = c(common_valid_props, "x2", "y2", "width", "height", "url", "align",
    "baseline"),
  line = c(common_valid_props, "interpolate", "tension"),
  rect = c(common_valid_props, "x2", "y2", "width", "height"),
  symbol = c(common_valid_props, "size", "shape"),
  text = c(common_valid_props, "text", "align", "baseline", "dx", "dy", "angle",
    "font", "fontSize", "fontWeight", "fontStyle")
)

# Hack to stop spurious warnings in R CMD check. Used in prop.
known_props <- sort(unique(unlist(valid_props)))
globalVariables(known_props)

# Some marks need more detailed validity checks of their props
mark_props_validity_checks <- list(
  image = function(props) {
    url <- props$url.update

    if (!is.null(url) &&
        is.prop_constant(url) &&
        !grepl("http(s)?://", url$value)) {

      warning("image mark's url prop '", props$url.update$value,
        "' should be an absolute URL (http://... or https://...).",
        " Referencing local files for url not yet implemented.")
    }
  }
)

default_props <- function(type) {
  switch(type,
    arc = props(fill := "#333333"),
    area = props(fill := "#333333"),
    line = props(stroke := "#000000"),
    image = props(fill := "#000000"),
    rect = props(stroke := "#000000", fill := "#333333"),
    symbol = props(fill := "#000000", size := 50),
    text = props(fill := "#333333"),
    stop("Unknown type")
  )
}

context("Flatten")

test_that("props inherited from parent", {
  p <- data.frame() %>% ggvis(x := 1) %>%
    add_props(y := 2) %>%
    layer_paths(x := 3)

  props <- p$marks[[1]]$props

  expect_equal(sort(names(props)), c("stroke.update", "x.update", "y.update"))
  expect_equal(props$x.update$value, 3)
  expect_equal(props$y.update$value, 2)
})

test_that("no data is an error", {
  base <- NULL %>% ggvis(x = ~x, y = ~y)
  expect_error(base %>% layer_paths(), "No data supplied to mark")
})

test_that("reactive source data only run once", {
  runs <- 0
  df <- data.frame(x = 1, y = 2)
  rdf <- shiny::reactive({
    runs <<- runs + 1
    df
  })

  p <- rdf %>% ggvis(~x, ~y) %>% layer_paths() %>% layer_points()
  expect_equal(length(p$data), 1)

  out_df <- shiny::isolate(p$data[[1]]())
  expect_equal(out_df, df)
  expect_equal(runs, 1)
})

context("is.dynamic")
library(shiny)

test_that("regular plot is not dynamic", {
  p <- mtcars %>% ggvis(x = ~wt, y = ~cyl) %>% layer_points()
  expect_false(is.dynamic(p))
})

test_that("plot with reactive data source is dynamic", {
  p <- shiny::reactive(mtcars) %>% ggvis(x = ~wt, y = ~cyl) %>% layer_points()
  expect_true(is.dynamic(p))
})

test_that("plot with reactive transform param is dynamic", {
  p <- mtcars %>% ggvis(~mpg, ~wt) %>% layer_smooths(span = input_slider(0, 1))
  expect_true(is.dynamic(p))
})

#' Manage a list of properties.
#'
#' \code{props()} provides a tool for concise creation of \code{prop} objects
#' using a set of conventions designed to capture the most common use cases.
#' If you need something less common, you'll need to use \code{\link{prop}} to
#' access all possible options.
#'
#' @section Heuristics:
#'
#' If the values are not already objects of class \code{prop}, \code{props}
#' uses the following heuristics to when creating the prop:
#'
#' \itemize{
#'  \item atomic vectors, e.g. \code{x = 1}: scaled = FALSE
#'  \item an interative input, e.g. \code{x = input_slider}:
#'     scaled = FALSE
#'  \item a formula containing a single value, e.g. \code{x ~ 1}:
#'     scaled = TRUE
#'  \item a formula containing a name or expression, \code{x ~ mpg}:
#'     scaled = TRUE
#' }
#'
#' @section Non-standard evaluation:
#'
#' \code{props} uses non-standard evaluation in a slightly unusual way:
#' if you provide a formula input, the LHS of the formula will provide the
#' name of the component. In otherwise, \code{props(x = y ~ 1)} is the
#' same as \code{props(y ~ 1)}.
#'
#' You can combine variables from the dataset and variables defined in the
#' local environment: expressions will be evaluated in the environment which
#' the formula was defined.
#'
#' If you have the name of a variable in a string, see the
#' props vignette for how to create the needed property mapping.
#'
#' @section Enter, exit, hover, and update events:
#'
#' There are four different property events that the marks
#' can use. These can, for example, be used to change the appearance of a mark
#' when the mouse cursor is hovering over it: when the mark is hovered over, it
#' uses the hover event, and when the mark isn't hovered over, it uses the
#' update event
#'
#' \itemize{
#'   \item enter: This event is used by marks when they are added to a plot.
#'   \item update: This event is used by marks after they have entered, and
#'     also after they have been hovered over.
#'   \item exit: This event is used by marks as they are removed from a plot.
#'   \item hover: This event is used when the mouse cursor is over the mark.
#' }
#'
#' You can specify the event for a property, by putting a period and the
#' event after the property name. For example,
#' \code{props(fill.update := "black", fill.hover := "red")} will make a mark
#' have a black fill normally, and red fill when it is hovered over.
#'
#' The default event is update, so if you run \code{props(fill := "red")},
#' this is equivalent to \code{props(fill.update := "red")}.
#'
#' In practice, the enter and exit events are useful only when the update has
#' a duration (and is therefore not instantaneous). The update event can be
#' thought of as the "default" state.
#'
#' @section Key property:
#'
#' In addition to the standard properties, there is a special optional property
#' called \code{key}. This is useful for plots with dynamic data and smooth
#' transitions: as the data changes, the key is used to tell the plot how the
#' new data rows should be matched to the old data rows. Note that the key must
#' be an unscaled value. Additionally, the key property doesn't have a event,
#' since it is independent of enter, update, exit, and hover events.
#'
#' @template properties
#' @param ... A set of name-value pairs. The name should be a valid vega
#'   property.
#'
#'   The first two unnamed components are taken to be \code{x} and \code{y}.
#'   Any additional unnamed components will raise an error.
#' @param .props When calling \code{props} from other functions, you'll
#'   often have a list of quoted function functions. You can pass that function
#'   to the \code{.props} argument instead of messing around with
#'   substitute. In other words, \code{.props} lets you opt out of the
#'   non-standard evaluation that \code{props} does.
#' @param inherit If \code{TRUE}, the defaults, will inherit from properties
#'   from the parent layer If \code{FALSE}, it will start from nothing.
#' @param env The environment in which to evaluate variable properties.
#' @export
#' @examples
#' # Set to constant values
#' props(x := 1, y := 2)
#' # Map to variables in the dataset
#' props(x = ~mpg, y = ~cyl)
#' # Set to a constant value in the data space
#' props(x = 1, y = 1)
#' # Use an interactive slider
#' props(opacity := input_slider(0, 1))
#'
#' # To control other settings (like custom scales, mult and offset)
#' # use a prop object
#' props(prop("x", "old", scale = "x", offset = -1))
#'
#' # Red when hovered over, black otherwise (these are equivalent)
#' props(fill := "black", fill.hover := "red")
#' props(fill.update := "black", fill.hover := "red")
#'
#' # Use a column called id as the key (for dynamic data)
#' props(key := ~id)
#'
#' # Explicitly create prop objects. The following are equivalent:
#' props(fill = ~cyl)
#' props(fill.update = ~cyl)
#' props(prop("fill", ~cyl))
#' props(prop("fill", ~cyl, scale = "fill", event = "update"))
#'
#' # Prop objects can be programmatically created and added:
#' property <- "fill"
#' expr <- parse(text = "wt/mpg")[[1]]
#' p <- prop(property, expr)
#' props(p)
#'
#' # Using .props
#' props(.props = list(x = 1, y = 2))
#' props(.props = list(x = ~mpg, y = ~cyl))
#' props(.props = list(quote(x := ~mpg)))
props <- function(..., .props = NULL, inherit = TRUE, env = parent.frame()) {
  check_empty_args()

  args <- dots(...)
  all <- args_to_props(c(args, .props), env)

  structure(
    all,
    inherit = inherit,
    class = "ggvis_props"
  )
}

dots <- function(...) {
  as.list(substitute(...()))
}

uses_colon_equals <- function(x) {
  is.call(x) && identical(x[[1]], quote(`:=`))
}

# Given a list of unevaluated expressions, return a list of prop objects
args_to_props <- function(args, env) {
  # Given a name and expression, create a prop
  expr_to_prop <- function(name, expr, scale = NULL) {
    name <- strsplit(name, ".", fixed = TRUE)[[1]]
    property <- name[1]
    event <- if (length(name) > 1) name[2] else NULL
    val <- eval(expr, env)
    prop(property, val, scale = scale, event = event, label = as.character(val))
  }

  # Given a prop, get the full name, like x.update
  prop_full_name <- function(p) {
    # Use this form so that if event is NULL, there won't be trailing .
    paste(c(p$property, p$event), collapse = ".")
  }

  arg_names <- names2(args)
  named_args <- args[arg_names != ""]
  unnamed_args <- args[arg_names == ""]

  # First pass: Convert named arguments to props
  named_args <- Map(named_args, names(named_args),
    f = function(x, name) expr_to_prop(name, x, scale = TRUE)
  )

  # Second pass: Convert unnamed arguments to prop objects, or raw value
  unnamed_args <- lapply(unnamed_args, function(x) {
    if (uses_colon_equals(x)) {
      expr_to_prop(deparse(x[[2]]), x[[3]], scale = FALSE)
    } else {
      # It's either prop() call, or an unnamed value
      eval(x, env)
    }
  })

  # Some of the unnamed items are now props, others are raw values.
  is_prop <- vapply(unnamed_args, is.prop, logical(1))
  unnamed_props <- unnamed_args[is_prop]
  unnamed_values <- unnamed_args[!is_prop]

  # Assign full name, like x.update
  names(named_args) <- vapply(named_args, prop_full_name, character(1))
  names(unnamed_props) <- vapply(unnamed_props, prop_full_name, character(1))

  # Assign missing names to unnamed values
  missing_names <- setdiff(c("x.update", "y.update"),
                           c(names(named_args), names(unnamed_props)))
  if (length(unnamed_values) > length(missing_names)) {
    stop("Too many unnamed properties (can only have x and y)", call. = FALSE)
  }
  names(unnamed_values) <- missing_names[seq_along(unnamed_values)]

  # Final pass: Convert the now-named values to props
  unnamed_values <- Map(unnamed_values, names(unnamed_values),
    f = function(x, name) expr_to_prop(name, x, scale = TRUE)
  )

  c(named_args, unnamed_props, unnamed_values)
}

#' @export
format.ggvis_props <- function(x, ...) {
  labels <- lapply(x, format, ...)
  if (length(labels) > 0) {
    inherit <- if (!attr(x, "inherit", TRUE)) "\ninherit: FALSE" else ""
    paste0(
      paste0("* ", names(x), ": ", labels, collapse = "\n"),
      inherit,
      sep = "\n")
  } else {
    "props()"
  }
}
#' @export
print.ggvis_props <- function(x, ...) cat(format(x, ...))

#' @rdname props
#' @export
#' @param x an object to test for props-ness.
is.ggvis_props <- function(x) inherits(x, "ggvis_props")

#' @export
`[.ggvis_props` <- function(x, idx) {
  structure(NextMethod(), inherit = attr(x, "inherit", TRUE), class = "ggvis_props")
}

# Merge two ggvis_props objects
#
# merge_props(props(x = ~x))
# merge_props(props(x = ~x), props(x = ~y))
# merge_props(props(x = ~x, y = 1), props(x = ~y))
# merge_props(props(x = ~x, y = 1), props(x = ~y, inherit = FALSE))
merge_props <- function(parent = NULL, child = NULL,
                        inherit = attr(child, "inherit", TRUE)) {
  if (is.null(parent)) return(child)
  if (is.null(child)) return(parent)
  stopifnot(is.ggvis_props(parent), is.ggvis_props(child))

  if (identical(inherit, FALSE)) return(child)

  structure(merge_vectors(parent, child),
    inherit = attr(parent, "inherit", TRUE),
    class = "ggvis_props")
}

is.formula <- function(x) inherits(x, "formula")

find_prop_var <- function(props, name) {
  prop <- props[[name]]
  if (is.null(prop)) {
    stop("Can't find prop ", name, call. = FALSE)
  }

  if (!is.prop_variable(prop)) {
    stop("Visual property ", name, " is not a variable", call. = FALSE)
  }

  stats::formula(prop)
}

#' Export a PNG or SVG from a ggvis object
#'
#' This requires that the external program \code{vg2png} is installed. This is
#' part of the \code{vega} node.js module.
#'
#' @seealso \url{https://github.com/trifacta/vega} for information on installing
#'   \code{vg2png} and \code{vg2svg}.
#'
#' @param vis A ggvis object.
#' @param file Output file name. If NULL, defaults to "plot.svg" or "plot.png".
#' @examples
#' \dontrun{
#' mtcars %>% ggvis(x = ~wt) %>% export_png()
#' }
#' @export
export_png <- function(vis, file = NULL) {
  vega_file(vis, file = file, type = "png")
}

#' @rdname export_png
#' @export
export_svg <- function(vis, file = NULL) {
  vega_file(vis, file = file, type = "svg")
}

# Generate an output image file from a ggvis object
#
# @param vis A ggvis object.
# @param file Output file name. If NULL, defaults to "plot.png".
# @param type Output file type.
vega_file <- function(vis, file = NULL, type = "png") {

  if (!(type %in% c("png", "svg")))  stop("type must be 'png' or 'svg'")

  if (is.null(file)) {
    file <- paste0("plot.", type)
    message("Writing to file ", file)
  }

  temp_dir <- tempfile(pattern = "ggvis")
  dir.create(temp_dir)

  # Try to find the external program that generates the images
  cmd <- paste0("vg2", type)

  # Search in these paths and use the first one found
  cmdsearch <- Sys.which(paste0(c("", "./bin/", "./node_modules/.bin/"), cmd))
  found_idx <- which(nzchar(cmdsearch))
  if (length(found_idx) == 0)
    stop("Conversion program ", cmd, "not found.")
  cmd <- cmdsearch[min(found_idx)]

  # Generate the Vega JSON spec
  json_file <- file.path(temp_dir, "plot.json")
  vega_json <- save_spec(vis, json_file)
  on.exit(unlink(json_file))

  # Create the image file
  system2(cmd, args = c(json_file, file))
}

#' Add a scale to a ggvis plot
#'
#' This creates a scale object for a given scale and variable type, and adds it
#' to a ggvis plot. The scale object is populated with default settings, which
#' depend on the scale (e.g. fill, x, opacity) and the type of variable (e.g.
#' numeric, nominal, ordinal). Any settings that are passed in as arguments
#' will override the defaults.
#'
#' @section Scale selection:
#'
#' ggvis supports the following types of scales. Typical uses for each scale
#' type are listed below:
#' \itemize{
#'   \item numeric For continuous numeric values.
#'   \item nominal For character vectors and factors.
#'   \item ordinal For ordered factors (these presently behave the same as
#'     nominal).
#'   \item logical For logical (TRUE/FALSE) values.
#'   \item datetime For dates and date-times.
#' }
#'
#' Each type has a corresponding function: \code{scale_numeric},
#' \code{scale_nominal}, and so on.
#'
#' The scale types for ggvis are mapped to scale types for Vega, which include
#' "ordinal", "quantitative", and "time". See \code{\link{ggvis_scale}} for more
#' details.
#'
#' Given a scale and type, the range is selected based on the combination of the
#' \code{scale} and \code{type}. For example, you get a different range of
#' colours depending on whether the data is numeric, ordinal, or nominal. Some
#' scales also set other properties. For example, nominal/ordinal position
#' scales also add some padding so that points are spaced away from plot edges.
#'
#' Not all combinations have an existing default scale. If you use a
#' combination that does not have an existing combination, it may suggest
#' you're displaying the data in a suboptimal way. For example, there is
#' no default for a numeric shape scale, because there's no obvious way to
#' map continuous values to discrete shapes.
#'
#' @param vis A ggvis object.
#' @param scale The name of a scale, such as "x", "y", "fill", "stroke", etc.
#' @param type A variable type. One of "numeric", "nominal", "ordinal",
#'   "logical", "datetime".
#' @param ... other arguments passed to the scale function. See the help for
#'   \code{\link{scale_numeric}}, \code{\link{scale_ordinal}} and
#'   \code{\link{scale_datetime}} for more details. For example, you might
#'   supply \code{trans = "log"} to create a log scale.
#' @param name If \code{NULL}, the default, the scale name is the same as
#'   \code{scale}. Set this to a custom name to create multiple scales for
#'   stroke or fill, or (god forbid) a secondary y scale.
#' @examples
#' p <- mtcars %>%
#'   ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl), stroke = ~hp) %>%
#'   layer_points()
#'
#' p %>% scale_numeric("x")
#' p %>% scale_numeric("stroke")
#' p %>% scale_nominal("fill")
#'
#' # You can also supply additional arguments or override the defaults
#' p %>% scale_numeric("x", trans = "log")
#' p %>% scale_numeric("stroke", range = c("red", "blue"))
#' @name scales
#' @aliases set_default_scale set_dscale
NULL

#' Add a numeric scale to a ggvis object.
#'
#' A numeric (quantitative) scale controls the mapping of continuous variables
#' to visual properties.
#'
#' The default values for most of the arguments is NULL. When the plot is
#' created, these NULL values will be replaced with default values, as indicated
#' below.
#'
#' @param vis A ggvis object.
#' @param property The name of a visual property, such as "x", "y", "fill",
#'   "stroke". Note both x and x2 use the "x" scale (similarly for y and y2).
#'   fillOpacity, opacity and strokeOpacity use the "opacity" scale.
#' @inheritParams ggvis_scale
#' @param trans A scale transformation: one of "linear", "log", "pow", "sqrt",
#'   "quantile", "quantize", "threshold". Default is "linear".
#' @param exponent Sets the exponent of the scale transformation. For pow
#'   transform only.
#' @param clamp  If \code{TRUE}, values that exceed the data domain are clamped
#'   to either the minimum or maximum range value. Default is \code{FALSE}.
#' @param nice If \code{TRUE}, modifies the scale domain to use a more
#'   human-friendly number range (e.g., 7 instead of 6.96). Default is
#'   \code{FALSE}.
#' @param zero If \code{TRUE}, ensures that a zero baseline value is included
#'   in the scale domain. This option is ignored for non-quantitative scales.
#'   Default is \code{FALSE}.
#' @param expand A multiplier for how much the scale should be expanded beyond
#'   the domain of the data. For example, if the data goes from 10 to 110, and
#'   \code{expand} is 0.05, then the resulting domain of the scale is 5 to 115.
#'   Set to 0 and use \code{nice=FALSE} if you want exact control over the
#'   domain. If left \code{NULL}, behavior will depend on the scale type. For
#'   positional scales (x and y), \code{expand} will default to 0.05. For other
#'   scales, it will default to 0.
#' @seealso \code{\link{scales}}, \code{\link{scale_ordinal}},
#'   \url{https://vega.github.io/vega/docs/scales/#quantitative}
#' @family scales
#' @export
#' @examples
#' p <- mtcars %>% ggvis(~wt, ~mpg, fill = ~hp) %>% layer_points()
#'
#' p %>% scale_numeric("y")
#'
#' p %>% scale_numeric("y", trans = "pow", exponent = 0.5)
#'
#' p %>% scale_numeric("y", trans = "log")
#'
#' # Can control other properties other than x and y
#' p %>% scale_numeric("fill", domain = c(0, 120), clamp = TRUE)
#'
#' # Set range of data from 0 to 3
#' p %>% scale_numeric("x", domain = c(0, 3), clamp = TRUE, expand = 0,
#'                      nice = FALSE)
#'
#' # Lower bound is set to lower limit of data, upper bound set to 3.
#' p %>% scale_numeric("x", domain = c(NA, 3), clamp = TRUE, nice = FALSE)
scale_numeric <- function(vis, property, domain = NULL, range = NULL,
                          reverse = NULL, round = NULL,
                          trans = NULL, clamp = NULL, exponent = NULL,
                          nice = NULL, zero = NULL, expand = NULL,
                          name = property, label = NULL, override = NULL) {
  assert_that(is.null(trans) || trans %in% c("linear", "log", "pow", "sqrt",
    "quantile", "quantize", "threshold"))
  if (!is.null(exponent) && !identical(trans, "pow")) {
    stop("May only set exponent when pow = 'trans'")
  }
  assert_that(is.null(exponent) || (is.numeric(exponent) && length(exponent) == 1))
  assert_that(is.null(reverse) || is.flag(reverse))
  assert_that(is.null(round) || is.flag(round))
  assert_that(is.null(clamp) || is.flag(clamp))
  assert_that(is.null(nice) || is.flag(nice))
  assert_that(is.null(zero) || is.flag(zero))
  assert_that(is.null(expand) || (is.numeric(expand) && length(expand) <= 2))

  vscale <- ggvis_scale(
    property = property,
    name = name,
    label = label,
    type = trans,
    subclass = "numeric",
    exponent = exponent,
    clamp = clamp,
    nice = nice,
    zero = zero,
    domain = domain,
    range = range,
    reverse = reverse,
    round = round,
    expand = expand,
    override = override
  )
  add_scale(vis, vscale)
}

#' Add a date-time scale to a ggvis object.
#'
#' A date/time scale controls the mapping of date and time variables to
#' visual properties.
#'
#' @param vis A ggvis object.
#' @param property The name of a property, such as "x", "y", "fill", "stroke", etc.
#' @inheritParams ggvis_scale
#' @param clamp  If \code{TRUE}, values that exceed the data domain are clamped
#'   to either the minimum or maximum range value. Default is \code{FALSE}.
#' @param nice If specified, modifies the scale domain to use a more
#'   human-friendly value range. Should be a string indicating the desired time
#'   interval; legal values are "second", "minute", "hour", "day", "week",
#'   "month", or "year".
#' @param expand A multiplier for how much the scale should be expanded beyond
#'   the domain of the data. For example, if the data goes from 10 to 110, and
#'   \code{expand} is 0.05, then the resulting domain of the scale is 5 to 115.
#'   Set to 0 and use \code{nice=FALSE} if you want exact control over the
#'   domain.
#' @param utc if \code{TRUE}, uses UTC times. Default is \code{FALSE}.
#' @seealso \code{\link{scales}}, \code{\link{scale_numeric}},
#'   \url{https://vega.github.io/vega/docs/scales/#time}
#' @family scales
#' @export
#' @examples
#' set.seed(2934)
#' dat <- data.frame(
#'   time = as.Date("2013-07-01") + 1:100,
#'   value = seq(1, 10, length.out = 100) + rnorm(100)
#' )
#' p <- dat %>% ggvis(~time, ~value) %>% layer_points()
#'
#' # Start and end on month boundaries
#' p %>% scale_datetime("x", nice = "month")
#'
#'
#' dist <- data.frame(times = as.POSIXct("2013-07-01", tz = "GMT") +
#'                            rnorm(200) * 60 * 60 * 24 * 7)
#' p <- dist %>% ggvis(x = ~times) %>% layer_histograms()
#' p
#'
#' # Start and end on month boundaries
#' p %>% scale_datetime("x", nice = "month")
#'
#' p %>% scale_datetime("x", utc = TRUE)
scale_datetime <- function(vis, property, domain = NULL, range = NULL,
                           reverse = NULL, round = NULL, utc = NULL,
                           clamp = NULL, nice = NULL, expand = NULL,
                           name = property, label = NULL, override = NULL) {
  assert_that(is.null(reverse) || is.flag(reverse))
  assert_that(is.null(round) || is.flag(round))
  assert_that(is.null(utc) || is.flag(utc))
  assert_that(is.null(clamp) || is.flag(clamp))
  assert_that(is.null(nice) || nice %in% c("second", "minute", "hour", "day",
    "week", "month", "year"))
  assert_that(is.null(expand) || (is.numeric(expand) && length(expand) <= 2))

  if (!is.null(utc)) {
    utc <- if (isTRUE(utc)) "utc" else "time"
  }

  vscale <- ggvis_scale(
    property = property,
    name = name,
    label = label,
    type = utc,
    subclass = "datetime",
    clamp = clamp,
    nice = nice,
    domain = domain,
    range = range,
    reverse = reverse,
    round = round,
    expand = expand,
    override = override
  )
  add_scale(vis, vscale)
}

#' Add a ordinal, nominal, or logical scale to a ggvis object.
#'
#' Ordinal, nominal, and logical scales are all categorical, and are treated
#' similarly by ggvis.
#'
#' @param vis A ggvis object.
#' @param property The name of a property, such as "x", "y", "fill", "stroke", etc.
#' @inheritParams ggvis_scale
#' @param points If \code{TRUE} (default), distributes the ordinal values over a
#'   quantitative range at uniformly spaced points. The spacing of the points
#'   can be adjusted using the padding property. If \code{FALSE}, the ordinal
#'   scale will construct evenly-spaced bands, rather than points. Note that
#'   if any mark is added with a \code{\link{band}()} prop, then the scale for
#'   that prop will automatically have \code{points} set to \code{FALSE}.
#' @param padding Applies spacing among ordinal elements in the scale range.
#'   The actual effect depends on how the scale is configured. If the points
#'   parameter is true, the padding value is interpreted as a multiple of the
#'   spacing between points. A reasonable value is 1.0, such that the first and
#'   last point will be offset from the minimum and maximum value by half the
#'   distance between points. Otherwise, padding is typically in the range
#'   [0, 1] and corresponds to the fraction of space in the range interval to
#'   allocate to padding. A value of 0.5 means that the range band width will
#'   be equal to the padding width. For positional (x and y) scales, the default
#'   padding is 0.1. For other scales, the default padding is 0.5.
#' @param sort  If \code{TRUE}, the values in the scale domain will be sorted
#'   according to their natural order. Default is \code{FALSE}.
#' @seealso \code{\link{scales}}, \code{\link{scale_numeric}},
#'   \url{https://vega.github.io/vega/docs/scales/#ordinal}.
#' @family scales
#' @export
#' @examples
#' p <- PlantGrowth %>% ggvis(~group, ~weight) %>% layer_points()
#'
#' p
#' p %>% scale_nominal("x", padding = 0)
#' p %>% scale_nominal("x", padding = 1)
#'
#' p %>% scale_nominal("x", reverse = TRUE)
#'
#' p <- ToothGrowth %>% group_by(supp) %>%
#'   ggvis(~len, fill = ~supp) %>%
#'   layer_histograms(width = 4, stack = TRUE)
#'
#' # Control range of fill scale
#' p %>% scale_nominal("fill", range = c("pink", "lightblue"))
#'
#' # There's no default range when the data is categorical but the output range
#' # is continuous, as in the case of opacity. In these cases, you can
#' # manually specify the range for the scale.
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, opacity = ~factor(cyl)) %>%
#'   layer_points() %>%
#'   scale_nominal("opacity", range = c(0.2, 1))
scale_ordinal <- function(vis, property, domain = NULL, range = NULL,
                          reverse = NULL, round = NULL,
                          points = NULL, padding = NULL, sort = NULL,
                          name = property, label = NULL, override = NULL) {
  assert_that(is.null(reverse) || is.flag(reverse))
  assert_that(is.null(round) || is.flag(round))
  assert_that(is.null(points) || is.flag(points))
  assert_that(is.null(padding) || (is.numeric(padding) && length(padding) == 1))
  assert_that(is.null(sort) || is.flag(sort))

  vscale <- ggvis_scale(
    property = property,
    name = name,
    label = label,
    type = "ordinal",
    points = points,
    padding = padding,
    sort = sort,
    subclass = "ordinal",
    domain = domain,
    range = range,
    reverse = reverse,
    round = round,
    override = override
  )
  add_scale(vis, vscale)
}

#' @rdname scale_ordinal
#' @export
scale_nominal <- function(vis, property, domain = NULL, range = NULL,
                          reverse = NULL, round = NULL,
                          points = NULL, padding = NULL, sort = NULL,
                          name = property, label = NULL, override = NULL) {
  assert_that(is.null(reverse) || is.flag(reverse))
  assert_that(is.null(round) || is.flag(round))
  assert_that(is.null(points) || is.flag(points))
  assert_that(is.null(padding) || (is.numeric(padding) && length(padding) == 1))
  assert_that(is.null(sort) || is.flag(sort))

  vscale <- ggvis_scale(
    property = property,
    name = name,
    label = label,
    type = "ordinal",
    points = points,
    padding = padding,
    sort = sort,
    subclass = "nominal",
    domain = domain,
    range = range,
    reverse = reverse,
    round = round,
    override = override
  )
  add_scale(vis, vscale)
}

#' @rdname scale_ordinal
#' @export
scale_logical <- scale_nominal


#' Add x_rel and y_rel scales
#'
#' This function adds scales named \code{x_rel} and \code{y_rel}, each of which
#' has a domain of 0 to 1, and the range is the plot's width or height.
#' These scales are useful for positioning visual elements relative to the
#' plotting area. For example, with legends.
#'
#' @seealso \code{\link{add_legend}} for a usage example.
#'
#' @param vis A ggvis object.
#' @export
add_relative_scales <- function(vis) {
  vis <- scale_numeric(vis, "x", name = "x_rel", domain = c(0, 1),
                       range = "width", expand = 0)
  vis <- scale_numeric(vis, "y", name = "y_rel", domain = c(0, 1),
                       range = "height", expand = 0)
  vis
}

scale_auto <- function(vis, scale, ..., quiet = FALSE) {
  info <- vis$scales[[scale]]

  if (!quiet) {
    message("Adding scale_", info$type, "(\"", scale, "\")")
  }
  scale_fun <- match.fun(paste0("scale_", info$type))
  vis <- scale_fun(vis, scale, ...)
}

#' Set the label for a scale
#' @param vis A ggvis object.
#' @param scale The name of a scale, like "x".
#' @param label Text to use for the label.
set_scale_label <- function(vis, scale, label) {
  add_scale(vis, ggvis_scale(scale, label = label))
}

# Is this scale object countable?
scale_countable <- function(scale) UseMethod("scale_countable")
#' @export
scale_countable.scale_numeric <- function(scale) FALSE
#' @export
scale_countable.scale_datetime <- function(scale) FALSE
#' @export
scale_countable.scale_nominal <- function(scale) TRUE
#' @export
scale_countable.scale_ordinal <- function(scale) TRUE
#' @export
scale_countable.scale_logical <- function(scale) TRUE
#' @export
scale_countable.default <- function(scale) NULL


# Make sure that scales are well-formed for a vega spec
check_scales_complete <- function(vis) {
  check_scale_complete <- function(scale) {
    if (is.null(scale$range)) {
      warning(paste(
        sprintf("Scale '%s' for property '%s' is missing a range.", scale$name, scale$property),
        "ggvis tries to automatically provide ranges for scales, but it doesn't know how in this case.",
        "You must specify the range manually.",
        "See ?scale_nominal or ?scale_numeric for more information and examples."
      ), call. = FALSE)
    }
  }

  lapply(vis$scales, check_scale_complete)
}

# There are five way to display a ggvis plot:
#   * static html file (static plot)
#     * print.ggvis -> view_static -> ggvisLayout -> ggvisElements
#     * Doesn't need shiny
#     * Needs embedded spec
#
#   * temporary shiny app (dynamic plot)
#     * print.ggvis -> view_dynamic -> ggvis_app -> ggvisLayout -> ggvisElements
#     * Needs shiny
#     * Doesn't need embedded spec
#
#   * static html embedded in Rmarkdown
#     * knit_print.ggvis -> ggvisElements
#     * Doesn't need shiny
#     * Needs embedded spec
#
#   * shiny app embedded in Rmarkdown
#     * knit_print.ggvis -> ggvis_app -> ggvisLayout -> ggvisElements
#     * Needs shiny
#     * Doesn't need embedded spec
#
#   * embedded in regular shiny app
#     * call ggvisOutput in ui.R
#     * Needs shiny
#     * Doesn't need embedded spec

#' View in a ggvis plot in the browser.
#'
#' \code{view_static} creates a static web page in a temporary directory;
#' \code{view_dynamic} generate a dynamic shiny app and launches it.
#' \code{print} automatically picks between the two.
#'
#' If \code{view_static} is used on a ggvis object that has dynamic components,
#' it will output a static plot.
#'
#' @param x A ggvis object.
#' @param dynamic Uses \code{view_dynamic} if \code{TRUE}, \code{view_static} if
#'   \code{FALSE}. The default, \code{NA}, chooses automatically based on the
#'   presence of reactives or interactive inputs in \code{x}.
#' @param launch If \code{TRUE}, will launch plot in a viewer/browser. If
#'   \code{FALSE} returns an object that you can \code{print()} to launch.
#' @param ... Other arguments passed on to \code{view_dynamic} and
#'   \code{view_static} ?from \code{print}.
#' @keywords internal
#' @method print ggvis
#' @export
print.ggvis <- function(x, dynamic = NA, launch = interactive(), ...) {
  if (is.na(dynamic)) {
    dynamic <- is.dynamic(x) && interactive()
  }

  if (dynamic) {
    out <- view_dynamic(x, ...)
  } else {
    out <- view_static(x, ...)
  }

  if (launch) print(out)
  out
}

#' @rdname print.ggvis
#' @export
#' @param plot_id Unique identifier used to identify the plot on the page.
#' @param dest Deprecated (this no longer works).
#' @examples
#' ## Run these examples only in interactive R sessions
#' if (interactive()) {
#' # In most cases view_static is unnecessary; these will do the same thing:
#' mtcars %>% ggvis(~wt, ~mpg)
#' mtcars %>% ggvis(~wt, ~mpg) %>% view_static()
#'
#' # Can find the output file with view_static() and html_print()
#' outfile <- mtcars %>% ggvis(~wt, ~mpg) %>%
#'   view_static() %>% htmltools::html_print(viewer = NULL)
#' }
view_static <- function(x, plot_id = rand_id("plot_"), dest = NULL) {


  if (!missing(dest)) {
    deprecated("dest", msg = " Please see ?view_static for more information.")
  }

  spec <- as.vega(x, dynamic = FALSE)
  htmltools::browsable(
    ggvisLayout(plot_id, length(x$controls) > 0, spec, shiny = FALSE)
  )
}

#' @rdname print.ggvis
#' @export
#' @param port the port on which to start the shiny app. If NULL (the default),
#'   Shiny will select a random port.
#' @param quiet If \code{TRUE} show status messages from Shiny. (Default is
#'   \code{FALSE}.)
view_dynamic <- function(x, plot_id = rand_id("plot_"), port = NULL,
                         quiet = FALSE) {

  ggvis_app(x,
    plot_id = plot_id,
    port = port,
    quiet = TRUE,
    launch.browser = function(url) {
      if (!quiet) {
        message("Showing dynamic visualisation. Press Escape/Ctrl + C to stop.")
      }
      view_plot(url, 350 + control_height(x))
    }
  )
}


# Dynamically registered. See zzz.R.
#' Knit print method for ggvis plots.
#'
#' @keywords internal
knit_print.ggvis <- function(x, options = list(), inline = FALSE, ...) {
  # Set height and width from knitr chunk options
  knitr_opts <- list(
    width = options$out.width.px,
    height = options$out.height.px
  )
  x <- add_options(x, knitr_opts, replace = FALSE)

  # If this is a dynamic object, check to see if we're rendering in a Shiny R
  # Markdown document and have an appropriate version of Shiny; emit a Shiny
  # application if we are, and a warning if we aren't.
  if (is.dynamic(x)) {
    if (identical(runtime(), "shiny")) {
      # create the application object and allocate space for the controls
      app <- ggvis_app(x,
        width = knitr_opts$width,
        height = knitr_opts$height + control_height(x)
      )
      return(knitr::knit_print(app))
    }

    warning(
      "Can't output dynamic/interactive ggvis plots in a knitr document.\n",
      "Generating a static (non-dynamic, non-interactive) version of the plot.",
      call. = FALSE
    )
  }

  # If we got here, it's static
  spec <- as.vega(x, dynamic = FALSE)
  knitr::knit_print(ggvisOutputElements(spec = spec, shiny = FALSE))
}

# Helper functions -------------------------------------------------------------

runtime <- function() {
  knitr::opts_knit$get("rmarkdown.runtime")
}

#' Determine if an ggvis is dynamic (i.e. needs to be run in a shiny app)
#'
#' @export
#' @keywords internal
is.dynamic <- function(x) {
  any_apply(x$data, shiny::is.reactive) || length(x$reactives) > 0
}

# View using either an internal viewer or fallback to utils::browseURL
view_plot <- function(url, height) {
  viewer <- getOption("viewer")
  if (!is.null(viewer)) {
    viewer(url, height)
  } else {
    utils::browseURL(url)
  }
}

# given a ggvis object, return the number of pixels to reserve for its controls.
control_height <- function(x) {
  n_controls <- length(x$controls)

  # Request 70 vertical pixels for each pair of control items, since there are
  # two on a row.
  70 * ceiling(n_controls / 2)
}

# Basic bar graphs
library(ggvis)
library(dplyr, warn.conflicts = FALSE)

pressure %>%
  ggvis(~factor(temperature), ~pressure) %>%
  layer_bars(width = 0.5) %>%
  save_spec("bar/categorical-x.json")

pressure %>%
  ggvis(x = ~temperature, y = ~pressure) %>%
  layer_bars(fill:="red") %>%
  save_spec("bar/continuous-x.json")

#' Handle brush events on a visualisation.
#'
#' Currently for brush events to be triggered on a visualisation, you must
#' use a \code{.brush} property. This limitation will be lifted in the future.
#'
#' @param vis Visualisation to listen to.
#' @param on_move Callback function with arguments:
#'   \describe{
#'    \item{items}{A data frame containing information about the items
#'      under the plot. An empty data.frame if no points under the brush.}
#'    \item{page_loc}{Location of the brush with repsect to the page}
#'    \item{plot_loc}{Location of the brush with respect to the plot}
#'    \item{session}{The session, used to communicate with the browser}
#'   }
#' @param fill Colour of the brush.
#' @export
#' @examples
#' # Display tooltip when objects are brushed
#' mtcars %>%
#'   ggvis(x = ~wt, y = ~mpg, size.brush := 400) %>%
#'   layer_points() %>%
#'   handle_brush(function(items, page_loc, session, ...) {
#'     show_tooltip(session, page_loc$r + 5, page_loc$t, html = nrow(items))
#'   })
handle_brush <- function(vis, on_move = NULL, fill = "black") {
  check_callback(on_move, c("items", "plot_loc", "page_loc", "session"))

  connect <- function(session, plot_id) {
    id <- paste0(plot_id, "_brush_move")
    shiny::observe({
      value <- session$input[[id]]
      if (is.null(value)) return()

      items <- tidy_items(value$items)
      page_loc <- list(
        t = value$pagey1, r = value$pagex2,
        b = value$pagey2, l = value$pagex1
      )
      plot_loc <- list(
        t = value$y1, r = value$x2,
        b = value$y2, l = value$x1
      )

      on_move(items = items, page_loc = page_loc, plot_loc = plot_loc,
        session = session)
    })
  }
  connector_label(connect) <- "brush"

  broker <- create_broker(
    reactive(NULL),
    connect = connect,
    spec = list(type = "brush")
  )
  vis <- register_reactive(vis, broker)

  layer_brush(vis, fill = fill)
}

layer_brush <- function(vis, fill = "black") {
  layer_f(vis, function(v) {
    init <- data.frame(x = 0, y = 0, width = 0, height = 0)
    v <- add_data(v, init, "ggvis_brush", add_suffix = FALSE)
    emit_rects(v, props(x := ~x, y := ~y,
      width := ~width, height := ~height,
      fill := fill, fillOpacity := 0.2,
      stroke := fill, strokeOpacity := 0.6,
      inherit = FALSE))
  })
}

tidy_items <- function(items) {
  if (length(items) == 0) {
    return(data.frame(keys__ = character()))
  }

  # FIXME: figure out more efficient way to do this
  dfs <- lapply(items, function(x) {
    class(x) <- "data.frame"
    attr(x, "row.names") <- .set_row_names(1L)
    x
  })
  items <- dplyr::bind_rows(dfs)

  if (is.numeric(items$key__)) {
    items$key__ <- as.character(items$key__ + 1)
  }
  items
}

#' Compute density of data.
#'
#' @param x Dataset (data frame, \code{grouped_df} or ggvis) object to work
#'   with.
#' @param x_var,w_var Names of variables to use for x position, and for
#'   weights.
#' @param kernel Smoothing kernel. See \code{\link{density}} for details.
#' @param trim If \code{TRUE}, the default, density estimates are trimmed to the
#'   actual range of the data.  If \code{FALSE}, they are extended by the
#'   default 3 bandwidths (as specified by the \code{cut} parameter to
#'   \code{\link{density}}).
#' @param ... Additional arguments passed on to \code{\link{density}}.
#' @param n Number of points (along x) to use in the density estimate.
#' @param na.rm If \code{TRUE} missing values will be silently removed,
#'   otherwise they will be removed with a warning.
#' @return A data frame with columns:
#'  \item{pred_}{regularly spaced grid of \code{n} locations}
#'  \item{resp_}{density estimate}
#' @export
#' @examples
#' mtcars %>% compute_density(~mpg, n = 5)
#' mtcars %>% group_by(cyl) %>% compute_density(~mpg, n = 5)
#' mtcars %>% ggvis(~mpg) %>% compute_density(~mpg, n = 5) %>%
#'   layer_points(~pred_, ~resp_)
compute_density <- function(x, x_var, w_var = NULL, kernel = "gaussian",
                            trim = FALSE, n = 256L, na.rm = FALSE, ...) {

  UseMethod("compute_density")
}

#' @export
compute_density.data.frame <- function(x, x_var, w_var = NULL,
                                       kernel = "gaussian",
                                       trim = FALSE, n = 256L,
                                       na.rm = FALSE, ...) {

  assert_that(is.formula(x_var))

  # Extract variables from data frame
  x_val <- eval_vector(x, x_var)

  # Special case zero-row input
  if (length(x_val) == 0) {
    return(data.frame(pred_ = numeric(0), resp_ = numeric(0)))
  }

  if (is.null(w_var)) {
    w_val <- NULL
  } else {
    w_val <- eval_vector(x, w_var)
  }

  # Build call to density()
  call <- make_call("density", quote(x_val), weights = quote(w_val),
    kernel = kernel, n = n, na.rm = na.rm, ...)

  if (trim) {
    call$from <- min(x_val)
    call$to   <- max(x_val)
  }
  dens <- eval(call)

  # Standardise output
  data.frame(pred_ = dens$x, resp_ = dens$y)
}

#' @export
compute_density.grouped_df <- function(x, x_var, w_var = NULL,
                                       kernel = "gaussian", trim = FALSE,
                                       n = 256L, na.rm = FALSE, ...) {
  dplyr::do(x, compute_density(., x_var = x_var, w_var = w_var,
    kernel = kernel, trim = trim, n = n, na.rm = na.rm, ...))
}

#' @export
compute_density.ggvis <- function(x, x_var, w_var = NULL,
                                  kernel = "gaussian", trim = FALSE,
                                  n = 256L, na.rm = FALSE, ...) {
  args <- list(x_var = x_var, w_var = w_var, kernel = kernel,
    trim = trim, n = n, na.rm = na.rm, ...)

  register_computation(x, args, "density", function(data, args) {
    output <- do_call(compute_density, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

context("compute_density")

test_that("compute_density respects arguments", {
  # Uses n
  res <- mtcars %>% compute_density(~mpg, n = 10)
  expect_equal(10, nrow(res))

  # When trim = FALSE (default), result goes past bounds of original data
  expect_true(min(res$pred_) < min(mtcars$mpg) && max(res$pred_) > max(mtcars$mpg))

  # When trim = TRUE, bounds of result match bounds of original data
  res <- mtcars %>% compute_density(~mpg, n = 10, trim = TRUE)
  expect_true(all(range(res$pred_) == range(mtcars$mpg)))
})


test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_density(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("pred_", "resp_")))

  # Grouped
  res <- mtcars %>% group_by(cyl) %>% dplyr::filter(FALSE) %>% compute_density(~mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("cyl", "pred_", "resp_")))
})

#' Handlers and interactive inputs for plot sizing.
#'
#' @param vis Visualisation to listen to.
#' @param on_resize Callback function with arguments:
#'   \describe{
#'    \item{width,height}{Width and height in pixels}
#'    \item{padding}{A named list of four components giving the padding in
#'      each direction}
#'    \item{session}{The session, used to communicate with the browser}
#'   }
#' @export
#' @examples
#' # This example just prints out the current dimensions to the console
#' mtcars %>% ggvis(~mpg, ~wt) %>%
#'   layer_points() %>%
#'   handle_resize(function(width, height, ...) cat(width, "x", height, "\n"))
#'
#' # Use plot_width() and plot_height() to dynamically get the plot size
#' # inside the plot.
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_text(text := plot_width())
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_text(text := plot_height())
handle_resize <- function(vis, on_resize) {
  broker <- create_broker(
    reactive(NULL),
    connect = connect_resize(on_resize),
    spec = list(type = "resize")
  )
  register_reactive(vis, broker)
}


#' @export
#' @rdname handle_resize
plot_width <- function(vis) {
  vals <- shiny::reactiveValues()
  vals$x <- 100
  on_resize <- function(width, ...) {
    vals$x <- width
  }

  create_broker(
    reactive(vals$x),
    connect = connect_resize(on_resize),
    spec = list(type = "resize")
  )
}

#' @export
#' @rdname handle_resize
plot_height <- function(vis) {
  vals <- shiny::reactiveValues()
  vals$x <- 100
  on_resize <- function(height, ...) {
    vals$x <- height
  }

  create_broker(
    reactive(vals$x),
    connect = connect_resize(on_resize),
    spec = list(type = "resize")
  )
}


connect_resize <- function(on_resize) {
  check_callback(on_resize, c("width", "height", "padding", "session"))

  function(session, plot_id) {
    id <- paste0(plot_id, "_resize")
    shiny::observe({
      value <- session$input[[id]]
      if (is.null(value)) return()

      on_resize(width = value$width, height = value$height,
        padding = value$padding, session = value$session)
    })
  }
}

#' Count data at each location of a categorical variable
#'
#' @param x Dataset-like object to count. Built-in methods for data frames,
#'   grouped data frames and ggvis visualisations.
#' @param x_var,w_var Names of x and weight variables.
#' @seealso \code{\link{compute_bin}} For counting cases within ranges of
#'   a continuous variable.
#' @seealso \code{\link{compute_count}} For counting cases at specific locations
#'   of a continuous variable. This is useful when the variable is continuous
#'   but the data is granular.
#' @export
#' @return A data frame with columns:
#'  \item{count_}{the number of points}
#'  \item{x_}{value of bin}
#' @examples
#' library(dplyr)
#' # The tabulated column must be countable (not numeric)
#' \dontrun{mtcars %>% compute_tabulate(~cyl)}
#' mtcars %>% mutate(cyl = factor(cyl)) %>% compute_tabulate(~cyl)
#'
#' # Or equivalently:
#' mtcars %>% compute_tabulate(~factor(cyl))
#'
#' # If there's one weight value at each x, it effectively just renames columns.
#' pressure %>% compute_tabulate(~factor(temperature), ~pressure)
#'
#' # It doesn't matter whether you transform inside or outside of a vis
#' mtcars %>% compute_tabulate(~factor(cyl)) %>%
#'   ggvis(x = ~x_, y = ~count_, y2 = 0) %>%
#'   layer_rects(width = band())
#'
#' mtcars %>%
#'   ggvis(x = ~x_, y = ~count_, y2 = 0) %>%
#'   compute_tabulate(~factor(cyl)) %>%
#'   layer_rects(width = band())
#'
#' # compute_tabulate is used automatically in layer_bars when no y prop
#' # is supplied.
#' mtcars %>% ggvis(x = ~factor(cyl)) %>% layer_bars()
compute_tabulate <- function(x, x_var, w_var = NULL) {
  UseMethod("compute_tabulate")
}

#' @export
compute_tabulate.data.frame <- function(x, x_var, w_var = NULL) {
  assert_that(is.formula(x_var))

  x_val <- eval_vector(x, x_var)
  if (!vector_countable(x_val)) {
    stop("compute_tabulate requires categorical (countable) data.",
      call. = FALSE)
  }

  if (is.null(w_var)) {
    w_val <- NULL
  } else {
    w_val <- eval_vector(x, w_var)
  }

  tabulate_vector(x_val, weight = w_val)
}

#' @export
compute_tabulate.grouped_df <- function(x, x_var, w_var = NULL) {
  dplyr::do(x, compute_tabulate(., x_var, w_var = w_var))
}

#' @export
compute_tabulate.ggvis <- function(x, x_var, w_var = NULL) {
  args <- list(x_var = x_var, w_var = w_var)

  register_computation(x, args, "count", function(data, args) {
    output <- do_call(compute_tabulate, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

# Tabulate individual vector ---------------------------------------------------

tabulate_vector <- function(x, weight = NULL, ...) {
  if (is.null(weight)) {
    weight <- rep.int(1, length(x))
  }
  counts <- unname(as.vector(tapply(weight, x, sum, na.rm = TRUE)))

  if (is.factor(x)) {
    # Get the factor levels, preserving factor-ness. Order should align
    # with counts.
    values <- unique(x)
  } else {
    # Need to get unique values this way instead of using names(counts),
    # because names are strings but the x values aren't always strings.
    values <- unname(as.vector(tapply(x, x, unique, na.rm = TRUE)))
  }

  data.frame(
    count_ = counts,
    x_ = values,
    stringsAsFactors = FALSE
  )
}

context("compute_tabulate")

test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_tabulate(~factor(cyl))
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("count_", "x_")))

  # Grouped
  res <- mtcars %>% group_by(cyl) %>% dplyr::filter(FALSE) %>% compute_tabulate(~factor(cyl))
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("cyl", "count_", "x_")))
})

test_that("weights are added", {
  df <- data.frame(x = factor(rep(1:3, each = 3)), y = rep(2, 9))

  out <- df %>% compute_tabulate(~x, ~y)
  expect_equal(out$count_, c(6, 6, 6))
})

library(ggvis)

shinyServer(function(input, output, session) {
  reactive({
    mtcars %>% ggvis(~disp, ~mpg) %>%
      layer_points() %>%
      scale_numeric("x", domain = input$x_domain, nice = FALSE, clamp = TRUE) %>%
      scale_numeric("y", domain = input$y_domain, nice = FALSE, clamp = TRUE)
  }) %>%
  bind_shiny("zoom")
})

#' Create a subvisualisation.
#'
#' @keywords internal
#' @examples
#' # Examples don't work yet
#' \dontrun{
#' library(dplyr, warn.conflicts = FALSE)
#'
#' small <- nasaweather::atmos %>%
#'   filter(lat <= -11.217391, long <= -106.287, year == 1995) %>%
#'   group_by(long, lat)
#' small %>%
#'   ggvis(~long, ~lat) %>%
#'   layer_points()
#'
#' small %>%
#'   ggvis(~long, ~lat) %>%
#'   subvis(width := 100, height := 100, stroke := "red") %>%
#'     layer_points(~month, ~ozone)
#'
#' small %>%
#'   ggvis(~long, ~lat) %>%
#'   subvis(width := 100, height := 100, stroke := "red") %>%
#'     layer_points(~month, ~ozone) %>%
#'     add_axis("x", ticks = 3) %>%
#'     add_axis("y", ticks = 3)
#' }
subvis <- function(vis, ..., data = NULL, width = NULL, height = NULL) {
  # Very similar to add_mark, but changes to cur_data and cur_props are
  # persistent, we use the special mark_group() and update cur_vis
  vis <- add_data(vis, data, deparse2(substitute(data)))
  vis <- add_props(vis, .props = props(...))
  vis <- register_scales_from_props(vis, cur_props(vis))

  new_mark <- mark_group(vis$cur_props, vis$cur_data)
  vis$marks <- append(vis$marks, list(new_mark))

  # Parent properties control apperance mark_group, and are not
  # inherited by child
  vis$cur_props <- NULL
  vis$cur_vis <- c(vis$cur_vis, length(vis$marks))
  vis

  vis
}

#' Pipe graphics
#'
#' Like dplyr, ggvis also uses the pipe function, \code{\%>\%} to turn
#' function composition into a series of imperative statements.
#'
#' @importFrom magrittr %>%
#' @name %>%
#' @rdname pipe
#' @export
#' @param lhs,rhs A visualisation and a function to apply to it
#' @examples
#' # Instead of
#' layer_points(ggvis(mtcars, ~mpg, ~wt))
#' # you can write
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points()
NULL

#' Create a new layering function.
#'
#' The layer function is run, and then the state before the code was run
#' is restored - this allows layers to be effectively isolated from
#' the rest of the plot.
#'
#' @param vis The ggvis visualisation to modify.
#' @param fun A function that takes a single argument, the current
#'   visualisation as input, and returns a modified visualisation.
#' @keywords internal
#' @export
#' @examples
#' mtcars %>% ggvis(~mpg) %>%
#'   layer_f(function(v) {
#'      v %>% compute_bin(~mpg) %>% layer_points(x = ~x_, y = ~count_)
#'   }) %>%
#'   layer_points(y = ~wt)
layer_f <- function(vis, fun) {
  # Save current data and props
  old_data  <- vis$cur_data
  old_props <- vis$cur_props

  vis <- fun(vis)

  # Restore previous data and props
  vis$cur_data  <- old_data
  vis$cur_props <- old_props

  vis
}

library(ggvis)


fluidPage(sidebarLayout(
  sidebarPanel(
    sliderInput("n", "Number of points", min = 1, max = nrow(mtcars),
                value = 10, step = 1),
    uiOutput("plot_ui")
  ),
  mainPanel(
    ggvisOutput("plot"),
    tableOutput("mtc_table")
  )
))

#' Connect a ggvis graphic to a shiny app.
#'
#' Embedding ggvis in a shiny app is easy. You need to make a place for it in
#' your \code{ui.r} with \code{ggvisOutput}, and tell your \code{server.r}
#' where to draw it with \code{bind_shiny}. It's easiest to learn by example:
#' there are many shiny apps in \code{demo/apps/} that you can learn from.
#'
#' @section Client-side:
#' In your UI, use \code{ggvisOutput()} in \code{ui.r} to insert an html
#' placeholder for the plot.
#'
#' If you're going to be using interactive controls generated by ggvis,
#' use \code{\link[shiny]{renderUI}()} to add a place holder. By convention,
#' if the id of plot placehold is called "plot", call the controls placeholder
#' "plot_ui".
#'
#' @section Server-side:
#' When you run ggvis plot interactively, it is automatically plotted because
#' it triggers the default print method. In shiny apps, you need to
#' explicitly render the plot to a specific placeholder with
#' \code{bind_shiny}:
#'
#' \code{p \%>\% bind_shiny("plot")}
#'
#' If the plot has controls, and you've reserved space for them in the UI,
#' supply the name of the placeholder as the third argument:
#'
#' \code{p \%>\% bind_shiny("plot", "plot_ui")}
#' @examples
#' ## Run these examples only in interactive R sessions
#' if (interactive()) {
#'
#' # Simplest possible app:
#' library(shiny)
#' runApp(list(
#'   ui = bootstrapPage(
#'     ggvisOutput("p"),
#'     uiOutput("p_ui")
#'   ),
#'   server = function(..., session) {
#'     mtcars %>%
#'       ggvis(~wt, ~mpg) %>%
#'       layer_points() %>%
#'       layer_smooths(span = input_slider(0, 1)) %>%
#'       bind_shiny("p", "p_ui")
#'   }
#' ))
#'
#' }
#' @name shiny-ggvis
NULL

#' @rdname shiny-ggvis
#' @param vis A ggvis object, or a reactive expression that returns a ggvis
#'   object.
#' @param session A Shiny session object.
#' @param ... Other arguments passed to \code{as.vega}.
#' @export
bind_shiny <- function(vis, plot_id, controls_id = NULL, ...,
                       session = shiny::getDefaultReactiveDomain()) {

  validate_plot_id(plot_id)

  if (is.null(session)) {
    stop("bind_shiny() must be run inside a shiny app.", call. = FALSE)
  }

  if (shiny::is.reactive(vis)) {
    visf <- vis
  } else if (is.ggvis(vis)) {
    visf <- function() vis
  } else {
    stop("bind_shiny requires a ggvis object or a reactive expression that returns a ggvis object",
      call. = FALSE)
  }

  r_spec <- shiny::reactive({
    as.vega(visf(), session = session, dynamic = TRUE, ...)
  })

  observe_spec(r_spec, plot_id, session)
  observe_data(r_spec, plot_id, session)
  exec_connectors(r_spec, plot_id, session)

  if (!is.null(controls_id)) {
    bind_shiny_ui(vis, controls_id, session = session)
  }

  vis
}

#' @param controls_id Unique identifier for controls div.
#' @rdname shiny-ggvis
#' @export
bind_shiny_ui <- function(vis, controls_id,
  session = shiny::getDefaultReactiveDomain()) {
  if (is.null(session)) {
    stop("bind_shiny_ui() must be run inside a shiny app.", call. = FALSE)
  }

  if (shiny::is.reactive(vis)) {
    visf <- vis
  } else if (is.ggvis(vis)) {
    visf <- function() vis
  } else {
    stop("bind_shiny_ui requires a ggvis object or a reactive expression that returns a ggvis object",
      call. = FALSE)
  }

  shiny::observe({
    controls <- visf()$controls
    if (empty(controls)) return()

    # Wrap each control in a div, for layout purposes
    divs <- lapply(controls, htmltools::div,  class = "ggvis-input-container")
    session$output[[controls_id]] <- shiny::renderUI(htmltools::tagList(divs))
  })

  vis
}

# Tell an observer to suspend or resume, depending on the hidden state of an
# output object on the client.
sync_with_hidden_state <- function(obs, id, session) {
  force(obs)

  shiny::observe({
    isHidden <- session$clientData[[paste0('output_', id, '_hidden')]]

    if (identical(isHidden, FALSE)) {
      obs$resume()
    } else {
      obs$suspend()
    }
  })
}

# Create an observer for a reactive vega spec
observe_spec <- function(r_spec, id, session) {
  obs <- shiny::observe(suspended = TRUE, {
    session$sendCustomMessage("ggvis_vega_spec", list(
      plotId = id,
      spec = r_spec()
    ))
  })

  sync_with_hidden_state(obs, id, session)
}

# Create observers for the data objects attached to a reactive vega spec
observe_data <- function(r_spec, id, session) {
  # A list for keeping track of each data observer
  data_observers <- list()

  outer_obs <- shiny::observe(suspended = TRUE, {
    # If data_observers list is nonempty, that means there are old observers
    # which need to be suspended before we create new ones. This can happen when
    # the reactive containing the ggvis() call is invalidated.
    for (obs in data_observers) obs$suspend()

    data_table <- c(attr(r_spec(), "data_table", TRUE),
                    attr(r_spec(), "scale_data_table", TRUE))

    # Create observers for each of the data objects
    data_observers <<- lapply(names(data_table), function(data_name) {
      # The data_table list contains named objects. The names are synthetic IDs
      # that are present in the vega spec.

      force(data_name)
      obs <- shiny::observe(suspended = TRUE, {
        data_reactive <- data_table[[data_name]]

        session$sendCustomMessage("ggvis_data", list(
          plotId = id,
          name = data_name,
          value = as.vega(data_reactive(), data_name)
        ))
      })
      sync_with_hidden_state(obs, id, session)

      obs
    })

    # Tell the plot to update _after_ all the data has been sent
    data_observers[[length(data_observers) + 1]] <<- shiny::observe(suspended = TRUE, {
      # Take dependency on all data objects
      for (name in names(data_table)) {
        data_table[[name]]()
      }

      session$sendCustomMessage("ggvis_command", list(
        plotId = id,
        command = "update"
      ))
    }, priority = -1)
    sync_with_hidden_state(data_observers[[length(data_observers)]], id, session)
  })

  sync_with_hidden_state(outer_obs, id, session)
}

# Run the connector functions
exec_connectors <- function(r_spec, plot_id, session) {
  connectors <- shiny::isolate(attr(r_spec(), "connectors", TRUE))

  lapply(connectors, function(connect) {
    if (!is.null(connect)) {
      connect(session, plot_id)
    }
  })
}

validate_plot_id <- function(id) {
  if (grepl(".", id, fixed = TRUE)) {
    stop("Plot ID '", id, "' is not valid. The ID must not contain a dot (.) character.")
  }
}

camelCase <- function(x) {
  words <- strsplit(x, "_")
  
  caps <- lapply(words, function(x) {
    if (length(x) == 1) return(x)
    x[-1] <- capitalise(x[-1])
    x
  })
  
  vapply(caps, paste0, collapse = "", FUN.VALUE = character(1))
}

under_score <- function(x) {
  substr(x, 1, 1) <- tolower(substr(x, 1, 1))
  tolower(gsub("([A-Z])", "_\\1", x))
}

capitalise <- function(x, first = TRUE) {
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

last <- function(x) {
  if (length(x) == 0) return(NULL)
  x[[length(x)]]
}

make_call <- function(f, ..., .args = list()) {
  if (is.character(f)) f <- as.name(f)
  as.call(c(list(f, ...), .args))
}
do_call <- function(f, ..., .args = list(), .env = parent.frame(), .debug = FALSE) {
  f <- substitute(f)

  call <- make_call(f, ..., .args = .args)
  if (.debug) print(call)
  eval(call, .env)
}


deparse2 <- function(x) paste(deparse(x, 500L), collapse = "")

names2 <- function(x) names(x) %||% rep("", length(x))
named <- function(x) names2(x) != ""

"%||%" <- function(a, b) if (!is.null(a)) a else b

# Given a vector or list, drop all the NULL items in it
drop_nulls <- function(x) {
  x[!vapply(x, is.null, FUN.VALUE=logical(1))]
}

# Given a string, indent every line by some number of spaces.
# The exception is to not add spaces after a trailing \n.
indent <- function(str, indent = 0) {
  gsub("(^|\\n)(?!$)",
    paste0("\\1", paste(rep(" ", indent), collapse = "")),
    str,
    perl = TRUE
  )
}

# Given two named vectors, join them together, and keep only the last element
# with a given name in the resulting vector. If b has any elements with the
# same name as elements in a, the element in a is dropped. Also, if there are
# any duplicated names in a or b, only the last one with that name is kept.
merge_vectors <- function(a, b) {
  if ((!is.null(a) && length(a) > 1 && is.null(names(a))) ||
      (!is.null(b) && length(b) > 1 && is.null(names(b)))) {
    stop("merge_vectors: vectors must be either NULL or named vectors")
  }

  x <- c(a, b)
  drop_idx <- duplicated(names(x), fromLast = TRUE)
  x[!drop_idx]
}

# Tests whether all elements in a vector are the same, respecting NA.
# Returns TRUE for zero-length vectors
# Returns FALSE for non-atomic vectors
all_same <- function(x) {
  if (!is.atomic(x)) {
    return(FALSE)
  }

  nas <- is.na(x)
  if (length(x) == 0 || all(nas))
    TRUE
  else if (any(nas))
    FALSE
  else
     all(x == x[1])
}

# Test whether a file exists and is a directory
dir.exists <- function(x) {
  res <- file.exists(x) & file.info(x)$isdir
  stats::setNames(res, x)
}

# Check whether a package is installed, and stop if not
assert_installed <- function(pkg) {
  if (nchar(system.file(package = pkg)) == 0) {
    stop("The '", pkg, "' package is required for this functionality")
  }
}

# Determine if an object is the result of quote()
is.quoted <- function(x) {
  is.call(x) || is.name(x)
}

# Drop all empty items from a list - except environments, which stay even if
# they are empty.
compact <- function(x) {
  non_empty <- !vapply(x, empty, logical(1))
  env <- vapply(x, is.environment, logical(1))
  x[non_empty | env]
}

param_string <- function(x, collapse = TRUE) {
  is_reactive <- vapply(x, shiny::is.reactive, logical(1))
  is_env <- vapply(x, is.environment, logical(1))
  is_string <- vapply(x, is.character, logical(1))

  x[is_reactive] <- "<reactive>"
  x[is_env] <- vapply(x[is_env], format, character(1))
  values <- vapply(x, toString, character(1))
  values[is_string] <- paste0("'", encodeString(values[is_string]), "'")

  if (!collapse) return(values)
  paste0("(", paste0(names(x), " = ", values, collapse = ", "), ")")
}

# Convert various objects to char strings.
as_char <- function(x) UseMethod("as_char")
#' @export
as_char.name <- function(x) as.character(x)
#' @export
as_char.call <- function(x) deparse2(x)
#' @export
as_char.default <- function(x) as.character(x)

# Given a string, return a string that is safe as a vega variable.
# Replaces . with \.
safe_vega_var <- function(x) {
  gsub(".", "\\.", x, fixed = TRUE)
}

empty <- function(x) UseMethod("empty")

#' @export
empty.default <- function(x) length(x) == 0

# Convert a list to a data frame, quickly
quickdf <- function(list) {
  if (length(list) == 0) return(data.frame())

  class(list) <- "data.frame"
  attr(list, "row.names") <- c(NA_integer_, -length(list[[1]]))

  list
}

# Generate a random number to use in IDs
rand_id <- function(prefix = "") {
  paste0(prefix, floor(stats::runif(1, 1e8, 1e9-1)))
}

is_missing <- function(x) identical(x, quote(expr = ))

# Check if the calling function had missing arguments when it was called, and
# throw an informative error if so. This happens when there are extra commas in
# the call, as in f(a, 2, ).
check_empty_args <- function() {
  call <- sys.call(-1)
  args <- as.list(call[-1])

  missing <- vapply(args, is_missing, logical(1))

  if (!any(missing)) return(invisible(TRUE))

  stop("Extra comma at position", if (sum(missing) > 1) "s", " ",
    paste0(which(missing), collapse = ", "),
    " in call to ", as.character(call[[1]]), "()",
    call. = FALSE)
}

# Report whether a package is installed
is_installed <- function(pkg) {
  system.file(package = pkg) != ""
}

# Try to load a package
try_require <- function(pkg) {
  result <- suppressPackageStartupMessages(suppressWarnings(
    require(pkg, character.only = TRUE)))

  if (!result) {
    stop(pkg,
      " package required for this functionality.  Please install and try again.",
      call. = FALSE)
  }
}


notify_guess <- function(x, explanation = NULL) {
  msg <- paste0(
    "Guessing ", deparse(substitute(x)), " = ", format(x, digits = 3),
    if (!is.null(explanation)) paste0(" # ", explanation)
  )
  message(msg)
}


any_apply <- function(xs, f) {
  for (x in xs) {
    if (f(x)) return(TRUE)
  }
  FALSE
}

pluck <- function(x, name) {
  lapply(x, `[[`, name)
}

vpluck <- function(x, name, type) {
  vapply(x, `[[`, name, FUN.VALUE = type)
}

# Like as.numeric, except that as.numeric(NULL) returns numeric(0), whereas
# as_numeric(NULL) returns NULL.
as_numeric <- function(x) {
  if (is.null(x)) NULL
  else as.numeric(x)
}

deprecated <- function(old, new = NULL, msg = NULL, version = NULL) {
  text <- paste0(
    sprintf("'%s' is deprecated.", old),
    if (!is.null(new)) sprintf(" Please use '%s' instead.", new),
    msg,
    if (!is.null(version)) sprintf(" (Last used in version %s)", version)
  )
  warning(text, call. = FALSE)
}

# Need this so R CMD check doesn't complain about "no visible global function
# definition"
`:=` <- function(x, value) {
  stop("This code should not be reached.", call. = FALSE)
}

context("utils")


test_that("merge_vectors behaves as expected", {
  expect_identical(merge_vectors(c(a=1, b=2), c(c=3, d=4)), c(a=1, b=2, c=3, d=4))

  # b appears twice; keep last version
  expect_identical(merge_vectors(c(a=1, b=2), c(b=3, c=4)), c(a=1, b=3, c=4))
  expect_identical(merge_vectors(c(a=1), c(b=2, b=3, c=4)), c(a=1, b=3, c=4))

  # NULL value
  expect_identical(merge_vectors(NULL, c(a=1, b=2)), c(a=1, b=2))
  expect_identical(merge_vectors(c(a=1, b=2), NULL), c(a=1, b=2))

  # Lists and empty lists
  expect_identical(merge_vectors(list(a=1, b=2), list(b=3, c=4)),
                   list(a=1, b=3, c=4))
  expect_identical(merge_vectors(list(a=1, b=2), list()),
                   list(a=1, b=2))
  expect_identical(merge_vectors(list(), list(a=1, b=2)),
                   list(a=1, b=2))

  expect_identical(merge_vectors(c(a=1, b=2), NULL), c(a=1, b=2))

  expect_error(merge_vectors(c(1, 2), c(1, 2)))
  expect_error(merge_vectors(c(1, 2), c(a=1, b=2)))
  expect_error(merge_vectors(c(a=1, b=2), c(1, 2)))
})


test_that("all_same behaves as expected", {
  expect_false(all_same(1:2))
  expect_true(all_same(1))
  expect_true(all_same(c(1, 1)))
  expect_true(all_same(NA))
  expect_true(all_same(c(NA, NA)))
  expect_false(all_same(c(NA, 1)))

  # Zero-length vector
  expect_true(all_same(character()))

  # List (instead of atomic vector)
  expect_false(all_same(list()))
  expect_false(all_same(list(numeric(0))))
})


test_that("empty behaves as expected", {
  expect_true(empty(character(0)))
  expect_true(empty(logical(0)))
  expect_true(empty(list()))
  expect_true(empty(new.env()))

  # Named list with no elements
  x <- list(a=1)
  x$a <- NULL
  expect_true(empty(x))

  expect_false(empty(100))
  expect_false(empty(list(40)))
  e <- new.env()
  e$x <- 10
  expect_false(empty(e))
})


test_that("compact behaves as expected", {
  # Compact drops empty vectors and lists, but keeps environments even if empty
  e <- new.env()
  e$x <- 10
  ne <- new.env()
  x <- list(a = 1, b = list(), c = character(0), e = e, ne = ne)
  expect_identical(compact(x), list(a = 1, e = e, ne = ne))
})

#' Create a broker object
#'
#' A broker is a subclass of reactive. It can hold extra information to
#' facilitate (or broker) communication between the client and the server.
#' For example, an input broker may contain HTML controls to be emitted on the
#' client web page, as well as a function to connect the inputs from the client
#' to the reactive expression.
#'
#' Other types of brokers are possible. Another broker may create reactive
#' observers and add information to the Vega spec, instead of having HTML
#' controls. In this case, a reactive expression is still needed, although
#' it can be a dummy value, like \code{reactive(NULL)}.
#'
#' @param r A reactive expression.
#' @param controls An HTML control, or a list of HTML controls.
#' @param connect A function to run at render time. This function takes the
#'   Shiny \code{session} object as its only argument, and is used to connect
#'   the session with the broker object.
#' @param spec Object to put in the Vega spec.
#' @export
#' @keywords internal
create_broker <- function(r, controls = NULL, connect = NULL, spec = NULL) {
  if (!shiny::is.reactive(r)) stop("r must be a reactive expression.")

  # If passed a bare control, wrap it into a list
  if (!is.null(controls) && inherits(controls, "shiny.tag")) {
    controls  <- list(controls)
    names(controls) <- paste0("unnamed input ", seq_len(length(controls)))
  }

  class(r) <- c("broker", class(r))

  attr(r, "broker") <- structure(list(
    controls = controls,
    connect = connect,
    spec = spec
  ))

  if (is.null(reactive_id(r))) {
    reactive_id(r) <- rand_id("reactive_")
  }

  r
}

#' Determine if an object is a broker object
#'
#' @param x An object to test.
#' @export
is.broker <- function(x) inherits(x, "broker")

# Get the label of a connector function
connector_label <- function(x) attr(x, "label", TRUE)

# Set the label of a connector function
`connector_label<-` <- function(x, value) {
  attr(x, "label") <- value
  x
}

#' Cocaine seizures in the US.
#'
#' This dataset comes from STRIDE, the System to Retrieve Information from Drug
#' Evidence. It contains all concaine seizures in the US from 2007 that have
#' a known weight.
#'
#' @section Variables:
#' \describe{
#'   \item{state}{State where seizure occured.}
#'   \item{potency}{Purity of cocaine, as percentage (100\% = pure cocaine,
#'     0\% = all filler)}
#'   \item{weight}{Weight, in grams.}
#'   \item{month}{Month in which seizure occured.}
#'   \item{price}{Estimated value in USD.}
#' }
#' @section Use:
#' Use of this data requires your agreement to refer to your analyses as
#' "unvalidated DEA data and to claim authorship and responsibility for any
#' inferences and/or conclusions you may draw from this information."
#' @format Data frame with 3380 observations of 5 variables.
"cocaine"

#' Display data with bars (a barchart).
#'
#' This will add bars to a plot. The exact behavior is complicated because
#' the term bar chart is used to describe four important variations on a theme.
#' The action of \code{layer_bars} depends on two factors: whether or not a
#' \code{y} prop has been specified, and whether the \code{x} props is
#' continuous or categorical.
#'
#' @section Visualisations:
#'
#' If no y prop has been specified, then this will count the number of entries
#' at each unique x value. There will be one bar at each unique x value, and
#' the y value (or height) of each bar will represent the count at that x value.
#'
#' If a y prop has been specified, then those y values will be used as weights
#' for a weighted count at each unique x value. If no x values appear more than
#' once in the data, then the end result is a plot where the height of the bar
#' at each x value is simply the y value. However, if an x value appear more
#' than once in the data, then this will sum up the y values at each x.
#'
#' If the x variable is continuous, then a continuous x axis will be used, and
#' the width of each bar is by default equal to the resolution of the data --
#' that is, the smallest difference between any two x values.
#'
#' If the x variable is categorical, then a categorical x axis will be used. By
#' default, the width of each bar is 0.9 times the space between the items.
#'
#' @param vis Visualisation to modify
#' @param ... Visual properties used to override defaults.
#' @param width Width of each bar. When x is continuous, this controls the width
#'   in the same units as x. When x is categorical, this controls the width as a
#'   proportion of the spacing between items (default is 0.9).
#' @param stack If there are multiple bars to be drawn at an x location, should
#'   the bars be stacked? If FALSE, the bars will be overplotted on each other.
#' @seealso \code{\link{layer_histograms}} For bar graphs of counts at each unique
#'   x value, in contrast to a histogram's bins along x ranges.
#' @seealso \code{\link{compute_count}} and \code{\link{compute_tabulate}} for
#'   more information on how data is transformed.
#' @export
#' @examples
#' # Discrete x: bar graph of counts at each x value
#' cocaine %>% ggvis(~state) %>% layer_bars()
#' # Continuous x: bar graph of counts at unique locations
#' cocaine %>% ggvis(~month) %>% layer_bars()
#'
#' # Use y prop to weight by additional variable. This is also useful
#' # if you have pretabulated data
#' cocaine %>% ggvis(~state, ~weight) %>% layer_bars()
#' cocaine %>% ggvis(~month, ~weight) %>% layer_bars()
#'
#' # For continuous x, layer_bars is useful when the variable has a few
#' # unique values that you want to preserve. If you have many unique
#' # values and you want to bin, use layer_histogram
#' cocaine %>% ggvis(~price) %>% layer_bars()
#' cocaine %>% ggvis(~price) %>% layer_histograms(width = 100)
#'
#' # If you have unique x values, you can use layer_bars() as an alternative
#' # to layer_points()
#' pressure %>% ggvis(~temperature, ~pressure) %>% layer_points()
#' pressure %>% ggvis(~temperature, ~pressure) %>% layer_bars()
#'
#' # When x is continuous, width controls the width in x units
#' pressure %>% ggvis(~temperature, ~pressure) %>% layer_bars(width = 10)
#' # When x is categorical, width is proportional to spacing between bars
#' pressure %>% ggvis(~factor(temperature), ~pressure) %>%
#'   layer_bars(width = 0.5)
#'
#' # Stacked bars
#' # If grouping var is continuous, you need to manually specify grouping
#' ToothGrowth %>% group_by(dose) %>%
#'   ggvis(x = ~supp, y = ~len, fill = ~dose) %>% layer_bars()
#' # If grouping var is categorical, grouping is done automatically
#' cocaine %>% ggvis(x = ~state, fill = ~as.factor(month)) %>%
#'   layer_bars()
layer_bars <- function(vis, ..., stack = TRUE, width = NULL) {
  new_props <- merge_props(cur_props(vis), props(...))

  check_unsupported_props(new_props, c("x", "y", "x2", "y2"),
                          c("enter", "exit", "hover"), "layer_bars")

  x_var <- find_prop_var(new_props, "x.update")
  discrete_x <- prop_countable(cur_data(vis), new_props$x.update)

  vis <- set_scale_label(vis, "x", prop_label(cur_props(vis)$x.update))

  if (!is.null(new_props$y.update)) {
    if (prop_countable(cur_data(vis), new_props$y.update)) {
      stop("y variable (weights) must be numeric.")
    }
    y_var <- find_prop_var(new_props, "y.update")
    vis <- set_scale_label(vis, "y", prop_label(cur_props(vis)$y.update))

  } else {
    y_var <- NULL
    vis <- set_scale_label(vis, "y", "count")
  }

  if (discrete_x) {
    if (is.null(width)) {
      width <- 0.9
    }

    vis <- layer_f(vis, function(v) {
      v <- add_props(v, .props = new_props)
      v <- auto_group(v, exclude = c("x", "y"))
      v <- compute_count(v, x_var, y_var)

      if (stack) {
        v <- compute_stack(v, stack_var = ~count_, group_var = ~x_)
        v <- layer_rects(v, x = ~x_, y = ~stack_lwr_, y2 = ~stack_upr_,
                         width = band())
      } else {
        v <- layer_rects(v, x = ~x_, y = 0, y2 = ~count_, width = band())
      }
      v
    })
    vis <- scale_nominal(vis, "x", padding = 1 - width, points = FALSE)

  } else {
    vis <- layer_f(vis, function(v) {
      v <- add_props(v, .props = new_props)
      v <- compute_count(v, x_var, y_var)
      v <- compute_align(v, ~x_, length = width)
      if (stack) {
        v <- compute_stack(v, stack_var = ~count_, group_var = ~x_)
        v <- layer_rects(v, x = ~xmin_, x2 = ~xmax_, y = ~stack_upr_,
                         y2 = ~stack_lwr_)
      } else {
        v <- layer_rects(v, x = ~xmin_, x2 = ~xmax_, y = 0, y2 = ~count_)
      }
      v
    })
  }

  vis <- scale_numeric(vis, "y", domain = c(0, NA), expand = c(0, 0.05))
  vis
}

#' Creates a named list, giving the properties used by each dataset.
#'
#' @noRd
#' @examples
#' base <- mtcars %>% ggvis(~wt, ~mpg)
#' base %>% layer_points() %>% combine_data_props()
#' base %>% layer_points() %>% layer_points() %>% combine_data_props()
#' base %>% layer_points(y = ~cyl) %>% layer_points(y = ~disp) %>% combine_data_props()
#' base %>% layer_points() %>% layer_points(~conc, ~uptake, data = CO2) %>%
#'  combine_data_props()
combine_data_props <- function(mark) {
  if (is.ggvis(mark)) return(combine_data_props(mark$marks))
  if (identical(class(mark), "mark")) {
    return(stats::setNames(list(mark$props), data_id(mark$data)))
  }

  if (is.mark_group(mark)) {
    children <- mark$marks
  } else if (is.list(mark)) {
    children <- mark
  } else {
    stop("Invalid input")
  }
  all_props <- unlist(lapply(children, combine_data_props), recursive = FALSE)

  # Combine together props for data with same name
  props_by_id <- split(all_props, names(all_props))
  props_by_id <- lapply(props_by_id, unlist, recursive = FALSE, use.names = FALSE)

  # Remove duplicates, and props that don't appear in the data
  lapply(props_by_id, function(props) {
    names <- safe_vega_var(vapply(props, prop_label, character(1)))
    ok <- !duplicated(names) & names != ""

    stats::setNames(props[ok], names[ok])
  })
}

# Create a new reactive dataset containing only the data actually used
# by properties.
active_props <- function(data, props) {
  reactive_prop <- function(props, parent_data) {
    force(props)
    force(parent_data)
    reactive({
      apply_props(parent_data(), props)
    })
  }

  data_out <- list()
  for (data_n in names(props)) {
    data_out[[data_n]] <- reactive_prop(props[[data_n]], data[[data_n]])
  }

  data_out
}

# Apply properties to a data object, creating calculated columns and dropping
# unused columns.
apply_props <- function(data, props) {
  UseMethod("apply_props")
}

#' @export
apply_props.data.frame <- function(data, props) {
  cols <- lapply(props, prop_value, data = data)
  names(cols) <- vapply(props, prop_label, character(1))
  quickdf(cols)
}

#' @export
apply_props.grouped_df <- function(data, props) {
  dplyr::do(data, apply_props(., props))
}

#' Display data with a boxplot.
#'
#' This will add boxplots to a plot. The action of \code{layer_boxplots} depends
#' on whether the \code{x} prop is continuous or categorical.
#'
#' The upper and lower "hinges" correspond to the first and third quartiles (the
#' 25th and 75th percentiles). This differs slightly from the method used by the
#' \code{boxplot} function, and may be apparent with small samples. See
#' \code{\link{boxplot.stats}} for more information on how hinge positions are
#' calculated for \code{boxplot}.
#'
#' The upper whisker extends from the hinge to the highest value that is within
#' 1.5 * IQR of the hinge, where IQR is the inter-quartile range, or distance
#' between the first and third quartiles. The lower whisker extends from the
#' hinge to the lowest value within 1.5 * IQR of the hinge. Data beyond the end
#' of the whiskers are outliers and plotted as points (as specified by Tukey).
#'
#' @param vis Visualisation to modify
#' @param ... Visual properties used to override defaults.
#' @param width Width of each bar. When x is continuous, this controls the width
#'   in the same units as x. When x is categorical, this controls the width as a
#'   proportion of the spacing between items (default is 0.9).
#' @inheritParams compute_boxplot
#' @seealso \code{\link{compute_boxplot}} for more information on how data is
#'   transformed.
#' @export
#' @examples
#' library(dplyr)
#'
#' mtcars %>% ggvis(~factor(cyl), ~mpg) %>% layer_boxplots()
#' # Set the width of the boxes to half the space between tick marks
#' mtcars %>% ggvis(~factor(cyl), ~mpg) %>% layer_boxplots(width = 0.5)
#'
#' # Continuous x: boxes fill width between data values
#' mtcars %>% ggvis(~cyl, ~mpg) %>% layer_boxplots()
#' # Setting width=0.5 makes it 0.5 wide in the data space, which is 1/4 of the
#' # distance between data values in this particular case.
#' mtcars %>% ggvis(~cyl, ~mpg) %>% layer_boxplots(width = 0.5)
#'
#' # Smaller outlier points
#' mtcars %>% ggvis(~factor(cyl), ~mpg) %>% layer_boxplots(size := 20)
#' @export
layer_boxplots <- function(vis, ..., coef = 1.5, width = NULL) {

  new_props <- merge_props(cur_props(vis), props(...))

  check_unsupported_props(new_props, c("x", "y", "x2", "y2"),
                          c("enter", "exit", "hover"), "layer_boxplots")

  x_var <- find_prop_var(new_props, "x.update")
  discrete_x <- prop_countable(cur_data(vis), new_props$x.update)
  vis <- set_scale_label(vis, "x", prop_label(cur_props(vis)$x.update))

  y_var <- find_prop_var(new_props, "y.update")
  if (prop_countable(cur_data(vis), new_props$y.update)) {
    stop("y variable (weights) must be numeric.")
  }
  vis <- set_scale_label(vis, "y", prop_label(cur_props(vis)$y.update))

  new_box_props <- merge_props(props(fill := "white"), new_props)
  new_box_props <- drop_props(new_box_props, "size")

  new_outlier_props <- merge_props(new_props, props(fill := "black"))
  new_outlier_props <- drop_props(new_outlier_props,
                                  c("stroke", "strokeWidth", "strokeOpacity"))


  if (discrete_x) {
    if (is.null(width)) {
      width <- 0.9
    }
    # Whiskers and outliers are on a centered x scale
    whisker_props <- props(prop("x", x_var, scale = "xcenter"),
                           y = ~min_, y2 = ~max_, width := 0.5)
    outlier_props <- props(prop("x", x_var, scale = "xcenter"), y = ~value_)

    rect_props <- props(x = x_var, y = ~lower_, y2 = ~upper_, width = band())
    median_props <- props(x = x_var, y = ~median_, height := 1, width = band())

    vis <- scale_nominal(vis, "x", padding = 1 - width, points = FALSE)

    # To get whiskers to line up with centers of x, need to use these padding
    # and points settings.
    vis <- scale_nominal(vis, "x", name = "xcenter", padding = 2 - width,
                         points = TRUE)
  } else {
    whisker_props <- props(x = x_var, y = ~min_, y2 = ~max_, width := 0.5)
    rect_props <- props(x = ~xmin_, x2 = ~xmax_, y = ~lower_, y2 = ~upper_)
    median_props <- props(x = ~xmin_, x2 = ~xmax_, y = ~median_, height := 1)
    outlier_props <- props(x = x_var, y = ~value_)
  }

  vis <- layer_f(vis, function(v) {
    v <- add_props(v, .props = new_box_props)

    # Group by x variable
    v <- dplyr::group_by(v, !!x_var[[2]])
    v <- compute_boxplot(v, y_var, coef = coef)
    if (!discrete_x) {
      v <- compute_align(v, x_var, length = width)
    }

    v <- emit_rects(v, merge_props(new_box_props, whisker_props))
    v <- emit_rects(v, merge_props(new_box_props, rect_props))
    # Median line
    v <- emit_rects(v, merge_props(new_box_props, median_props))

    # Outlier points need their own data set
    v <- compute_boxplot_outliers(v)
    v <- emit_points(v, merge_props(new_outlier_props, outlier_props))

    v
  })

  vis
}

#' Create an axis_props object for controlling axis properties.
#'
#' The items in each of the lists can be a literal value, like \code{5} or
#' "blue", or they can be a \code{\link{scaled_value}} object.
#'
#' @param axis A named list of line properties for the axis line.
#' @param ticks A named list of line properties for ticks.
#' @param majorTicks A named list of line properties for major ticks.
#' @param minorTicks A named list of line properties for minor ticks.
#' @param grid A named list of line properties for grid lines.
#' @param labels A named list of text properties for axis labels.
#' @param title A named list of text properties for the axis title.
#'
#' @export
axis_props <- function(ticks = NULL, majorTicks = NULL, minorTicks = NULL,
                       grid = NULL, labels = NULL, title = NULL, axis = NULL) {

  args <- list(ticks = ticks, majorTicks = majorTicks, minorTicks = minorTicks,
               grid = grid, labels = labels, title = title, axis = axis)

  # Validate properties
  prop_names <- lapply(args, function(arg) names(arg))
  types <- c("line", "line", "line", "line", "text", "text", "line")
  Map(check_mark_props, types, prop_names)

  structure(
    compact(args),
    class = "axis_props"
  )
}

#' Tests whether an object is an axis_props object
#' @param x an object to test
#' @export
#' @keywords internal
is.axis_props <- function(x) inherits(x, "axis_props")

#' @export
as.vega.axis_props <- function(x, ...) {
  as_value <- function(item) {
    lapply(item, function(val) {
      if (is.scaled_value(val)) {
        val
      } else {
        list(value = val)
      }
    })
  }

  lapply(x, as_value)
}

#' Create an axis_props object for controlling legend properties.
#'
#' The items in each of the lists can be a literal value, like \code{5} or
#' "blue", or they can be a \code{\link{scaled_value}} object.
#'
#' @param title A named list of text properties for the legend title.
#' @param labels A named list of text properties for legend labels.
#' @param symbols A named list of line properties for symbols (for discrete
#'   legend items).
#' @param gradient A named list of line properties a continuous color gradient.
#' @param legend A named list of line properties for the overall legend. The
#'   x and y position can be set here, which will override automatic
#'   positioning.
#'
#' @export
legend_props <- function(title = NULL, labels = NULL,
                         symbols = NULL, gradient = NULL, legend = NULL) {

  args <- list(title = title, labels = labels, symbols = symbols,
               gradient = gradient, legend = legend)

  # Validate properties
  prop_names <- lapply(args, function(arg) names(arg))
  types <- c("text", "text", "symbol", "rect", "rect")
  Map(check_mark_props, types, prop_names)

  structure(
    compact(args),
    class = "legend_props"
  )
}

#' Tests whether an object is a legend_props object
#' @param x an object to test
#' @export
#' @keywords internal
is.legend_props <- function(x) inherits(x, "legend_props")

#' @export
as.vega.legend_props <- as.vega.axis_props


#' Create a scaled_value object
#'
#' These are for use with legends and axes.
#' @param scale The name of a scale, e.g., "x", "fill".
#' @param value A value which will be transformed using the scale.
#'
#' @export
scaled_value <- function(scale, value) {
  structure(list(scale = scale, value = value), class = "scaled_value")
}

#' Tests whether an object is a scaled_value object
#' @param x an object to test
#' @export
#' @keywords internal
is.scaled_value <- function(x) inherits(x, "scaled_value")

#' @export
as.vega.scaled_value <- function(x, ...) x

#' Guess the right type of layer based on current properties.
#'
#' \code{layer_guess} provides the magic behind the default behaviour of
#' \code{\link{ggvis}}.
#'
#' @section Defaults:
#'
#' \itemize{
#'   \item Continuous x, \code{\link{layer_histograms}}
#'   \item Categorical x, \code{\link{layer_bars}}
#'   \item Continuous x and y, \code{\link{layer_points}}
#' }
#'
#' @param vis The visualisation to add the new layer to.
#' @param ... Other arguments passed on individual layers.
#' @export
#' @examples
#' # A scatterplot:
#' mtcars %>% ggvis(~mpg, ~wt)
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_guess()
#'
#' # A histogram:
#' mtcars %>% ggvis(~mpg)
#' mtcars %>% ggvis(~mpg) %>% layer_guess()
layer_guess <- function(vis, ...) {
  types <- lapply(vis$cur_props, function(x) {
    shiny::isolate(prop_type(x, data = cur_data(vis)))
  })

  types <- types[grepl("\\.update$", names(types))]
  names(types) <- sub("\\.update$", "", names(types))

  layer <- closest(unlist(types), templates)
  message("Guessing layer_", layer, "()")

  f <- match.fun(paste0("layer_", layer))
  f(vis, ...)
}

template <- function(layer, x = NA, y = NA, ...) {
  desc <- c(x = x, y = y, ...)
  stopifnot(is.character(desc))
  desc <- desc[!is.na(desc)]

  structure(list(layer = layer, desc = desc), class = "template")
}

#' @export
print.template <- function(x, ...) {
  cat("<template> ",
    paste0(names(x$desc), " = ", x$desc, collapse = ", "),
    " -> ",
    x$layer,
    "\n",
    sep = "")
}

templates <- list(
  template("bars",       "nominal"),
#   template("boxplots",   "nominal", "numeric"),
  template("bars",       "nominal", "nominal"),

  template("histograms", "numeric"),
  template("freqpolys",  "numeric", stroke = "nominal"),
#   template("boxplots",   "numeric", "nominal"),

  template("points",     "numeric", "numeric"),
  template("lines",      "datetime", "numeric"),
  template("points",     "datetime", "datetime")
)

closest <- function(data, templates) {
  ds <- vapply(templates, distance_n, data = data, FUN.VALUE = numeric(1))
  if (!any(is.finite(ds))) {
    stop("Don't now how to guess a layer for this type of data", call. = FALSE)
  }

  templates[[which.min(ds)]]$layer
}

distance_n <- function(data, template) {
  all <- union(names(data), names(template$desc))

  ds <- unlist(Map(distance_1, data[all], template$desc[all]))
  sum(ds)
}

distance_1 <- function(data, template) {
  if (is.na(data) && is.na(template)) return(0) # Both missing, so can match
  if (is.na(data) && !is.na(template)) return(Inf) # If in template, must be in data
  if (!is.na(data) && is.na(template)) return(0.01)   # If in data, but not template, minor penalty

  n_miss <- is.na(data) + is.na(template)
  if (n_miss == 1) return(Inf) # One missing, so can't match
  if (n_miss == 2) return(0)

  switch(data,
    numeric   = switch(template, numeric = 0, Inf),
    datetime  = switch(template, numeric = 1, datetime = 0, Inf),
    nominal   = switch(template, nominal = 0, Inf),
    ordinal   = switch(template, nominal = 1, ordinal = 0, Inf),
    Inf
  )
}

#' A band
#'
#' Bands are used to set the width or height on categorical scales - a band
#' represent the height or width allocated for one level of a factor.
#'
#' @param offset,mult Additive and multiplicate offsets used to adjust the
#'   band size. For example, use \code{mult = 0.9} to make a bar take up
#'   90\% of the space allocated for its category.
#' @export
#' @examples
#' df <- data.frame(label = c("a", "b", "c"), n = c(10, 9, 4))
#'
#' base <- df %>% ggvis(~label, y2 = 0, y = ~n)
#' base %>% layer_rects(width = band())
#' base %>% layer_rects(width = band(offset = -1))
#' base %>% layer_rects(width = band(mult = 0.9))
#'
#' # A nominal scale with padding is more symmetrical than band with a mult
#' base %>% layer_rects(width = band(mult = 0.75))
#' base %>% layer_rects(width = band()) %>%
#'   scale_nominal("x", padding = 0.25, points = FALSE)
band <- function(offset = NULL, mult = NULL) {
  structure(
    list(offset = offset, mult = mult),
    class = c("band")
  )
}

#' @export
new_prop.band <- function(x, property, scale, offset, mult, env, event,
                          label) {
  if (!(property %in% c("width", "height"))) {
    stop("band() can only be used for width and height properties.")
  }
  if (identical(scale, FALSE)) stop("band() must be scaled.")

  structure(
    list(
      property = property,
      scale = decide_scale(scale %||% TRUE, property),
      offset = offset,
      mult = mult,
      event = event,
      env = NULL
    ),
    class = c("prop_band", "prop")
  )
}

#' @export
as.character.prop_band <- function(x, ...) ""

#' @rdname band
#' @param x object to test for band-ness
is.prop_band <- function(x) inherits(x, "prop_band")

#' @export
prop_value.prop_band <- function(x, data) {
  NULL
}

#' @export
prop_label.prop_band <- function(x) {
  ""
}

#' @export
prop_domain.prop_band <- function(x, data) {
  NULL
}

#' @export
prop_vega.prop_band <- function(x, default_scale) {
  compact(list(
    scale = x$scale,
    mult = x$mult,
    offset = x$offset,
    band = TRUE
  ))
}

# Basic line graphs
library(ggvis)
library(dplyr, warn.conflicts = FALSE)

set.seed(1780)
df <- data.frame(x = runif(12), y = runif(12), z = gl(3, 4),
                 w = factor(rep(letters[1:4], 3)))

df %>%
  ggvis(x = ~x, y = ~y) %>%
  layer_paths() %>%
  save_spec("line/basic.json")

df %>%
  ggvis(x = ~x, y = ~y) %>%
  arrange(x) %>%
  layer_paths() %>%
  save_spec("line/sort.json")

df %>%
  ggvis(x = ~x, y = ~y, stroke = ~z) %>%
  layer_lines() %>%
  save_spec("line/layer-line.json")

# Auto-grouping for lines with nominal x
df %>%
  ggvis(x = ~w, y = ~y, stroke = ~z) %>%
  layer_lines() %>%
  save_spec("line/layer-line-nominal-x.json")

#' Compute the "resolution" of a data vector.
#'
#' The resolution is is the smallest non-zero distance between adjacent
#' values.  If there is only one unique value, then the resolution is defined
#' to be one.
#'
#' If x is an integer vector, then it is assumed to represent a discrete
#' variable, and the resolution is 1.
#'
#' @param x numeric vector
#' @param zero should a zero value be automatically included in the
#'   computation of resolution
#' @export
#' @examples
#' resolution(1:10)
#' resolution((1:10) - 0.5)
#' resolution((1:10) - 0.5, FALSE)
#' resolution(c(1,2, 10, 20, 50))
#' resolution(as.integer(c(1, 10, 20, 50)))  # Returns 1
resolution <- function(x, zero = TRUE) {
  if (is.integer(x) || zero_range(range(x, na.rm = TRUE)))
    return(1)

  x <- unique(as.numeric(x))
  if (zero) {
    x <- unique(c(0, x))
  }

  min(diff(sort(x)))
}


#' Determine if range of vector is close to zero, with a specified tolerance
#'
#' The machine epsilon is the difference between 1.0 and the next number
#' that can be represented by the machine. By default, this function
#' uses epsilon * 100 as the tolerance. First it scales the values so that
#' they have a mean of 1, and then it checks if the difference between
#' them is larger than the tolerance.
#'
#' @examples
#' eps <- .Machine$double.eps
#' zero_range(c(1, 1 + eps))       # TRUE
#' zero_range(c(1, 1 + 99 * eps))  # TRUE
#' zero_range(c(1, 1 + 101 * eps)) # FALSE - Crossed the tol threshold
#' zero_range(c(1, 1 + 2 * eps), tol = eps) # FALSE - Changed tol
#'
#' # Scaling up or down all the values has no effect since the values
#' # are rescaled to 1 before checking against tol
#' zero_range(100000 * c(1, 1 + eps))        # TRUE
#' zero_range(100000 * c(1, 1 + 200 * eps))  # FALSE
#' zero_range(.00001 * c(1, 1 + eps))        # TRUE
#' zero_range(.00001 * c(1, 1 + 200 * eps))  # FALSE
#'
#' # NA values
#' zero_range(c(1, NA))   # NA
#' zero_range(c(1, NaN))  # NA
#'
#' # Infinite values
#' zero_range(c(1, Inf))     # FALSE
#' zero_range(c(-Inf, Inf))  # FALSE
#' zero_range(c(Inf, Inf))   # TRUE
#'
#' @param x numeric range: vector of length 2
#' @param tol A value specifying the tolerance. Defaults to
#'   \code{.Machine$double.eps * 100}.
#' @return logical \code{TRUE} if the relative difference of the endpoints of
#' the range are not distinguishable from 0.
#' @export
zero_range <- function(x, tol = .Machine$double.eps * 100) {
  if (length(x) == 1) return(TRUE)
  if (length(x) != 2) stop("x must be length 1 or 2")
  if (any(is.na(x)))  return(NA)

  # Special case: if they are equal as determined by ==, then there
  # is zero range. Also handles (Inf, Inf) and (-Inf, -Inf)
  if (x[1] == x[2]) return(TRUE)

  # If we reach this, then x must be (-Inf, Inf) or (Inf, -Inf)
  if (all(is.infinite(x))) return(FALSE)

  # Take the smaller (in magnitude) value of x, and use it as the scaling
  # factor.
  m <- min(abs(x))

  # If we get here, then exactly one of the x's is 0. Return FALSE
  if (m == 0) return(FALSE)

  # If x[1] - x[2] (scaled to 1) is smaller than tol, then return
  # TRUE; otherwise return FALSE
  abs((x[1] - x[2])/m) < tol
}

library(ggvis)

var_range <- function(id, label, variable) {
  rng <- range(variable, na.rm = TRUE)
  sliderInput(id, label, rng[1], rng[2], rng)
}

shinyUI(pageWithSidebar(
  headerPanel("Zooming demo"),
  sidebarPanel(
    var_range("x_domain", "X", mtcars$disp),
    var_range("y_domain", "Y", mtcars$mpg)
  ),
  mainPanel(
    ggvisOutput("zoom")
  )
))

range_prop <- function(x, name) {
  if (is.null(x)) return(list())

  # Character vector always left as is
  if (is.character(x)) {
    return(named_list(name, x))
  }

  assert_that(is.numeric(x), length(x) <= 2)
  n_miss <- sum(is.na(x))

  if (n_miss == 0) {
    named_list(name, x)
  } else if (n_miss == 1) {
    if (is.na(x[1])) {
      named_list(paste0(name, "Max"), x[2])
    } else {
      named_list(paste0(name, "Min"), x[1])
    }
  } else if (n_miss == 2) {
    list()
  }

}

named_list <- function(names, ...) {
  stats::setNames(list(...), names)
}

#' Convert the name of a property to the name of its default scale.
#'
#' This is mainly used to ensure that similar properties share the same
#' scale by default - e.g. \code{x} and \code{x2} should use the same
#' scale.
#'
#' @param prop character vector of property names. Any unrecognised names
#'   are left unchanged.
#' @return character vector of default scale names.
#' @keywords internal
#' @export
#' @examples
#' propname_to_scale(c("x", "x2"))
#' propname_to_scale(c("foo", "bar"))
#' propname_to_scale(c("opacity", "fillOpacity", "strokeOpacity"))
propname_to_scale <- function(prop) {
  simplify <- c(
    "x2" = "x",
    "width" = "x",
    "y2" = "y",
    "height" = "y",
    "fillOpacity" = "opacity",
    "strokeOpacity" = "opacity",
    "innerRadius" = "radius",
    "outerRadius" = "radius",
    "startAngle" = "angle",
    "endAngle" = "angle"
  )

  matches <- match(prop, names(simplify))
  prop[!is.na(matches)] <- simplify[prop[!is.na(matches)]]
  prop
}

#' Given the type of a ggvis scale, get the name of its corresponding vega scale
#'
#' @param type property type: numeric, ordinal, nominal, logical or datetime.
#' @keywords internal
scaletype_to_vega_scaletype <- function(type) {
  unname(c(
    "numeric" = "quantitative",
    "ordinal" = "ordinal",
    "nominal" = "ordinal",
    "logical" = "ordinal",
    "datetime" = "time"
  )[type])
}

#' Handle mouse actions on marks.
#'
#' @param vis Visualisation to listen to.
#' @param on_click,on_mouse_over Callback function with arguments:
#'   \describe{
#'    \item{data}{A data frame with one row}
#'    \item{location}{A named list with components x and y}
#'    \item{session}{The session, used to communicate with the browser}
#'   }
#' @param on_mouse_out Callback function with argument:
#'   \describe{
#'    \item{session}{The session, used to communicate with the browser}
#'   }
#' @export
#' @examples
#' location <- function(location, ...) cat(location$x, "x", location$y, "\n")
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points() %>%
#'   handle_click(location)
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points() %>%
#'   handle_hover(function(...) cat("over\n"), function(...) cat("off\n"))
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points() %>%
#'   handle_hover(function(data, ...) str(data))
handle_click <- function(vis, on_click = NULL) {
  check_callback(on_click, c("data", "location", "session"))

  connect <- function(session, plot_id) {
    watch_mouse(session, paste0(plot_id, "_mouse_click"), on_click)
  }

  spec <- list(type = "click")
  broker <- create_broker(reactive(NULL), connect = connect, spec = spec)
  register_reactive(vis, broker)
}

#' @rdname handle_click
#' @export
handle_hover <- function(vis, on_mouse_over = NULL, on_mouse_out = NULL) {
  if (!is.null(on_mouse_over))
    check_callback(on_mouse_over, c("data", "location", "session"))
  if (!is.null(on_mouse_out))
    check_callback(on_mouse_out, "session")

  connect <- function(session, plot_id) {
    watch_mouse(session, paste0(plot_id, "_mouse_over"), on_mouse_over)

    if (is.null(on_mouse_out)) return()
    shiny::observe({
      value <- session$input[[paste0(plot_id, "_mouse_out")]]
      if (is.null(value)) return()

      on_mouse_out(session = session)
    })
  }

  spec <- list(type = "hover")
  broker <- create_broker(reactive(NULL), connect = connect, spec = spec)
  register_reactive(vis, broker)
}

watch_mouse <- function(session, id, fun) {
  shiny::observe({
    value <- session$input[[id]]
    if (is.null(value)) return()
    if (!is.list(value$data)) return() # axis ticks/labels etc

    df <- value$data
    class(df) <- "data.frame"
    attr(df, "row.names") <- .set_row_names(1L)

    fun(
      data = df,
      location = list(x = value$pagex, y = value$pagey),
      session = session
    )
  })
}

# Basic scatterplot
library(ggvis)

set.seed(1014)
df <- data.frame(x = 1:5, y = 5:1, z = runif(5))

df %>%
  ggvis(x = ~x, y = ~x) %>%
  layer_points() %>%
  save_spec("scatter/basic.json")

df %>%
  ggvis(x = ~factor(x), y = ~x / y) %>%
  layer_points() %>%
  save_spec("scatter/transform.json")

df %>%
  ggvis(x = ~x, y = ~y, fill = ~z) %>%
  layer_points() %>%
  save_spec("scatter/fill-continuous.json")

df %>%
  ggvis(x = ~x, y = ~y, fill = ~factor(z)) %>%
  layer_points() %>%
  save_spec("scatter/fill-discrete.json")

# Data handling tests
library(ggvis)
library(dplyr, warn.conflicts = FALSE)

# Using "." in names of grouped and non-grouped vars
mtcars %>%
  dplyr::mutate(cyl.new = factor(cyl), mpg.new = mpg) %>%
  group_by(cyl.new) %>%
  ggvis(~wt, ~mpg.new, fill = ~cyl.new) %>%
  layer_points() %>%
  save_spec("data/dots.json")

#' Add tooltips to a plot.
#'
#' @param vis Visualisation to add tooltips to.
#' @param html A function that takes a single argument as input. This argument
#'   will be a list containing the data in the mark currently under the
#'   mouse. It should return a string containing HTML or \code{NULL} to
#'   hide tooltip for the current element.
#' @param on Should tooltips appear on hover, or on click?
#' @export
#' @examples
#' ## Run these examples only in interactive R sessions
#' if (interactive()) {
#'
#' all_values <- function(x) {
#'   if(is.null(x)) return(NULL)
#'   paste0(names(x), ": ", format(x), collapse = "<br />")
#' }
#'
#' base <- mtcars %>% ggvis(x = ~wt, y = ~mpg) %>%
#'   layer_points()
#' base %>% add_tooltip(all_values, "hover")
#' base %>% add_tooltip(all_values, "click")
#'
#' # The data sent from client to the server contains only the data columns that
#' # are used in the plot. If you want to get other columns of data, you should
#' # to use a key to line up the item from the plot with a row in the data.
#' mtc <- mtcars
#' mtc$id <- 1:nrow(mtc)  # Add an id column to use ask the key
#'
#' all_values <- function(x) {
#'   if(is.null(x)) return(NULL)
#'   row <- mtc[mtc$id == x$id, ]
#'   paste0(names(row), ": ", format(row), collapse = "<br />")
#' }
#'
#' mtc %>% ggvis(x = ~wt, y = ~mpg, key := ~id) %>%
#'   layer_points() %>%
#'   add_tooltip(all_values, "hover")
#'
#' }
add_tooltip <- function(vis, html, on = c("hover", "click")) {
  on <- match.arg(on)

  show_tooltip2 <- function(data, location, session, ...) {
    if (is.null(data)) {
      hide_tooltip(session)
      return()
    }

    html <- html(data)
    if (is.null(html)) {
      hide_tooltip(session)
    } else {
      show_tooltip(session, location$x + 5, location$y + 5, html)
    }
  }
  hide_tooltip2 <- function(session) {
    hide_tooltip(session)
  }

  switch(on,
    click = handle_click(vis, show_tooltip2),
    hover = handle_hover(vis, show_tooltip2, hide_tooltip2)
  )
}

#' Send a message to the client to show or hide a tooltip
#'
#' @param session A Shiny session object.
#' @param l Pixel location of left edge of tooltip (relative to page)
#' @param t Pixel location of top edge of tooltip (relative to page)
#' @param html HTML to display in the tooltip box.
#'
#' @export
show_tooltip <- function(session, l = 0, t = 0, html = "") {
  ggvis_message(session, "show_tooltip",
    list(pagex = l, pagey = t, html = html))
}

#' @rdname show_tooltip
#' @export
hide_tooltip <- function(session) {
  ggvis_message(session, "hide_tooltip")
}

#' Add a vega axis specification to a ggvis plot
#'
#' Axis specifications allow you to either override the default axes,
#' or additional axes.
#'
#' More information about axes can be found in the "axes and legends" vignettes.
#'
#' @section Compared to ggplot2:
#'
#' In ggplot2, axis (and legend) properties are part of the scales
#' specification. In vega, they are separate, which allows the specification
#' of multiple axes, and more flexible linkage between scales and axes.
#'
#' @param vis A ggvis object.
#' @param type The type of axis. Either x or y.
#' @param scale The name of the scale backing the axis component. Defaults to
#'   the scale type - you will need to specify if you want (e.g.) a scale
#'   for a secondary y-axis.
#' @param orient The orientation of the axis. One of top, bottom, left or right.
#'   The orientation can be used to further specialize the axis type (e.g., a y
#'   axis oriented for the right edge of the chart) - defaults to bottom for
#'   x axes, and left for y axes.
#' @param title A title for the axis. By default, it uses the name of the field
#'   in the first data set used by the scale. Use \code{""} to suppress the
#'   title.
#' @param title_offset The offset (in pixels) from the axis at which to place
#'   the title.
#' @param format The formatting pattern for axis labels. Vega uses D3's format
#'   pattern.
#' @param ticks A desired number of ticks. The resulting number may be different
#'   so that values are "nice" (multiples of 2, 5, 10) and lie within the
#'   underlying scale's range.
#' @param values Explicitly set the visible axis tick values.
#' @param subdivide If provided, sets the number of minor ticks between major
#'   ticks (the value 9 results in decimal subdivision).
#' @param tick_padding The padding, in pixels, between ticks and text labels.
#' @param tick_size_major,tick_size_minor,tick_size_end
#'   The size, in pixels, of major, minor and end ticks.
#' @param offset The offset, in pixels, by which to displace the axis from the
#'   edge of the enclosing group or data rectangle.
#' @param layer A string indicating if the axis (and any gridlines) should be
#'   placed above or below the data marks. One of "front" or "back" (default).
#' @param grid A flag indicating if gridlines should be created in addition to
#'   ticks.
#' @param properties Optional mark property definitions for custom axis styling.
#'   Should be an object created by \code{\link{axis_props}}, with properties
#'   for ticks, majorTicks, minorTicks, grid, labels, title, and axis.
#' @seealso Vega axis documentation:
#'   \url{https://vega.github.io/vega/docs/axes/}
#' @export
#' @examples
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   add_axis("x", title = "Weight", orient = "top")
#'
#' # Suppress axis with hide_axis
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   hide_axis("x") %>% hide_axis("y")
#'
#' mtcars %>% ggvis(x = ~wt, y = ~mpg) %>% layer_points() %>%
#'   add_axis("x", title = "Weight", ticks = 40,
#'     properties = axis_props(
#'       ticks = list(stroke = "red"),
#'       majorTicks = list(strokeWidth = 2),
#'       grid = list(stroke = "red"),
#'       labels = list(
#'         fill = "steelblue",
#'         angle = 50,
#'         fontSize = 14,
#'         align = "left",
#'         baseline = "middle",
#'         dx = 3
#'       ),
#'       title = list(fontSize = 16),
#'       axis = list(stroke = "#333", strokeWidth = 1.5)
#'     )
#'   )
add_axis <- function(vis, type, scale = NULL, orient = NULL, title = NULL,
                       title_offset = NULL, format = NULL, ticks = NULL,
                       values = NULL, subdivide = NULL, tick_padding = NULL,
                       tick_size_major = NULL, tick_size_minor = tick_size_major,
                       tick_size_end = tick_size_major, offset = NULL,
                       layer = "back", grid = TRUE, properties = NULL) {

  if (is.null(scale)) {
    if (is.null(vis$cur_vis)) {
      scale <- type
    } else {
      scale <- paste0(type, paste0(vis$cur_vis, collapse = "-"))
    }
  }

  axis <- create_axis(type, scale, orient, title, title_offset, format,
                      ticks, values, subdivide, tick_padding,
                      tick_size_major, tick_size_minor, tick_size_end,
                      offset, layer, grid, properties)

  append_ggvis(vis, "axes", axis)
}

#' @rdname add_axis
#' @export
hide_axis <- function(vis, scale) {
  axis <- structure(list(scale = scale, hide = TRUE), class = "ggvis_axis")
  append_ggvis(vis, "axes", axis)
}

#' Defunct function for adding an axis
#'
#' This function has been replaced with \code{\link{add_axis}}.
#' @param ... Other arguments.
#' @export
add_guide_axis <- function(...) {
  stop("add_guide_axis() has been replaced by add_axis().")
}

# Create an axis object.
create_axis <- function(type, scale = type, orient = NULL, title = NULL,
                 title_offset = NULL, format = NULL, ticks = NULL,
                 values = NULL, subdivide = NULL, tick_padding = NULL,
                 tick_size_major = NULL, tick_size_minor = tick_size_major,
                 tick_size_end = tick_size_major, offset = NULL,
                 layer = "back", grid = TRUE, properties = NULL) {

  assert_that(type %in% c("x", "y"))
  assert_that(is.string(scale))

  if (is.null(orient)) orient <- c(x = "bottom", y = "left")[type]
  orient <- match.arg(orient, c("top", "right", "bottom", "left"))

  assert_that(is.null(title) || is.string(title))
#   assert_that(is.number(title_offset))

  layer <- match.arg(layer, c("front", "back"))
  assert_that(is.flag(grid))

  assert_that(is.null(properties) || is.axis_props(properties))

  structure(compact(list(
      type = type, scale = scale, orient = orient, title = title,
      titleOffset = title_offset, format = format, ticks = ticks,
      values = values, subdivide = subdivide, tickPadding = tick_padding,
      tickSizeMajor = tick_size_major, tickSizeMinor = tick_size_minor,
      tickSizeEnd = tick_size_end, offset = offset, layer = layer,
      grid = grid, properties = properties
  )), class = "ggvis_axis")
}


add_missing_axes <- function(vis) {
  axes <- vis$axes
  scales <- vis$scales

  present <- vapply(axes, "[[", "scale", FUN.VALUE = character(1))
  missing <- setdiff(intersect(names(scales), c("x", "y")), present)

  for (scale in missing) {
    vis <- add_axis(vis, scale)
  }
  vis
}

# Some axis settings require examining the scale
apply_axes_defaults <- function(vis) {
  axes <- vis$axes
  scales <- vis$scales

  axes <- lapply(axes, function(axis) {
    scale <- scales[[axis$scale]]

    # If we don't have a title, try to get it from the scale.
    # Use [[-indexing to avoid partial name matching of "titleOffset". (#269)
    if (is.null(axis[["title"]])) {
      axis$title <- scale$label
    }

    axis
  })

  # Replace the original axes with the new ones
  vis$axes <- axes
  vis
}

#' @export
format.ggvis_axis <- function(x, ...) {
  params <- param_string(x, collapse = FALSE)
  param_s <- paste0("  ", format(paste0(names(params), ":")), " ", format(params),
    collapse = "\n")

  paste0("<", class(x)[1], ">\n", param_s)
}

#' @export
print.ggvis_axis <- function(x, ...) cat(format(x, ...), "\n", sep = "")

#' Create a model of a data set and compute predictions.
#'
#' Fit a 1d model, then compute predictions and (optionally) standard errors
#' over an evenly spaced grid.
#'
#' \code{compute_model_prediction} fits a model to the data and makes
#' predictions with it. \code{compute_smooth} is a special case of model
#' predictions where the model is a smooth loess curve whose smoothness is
#' controlled by the \code{span} parameter.
#'
#' @param x Dataset-like object to model and predict. Built-in methods for data
#'   frames, grouped data frames and ggvis visualisations.
#' @param model Model fitting function to use - it must support R's standard
#'   modelling interface, taking a formula and data frame as input, and
#'   returning predictions with \code{\link{predict}}. If not supplied, will use
#'   \code{\link{loess}} for <= 1000 points, otherwise it will use
#'   \code{\link[mgcv]{gam}}. Other modelling functions that will work include
#'   \code{\link{lm}}, \code{\link{glm}} and \code{\link[MASS]{rlm}}.
#' @param formula Formula passed to modelling function. Can use any variables
#'   from data.
#' @param se include standard errors in output? Requires appropriate method of
#'   \code{predict_grid}, since the interface for returning predictions with
#'   standard errors is not consistent acrossing modelling frameworks.
#' @param level the confidence level of the standard errors.
#' @param n the number of grid points to use in the prediction
#' @param domain If \code{NULL} (the default), the domain of the predicted
#'   values will be the same as the domain of the prediction variable in the
#'   data. It can also be a two-element numeric vector specifying the min and
#'   max.
#' @param ... arguments passed on to \code{model} function
#' @param method Deprecated. Please use \code{model} instead.
#' @param span Smoothing span used for loess model.
#' @return A data frame with columns: \item{\code{resp_}}{regularly spaced grid
#'   of \code{n} locations} \item{\code{pred_}}{predicted value from model}
#'   \item{\code{pred_lwr_} and \code{pred_upr_}}{upper and lower bounds of
#'   confidence interval (if \code{se = TRUE})} \item{\code{pred_se_}}{the
#'   standard error (width of the confidence interval) (if \code{se = TRUE})}
#' @export
#' @examples
#' # Use a small value of n for these examples
#' mtcars %>% compute_model_prediction(mpg ~ wt, n = 10)
#' mtcars %>% compute_model_prediction(mpg ~ wt, n = 10, se = TRUE)
#' mtcars %>% group_by(cyl) %>% compute_model_prediction(mpg ~ wt, n = 10)
#'
#' # compute_smooth defaults to loess
#' mtcars %>% compute_smooth(mpg ~ wt)
#'
#' # Override model to suppress message or change approach
#' mtcars %>% compute_model_prediction(mpg ~ wt, n = 10, model = "loess")
#' mtcars %>% compute_model_prediction(mpg ~ wt, n = 10, model = "lm")
#'
#' # Set the domain manually
#' mtcars %>%
#'   compute_model_prediction(mpg ~ wt, n = 20, model = "lm", domain = c(0, 8))
#'
#' # Plot the results
#' mtcars %>% compute_model_prediction(mpg ~ wt) %>%
#'   ggvis(~pred_, ~resp_) %>%
#'   layer_paths()
#' mtcars %>% ggvis() %>%
#'   compute_model_prediction(mpg ~ wt) %>%
#'   layer_paths(~pred_, ~resp_)
compute_model_prediction <- function(x, formula, ..., model = NULL, se = FALSE,
                              level = 0.95, n = 80L, domain = NULL, method) {
  if (!missing(method)) {
    deprecated("method", version = "0.3")
    model <- method
  }
  UseMethod("compute_model_prediction")
}

#' @export
compute_model_prediction.data.frame <- function(x, formula, ..., model = NULL,
                                         se = FALSE, level = 0.95, n = 80L,
                                         domain = NULL, method) {
  assert_that(is.formula(formula))
  model <- model %||% guess_model(x)
  assert_that(is.string(model))

  if (nrow(x) == 0) {
    return(empty_smooth(se))
  }

  restore <- identity

  if (is.character(model)) {
    # This allows the use of e.g. MASS::rlm
    model <- parse(text = model)[[1]]
  }

  if (model == quote(loess)) {
    # loess can't handle POSIXct, so convert to numeric. Fortunately we know
    # that loess only has a single predictor to extract.
    pred_var <- formula[[3]]
    pred_vals <- eval(pred_var, x)
    if (inherits(pred_vals, "POSIXct")) {
      x[[as.character(pred_var)]] <- as.numeric(pred_vals)
      tz <- attr(pred_vals, "tzone", TRUE)

      restore <- function(data) {
        data$pred_ <- as.POSIXct(data$pred_, origin = "1970-01-01", tz = tz)
        data
      }
    } else if (inherits(pred_vals, "Date")) {
      x[[as.character(pred_var)]] <- as.numeric(pred_vals)

      restore <- function(data) {
        data$pred_ <- structure(data$pred_, class = "Date")
        data
      }
    }
  }

  # Create model environment & model call
  env <- new.env(parent = environment(formula))
  env$data <- x
  model_call <- make_call(model, formula, data = quote(data), ...)

  # Fit model and make predictions
  model <- eval(model_call, env)
  res <- pred_grid(model, x, se = se, level = level, n = n, domain = domain)
  restore(res)
}

empty_smooth <- function(se = FALSE) {
  res <- data.frame(resp_ = numeric(0), pred_ = numeric(0))
  if (se) {
    res$pred_lwr_ <- numeric(0)
    res$pred_upr_ <- numeric(0)
    res$pred_se_ <- numeric(0)
  }
  res
}

#' @export
compute_model_prediction.grouped_df <- function(x, formula, ..., model = NULL,
                                         se = FALSE, level = 0.95, n = 80L,
                                         domain = NULL, method) {
  dplyr::do(x, compute_model_prediction(., formula = formula, model = model,
    se = se, level = level, n = n, domain = domain, ...))
}

globalVariables(".")

#' @export
compute_model_prediction.ggvis <- function(x, formula, ..., model = NULL,
                                           se = FALSE, level = 0.95, n = 80L,
                                           domain = NULL, method) {
  args <- list(formula = formula, model = model, se = se, level = level,
               n = n, domain = domain, ...)

  register_computation(x, args, "model_prediction", function(data, args) {
    output <- do_call(compute_model_prediction, quote(data), .args = args)
    preserve_constants(data, output)
  })
}


#' @rdname compute_model_prediction
#' @export
compute_smooth <- function(x, formula, ..., span = 0.75, se = FALSE) {
  compute_model_prediction(x, formula, ..., model = "loess", span = span,
                           se = se)
}

guess_model <- function(data) {
  model <- if (max_rows(data) > 1000) "gam" else "loess"
  notify_guess(model)
  model
}

# Helper function to create data frame of predictions -------------------------

pred_grid <- function(model, data, domain = NULL, n = 80, se = FALSE,
                      level = 0.95) {
  assert_that(is.flag(se))
  assert_that(is.numeric(level), length(level) == 1, level >= 0, level <= 1)
  assert_that(length(n) == 1, n >= 0)
  assert_that(is.null(domain) || length(domain) == 2)

  UseMethod("pred_grid")
}

#' @export
pred_grid.loess <- function(model, data, domain = NULL, n = 80, se = FALSE,
                            level = 0.95) {
  if (length(model$xnames) > 1) {
    stop("Only know how to make grid for one variable", call. = FALSE)
  }

  x_rng <- domain %||% range(model$x, na.rm = TRUE)
  x_grid <- seq(x_rng[1], x_rng[2], length = n)
  grid <- stats::setNames(data.frame(x_grid), model$xnames)
  resp <- stats::predict(model, newdata = grid, se = se)

  if (!se) {
    data.frame(
      pred_ = x_grid,
      resp_ = as.vector(resp)
    )
  } else {
    ci <- resp$se.fit * stats::qt(level / 2 + .5, resp$df)
    data.frame(
      pred_ = x_grid,
      resp_ = resp$fit,
      resp_lwr_ = resp$fit - ci,
      resp_upr_ = resp$fit + ci,
      resp_se_ = resp$se.fit
    )
  }
}

#' @export
pred_grid.lm <- function(model, data, domain = NULL, n = 80, se = FALSE,
                         level = 0.95) {
  x_var <- get_predict_vars(stats::terms(model))
  if (length(x_var) > 1) {
    stop("Only know how to make grid for one variable", call. = FALSE)
  }

  x_rng <- domain %||% range(data[[x_var]], na.rm = TRUE)
  x_grid <- seq(x_rng[1], x_rng[2], length = n)
  grid <- stats::setNames(data.frame(x_grid), x_var)

  resp <- stats::predict(model, newdata = grid, se = se,
    level = level, interval = if(se) "confidence" else "none")

  if (!se) {
    data.frame(
      pred_ = x_grid,
      resp_ = as.vector(resp)
    )
  } else {
    data.frame(
      pred_ = x_grid,
      resp_ = resp$fit[, 1],
      resp_lwr_ = resp$fit[, 2],
      resp_upr_ = resp$fit[, 3],
      resp_se_ = resp$se.fit
    )
  }
}


# Given a formula object, return a character vector of predictor variables
get_predict_vars <- function(f) {
  if (!is.formula(f))
    stop("f must be a formula object")
  if (length(f) != 3)
    stop("Formula must have components on both sides of `~`")

  all.vars(f[[3]])
}

context("compute_model_prediction")

test_that("compute_model_prediction preserves datetimes", {
  # Data frame with POSIXct, and zigzag values
  dat <- data.frame(
    d = as.POSIXct('2001-06-11 21:00', tz = 'UTC') + seq(1, 1000, by = 10),
    value = 1:100 + rep(c(-3, 3), 50)
  )
  res <- dat %>% compute_model_prediction(value ~ d, n = 10, model = "lm")
  expect_equal(range(dat$d), range(res$pred_))
})

test_that("compute_model_prediction preserves dates", {
  # Data frame with Date, and zigzag values
  dat <- data.frame(
    d = as.Date('2001-06-11') + seq(1, 1000, by = 10),
    value = 1:100 + rep(c(-3, 3), 50)
  )
  res <- dat %>% compute_model_prediction(value ~ d, n = 10, model = "lm")
  expect_equal(range(dat$d), range(res$pred_))
})

test_that("compute_model_prediction works with datetimes", {
  # Perfectly linear data
  dat <- data.frame(
    d = as.POSIXct('2001-06-11 21:00', tz = 'America/New_York') + 1:10 * 100,
    value = 1:10
  )

  # Tests with various models
  res <- dat %>% compute_model_prediction(value ~ d, n = 10, model = "loess")
  expect_equal(range(dat$d), range(res$pred_))
  expect_equal(attr(dat$d, "tzone"), attr(res$pred_, "tzone"))
  expect_equal(range(dat$value), range(res$resp_))

  res <- dat %>% compute_model_prediction(value ~ d, n = 10, model = "lm")
  expect_equal(range(dat$d), range(res$pred_))
  expect_equal(attr(dat$d, "tzone"), attr(res$pred_, "tzone"))
  expect_equal(range(dat$value), range(res$resp_))

  res <- dat %>% compute_model_prediction(value ~ d, n = 10, model = "glm")
  expect_equal(range(dat$d), range(res$pred_))
  expect_equal(attr(dat$d, "tzone"), attr(res$pred_, "tzone"))
  expect_equal(range(dat$value), range(res$resp_))
})

test_that("compute_model_prediction works with more complex formulas", {
  dat <- data.frame(x = 1:10, y = (1:10 - 5)^2 + 4 * 1:10 + 100)
  res <- dat %>%
          compute_model_prediction(y ~ I(x^2) + x, n = 10, model = "lm") %>%
          setNames( c("x", "y"))
  expect_equal(dat, res)

  dat <- data.frame(x = 1:10, y = 2.5*(1:10)^3 + 7*(1:10)^2 + 4*(1:10) + 100)
  res <- dat %>%
          compute_model_prediction(y ~ poly(x, 3), n = 10, model = "lm") %>%
          setNames( c("x", "y"))
  expect_equal(dat, res)
})


test_that("Can control domain", {
  dat <- data.frame(x = 1:10, y = 5 * (1:10))
  res <- dat %>%
          compute_model_prediction(y ~ x, n = 10, model = "lm", domain = c(11, 20)) %>%
          setNames( c("x", "y"))
  expect_equal(res, data.frame(x = 11:20, y = 5 * (11:20)))
})


test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_model_prediction(wt ~ mpg, model = "lm")
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("pred_", "resp_")))

  res <- mtcars[0,] %>% compute_model_prediction(wt ~ mpg, model = "lm", se = TRUE)
  expect_equal(nrow(res), 0)
  expect_true(setequal(
    names(res),
    c("pred_", "resp_", "pred_lwr_", "pred_upr_", "pred_se_" )
  ))

  # Smooth
  res <- mtcars[0,] %>% compute_smooth(wt ~ mpg)
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("pred_", "resp_")))

  # Grouped
  res <- mtcars[0,] %>% group_by(cyl) %>%
    compute_model_prediction(wt ~ mpg, model = "lm", se = FALSE)
  expect_true(setequal(names(res), c("cyl", "pred_", "resp_")))

  res <- mtcars[0,] %>% group_by(cyl) %>%
    compute_model_prediction(wt ~ mpg, model = "lm", se = TRUE)
  expect_true(setequal(
    names(res),
    c("cyl", "pred_", "resp_", "pred_lwr_", "pred_upr_", "pred_se_" )
  ))
})

#' Explain details of an object
#'
#' This is a generic function which gives more details about an object than
#' print, and is more focussed on human readable output than str.
#'
#' @export
#' @seealso \code{dplyr::\link[dplyr]{explain}} for more information.
#' @importFrom dplyr explain
#' @name explain
#' @examples
#' p <- mtcars %>% ggvis(x = ~cyl) %>% layer_bars()
#' explain(p)
NULL

#' Print out the structure of a ggvis object in a friendly format
#'
#' @param x Visualisation to explain
#' @param ... Needed for compatibility with generic. Ignored by this method.
#' @method explain ggvis
#' @export
explain.ggvis <- function (x, ...) {
  cat("Marks:\n")
  for (mark in x$marks) {
    cat(indent(format(mark), 2))
  }
  cat("Data objects:\n")
  for (dat in x$data) {
    cat(indent(data_id(dat), 2), "\n")
  }
  cat("Reactives:\n")
  for (reactive in x$reactives) {
    cat(indent(reactive_id(reactive), 2))
    if (is.broker(reactive)) {
      cat(" <Broker>\n")
    } else {
      cat("\n")
    }
  }
  cat("Scales:\n")
  for (scale_name in names(x$scales)) {
    cat(indent(paste0(scale_name, ":\n"), 2))
    for (scale in x$scales[[scale_name]]) {
      cat(indent(format(scale), 4))
      cat("\n")
    }
  }
  cat("Axes:\n")
  for (axis in x$axes) {
    cat(indent(format(axis), 2))
    cat("\n")
  }
  cat("Legends:\n")
  for (legend in x$legends) {
    cat(indent(format(legend), 2))
    cat("\n")
  }
  cat("HTML controls:\n")
  for (control_name in names(x$controls)) {
    cat(indent(control_name, 2))
    cat("\n")
  }
  cat("Client-side handlers:\n")
  for (handler in x$handlers) {
    cat(indent(paste0("<", handler$type, "> ", handler$id), 2))
    cat("\n")
  }
  cat("Connector functions:\n")
  for (connector in x$connectors) {
    cat(indent(connector_label(connector), 2))
    cat("\n")
  }
  cat("Options:\n")
  if (length(x$options) > 0) {
    params <- param_string(x$options, collapse = FALSE)
    cat(paste0("  ", format(paste0(names(params), ":")), " ", format(params),
      collapse = "\n"))
    cat("\n")
  }
}

#' Create a new interactive "input" object.
#'
#' An interactive input object is a reactive expression which wraps a reactive
#' value. When the plot is rendered, an observer is created which pushes values
#' into the reactive value in response to changes of an input object. Those
#' changes invalidate the reactive expression, which will return the value,
#' optionally passed through a mapping function.
#'
#' This function is designed to be used by authors of new types of interactive
#' inputs. If you are a ggvis user, please use one of the more specific input
#' functions starting with the \code{input_}.
#'
#' @param id The name of the input object in the Shiny app, such as
#'   "slider_1338869".
#' @param default The default (starting) value for the input.
#' @param map A mapping function. Defaults to \code{identity}, which simply
#'   returns the value unchanged.
#' @param controls A Shiny HTML tag object representing the UI for the controls.
#' @export
#' @keywords internal
create_input <- function(id = rand_id("input_"), default = NULL,
                         map = identity, controls = NULL) {

  # Create a reactivevalues object to store the value. When a plot is rendered,
  # an observer will be set up to push values into val$x.
  vals <- shiny::reactiveValues()
  vals$x <- default

  # A reactive to wrap the reactive value
  res <- reactive({
    map(vals$x)
  })

  # This function is run at render time. It takes values from session$input$foo
  # and pushes them into val$foo.
  connect <- function(session, plot_id) {
    shiny::observe({
      value <- session$input[[id]]
      if (is.null(value)) {
        # Need to explicitly set it to default when input is NULL, because some
        # inputs give NULL when they're cleared. (#272)
        vals$x <- default
      } else {
        vals$x <- value
      }
    })
  }
  connector_label(connect) <- paste0("<", id, ">")

  # Wrap the shiny tag object into a list
  controls_l <- list()
  controls_l[[id]] <- controls

  create_broker(res, connect = connect, controls = controls_l)
}

#' Automatically group data by grouping variables
#'
#' Use \code{auto_group} to group up a dataset on all categorical variables
#' specified by props, and have each piece rendered by the same mark.
#'
#' @export
#' @param vis The ggvis visualisation to modify.
#' @param exclude A vector containing names of props to exclude from auto grouping.
#'   It is often useful to exclude \code{c("x", "y")}, when one of those variables
#'   is categorical.
#' @seealso To manually specify grouping variables, see \code{\link{group_by}}.
#' @examples
#' # One line
#' mtcars %>% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) %>% layer_paths()
#' # One line for each level of cyl
#' mtcars %>% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) %>% group_by(cyl) %>%
#'   layer_paths()
#' mtcars %>% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) %>% auto_group() %>%
#'   layer_paths()
#'
#' # The grouping column can already be stored as a factor
#' mtcars2 <- mtcars
#' mtcars2$cyl <- factor(mtcars2$cyl)
#' mtcars2 %>% ggvis(~disp, ~mpg, stroke = ~cyl) %>% auto_group() %>%
#'   layer_paths()
auto_group <- function(vis, exclude = NULL) {

  # Figure out grouping variable
  data <- cur_data(vis)
  props <- cur_props(vis)

  # Drop props named in exclude
  pnames <- trim_prop_event(names(props))
  props <- props[!(pnames %in% exclude)]

  countable <- vapply(props,
    function(prop) is.prop_variable(prop) && prop_countable(data, prop),
    logical(1)
  )
  if (!any(countable)) return(vis)

  group_vars <- lapply(unname(props[countable]), "[[", "value")
  dplyr::group_by(vis, !!!group_vars)
}

#' Add a vega legend specification to a ggvis plot
#'
#' Axis specifications allow you to either override the default legends,
#' or supply additional legends.
#'
#' More information about axes can be found in the "axes and legends" vignettes.
#'
#' @section Compared to ggplot2:
#'
#' In ggplot2, legend (and axis) properties are part of the scales
#' specification. In vega, they are separate, which allows the specification
#' of multiple legends, and more flexible linkage between scales and legends.
#'
#' @param vis A ggvis object.
#' @param scales The name of one or more scales for which to add a legend.
#'   Typically one of "size", "shape", "fill", "stroke", although custom scale
#'   names may also be used. Multiple names can also be used, like
#'   \code{c("fill", "shape")}.
#' @param orient The orientation of the legend. One of "left" or "right". This
#'   determines how the legend is positioned within the scene. The default is
#'   "right".
#' @param title A title for the legend. By default, it uses the name the fields
#'   used in the legend. Use \code{""} to suppress the title.
#' @param format The formatting pattern for axis labels. Vega uses D3's format
#'   pattern.
#' @param values  Explicitly set the visible legend values.
#' @param properties Optional mark property definitions for custom legend
#'   styling. Should be an object created by \code{\link{legend_props}}, with
#'   properties for title, label, symbols, gradient, legend.
#' @export
#' @examples
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   add_legend("fill", title = "Cylinders")
#'
#' # Suppress legend with hide_legend
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   hide_legend("fill")
#'
#' # Combining two properties in one legend
#' mtcars %>%
#'   ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl), shape = ~factor(cyl)) %>%
#'   layer_points() %>%
#'   add_legend(c("fill", "shape"))
#'
#' # Control legend properties with a continuous legend, with x and y position
#' # in pixels.
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   add_legend("fill", title = "Cylinders",
#'     properties = legend_props(
#'       title = list(fontSize = 16),
#'       labels = list(fontSize = 12, fill = "#00F"),
#'       gradient = list(stroke = "red", strokeWidth = 2),
#'       legend = list(x = 500, y = 50)
#'     )
#'   )
#'
#' # Control legend properties with a categorical legend, with x and y position
#' # in the scaled data space.
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl)) %>%
#'   layer_points() %>%
#'   add_legend("fill", title = "Cylinders",
#'     properties = legend_props(
#'       title = list(fontSize = 16),
#'       labels = list(fontSize = 14, dx = 5),
#'       symbol = list(stroke = "black", strokeWidth = 2,
#'         shape = "square", size = 200),
#'       legend = list(
#'         x = scaled_value("x", 4.5),
#'         y = scaled_value("y", 30)
#'       )
#'     )
#'   )
#'
#' # Control legend position using x_rel and y_rel which specify relative
#' # position, going from 0 to 1. (0, 0) is the bottom-left corner, and
#' # (1, 1) is the upper-right corner. The values control the position of
#' # the upper-left corner of the legend.
#' mtcars %>% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) %>%
#'   layer_points() %>%
#'   add_relative_scales() %>%
#'   add_legend("fill", title = "Cylinders",
#'     properties = legend_props(
#'       legend = list(
#'         x = scaled_value("x_rel", 0.8),
#'         y = scaled_value("y_rel", 1)
#'       )
#'     )
#'   )
add_legend <- function(vis, scales = NULL, orient = "right", title = NULL,
                       format = NULL, values = NULL, properties = NULL) {
  assert_that(!is.null(scale))

  # Create an unfortified legend
  legend <- structure(compact(list(
    scales = scales, orient = orient, title = title, format = format,
    values = values, properties = properties
  )), class = "ggvis_legend")

  append_ggvis(vis, "legends", legend)
}

#' @rdname add_legend
#' @export
hide_legend <- function(vis, scales) {
  legend <- structure(list(scales = scales, hide = TRUE), class = "ggvis_legend")
  append_ggvis(vis, "legends", legend)
}

#' Defunct function for adding a legend
#'
#' This function has been replaced with \code{\link{add_legend}}.
#' @param ... Other arguments.
#' @export
add_guide_legend <- function(...) {
  stop("add_guide_legend() has been replaced by add_legend().")
}

# Given a ggvis object, find all the unfortified legend and fortify them.
# The fortification process requires examining the scales in the ggvis object.
fortify_legends <- function(vis) {
  all_scales <- collapse_scales(gather_scales(vis))
  # Get a named vector where names are scales, values are properties
  scales_props <- vapply(all_scales, function(s) s$property, character(1))

  vis$legends <- lapply(vis$legends, fortify_legend, scales_props)
  vis
}

# Create a fortified legend object. An unfortified legend has only the names
# of the scales. A fortified legends maps those names of scales to property
# names.
# scales_props is a named list, where names are scales, values are properties
fortify_legend <- function(legend, scales_props) {
  if (inherits(legend, "fortified_legend")) return(legend)

  # Get scales and props that are actually used by this legend
  scales_props <- scales_props[legend$scales]

  assert_that(!is.null(scales_props) && length(scales_props) > 0)
  assert_that(is.null(legend$properties) || is.legend_props(legend$properties))

  legend <- structure(compact(list(
      orient = legend$orient, title = legend$title, format = legend$format,
      values = legend$values, properties = legend$properties,
      hide = legend$hide
  )), class = c("fortified_legend", "ggvis_legend"))

  legend[scales_props] <- names(scales_props)
  legend
}

add_missing_legends <- function(vis) {
  legends <- vis$legends
  scales <- vis$scales

  legs <- c("size", "shape", "fill", "stroke")
  # Get scales that are in some legend
  present <- unlist(lapply(legends, function(x) x$scales))
  # Find scales that don't have legend
  missing <- setdiff(intersect(names(scales), legs), present)

  for (scale in missing) {
    vis <- add_legend(vis, scale)
  }

  vis
}

# Some legend settings require examining the scale
apply_legends_defaults <- function(vis) {
  legends <- vis$legends
  scales <- vis$scales

  legs <- c("size", "shape", "fill", "stroke")

  legends <- lapply(legends, function(legend) {
    if (isTRUE(legend$hide)) return(legend)

    present <- unlist(legend[legs])
    present_scales <- scales[present]

    # Use [[-indexing to avoid partial name matching of "titleOffset". (#269)
    if (is.null(legend[["title"]])) {
      legend$title <- present_scales[[1]]$label
    }

    legend
  })

  # Replace the original legends with the new ones
  vis$legends <- legends
  vis
}

#' @export
format.ggvis_legend <- format.ggvis_axis

#' @export
print.ggvis_legend <- print.ggvis_axis

ggvis_path <- function(x) {
  system.file(package = "ggvis", "www", x)
}

# HTML dependencies of a ggvis plot
ggvis_dependencies <- function() {

  minified <- getOption("ggvis.js_minified", TRUE)
  adjust_min <- function(x) {
    if (minified) return(x)
    gsub("\\.min", "", x)
  }

  deps <- list(
    htmltools::htmlDependency(
      name = "jquery",
      version = "1.11.0",
      src = ggvis_path("lib/jquery"),
      script = "jquery.min.js"
    ),
    htmltools::htmlDependency(
      name = "detect-resize",
      version = "0.5.3",
      src = ggvis_path("lib/detect-resize"),
      script = "jquery.resize.js"
    ),
    htmltools::htmlDependency(
      name = "jquery-ui",
      version = "1.11.4",
      src = ggvis_path("lib/jquery-ui"),
      script = adjust_min("jquery-ui.min.js"),
      stylesheet = adjust_min("jquery-ui.min.css")
    ),
    htmltools::htmlDependency(
      name = "d3",
      version = "3.5.2",
      src = ggvis_path("lib/d3"),
      script = adjust_min("d3.min.js")
    ),
    htmltools::htmlDependency(
      name = "vega",
      version = "1.4.3",
      src = ggvis_path("lib/vega"),
      script = adjust_min("vega.min.js")
    ),
    htmltools::htmlDependency(
      name = "lodash",
      version = "4.17.21",
      src = ggvis_path("lib/lodash"),
      script = adjust_min("lodash.min.js"),
      head = "<script>var lodash = _.noConflict();</script>"
    ),
    htmltools::htmlDependency(
      name = "ggvis",
      version = as.character(utils::packageVersion("ggvis")),
      src = ggvis_path("ggvis"),
      script = "js/ggvis.js",
      stylesheet = "css/ggvis.css"
    )
  )

  deps
}

shiny_dependency <- function() {
  htmltools::htmlDependency(
    name = "shiny-ggvis",
    version = as.character(utils::packageVersion("ggvis")),
    src = ggvis_path("ggvis"),
    script = "js/shiny-ggvis.js"
  )
}

ggvis_app <- function(x, plot_id = rand_id("plot_"),
                      ...) {

  ui <- ggvisLayout(plot_id, length(x$controls) > 0, spec = NULL, shiny = TRUE)

  server <- function(input, output, session) {
    r_gv <- reactive(x)
    bind_shiny(r_gv, session = session, plot_id = plot_id,
      controls_id = "ggvis_controls")
  }

  options <- compact(list(...))

  shiny::shinyApp(ui = ui, server = server, options = options)
}

#' Transformation: density estimate
#'
#' \code{transform_density} is a data transformation that computes a kernel
#' density estimate from a dataset. \code{layer_density} combines
#' \code{transform_density} with \code{mark_path} and \code{mark_area}
#' to display a smooth line and its standard errror.
#'
#' @param vis The visualisation to modify
#' @param ... Visual properties, passed on to \code{\link{props}}.
#' @param adjust Multiple the default bandwidth by this amount. Useful for
#'   controlling wiggliness of density.
#' @inheritParams compute_density
#' @param density_args Other arguments passed on to
#'   \code{\link{compute_density}} and thence to \code{\link{density}}.
#' @param area Should there be a shaded region drawn under the curve?
#' @export
#' @examples
#' # Basic density estimate
#' faithful %>% ggvis(~waiting) %>% layer_densities()
#' faithful %>% ggvis(~waiting) %>% layer_densities(area = FALSE)
#'
#' # Control bandwidth with adjust
#' faithful %>% ggvis(~waiting) %>% layer_densities(adjust = .25)
#' faithful %>% ggvis(~waiting) %>%
#'   layer_densities(adjust = input_slider(0.1, 5))
#'
#' # Control stroke and fill
#' faithful %>% ggvis(~waiting) %>%
#'   layer_densities(stroke := "red", fill := "red")
#'
#' # With groups
#' PlantGrowth %>% ggvis(~weight, fill = ~group) %>% group_by(group) %>%
#'   layer_densities()
#' PlantGrowth %>% ggvis(~weight, stroke = ~group) %>% group_by(group) %>%
#'   layer_densities(strokeWidth := 3, area = FALSE)
layer_densities <- function(vis, ..., kernel = "gaussian", adjust = 1,
                            density_args = list(), area = TRUE) {

  x_var <- find_prop_var(cur_props(vis), "x.update")

  vis <- set_scale_label(vis, "x", prop_label(cur_props(vis)$x.update))
  vis <- set_scale_label(vis, "y", "density")

  props <- stroke_fill_defaults(props(...),
    stroke = props(~pred_, ~resp_),
    fill   = props(~pred_, ~resp_, y2 = 0, fillOpacity := 0.2)
  )

  pipeline <- function(x) {
    x <- do_call(compute_density, quote(x), x_var = x_var, kernel = kernel,
      adjust = adjust, .args = density_args)

    if (identical(area, TRUE)) {
      x <- emit_ribbons(x, props$fill)
    }
    x <- emit_paths(x, props$stroke)
    x
  }
  layer_f(vis, pipeline)
}

# Given a character vector like c("x", "x.update", "x.enter"), report which ones
# have .update, .enter, etc.
has_prop_event <- function(x) {
  sub("^.*\\.", "", x) %in% c("enter", "exit", "update", "hover", "brush")
}

# Remove the trailing event from a prop name
trim_prop_event <- function(x) {
  sub("\\.(enter|exit|update|hover|brush)$", "", x)
}

# Given a list with objects x.enter, x.update, y.update, return a list p of
# ggvis_props objects, with structure p$enter$x, p$update$x, p$update$y.
prop_event_sets <- function(props) {
  sets <- c("enter", "exit", "update", "hover", "brush")
  names(sets) <- sets

  x <- lapply(sets, function(set) {
    searchstr <- paste0("\\.", set, "$")
    matches <- props[grep(searchstr, names(props))]
    names(matches) <- trim_prop_event(names(matches))
    matches
  })

  compact(x)
}

# Given a props object, trim the .update, .enter, etc, and drop all those which
# are named in `drop`.
# drop_props(
#   props(x  = ~wt, x.enter = 0, stroke.enter := "black", stroke.hover := "red"),
#   c("stroke", "strokeOpacity")
# )
drop_props <- function(props, drop) {
  pnames <- trim_prop_event(names(props))
  props[!(pnames %in% drop)]
}

stroke_fill_defaults <- function(props, stroke = list(), fill = list()) {
  stroke_props <- merge_props(stroke, props)
  stroke_props <- drop_props(stroke_props, c("fill", "fillOpacity"))
  stroke_props <- merge_props(stroke_props, props(fill := "transparent"))

  fill_props <- merge_props(fill, props)
  fill_props <- drop_props(fill_props, c("stroke", "strokeOpacity"))
  fill_props <- merge_props(fill_props, props(stroke := "transparent"))

  list(stroke = stroke_props, fill = fill_props)
}

# @param props A props object.
# @param properties A character vector of properties to check.
# @param events A character vector of events that are not supported
# @param label A string naming the type of layer (like layer_bars), for error
#   messages.
check_unsupported_props <- function(props, properties, events, label = "") {
  unsupported <- vapply(props, function(p) {
    (p$property %in% properties) && (p$event %in% events)
  }, logical(1))

  if (any(unsupported)) {
    if (label == "") label <- "layer"
    stop(label, " presently cannot use properties ",
         paste(properties, collapse = ", "), " for ",
         paste(events, collapse = ", "), " events.")
  }
  invisible(NULL)
}

#' Waggle back and forth between two numbers
#'
#' @param min A minimum value.
#' @param max A maximum value.
#' @param value Starting value. Defaults to half-way between \code{min} and
#'   \code{max}.
#' @param step How much value changes at each frame. Defaults to 50 steps
#'   between min and max so it takes 5 seconds to waggle once.
#' @param fps number of frames per second.
#' @export
#' @examples
#' span <- waggle(0.2, 1)
#' mtcars %>% ggvis(~mpg, ~wt) %>%
#'  layer_points() %>%
#'  layer_smooths(span = span)
waggle <- function(min, max, value = (min + max) / 2, step = (max - min) / 50,
                   fps = 10) {

  vals <- shiny::reactiveValues()
  vals$x <- value

  connect <- function(session, plot_id) {
    direction <- 1
    shiny::observe({
      shiny::invalidateLater(1000 / fps, NULL)

      next_value <- shiny::isolate(vals$x) + direction * step
      if (next_value < min || next_value > max) {
        direction <<- -1 * direction
        next_value <- pmax(pmin(next_value, max), min)
      }

      vals$x <<- next_value
    })
  }

  create_broker(reactive(vals$x), connect = connect)
}

#' Display binned data
#'
#' @param vis Visualisation to modify
#' @param ... Visual properties used to override defaults.
#' @param stack If \code{TRUE}, will automatically stack overlapping bars.
#' @inheritParams compute_bin
#' @seealso \code{\link{layer_bars}} For bar graphs of counts at each unique
#'   x value, in contrast to a histogram's bins along x ranges.
#' @export
#' @examples
#' # Create histograms and frequency polygons with layers
#' mtcars %>% ggvis(~mpg) %>% layer_histograms()
#' mtcars %>% ggvis(~mpg) %>% layer_histograms(width = 2)
#' mtcars %>% ggvis(~mpg) %>% layer_freqpolys(width = 2)
#'
#' # These are equivalent to combining compute_bin with the corresponding
#' # mark
#' mtcars %>% compute_bin(~mpg) %>% ggvis(~x_, ~count_) %>% layer_paths()
#'
#' # With grouping
#' mtcars %>% ggvis(~mpg, fill = ~factor(cyl)) %>% group_by(cyl) %>%
#'   layer_histograms(width = 2)
#' mtcars %>% ggvis(~mpg, stroke = ~factor(cyl)) %>% group_by(cyl) %>%
#'   layer_freqpolys(width = 2)
layer_histograms <- function(vis, ..., width = NULL, center = NULL,
                             boundary = NULL, closed = c("right", "left"),
                             stack = TRUE, binwidth) {
  if (!missing(binwidth)) {
    width <- binwidth
    deprecated("binwidth", "width", version = "0.3.0")
  }

  closed <- match.arg(closed)
  new_props <- merge_props(cur_props(vis), props(...))

  check_unsupported_props(new_props, c("x", "y", "x2", "y2"),
                          c("enter", "exit", "hover"), "layer_histograms")

  x_var <- find_prop_var(new_props, "x.update")
  x_val <- eval_vector(cur_data(vis), x_var)

  vis <- set_scale_label(vis, "x", prop_label(new_props$x.update))
  vis <- scale_numeric(vis, "y", domain = c(0, NA), expand = c(0, 0.05),
                       label = "count")

  layer_f(vis, function(x) {
    x <- compute_bin(x, x_var, width = width, center = center,
      boundary = boundary, closed  = closed)

    if (stack) {
      x <- compute_stack(x, stack_var = ~count_, group_var = ~x_)

      rect_props <- merge_props(
        new_props,
        props(x = ~xmin_, x2 = ~xmax_, y = ~stack_upr_, y2 = ~stack_lwr_)
      )
      x <- emit_rects(x, rect_props)

    } else {
      rect_props <- merge_props(
        new_props,
        props(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0)
      )
      x <- emit_rects(x, rect_props)
    }
    x
  })
}

#' @rdname layer_histograms
#' @export
layer_freqpolys <- function(vis, ..., width = NULL, center = NULL, boundary = NULL,
                            closed = c("right", "left"), binwidth) {
  if (!missing(binwidth)) {
    width <- binwidth
    deprecated("binwidth", "width", version = "0.3.0")
  }

  closed <- match.arg(closed)

  new_props <- merge_props(cur_props(vis), props(...))

  check_unsupported_props(new_props, c("x", "y"),
                          c("enter", "exit", "hover"), "layer_freqpolys")

  x_var <- find_prop_var(new_props, "x.update")
  x_val <- eval_vector(cur_data(vis), x_var)

  vis <- set_scale_label(vis, "x", prop_label(new_props$x.update))
  vis <- set_scale_label(vis, "y", "count")

  params <- bin_params(range(x_val, na.rm = TRUE), width = value(width),
                       center = value(center), boundary = value(boundary),
                       closed = value(closed))

  layer_f(vis, function(x) {
    x <- compute_bin(x, x_var, width = params$width,
      boundary = params$origin, closed = params$closed, pad = TRUE)

    path_props <- merge_props(new_props, props(x = ~x_, y = ~count_))
    x <- emit_paths(x, path_props)
    x
  })
}

library(ggvis)

set.seed(1233)
cocaine <- cocaine[sample(1:nrow(cocaine), 500), ]
cocaine$id <- seq_len(nrow(cocaine))

shinyServer(function(input, output, session) {

  lb <- linked_brush(keys = cocaine$id, "red")

  cocaine %>%
    ggvis(~weight, ~price, key := ~id) %>%
    layer_points(fill := lb$fill, fill.brush := "red", opacity := 0.3) %>%
    lb$input() %>%
    set_options(width = 300, height = 300) %>%
    bind_shiny("plot1") # Very important!


  # A subset of cocaine, of only the selected points
  selected <- lb$selected
  cocaine_selected <- reactive({
    cocaine[selected(), ]
  })

  cocaine %>%
    ggvis(~potency) %>%
    layer_histograms(width = 5, boundary = 0) %>%
    add_data(cocaine_selected) %>%
    layer_histograms(width = 5, boundary = 0, fill := "#dd3333") %>%
    set_options(width = 300, height = 300) %>%
    bind_shiny("plot2")
})

#' Count data at each location
#'
#' @param x Dataset-like object to count. Built-in methods for data frames,
#'   grouped data frames and ggvis visualisations.
#' @param x_var,w_var Names of x and weight variables.
#' @seealso \code{\link{compute_bin}} For counting cases within ranges of
#'   a continuous variable.
#' @seealso \code{\link{compute_align}} For calculating the "width" of data.
#' @export
#' @return A data frame with columns:
#'  \item{count_}{the number of points}
#'  \item{x_}{the x value where the count was made}
#'
#' The width of each "bin" is set to the resolution of the data -- that is, the
#' smallest difference between two x values.
#'
#' @examples
#' mtcars %>% compute_count(~cyl)
#'
#' # Weight the counts by car weight value
#' mtcars %>% compute_count(~cyl, ~wt)
#'
#' # If there's one weight value at each x, it effectively just renames columns.
#' pressure %>% compute_count(~temperature, ~pressure)
#' # Also get the width of each bin
#' pressure %>% compute_count(~temperature, ~pressure) %>% compute_align(~x_)
#'
#' # It doesn't matter whether you transform inside or outside of a vis
#' mtcars %>% compute_count(~cyl, ~wt) %>%
#'   compute_align(~x_) %>%
#'   ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) %>%
#'   layer_rects()
#'
#' mtcars %>%
#'   ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) %>%
#'   compute_count(~cyl, ~wt) %>%
#'   compute_align(~x_) %>%
#'   layer_rects()
compute_count <- function(x, x_var, w_var = NULL) {
  UseMethod("compute_count")
}

#' @export
compute_count.data.frame <- function(x, x_var, w_var = NULL) {
  assert_that(is.formula(x_var))

  x_val <- eval_vector(x, x_var)

  if (is.null(w_var)) {
    w_val <- NULL
  } else {
    w_val <- eval_vector(x, w_var)
  }

  count_vector(x_val, weight = w_val)
}

#' @export
compute_count.grouped_df <- function(x, x_var, w_var = NULL) {
  dplyr::do(x, compute_count(., x_var, w_var = w_var))
}

#' @export
compute_count.ggvis <- function(x, x_var, w_var = NULL) {
  args <- list(x_var = x_var, w_var = w_var)

  register_computation(x, args, "count", function(data, args) {
    output <- do_call(compute_count, quote(data), .args = args)
    preserve_constants(data, output)
  })
}

# Count individual vector ------------------------------------------------------

count_vector <- function(x, weight = NULL, ...) {
  if (is.null(weight)) {
    weight <- rep.int(1, length(x))
  }

  # Preserve date and time types
  if (inherits(x, "POSIXct")) {
    tz <- attr(x, "tzone", TRUE)
    restore <- function(x) as.POSIXct(x, origin = "1970-01-01", tz = tz)
  } else if (inherits(x, "Date")) {
    restore <- function(x) structure(x, class = "Date")
  } else {
    restore <- identity
  }

  counts <- unname(as.vector(tapply(weight, x, sum, na.rm = TRUE)))

  if (is.factor(x)) {
    na_idx <- is.na(counts)
    # Factor levels that aren't represented in x result in NA, which we'll
    # drop.
    counts <- counts[!na_idx]
    # Get the factor levels, preserving factor-ness and order of factor levels.
    # Order should align with counts.
    values <- factor(levels(x), levels = levels(x))[!na_idx]
  } else {
    # Need to get unique values this way instead of using names(counts),
    # because names are strings but the x values aren't always strings.
    values <- unname(as.vector(tapply(x, x, unique, na.rm = TRUE)))
  }

  data.frame(
    count_ = counts,
    x_ = restore(values),
    stringsAsFactors = FALSE
  )
}

context("compute_count")

test_that("count_vector correctly handles factors", {
  fac <- factor(c("A", "A", "C"), levels = c("A", "C"))
  expect_identical(
    count_vector(fac),
    data.frame(
      count_ = c(2, 1),
      x_ = factor(c("A", "C"), levels = c("A", "C"))
    )
  )

  # Levels that aren't represented are dropped from the count
  fac <- factor(c("A", "A", "C"), levels = c("A", "B", "C"))
  expect_identical(
    count_vector(fac),
    data.frame(
      count_ = c(2, 1),
      x_ = factor(c("A", "C"), levels = c("A", "B", "C"))
    )
  )

  # Factor with levels in non-lexical order
  fac <- factor(c("A", "A", "C"), levels = c("C", "A"))
  expect_identical(
    count_vector(fac),
    data.frame(
      count_ = c(1, 2),
      x_ = factor(c("C", "A"), levels = c("C", "A"))
    )
  )
})

test_that("count_vector preserves dates and times", {
  dates <- as.Date("2013-07-01") + 1:100
  res <- count_vector(dates)
  # May not be identical because res$x_ gets coerced to int
  expect_equal(dates, res$x_)

  times <- as.POSIXct('2001-06-11 21:00', tz = 'America/New_York') + 1:10 * 100
  res <- count_vector(times)
  expect_identical(times, res$x_)

  times <- as.POSIXct('2001-06-11 21:00', tz = 'UTC') + seq(1, 1000, by = 10)
  res <- count_vector(times)
  expect_identical(times, res$x_)
})


test_that("Zero-row inputs", {
  res <- mtcars[0,] %>% compute_count(~cyl)
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("count_", "x_")))

  # Grouped
  res <- mtcars %>% group_by(cyl) %>% dplyr::filter(FALSE) %>% compute_count(~cyl)
  expect_equal(nrow(res), 0)
  expect_true(setequal(names(res), c("cyl", "count_", "x_")))
})

#!/usr/bin/env Rscript

# This script copies the resize detection script from:
#   https://github.com/sdecima/javascript-detect-element-resize
# to the inst/ directory of this package.

# This script can be sourced from RStudio, or run with Rscript.

# Returns the file currently being sourced or run with Rscript
thisFile <- function() {
  cmdArgs <- commandArgs(trailingOnly = FALSE)
  needle <- "--file="
  match <- grep(needle, cmdArgs)
  if (length(match) > 0) {
    # Rscript
    return(normalizePath(sub(needle, "", cmdArgs[match])))
  } else {
    # 'source'd via R console
    return(normalizePath(sys.frames()[[1]]$ofile))
  }
}

url <- "https://raw.githubusercontent.com/sdecima/javascript-detect-element-resize/master/jquery.resize.js"
destfile <- file.path(dirname(thisFile()), "../inst/www/lib/detect-resize/",
                      basename(url))

downloader::download(url, destfile)

#' Send a message to ggvis running on client
#'
#' This will be sent to the client and passed to a handler in ggvis.messages on
#' the client side. The handler is specified by \code{type}.
#'
#' @param session A session object.
#' @param type A string representing the type of the message.
#' @param data An object (typically a list) containing information for the client.
#' @param id A unique identifier for ggvis message handler (optional).
#' @export
ggvis_message <- function(session, type, data = NULL, id = NULL) {
  if (is.null(session)) {
    stop("Need an active Shiny session to send ggvis message")
  }
  session$sendCustomMessage("ggvis_message",
    list(type = type, id = id, data = data))
}

#' Create a linked brush object.
#'
#' A linked brush has two sides: input and output
#'
#' @note \code{linked_brush} is very new and is likely to change substantially
#'   in the future
#' @param keys vector of all possible keys, if known.
#' @param fill brush colour
#' @return A list with components:
#'   \item{input}{A function that takes a visualisation as an argument and
#'      adds an input brush to that plot}
#'   \item{selected}{A reactive providing a logical vector that describes
#'     which points are under the brush}
#'   \item{fill}{A reactive that gives the fill colour of points under the
#'     brush}
#' @export
#' @examples
#' lb <- linked_brush(keys = 1:nrow(mtcars), "red")
#'
#' # Change the colour of the points
#' mtcars %>%
#'  ggvis(~disp, ~mpg) %>%
#'  layer_points(fill := lb$fill, size.brush := 400) %>%
#'  lb$input()
#'
#' # Display one layer with all points and another layer with selected points
#' library(shiny)
#' mtcars %>%
#'  ggvis(~disp, ~mpg) %>%
#'  layer_points(size.brush := 400) %>%
#'  lb$input() %>%
#'  layer_points(fill := "red", data = reactive(mtcars[lb$selected(), ]))
linked_brush <- function(keys, fill = "red") {
  stopifnot(is.character(fill), length(fill) == 1)

  rv <- shiny::reactiveValues(under_brush = character())

  input <- function(vis) {
    handle_brush(vis, fill = fill, on_move = function(items, ...) {
      rv$under_brush <- items$key__
    })
  }

  selected_r <- reactive(keys %in% rv$under_brush)
  fill_r <- reactive(c("black", fill)[selected_r() + 1])

  list(
    input = input,
    selected = create_broker(selected_r),
    fill = create_broker(fill_r)
  )
}

#' Layer lines on a plot.
#'
#' \code{layer_lines} differs from \code{layer_paths} in that \code{layer_lines}
#' sorts the data on the x variable, so the line will always proceed from left
#' to right, whereas \code{layer_paths} will draw a line in whatever order
#' appears in the data.
#'
#' @seealso \code{\link{layer_paths}}
#' @export
#' @param vis Visualisation to modify.
#' @param ... Visual properties.
#' @examples
#' mtcars %>% ggvis(~wt, ~mpg, stroke = ~factor(cyl)) %>% layer_lines()
#'
#' # Equivalent to
#' mtcars %>% ggvis(~wt, ~mpg, stroke = ~factor(cyl)) %>%
#'   group_by(cyl) %>% dplyr::arrange(wt) %>% layer_paths()
layer_lines <- function(vis, ...) {

  x_var <- vis$cur_props$x.update$value

  layer_f(vis, function(x) {
    x <- auto_group(x, exclude = c("x", "y"))
    x <- dplyr::arrange(x, !!x_var)
    emit_paths(x, props(...))
  })
}

# Given an two data objects, input and output, this will return output cbind'ed
# with the columns in input that are constant, that is where all the values in
# a column have the same value within each group (if grouped). For any columns
# names that exist in both input and output, the value from output will
# supersede the value from input.
preserve_constants <- function(input, output) UseMethod("preserve_constants")

#' @export
preserve_constants.data.frame <- function(input, output) {
  # Merge data frame 'a' into data frame 'b'. If a and b share any columns, use
  # the column from b. If the second data frame has zero rows, return a zero-row
  # data frame.
  merge_df <- function(a, b) {
    if (is.null(a) || ncol(a) == 0) return(b)
    if (is.null(b) || ncol(b) == 0) return(a)

    a_new <- a[setdiff(names(a), names(b))]
    if (nrow(b) == 0) {
      a_new <- a_new[0, , drop = FALSE]
    }
    cbind(a_new, b)
  }

  is_constant <- constant_vars(input)
  constants <- input[1, is_constant, drop = FALSE]
  rownames(constants) <- NULL

  merge_df(constants, output)
}

#' @export
preserve_constants.grouped_df <- function(input, output) {
  is_constant <- constant_vars(input)

  # Get data frame of constants with one row per group
  constants <- dplyr::do(input, `[`(., 1, is_constant, drop = FALSE))

  group_vars <- unlist(lapply(dplyr::groups(constants), as.character))
  # From input, drop any columns that also exist in output, except grouping
  # vars. This is so that can later do a join without duplicate columns.
  keep_vars <- setdiff(names(constants), setdiff(names(output), group_vars))

  constants <- constants[, keep_vars, drop = FALSE]

  dplyr::inner_join(constants, output, by = group_vars)
}


# Returns a logical vector reporting which columns are constant - that is, for
# that column, all rows have the same value. If the data is grouped, this
# reports whether all rows have the same value _within each group_.
constant_vars <- function(data) UseMethod("constant_vars")

#' @export
constant_vars.data.frame <- function(data) {
  vapply(data, all_same, logical(1), USE.NAMES = FALSE)
}

#' @export
constant_vars.grouped_df <- function(data) {
  # Get number of groups
  n <- length(dplyr::group_size(data))

  # Get a list of boolean vectors
  vecs <- dplyr::do(data, constant_var__ = constant_vars(.))
  vecs <- vecs[["constant_var__"]]

  # Don't create ridiculously long column names
  names(vecs) <- seq_len(length(vecs))
  # Convert to matrix for faster computation
  mat <- as.matrix(as.data.frame(vecs))

  rowSums(mat) == n
}

# Returns a string representing the transform type. For example, if it has
# class "transform_smooth", then this returns "smooth".
transform_type <- function(transform) {
  classes <- class(transform)
  type <- classes[grep("^transform_", classes)][1]
  sub("^transform_", "", type)
}

#' Determine the "type" of a vector
#'
#' The \code{vector_type} collapses down the class of base vectors into
#' something useful more for visualisation, yielding one of "datetime",
#' "numeric", "ordinal", "nominal" or "logical".
#'
#' @param x a vector
#' @export
#' @seealso \code{default_scale}, which uses this when picking the default
#'   scale.
vector_type <- function(x) UseMethod("vector_type")

#' @export
vector_type.POSIXt <- function(x) "datetime"
#' @export
vector_type.Date <- function(x) "datetime"
#' @export
vector_type.numeric <- function(x) "numeric"
#' @export
vector_type.integer <- function(x) "numeric"
#' @export
vector_type.character <- function(x) "nominal"
#' @export
vector_type.logical <- function(x) "logical"
#' @export
vector_type.factor <- function(x) "nominal"
#' @export
vector_type.ordered <- function(x) "ordinal"
#' @export
vector_type.NULL <- function(x) "NULL"
#' @export
vector_type.default <- function(x) {
  stop("Unknown variable type: ", paste0(class(x), collapse = "/"))
}

# Reports whether a vector maps to a countable prop type
vector_countable <- function(x) {
  countable_prop_type(vector_type(x))
}

#' Determine the vega data type for a vector
#'
#' This is used to specify the data type so that the appropriate parser is used
#' when Vega receives the data.
#' @param x A vector.
vega_data_parser <- function(x) UseMethod("vega_data_parser")

#' @export
vega_data_parser.POSIXt <- function(x) "number"
#' @export
vega_data_parser.Date <- function(x) "number"
#' @export
vega_data_parser.numeric <- function(x) "number"
#' @export
vega_data_parser.integer <- function(x) "number"
#' @export
vega_data_parser.character <- function(x) NULL
#' @export
vega_data_parser.logical <- function(x) "boolean"
#' @export
vega_data_parser.factor <- function(x) NULL
#' @export
vega_data_parser.ordered <- function(x) NULL
#' @export
vega_data_parser.NULL <- function(x) NULL
#' @export
vega_data_parser.default <- function(x) {
  stop("Unknown variable type: ", paste0(class(x), collapse = "/"))
}

library(ggvis)
data(diamonds, package = "ggplot2")

shinyServer(function(input, output, session) {

  values <- reactiveValues(selected = rep(TRUE, nrow(diamonds)))

  diamonds %>% ggvis(~carat) %>%
    layer_histograms(fill.hover := "red", width = 0.1) %>%
    handle_hover(function(data, ...) {
      values$selected <- diamonds$carat >= data$xmin_ &
        diamonds$carat < data$xmax_
    }) %>%
    set_options(width = 400, height = 200) %>%
    bind_shiny("plot1")

  # Sub-histogram
  reactive(diamonds[values$selected, , drop = FALSE]) %>%
    ggvis(~carat) %>%
    layer_histograms(width = 0.01) %>%
    set_options(width = 400, height = 200) %>%
    bind_shiny("plot2")

})

#' Interactive inputs bound to arrow keys.
#'
#' @param min A minimum value.
#' @param max A maximum value.
#' @param value The initial value before any keys are pressed. Defaults to
#'   half-way between \code{min} and \code{max}.
#' @param step How much each key press changes \code{value}. Defaults to
#'   40 steps along range
#' @export
#' @examples
#' size <- left_right(1, 801, value = 51, step = 50)
#' opacity <- up_down(0, 1, value = 0.9, step = 0.05)
#'
#' mtcars %>% ggvis(~mpg, ~wt, size := size, opacity := opacity) %>%
#'   layer_points()
left_right <- function(min, max, value = (min + max) / 2,
                       step = (max - min) / 40) {

  # Given the key_press object and current value, return the next value
  map <- function(key_press, current_value) {
    key <- key_press$value

    if (key == "left") {
      pmax(min, current_value - step)
    } else if (key == "right") {
      pmin(max, current_value + step)
    } else {
      current_value
    }
  }

  create_keyboard_event(map, value)
}

#' @export
#' @rdname left_right
up_down <- function(min, max, value = (min + max) / 2,
                    step = (max - min) / 40) {

  map <- function(key_press, current_value) {
    key <- key_press$value

    if (key == "down") {
      pmax(min, current_value - step)
    } else if (key == "up") {
      pmin(max, current_value + step)
    } else {
      current_value
    }
  }

  create_keyboard_event(map, value)
}

#' Event broker for keyboard events.
#'
#' The keyboard event broker is useful if you want your shiny app to respond
#' to keyboard events: \code{key_down}, \code{key_press} and \code{key_up}.
#' This returns a broker object, similar to \code{\link{create_input}}.
#'
#' The \code{map} function takes a \code{key_press} list object which is
#' converted from the JSON object sent from the client, and the current value
#' of this reactive. Keyboard events differ from inputs in that when a key
#' is pressed, the new value can depend on the previous value. For example,
#' pressing the right arrow might increment the value each time. The value of
#' an input contrast, in contrast, takes the value directly from the client,
#' without using the previous value. The map function for keyboard events
#' therefore take the \code{current_value} as an argument.
#'
#' The \code{map} function takes two arguments. The first is a list containing
#' an item named \code{value}, which is a string representation of the key that
#' was pressed. The second is the current value of the reactive.
#'
#' The keyboard event handling in ggvis is implemented with
#'   \href{mousetrap}{https://craig.is/killing/mice}, so you can specify keys
#'   like \code{c("C", "Shift + X", "F2", "up"))}.
#'
#' Also, unlike inputs, keyboard events add no HTML controls, but they do add
#' an object to insert into the Vega spec.
#'
#' @param map A function which takes the key_press list object (a list
#' @param default Default value before first key pressed
#' @noRd
create_keyboard_event <- function(map, default = NULL) {
  if (!is.function(map)) stop("map must be a function")

  vals <- shiny::reactiveValues()
  vals$x <- default

  # A reactive to wrap the reactive value
  res <- reactive({
    vals$x
  })

  # This function is run at render time.
  connect <- function(session, plot_id) {
    key_press_id  <- paste0(plot_id, "_key_press")

    shiny::observe({
      key_press <- session$input[[key_press_id]]

      if (!is.null(key_press)) {
        # Get the current value of the reactive, without taking a dependency
        current_value <- shiny::isolate(vals$x)

        vals$x <- map(key_press, current_value)
      }

    })
  }
  connector_label(connect) <- "key_press"

  spec <- list(type = "keyboard")
  create_broker(res, connect = connect, spec = spec)
}

#' Create a new ggvis_scale object.
#'
#' A scale object is a close mapping to a vega mark object. Vega scales
#' are documented in \url{https://vega.github.io/vega/docs/scales/}.
#'
#' This function is designed to be used by authors of new types of scale.
#' If you are a ggvis user, please use one of the more specific scale
#' functions starting with the \code{scale_}.
#'
#' This is very close, but not exactly a vega scale object. Instead of being a
#' named list with a set of values, the domain can be  a vector of values, or a
#' reactive that returns such values.
#'
#' @param property The property to which the scale applies, such as "x", "y",
#'   "fill", etc.
#' @param name Name of the scale, such as "x", "y", "fill", etc. Can also be an
#'   arbitrary name like "foo".
#' @param label Label for the scale. Used for axis or legend titles.
#' @param type Type of scale. Should be one of "linear", "ordinal", "time",
#'   "utc", "linear", "log", "pow", "sqrt", "quantile", "quantize", "threshold".
#' @param domain The domain of the scale, representing the set of data values.
#'   For ordinal scales, a character vector; for quantitative scales, a numeric
#'   vector of length two. Either value (but not both) may be NA, in which
#'   case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
#'   can also be a reactive which returns the appropriate type of vector.
#' @param range The range of the scale, representing the set of visual values.
#'   For numeric values, the range can take the form of a two-element array with
#'   minimum and maximum values. For ordinal data, the range may by an array of
#'   desired output values, which are mapped to elements in the specified
#'   domain. The following range literals are also available: "width", "height",
#'   "shapes", "category10", "category20".
#' @param reverse  If true, flips the scale range.
#' @param round If true, rounds numeric output values to integers. This can be
#'   helpful for snapping to the pixel grid.
#' @param ... other named arguments.
#' @param subclass Class name for subclass.  Will have \code{scale_} prepended.
#' @param override Should the domain specified by this ggvis_scale object
#'   override other ggvis_scale objects for the same scale? Useful when domain is
#'   manually specified. For example, by default, the domain of the scale
#'   will contain the range of the data, but when this is TRUE, the specified
#'   domain will override, and the domain can be smaller than the range of the
#'   data. If \code{FALSE}, the \code{domain} will not behave this way. If
#'   left \code{NULL}, then it will be treated as \code{TRUE} whenever
#'   \code{domain} is non-NULL.
#' @seealso \url{https://vega.github.io/vega/docs/scales/}
#' @export
#' @keywords internal
#' @examples
#' ggvis_scale("x", type = "linear")
#' ggvis_scale("x", "ord")
ggvis_scale <- function(property, name = property, label = name, type = NULL,
                        domain = NULL, range = NULL, reverse = NULL,
                        round = NULL, ..., subclass = NULL, override = NULL) {
  assert_that(is.string(name))
  assert_that(is.null(type) ||
              type %in% c("linear", "ordinal", "time", "utc", "log", "pow",
                          "sqrt", "quantile", "quantize", "threshold"))
  assert_that(is.null(reverse) || is.flag(reverse),
              is.null(round) || is.flag(round))

  if (!is.null(subclass)) {
    assert_that(is.string(subclass))
    subclass <- paste0("scale_", subclass)
  }

  # By default, if domain is specified, it should override other domains
  override <- override %||% !is.null(domain)

  structure(
    drop_nulls(c(
      list(
        property = property, name = name, label = label, type = type,
        reverse = reverse, round = round, domain = domain, override = override,
        ...
      ),
      range_prop(range, "range")
    )),
    class = c(subclass, "ggvis_scale")
  )
}

#' @export
#' @rdname ggvis_scale
#' @param x object to test for scale-ness
is.ggvis_scale <- function(x) inherits(x, "ggvis_scale")

#' @export
format.ggvis_scale <- format.ggvis_axis

#' @export
print.ggvis_scale <- print.ggvis_axis

#' @export
as.vega.ggvis_scale <- function(x, ...) {
  if (shiny::is.reactive(x$domain)) {
    # Replace the domain with something that grabs it from the domain data
    x$domain <- list(
      data = paste0("scale/", x$name),
      field = "data.domain"
    )
  }

  # Remove the non-vega items
  x$property <- NULL
  x$label <- NULL
  x$override <- NULL
  x$expand <- NULL

  x
}

# Collapse all scales inside each vis
collapse_scales <- function(x) {
  if (is.ggvis(x) || is.mark_group(x)) {
    x$scales <- collapse_scales_by_name(x$scales)
    x$marks <- lapply(x$marks, collapse_scales)
    x
  } else {
    x
  }
}


collapse_scales_by_name <- function(x) {
  by_name <- split(x, vpluck(x, "name", character(1)))
  lapply(by_name, collapse_ggvis_scales)

}

# Takes a list of ggvis_scale objects and collapses them into a single
# ggvis_scale object.
collapse_ggvis_scales <- function(scales) {
  if (empty(scales)) return(NULL)

  countable <- unique(unlist(lapply(scales, scale_countable)))
  if (length(unique(countable)) > 1) {
    stop("Scales must all be countable, or all not countable.")
  }

  collapse_domains <- function(domains, overrides, countable) {
    # Set the domain based on whether there is an override domain, and the type
    # of domain.
    if (any(overrides)) {
      # Use the last overriding domain, if more than one
      over <- domains[[last(which(overrides))]]
      under <- domains[!overrides]

      if (countable) {
        # For categorical props, just use the override domain
        domain <- reactive(value(over))

      } else {
        # For continuous props, if either of the values in the override domain
        # is NA, then use the upper/lower bound of the non-override domains for
        # that value.
        domain <- reactive({
          over_vals <- value(over)

          if (any(is.na(over_vals))) {
            under_vals <- data_range(concat(values(under)))
            if (is.na(over_vals[1])) over_vals[1] <- under_vals[1]
            if (is.na(over_vals[2])) over_vals[2] <- under_vals[2]
          }
          over_vals
        })
      }

    } else {
      # If no overrides, just get range from the data
      domain <- reactive({
        data_range(concat(values(domains)))
      })
    }
    domain
  }

  domain <- collapse_domains(
    domains = pluck(scales, "domain"),
    overrides = vpluck(scales, "override", logical(1)),
    countable = countable
  )

  # Merge scales from left to right. A couple fields need special treatment.
  new_scale <- Reduce(merge_ggvis_scales, scales)
  # Get last non-NULL label
  new_scale$label <- last(compact(pluck(scales, "label")))

  new_scale$domain <- domain
  new_scale$override <- NULL

  new_scale <- apply_scale_defaults(new_scale)

  # Must expand domains after the scale defaults are applied
  expand <- new_scale$expand
  if (!scale_countable(new_scale) && !is.null(expand) && any(expand != 0)) {
    old_domain <- new_scale$domain
    new_scale$domain <- reactive({
      expand_range(old_domain(), expand)
    })
  }

  new_scale
}

# Takes a named list, where each name is the name of a scale, and each item is
# a scale object. Returns a named list where there's one item per scale,
# and each item is a reactive that returns a data frame with values for the
# domain.

scale_domain_data <- function(x) {
  # Find scales in all vises
  scales <- gather_scales(x)
  # Only need to make data domain for reactive scales
  scales <- Filter(function(scale) shiny::is.reactive(scale$domain), scales)
  names(scales) <- paste0("scale/", vpluck(scales, "name", character(1)))

  lapply(scales, function(scale) {
    force(scale)
    reactive({
      data.frame(domain = value(scale$domain))
    })
  })
}


expand_range <- function(range, mult = 0) {
  if (length(range) == 0) return(range)
  if (length(range) != 2) stop("range must have either 0 or 2 values")
  if (length(mult) == 1) mult <- c(mult, mult)

  range + diff(range) * mult * c(-1, 1)
}

merge_ggvis_scales <- function(a, b) {
  # a and b may have more than one class. If there's more than one, b's class
  # has precedence.
  if (length(class(b)) > 1) {
    new_class <- class(b)
  } else {
    new_class <- class(a)
  }

  structure(merge_vectors(a, b), class = new_class)
}

library(ggvis)

function(input, output, session) {
  # A reactive subset of mtcars
  mtc <- reactive({ mtcars[1:input$n, ] })

  # A simple visualisation. In shiny apps, need to register observers
  # and tell shiny where to put the controls
  mtc %>%
    ggvis(~wt, ~mpg) %>%
    layer_points() %>%
    bind_shiny("plot", "plot_ui")

  output$mtc_table <- renderTable({
    mtc()[, c("wt", "mpg")]
  })
}

#' Visualise a data set with a ggvis graphic.
#'
#' \code{ggvis} is used to turn a dataset into a visualisation, setting up
#' default mappings between variables in the dataset and visual properties.
#' Nothing will be displayed until you add additional layers.
#'
#' @param data A data object.
#' @param ... Property mappings. If not named, the first two mappings are
#'   taken to be \code{x} and \code{y}. Common properties are \code{x},
#'   \code{y}, \code{stroke}, \code{fill}, \code{opacity}, \code{shape}
#' @param env Environment in which to evaluate properties.
#' @import assertthat
#' @importFrom shiny reactive
#' @export
#' @examples
#' # If you don't supply a layer, ggvis uses layer_guess() to guess at
#' # an appropriate type:
#' mtcars %>% ggvis(~mpg, ~wt)
#' mtcars %>% ggvis(~mpg, ~wt, fill = ~cyl)
#' mtcars %>% ggvis(~mpg, ~wt, fill := "red")
#' mtcars %>% ggvis(~mpg)
#'
#' # ggvis has a functional interface: every ggvis function takes a ggvis
#' # an input and returns a modified ggvis as output.
#' layer_points(ggvis(mtcars, ~mpg, ~wt))
#'
#' # To make working with this interface more natural, ggvis imports the
#' # pipe operator from magrittr. x %>% f(y) is equivalent to f(x, y) so
#' # we can rewrite the previous command as
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points()
#'
#' # For more complicated plots, add a line break after %>%
#' mtcars %>%
#'   ggvis(~mpg, ~wt) %>%
#'   layer_points() %>%
#'   layer_smooths()
ggvis <- function(data = NULL, ..., env = parent.frame()) {
  vis <- structure(
    list(
      marks = list(),
      data = list(),
      props = list(),
      reactives = list(),
      scales = list(),
      axes = list(),
      legends = list(),
      controls = list(),
      connectors = list(),
      handlers = list(),
      options = list(),
      cur_data = NULL,
      cur_props = NULL,
      cur_vis = NULL
    ),
    class = "ggvis"
  )

  vis <- add_data(vis, data, deparse2(substitute(data)))
  vis <- add_props(vis, ..., env = env)
  vis
}


#' Add visual properties to a visualisation
#'
#' @param vis Visualisation to modify.
#' @inheritParams props
#' @export
#' @examples
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points()
#' mtcars %>% ggvis() %>% add_props(~wt, ~mpg) %>% layer_points()
#' mtcars %>% ggvis(~wt) %>% add_props(y = ~mpg) %>% layer_points()
add_props <- function(vis, ..., .props = NULL, inherit = NULL,
                      env = parent.frame()) {

  # Get value of inherit from inherit arg, then .props$inherit, then TRUE
  if (!is.null(.props)) inherit <- attr(.props, "inherit", TRUE)
  inherit <- inherit %||% TRUE

  new_props <- props(..., .props = .props, inherit = inherit, env = env)
  both_props <- merge_props(cur_props(vis), new_props)

  vis$props[[length(vis$props) + 1]] <- both_props
  vis$cur_props <- both_props

  vis <- register_reactives(vis, extract_reactives(both_props))
  vis
}

#' Add dataset to a visualisation
#'
#' @param vis Visualisation to modify.
#' @param data Data set to add.
#' @param name Data of data - optional, but helps produce informative
#'  error messages.
#' @param add_suffix Should a unique suffix be added to the data object's ID?
#'   This should only be FALSE when the spec requires a data set with a
#'    specific name.
#' @export
#' @examples
#' mtcars %>% ggvis(~mpg, ~wt) %>% layer_points()
#' NULL %>% ggvis(~mpg, ~wt) %>% add_data(mtcars) %>% layer_points()
add_data <- function(vis, data, name = deparse2(substitute(data)),
                     add_suffix = TRUE) {
  if (is.null(data)) return(vis)

  # Make sure data is reactive
  if (!shiny::is.reactive(data)) {
    static_data <- data
    data <- function() static_data
  }

  if (add_suffix) name <- paste0(name, length(vis$data))
  data_id(data) <- name
  vis$data[[name]] <- data
  vis$cur_data <- data

  vis
}

#' Is an object a ggvis object?
#'
#' @export
#' @param x an object to test
#' @keywords internal
is.ggvis <- function(x) inherits(x, "ggvis")


# Add a mark to a ggvis object.
add_mark <- function(vis, type = NULL, props = NULL, data = NULL,
                     data_name = "unnamed_data") {

  # Save current data
  old_data <- vis$cur_data
  old_props <- vis$cur_props

  # If we're in a subvis, modify scale names to include prefix
  # FIXME: figure out how to avoid this in order to specify parent scales
  # Maybe some attribute? e.g. scale = parent("x")
  if (!is.null(vis$cur_vis)) {
    suffix <- paste0(vis$cur_vis, collapse = "-")
    props <- lapply(props, function(x) {
      if (identical(x$scale, FALSE)) return(x)
      x$scale <- paste0(x$scale, suffix)
      x
    })
  }

  vis <- add_data(vis, data, data_name)
  vis <- add_props(vis, .props = props)


  vis <- register_scales_from_props(vis, cur_props(vis))

  new_mark <- mark(type, props = cur_props(vis), data = vis$cur_data)
  vis <- append_ggvis(vis, "marks", new_mark)

  # Restore old data
  vis$cur_data <- old_data
  vis$cur_props <- old_props
  vis
}

#' Add arbitrary scales to ggvis.
#'
#' @param vis Visualisation to modify.
#' @param scale Scale object
#' @param data_domain Should the domain be controlled by a data set which is
#'   added to the spec? Should only be set to FALSE in special cases.
#' @keywords internal
#' @export
add_scale <- function(vis, scale, data_domain = TRUE) {
  if (data_domain && shiny::is.reactive(scale$domain)) {
    vis <- register_reactive(vis, scale$domain)
  }
  vis <- append_ggvis(vis, "scales", scale)
  vis
}

# If replace is TRUE, new options overwrite existing options; if FALSE, they don't.
add_options <- function(vis, options, replace = TRUE) {
  if (replace) {
    vis$options <- merge_vectors(vis$options, options)
  } else {
    vis$options <- merge_vectors(options, vis$options)
  }
  vis
}

register_computation <- function(vis, args, name, transform = NULL) {
  vis <- register_reactives(vis, args)

  if (is.null(transform)) return(vis)

  parent_data <- vis$cur_data
  # For the ID, append to the parent's ID, along with a unique number.
  id <- paste0(data_id(parent_data), "/", name, length(vis$data))

  if (shiny::is.reactive(parent_data) || any_apply(args, shiny::is.reactive)) {
    empty <- NULL

    # First time computation is executed, it must succeed. That's used to
    # determine the specification of the data, and if an error occurs in
    # subequent run, that specification is sent and the error is printed to
    # the console
    new_data <- reactive({
      if (is.null(empty)) {
        out <- transform(parent_data(), values(args))
        empty <<- out[0, , drop = FALSE]
        out
      } else {
        tryCatch(
          transform(parent_data(), values(args)),
          error = function(e) {
            message("Error: ", e$message)
            data.frame
          }
        )
      }
    })

  } else {
    cache <- transform(parent_data(), args)
    new_data <- function() cache
  }

  data_id(new_data) <- id
  vis$data[[id]] <- new_data
  vis$cur_data <- new_data

  vis
}


# Register a list of reactives in the ggvis object's reactives list
# @param vis A ggvis object.
# @param reactives A list of reactives.
register_reactives <- function(vis, reactives = NULL) {
  # Drop any objects from the 'reactives' list which aren't actually reactive
  reactives <- reactives[vapply(reactives, shiny::is.reactive, logical(1))]

  for (reactive in reactives) {
    vis <- register_reactive(vis, reactive)
  }
  vis
}

register_reactive <- function(vis, reactive) {
  # Some reactives are marked so that they're not registered
  if (identical(attr(reactive, "register"), FALSE)) return(vis)

  # Add reactive id if needed
  if (is.null(reactive_id(reactive))) {
    reactive_id(reactive) <- rand_id("reactive_")
  }

  label <- reactive_id(reactive)

  # Don't add if already registered
  if (label %in% names(vis$reactives)) return(vis)

  vis$reactives[[label]] <- reactive

  # If it's a broker, add controls, connector, and spec as needed
  if (is.broker(reactive)) {
    broker <- attr(reactive, "broker", TRUE)

    vis <- register_controls(vis, broker$controls)
    vis <- register_connector(vis, broker$connect)
    vis <- register_handler(vis, broker$spec)
  }

  vis
}

# Given a set of props, register a scale for each one.
register_scales_from_props <- function(vis, props) {
  # Strip off .update, .enter, etc.
  names(props) <- trim_prop_event(names(props))

  # Get a reactive for each scaled prop
  data <- vis$cur_data

  add_scale_from_prop <- function(vis, prop) {
    # Automatically add label, unless it's blank or has a trailing '_'
    label <- prop_label(prop)
    if (label == "" || grepl("_$", label)) {
      label <- NULL
    }

    if (is.prop_band(prop)) {
      # band() requires points = FALSE
      vis <- add_scale(
        vis,
        ggvis_scale(property = propname_to_scale(prop$property),
          name = prop$scale, points = FALSE, label = label)
      )
      return(vis)
    }

    if (is.null(prop$value) || !prop_is_scaled(prop) || is.null(data)) {
      return(vis)
    }

    type <- vector_type(shiny::isolate(prop_value(prop, data())))
    domain <- reactive({
      data_range(prop_value(prop, data()))
    })
    # Flag to not register this reactive in the ggvis reactives list. This is
    # so that these reactives don't make is.dynamic() think that the plot is
    # dynamic.
    attr(domain, "register") <- FALSE

    # e.g. scale_quantitative, scale_nominal
    scale_fun <- match.fun(paste0("scale_", type))

    vis <- scale_fun(vis, property = prop$property, name = prop$scale,
                     label = label, domain = domain, override = FALSE)
    vis
  }

  # Add them to the vis
  for (i in seq_along(props)) {
    vis <- add_scale_from_prop(vis, props[[i]])
  }

  vis
}

# Takes a list of controls
register_controls <- function(vis, controls) {
  if (empty(controls)) return(vis)

  # If passed a bare control, wrap it into a list
  if (inherits(controls, "shiny.tag")) {
    controls <- list(controls)
  }
  vis$controls <- c(vis$controls, controls)
  vis
}

register_connector <- function(vis, connector) {
  vis$connectors <- c(vis$connectors, connector)
  vis
}

register_handler <- function(vis, handler) {
  if(empty(handler)) return(vis)

  vis$handlers <- c(vis$handlers, list(handler))
  vis
}

#' Print out the vega plot specification
#'
#' @param vis Visualisation to print
#' @param pieces Optional, a character or numeric vector used to
#'   pull out selected pieces of the spec
#' @export
#' @examples
#' base <- mtcars %>% ggvis(~mpg, ~wt) %>% layer_points()
#' base %>% show_spec()
#' base %>% show_spec("scales")
show_spec <- function(vis, pieces = NULL) {
  out <- as.vega(vis, dynamic = FALSE)

  if (!is.null(pieces)) {
    out <- out[pieces]
  }

  json <- jsonlite::toJSON(out, pretty = TRUE, auto_unbox = TRUE, force = TRUE,
                           null = "null")
  cat(gsub("\t", " ", json), "\n", sep = "")

  invisible(vis)
}

#' Tools to save and view static specs.
#'
#' These functions are mainly useful for testing.
#'
#' @param path location to save spec to, or load spec from
#' @param x a ggvis object
#' @param ... other arguments passed to \code{as.vega}
#' @keywords internal
#' @export
save_spec <- function(x, path, ...) {
  assert_that(is.ggvis(x), is.string(path))

  json <- jsonlite::toJSON(as.vega(x, ...), pretty = TRUE, auto_unbox = TRUE,
                           force = TRUE, null = "null")
  writeLines(json, path)
}

#' @rdname save_spec
view_spec <- function(path, ...) {
  contents <- paste0(readLines(path), collapse = "\n")
  spec <- jsonlite::fromJSON(contents)
  view_static(spec)
}

append_ggvis <- function(vis, field, x) {
  i <- vis$cur_vis
  if (length(i) == 0) {
    vis[[field]] <- c(vis[[field]], list(x))
  } else if (length(i) == 1) {
    vis$marks[[i]][[field]] <- c(vis$marks[[i]][[field]], list(x))
  } else if (length(i) == 2) {
    vis$marks[[i[1]]]$marks[[i[2]]][[field]] <-
      c(vis$marks[[i[1]]]$marks[[i[2]]][[field]], list(x))
  } else {
    stop(">3 levels deep? You must be crazy!", call. = FALSE)
  }

  vis
}

context("ggvis")
# Recursively crawl a list and replace any environments with a special
# environment. This is a workaround for a change in behavior in R 3.2.0
# for all.equal when given two environments.
blank_envs <- function(x) {
  replacement_env <- new.env(parent = emptyenv())

  # Use `x[]<-` to preserve any attributes on x
  x[] <- lapply(x, function(val) {
    if (is.environment(val)) replacement_env
    else if (is.list(val)) blank_envs(val)
    else val
  })

  x
}

test_that("plot the same regardless of order data/props added", {

  df <- data.frame(x = 1:10, y = 10:1)
  plots <- list(
    df %>% ggvis(~x, ~y) %>% layer_points(),
    NULL %>% ggvis(~x, ~y) %>% add_data(df) %>% layer_points(),
    NULL %>% ggvis(~x, ~y) %>% layer_points(data = df),
    ggvis() %>% add_data(df) %>% add_props(~x, ~y) %>% layer_points(),
    ggvis() %>% add_data(df) %>% add_props(~x) %>% layer_points(y = ~y)
  )
  plots <- blank_envs(plots)

  marks <- lapply(plots, function(x) x$marks[[1]])
  props <- lapply(marks, "[[", "props")
  data  <- lapply(marks, function(x) x$data())

  for (i in seq_along(marks)) {
    expect_equal(
      props[[i]][c("x.update", "y.update")],
      blank_envs(props(~x, ~y))
    )
    expect_equal(data[[i]], df)
  }
})

#' Get data from a ggvis object
#'
#' This function is useful for inspecting the data in a ggvis object.
#' @param vis A ggvis object.
#' @examples
#' p <- cocaine %>% ggvis(~price) %>% layer_bars()
#' get_data(p)
#'
#' @export
get_data <- function(vis) {
  if (!is.ggvis(vis)) stop("vis must be a ggvis object.")
  lapply(vis$data, function(x) shiny::isolate(x()))
}

data_id <- function(x) {
  return(attr(x, "data_id", TRUE))
}

`data_id<-` <- function(x, value) {
  attr(x, "data_id") <- value
  x
}

# Convenience function to remove missing values from a data.frame
# Remove all non-complete rows, with a warning if \code{warn_na = FALSE}.
#
# @param x An input data frame or split_df
# @param warn_na Warn when rows are removed?
# @param finite When FALSE, remove NA and NaN. When TRUE, also remove Inf and -Inf.
# @param vars Columns to check for missing values (for data.frame and split_df)
remove_missing <- function(x, warn_na = TRUE, finite = FALSE, ...) {
  UseMethod("remove_missing")
}

#' @export
remove_missing.default <- function(x, warn_na = TRUE, finite = FALSE, ...) {
  if (!is.atomic(x)) {
    stop("Input to remove_missing.default must be a vector.")
  }

  if (finite) {
    missing <- !is.finite(x) | is.na(x)
    str <- "non-finite"
  } else {
    missing <- is.na(x)
    str <- "missing"
  }

  if (any(missing)) {
    x <- x[!missing]
    if (warn_na) warning("Removed ", sum(missing), " rows containing ", str,
      " values", ".", call. = FALSE)
  }
  x
}

#' @export
remove_missing.data.frame <- function(x, warn_na = TRUE, finite = FALSE,
    vars = names(x), ...) {

  vars <- intersect(vars, names(x))

  if (finite) {
    missing <- !finite.cases(x[, vars, drop = FALSE])
    str <- "non-finite"
  } else {
    missing <- !stats::complete.cases(x[, vars, drop = FALSE])
    str <- "missing"
  }

  if (any(missing)) {
    x <- x[!missing, , drop = FALSE]
    if (warn_na) warning("Removed ", sum(missing), " rows containing ", str,
      " values", ".", call. = FALSE)
  }
  x
}

#' @export
remove_missing.split_df <- function(x, warn_na = TRUE, finite = FALSE,
    vars = names(x), ...) {
  x[] <- lapply(x, remove_missing, warn_na, vars, finite)
  x
}


# Returns a logical vector of same length as nrow(x). If all data on a row
# is finite (not NA, NaN, Inf, or -Inf) return TRUE; otherwise FALSE.
finite.cases <- function(x) {
  finite_cases <- vapply(x, is.finite, logical(nrow(x)))

  # Need a special case test when x has exactly one row, because rowSums
  # doesn't respect dimensions for 1x1 matrices. vapply returns a vector (not
  # a matrix when the input has one row.
  if (is.vector(finite_cases)) {
    all(finite_cases)
  } else {
    # Find all the rows where all are TRUE
    rowSums(as.matrix(finite_cases)) == ncol(x)
  }
}


to_csv <- function(x, header = TRUE, ...) UseMethod("to_csv")

#' @export
to_csv.data.frame <- function(x, header = TRUE, ...) {
  x <- lapply(x, format_vec_csv)

  # Collapse across rows, yielding each row of CSV text
  rows <- do.call(paste, c(x, sep = ","))
  rows <- paste0(rows, collapse = "\n")

  if (header) {
    rows <- paste(
      paste(quote_text(names(x)), collapse = ","),
      rows,
      sep = "\n"
    )
  }

  rows
}

# Format a vector for csv output
format_vec_csv <- function(vec) UseMethod("format_vec_csv")
#' @export
format_vec_csv.numeric <- function(vec) vec
#' @export
format_vec_csv.character <- function(vec) quote_text(vec)
#' @export
format_vec_csv.factor <- function(vec) quote_text(vec)
# Represent dates and times as numbers (ms from epoch). If they're represented
# as date-time strings, this will cause problems when the client is in a
# different time zone from the server.
#' @export
format_vec_csv.POSIXt <- function(vec) floor(as.numeric(vec) * 1000)
#' @export
format_vec_csv.Date <- function(vec) as.numeric(as.POSIXct(vec)) * 1000


# Format a vector for d3 json output
format_vec_d3json <- function(vec) UseMethod("format_vec_d3json")
#' @export
format_vec_d3json.numeric <- function(vec) vec
#' @export
format_vec_d3json.character <- function(vec) vec
#' @export
format_vec_d3json.factor <- function(vec) as.character(vec)
#' @export
format_vec_d3json.POSIXt <- function(vec) floor(as.numeric(vec) * 1000)
#' @export
format_vec_d3json.Date <- function(vec) as.numeric(as.POSIXct(vec)) * 1000


# Replace \. with . , " with \", and add " to start and end
quote_text <- function(txt) {
  if (length(txt) == 0)
    return(txt)

  txt <- gsub("\\.", ".", txt, fixed = TRUE)
  txt <- gsub('"', '\\\\"', txt, fixed = TRUE)
  paste0('"', txt, '"')
}


cur_data <- function(x) shiny::isolate(x$cur_data())
cur_props <- function(x) x$cur_props

eval_vector <- function(x, f) UseMethod("eval_vector")
#' @export
eval_vector.data.frame <- function(x, f) {
  eval(f[[2]], x, environment(f))
}

# Find the range of values for a vector
data_range <- function(x) UseMethod("data_range")
#' @export
data_range.default <- function(x) range2(x, na.rm = TRUE)
#' @export
data_range.character <- function(x) unique(stats::na.omit(x))
#' @export
data_range.factor <- function(x) levels(x)

# Takes a list of vectors, and puts them all together into one vector.
# For POSIXct, this preserves time zone.
# For factors, this preserves all levels (but not necessarily order)
concat <- function(x) {
  x <- drop_nulls(x)
  if (length(x) == 0) {
    return(NULL)
  }
  if (inherits(x[[1]], "POSIXct")) {
    vec <- do_call(c, .args = x)
    structure(vec, tzone = attr(x[[1]], "tzone"))
  } else if (inherits(x[[1]], "Date")) {
    structure(unlist(x, recursive = FALSE), class = "Date")
  } else {
    unlist(x, recursive = FALSE)
  }
}

# Does the same as base::range, except that for for 0-length vectors, it returns
# a zero-length vector of appropriate type, instead of throwing an error.
range2 <- function(..., na.rm = FALSE) {
  vals <- c(...)
  if (length(vals) == 0) {
    return(vals)
  }
  range(..., na.rm = na.rm)
}

context("utils-data")

test_that("finite.cases.data.frame", {
  # All finite --------------------------------------------------------------
  expect_identical(finite.cases(data.frame(x=4)),            TRUE)          # 1x1
  expect_identical(finite.cases(data.frame(x=4, y=11)),      TRUE)          # 1x2
  expect_identical(finite.cases(data.frame(x=4:5)),          c(TRUE, TRUE)) # 2x1
  expect_identical(finite.cases(data.frame(x=4:5, y=11:12)), c(TRUE, TRUE)) # 2x2

  # Has one NA --------------------------------------------------------------
  expect_identical(finite.cases(data.frame(x=NA)),                  FALSE)           # 1x1
  expect_identical(finite.cases(data.frame(x=4, y=NA)),             FALSE)           # 1x2
  expect_identical(finite.cases(data.frame(x=c(4,NA))),             c(TRUE,  FALSE)) # 2x1
  expect_identical(finite.cases(data.frame(x=c(4,NA), y=c(11,NA))), c(TRUE,  FALSE)) # 2x2
  expect_identical(finite.cases(data.frame(x=c(4,NA), y=c(NA,12))), c(FALSE, FALSE)) # 2x2
  expect_identical(finite.cases(data.frame(x=c(4,5),  y=c(NA,12))), c(FALSE, TRUE))  # 2x2

  # Testing NaN and Inf, using miscellaneous data shapes --------------------
  expect_identical(finite.cases(data.frame(x=c(4,NaN))),              c(TRUE, FALSE))
  expect_identical(finite.cases(data.frame(x=Inf)),                   FALSE)
  expect_identical(finite.cases(data.frame(x=c(4,5),  y=c(-Inf,12))), c(FALSE, TRUE))
})


test_that("remove_missing", {
  all_vals <- c(1, NA, NaN, Inf, -Inf)

  expect_warning(remove_missing(all_vals))
  expect_warning(remove_missing(all_vals, warn_na = FALSE), NA)

  # Vectors
  expect_identical(remove_missing(all_vals, warn_na = FALSE), c(1, Inf, -Inf))
  expect_identical(remove_missing(all_vals, warn_na = FALSE, finite = TRUE), 1)

  # Data frames
  # Get all combinations
  all_combs <- data.frame(x = rep(all_vals, 5), y = rep(all_vals, each = 5))

  res <- remove_missing(all_combs, warn_na = FALSE)
  rownames(res) <- NULL
  expect_identical(
    res,
    data.frame(x = rep(c(1, Inf, -Inf), 3), y = rep(c(1, Inf, -Inf), each = 3))
  )

  expect_identical(
    remove_missing(all_combs, warn_na = FALSE, finite = TRUE),
    data.frame(x=1, y=1)
  )

})


test_that("concat preserves types and timezones", {
  expect_identical(concat(list(1:3, 3:5)), c(1:3, 3:5))
  expect_identical(
    concat(list(c('a', 'b'), c('a', 'c'))),
    c('a', 'b', 'a', 'c')
  )
  expect_identical(
    concat(list(factor(c('a', 'b')), factor(c('c', 'a')))),
    factor(c('a', 'b', 'c', 'a'))
  )
  # Factors with different level order
  expect_identical(
    concat(list(factor(c('a', 'b'), levels = c('a', 'c', 'b')),
                factor(c('c', 'a')))),
    factor(c('a', 'b', 'c', 'a'), levels = c('a', 'c', 'b'))
  )
  expect_identical(
    concat(list(factor(c('a', 'b'), levels = c('b', 'a')),
                factor(c('c', 'd'), levels = c('d', 'c')))),
    factor(c('a', 'b', 'c', 'd'), levels = c('b', 'a', 'd', 'c'))
  )


  # Preserves time zone
  t1 <- as.POSIXct('2001-06-11 21:00', tz = 'UTC') + c(0, 2000)
  t2 <- t1 + 5000
  expect_identical(
    concat(list(t1, t2)),
    as.POSIXct('2001-06-11 21:00', tz = 'UTC') + c(0, 2000, 5000, 7000)
  )


  # Lists with 3 items
  expect_identical(concat(list(1:3, 3:5, 1:2)), c(1:3, 3:5, 1:2))
  expect_identical(
    concat(list(factor(c('a', 'b')), factor(c('a', 'c')), factor(c('b', 'd')))),
    factor(c('a', 'b', 'a', 'c', 'b', 'd'))
  )
  expect_identical(
    concat(list(t1, t2, t2)),
    as.POSIXct('2001-06-11 21:00', tz = 'UTC') + c(0, 2000, 5000, 7000, 5000, 7000)
  )
})


test_that("concat handles NULLs and zero-length vectors", {
  expect_identical(concat(list(NULL, NULL)), NULL)
  expect_identical(concat(list(NULL, character(0))), character(0))
  expect_identical(concat(list(NULL, 1:10)), 1:10)
  expect_identical(concat(list(1:10, NULL, integer(0))), 1:10)
})


test_that("preserve_constants", {
  # Input data frames with various numbers of rows
  input0 <- data.frame(a = numeric(0), b = character(0), stringsAsFactors = FALSE)
  input1 <- data.frame(a = 1, b = "txt", stringsAsFactors = FALSE)
  input3 <- data.frame(a = 1:3, b = rep("txt", 3), stringsAsFactors = FALSE)

  # Output data frames with various numbers of rows
  output0 <- data.frame(foo = numeric(0))
  output1 <- data.frame(foo = 11)
  output2 <- data.frame(foo = 11:12)

  # Some things that the input data frames get reduced to
  abNA <- data.frame(a = NA_real_, b = NA_character_, stringsAsFactors = FALSE)
  b0 <- data.frame(b = character(0), stringsAsFactors = FALSE)
  b1 <- data.frame(b = "txt", stringsAsFactors = FALSE)

  # Test all the combinations
  expect_identical(preserve_constants(input0, output0), cbind(input0, output0))
  expect_identical(preserve_constants(input1, output0), cbind(input0, output0))
  expect_identical(preserve_constants(input3, output0), cbind(b0, output0))

  expect_identical(preserve_constants(input0, output1), cbind(abNA, output1))
  expect_identical(preserve_constants(input1, output1), cbind(input1, output1))
  expect_identical(preserve_constants(input3, output1), cbind(b1, output1))

  expect_identical(preserve_constants(input0, output2), cbind(abNA, output2))
  expect_identical(preserve_constants(input1, output2), cbind(input1, output2))
  expect_identical(preserve_constants(input3, output2), cbind(b1, output2))

  # grouped_df with no rows in some groups - output shouldn't have NA rows for
  # those that are missing in one or the other
  input3g <- group_by(input3, a)
  expect_equal(
    preserve_constants(input3g, data.frame(a=1:2, v=5:6)),
    group_by(data.frame(a=1:2, b=c("txt","txt"), v=5:6, stringsAsFactors=FALSE), a)
  )
})

test_that("preserve_constants preserves factor level order", {
  input <- data.frame(g1 = factor(c('A','B','C'), levels = c('B','A','C')))
  output <- data.frame(
    g1 = factor(c('A','B','C'), levels = c('B','A','C')),
    g2 = factor(c('A','B','C'), levels = c('B','A','C'))
  )

  expect_identical(preserve_constants(input, output), output)

  # grouped
  res <- preserve_constants(group_by(input, g1), group_by(output, g1))
  expect_identical(levels(res$g1), c('B','A','C'))
  expect_identical(levels(res$g2), c('B','A','C'))
})

test_that("to_csv", {
  # Zero-row data frame. The trailing \n should be optional.
  expect_identical(
    to_csv(data.frame(x = numeric(0), c = character(0))),
    "\"x\",\"c\"\n"
  )
})

#' Create an interactive slider.
#'
#' @inheritParams shiny::sliderInput
#' @param id A unique identifier for this input. Usually generated
#'   automatically.
#' @param map A function with single argument \code{x}, the value of the
#'   control on the client. Returns a modified value.
#' @param format A string specifying how to format the value.
#' @param locale A string specifying the locale to use for formatting.
#' @family interactive input
#' @export
#' @examples
#' input_slider(0, 100)
#' input_slider(0, 100, label = "binwidth")
#' input_slider(0, 100, value = 50)
#'
#' # Supply two values to value to make a double-ended sliders
#' input_slider(0, 100, c(25, 75))
#'
#' # You can use map to transform the outputs
#' input_slider(-5, 5, label = "Log scale", map = function(x) 10 ^ x)
input_slider <- function(min, max, value = (min + max) / 2, step = NULL,
                         round = FALSE, format = NULL, locale = "us",
                         ticks = TRUE, animate = FALSE,
                         sep = ",", pre = NULL, post = NULL, label = "",
                         id = rand_id("slider_"), map = identity) {

  assert_that(is.string(label), is.string(id))

  if (!is.null(step)) {
    value <- round_any(value - min, step) + min
  }

  # Older versions of shiny use `format` and `locale`; newer versions use
  # `sep`, `pre`, and `post`.
  if (utils::packageVersion("shiny") >= "0.11") {
    control <- shiny::sliderInput(id, label, min = min, max = max,
      value = value, step = step, round = round,
      ticks = ticks, animate = animate, sep = sep, pre = pre, post = post)
  } else {
    control <- shiny::sliderInput(id, label, min = min, max = max,
      value = value, step = step, round = round, format = format, locale = locale,
      ticks = ticks, animate = animate)
  }

  create_input(id, value, map, control)
}


#' Create an interactive checkbox.
#'
#' @inheritParams shiny::checkboxInput
#' @inheritParams input_slider
#' @family interactive input
#' @export
#' @examples
#'
#' input_checkbox(label = "Confidence interval")
#' input_checkbox(label = "Confidence interval", value = TRUE)
#'
#' # Used in layer_smooths
#' mtcars %>% ggvis(~wt, ~mpg) %>%
#'   layer_smooths(se = input_checkbox(label = "Confidence interval"))
#'
#' # Used with a map function, to convert the boolean to another type of value
#' model_type <- input_checkbox(label = "Use flexible curve",
#'   map = function(val) if(val) "loess" else "lm")
#' mtcars %>% ggvis(~wt, ~mpg) %>%
#'   layer_model_predictions(model = model_type)
input_checkbox <- function(value = FALSE, label = "",
                           id = rand_id("checkbox_"), map = identity) {

  assert_that(is.string(label), is.string(id))

  create_input(id, value, map, shiny::checkboxInput(id, label, value))
}

#' Create an interactive text or numeric input box.
#'
#' \code{input_numeric} only allows numbers and comes with a spin box control.
#' \code{input_text} allows any type of input.
#'
#' @inheritParams shiny::textInput
#' @inheritParams input_slider
#' @family interactive input
#' @export
#' @examples
#' fill_text <- input_text(label = "Point color", value = "red")
#' mtcars %>% ggvis(~wt, ~mpg, fill := fill_text) %>% layer_bars()
#'
#' size_num <- input_numeric(label = "Point size", value = 25)
#' mtcars %>% ggvis(~wt, ~mpg, size := size_num) %>% layer_points()
input_text <- function(value, label = "", id = rand_id("text_"),
                       map = identity) {

  assert_that(is.string(label), is.string(id), is.string(value))

  create_input(id, value, map, shiny::textInput(id, label, value))
}

#' @rdname input_text
#' @export
input_numeric <- function(value, label = "", id = rand_id("numeric_"),
                          map = identity) {

  assert_that(is.string(label), is.string(id), is.numeric(value))

  create_input(id, value, map, shiny::numericInput(id, label, value))
}

#' Create interactive control to select one (or more options) from a list.
#'
#' \itemize{
#'  \item \code{input_radiobuttons} only ever selects one value
#'  \item \code{input_checkboxgroup} can alway select multiple values
#'  \item \code{input_select} can select only one if \code{multiple = FALSE},
#'    otherwise the user can select multiple by using modifier keys
#' }
#'
#' @inheritParams shiny::selectInput
#' @inheritParams input_slider
#' @family interactive input
#' @export
#' @examples
#' # Dropdown
#' input_select(c("a", "b", "c"))
#' input_select(c("a", "b", "c"), multiple = TRUE)
#' input_select(c("a", "b", "c"), selected = "c")
#'
#' # If you want to select variable names, you need to convert
#' # the output of the input to a name with map so that they get
#' # computed correctly
#' input_select(names(mtcars), map = as.name)
#'
#' # Radio buttons
#' input_radiobuttons(choices = c("Linear" = "lm", "LOESS" = "loess"),
#'                    label = "Model type")
#' input_radiobuttons(choices = c("Linear" = "lm", "LOESS" = "loess"),
#'                    selected = "loess",
#'                    label = "Model type")
#'
#' # Used in layer_model_predictions
#' mtcars %>% ggvis(~wt, ~mpg) %>%
#'   layer_model_predictions(model = input_radiobuttons(
#'     choices = c("Linear" = "lm", "LOESS" = "loess"),
#'     selected = "loess",
#'     label = "Model type"))
#'
#' # Checkbox group
#' mtcars %>% ggvis(x = ~wt, y = ~mpg) %>%
#'   layer_points(
#'     fill := input_checkboxgroup(
#'       choices = c("Red" = "r", "Green" = "g", "Blue" = "b"),
#'       label = "Point color components",
#'       map = function(val) {
#'         rgb(0.8 * "r" %in% val, 0.8 * "g" %in% val, 0.8 * "b" %in% val)
#'       }
#'     )
#'   )
input_select <- function(choices, selected = NULL, multiple = FALSE,
                         label = "", id = rand_id("select_"), map = identity,
                         selectize = FALSE) {

  assert_that(is.string(label), is.string(id))

  control <- shiny::selectInput(id, label, choices = choices,
                                selected = selected, multiple = multiple,
                                selectize = selectize)

  if (is.null(selected)) {
    if (multiple) value <- ""
    else value <- choices[1]
  } else {
    value <- selected
  }

  create_input(id, value, map, control)
}

#' @rdname input_select
#' @export
input_radiobuttons <- function(choices, selected = NULL, label = "",
                               id = rand_id("radio_"), map = identity) {

  assert_that(is.string(label), is.string(id))

  control <- shiny::radioButtons(id, label, choices = choices,
                                 selected = selected)

  if (is.null(selected)) {
    value <- choices[1]
  } else {
    value <- selected
  }

  create_input(id, value, map, control)
}

#' @rdname input_select
#' @export
input_checkboxgroup <- function(choices, selected = NULL, label = "",
                                id = rand_id("radio_"), map = identity) {

  assert_that(is.string(label), is.string(id))

  control <- shiny::checkboxGroupInput(id, label, choices = choices,
                                       selected = selected)

  if (is.null(selected)) {
    value <- character(0)
  } else {
    value <- selected
  }

  create_input(id, value, map, control)
}

#' Overlay model predictions or a smooth curve.
#'
#' \code{layer_model_predictions} fits a model to the data and draw it with
#' \code{layer_paths} and, optionally, \code{layer_ribbons}.
#' \code{layer_smooths} is a special case of layering model predictions where
#' the model is a smooth loess curve whose smoothness is controlled by the
#' \code{span} parameter.
#'
#' @param vis Visualisation to modify
#' @param ... Visual properties. Stroke properties control only affect line,
#'   fill properties only affect standard error band.
#' @param model Name of the model as a string, e.g. \code{"loess"}, \code{"lm"},
#'   or \code{"MASS::rlm"}. Must be the name of a function that produces a
#'   standard model object with a \code{\link{predict}} method. For
#'   \code{layer_smooth} this is always "loess".
#' @param formula Model formula. If not supplied, guessed from the visual
#'   properties, constructing \code{y ~ x}.
#' @param model_args A list of additional arguments passed on to the
#'   \code{model} function.
#' @param se Also display a point-wise standard error band? Defaults to
#'   \code{FALSE} because interpretation is non-trivial.
#' @param span For \code{layer_smooth}, the span of the loess smoother.
#' @param domain If \code{NULL} (the default), the domain of the predicted
#'   values will be the same as the domain of the prediction variable in the
#'   data. It can also be a two-element numeric vector specifying the min and
#'   max.
#' @export
#' @examples
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_smooths()
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_smooths(se = TRUE)
#'
#' # Use group by to display multiple smoothes
#' mtcars %>% ggvis(~wt, ~mpg) %>% group_by(cyl) %>% layer_smooths()
#'
#' # Control appearance with props
#' mtcars %>% ggvis(~wt, ~mpg) %>%
#'   layer_smooths(se = TRUE, stroke := "red", fill := "red", strokeWidth := 5)
#'
#' # Control the wiggliness with span. Default is 0.75
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points() %>%
#'   layer_smooths(span = 0.2)
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points() %>%
#'   layer_smooths(span = 1)
#' # Map to an input to modify interactively
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points() %>%
#'   layer_smooths(span = input_slider(0.2, 1))
#'
#' # Use other modelling functions with layer_model_predictions
#' mtcars %>% ggvis(~wt, ~mpg) %>%
#'   layer_points() %>%
#'   layer_model_predictions(model = "lm") %>%
#'   layer_model_predictions(model = "MASS::rlm", stroke := "red")
#'
#' # Custom domain for predictions
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points() %>%
#'   layer_model_predictions(model = "lm", domain = c(0, 8))
#' mtcars %>% ggvis(~wt, ~mpg) %>% layer_points() %>%
#'   layer_model_predictions(model = "lm",
#'     domain = input_slider(0, 10, value = c(1, 4)))
#'
#' # layer_smooths() is just compute_smooth() + layer_paths()
#' # Run loess or other model outside of a visualisation to see what variables
#' # you get.
#' mtcars %>% compute_smooth(mpg ~ wt)
#' mtcars %>% compute_model_prediction(mpg ~ wt, model = "lm")
#'
#' mtcars %>%
#'   ggvis(~wt, ~mpg) %>%
#'   layer_points() %>%
#'   compute_smooth(mpg ~ wt) %>%
#'   layer_paths(~pred_, ~resp_, strokeWidth := 2)
layer_model_predictions <- function(vis, ..., model, formula = NULL,
                                    model_args = NULL, se = FALSE,
                                    domain = NULL) {

  vis <- set_scale_label(vis, "x", prop_label(cur_props(vis)$x.update))
  vis <- set_scale_label(vis, "y", prop_label(cur_props(vis)$y.update))

  formula <- formula %||% guess_formula(vis$cur_props, model)
  props <- stroke_fill_defaults(props(...),
    stroke = props(~pred_, ~resp_, strokeWidth := 2),
    fill =   props(~pred_, ~resp_lwr_, y2 = ~resp_upr_, fillOpacity := 0.2)
  )

  pipeline <- function(x) {
    x <- do_call(compute_model_prediction, quote(x), formula = formula,
                 model = model, se = se, domain = domain, .args = model_args)

    if (identical(se, TRUE)) {
      x <- emit_ribbons(x, props$fill)
    }
    x <- emit_paths(x, props$stroke)
    x
  }
  layer_f(vis, pipeline)
}

#' @export
#' @rdname layer_model_predictions
layer_smooths <- function(vis, ..., span = 0.75, se = FALSE) {
  formula <- guess_formula(vis$cur_props, "loess", quiet = TRUE)
  layer_model_predictions(vis, ..., model = "loess", formula = formula,
    model_args = list(span = span), se = se)
}

guess_formula <- function(props, model, quiet = FALSE) {
  vars <- list(
    x = find_prop_var(props, "x.update")[[2]],
    y = find_prop_var(props, "y.update")[[2]]
  )

  if (identical(model, "gam")) {
    f <- substitute(y ~ s(x), vars)
  } else {
    f <- substitute(y ~ x, vars)
  }
  formula <- eval(f, parent.frame())
  if (!quiet) notify_guess(formula)
  formula
}


# Helper function to determin maximum number of rows ---------------------------

max_rows <- function(x) UseMethod("max_rows")
#' @export
max_rows.data.frame <- function(x) nrow(x)
#' @export
max_rows.grouped_df <- function(x) max(dplyr::group_size(x))

library(ggvis)

mtcars %>%
  ggvis(~cyl, ~mpg) %>%
  layer_boxplots() %>%
  save_spec("boxplot/boxplot-continuous.json")

mtcars %>%
  ggvis(~factor(cyl), ~mpg) %>%
  layer_boxplots() %>%
  save_spec("boxplot/boxplot-categorical.json")


dat <- data.frame(x = c('a','a','a','b','b','b'), y = c(1:3, 2:4))
dat %>%
  ggvis(x = ~x, y = ~y) %>%
  layer_boxplots() %>%
  save_spec("boxplot/boxplot-no-outliers.json")

#' @export
set_default_scale <- function(...) {
  stop("set_default_scale has been removed, and has been replaced by scale_numeric, ",
       "scale_nominal, and so on. See ?scales for more information")
}
#' @export
set_dscale <- set_default_scale

# Given a ggvis_scale object, apply defaults and then return the modified object.
apply_scale_defaults <- function(x) UseMethod("apply_scale_defaults")

#' @export
apply_scale_defaults.ggvis_scale <- function(x) x

#' @export
apply_scale_defaults.scale_numeric <- function(x) {
  x$zero <- x$zero %||% FALSE
  x$nice <- x$nice %||% FALSE
  x$clamp <- x$clamp %||% FALSE

  if (is.null(x$range)) {
    x$range <- switch(propname_to_scale(x$property),
      x = "width",
      y = "height",
      stroke = c("#132B43", "#56B1F7"),
      fill = c("#132B43", "#56B1F7"),
      size = c(20, 100),
      fontSize = c(10, 20),
      opacity = c(0, 1),
      angle = c(0, 2 * pi),
      radius = c(0, 50),
      stop("Don't know how to automatically set range for ", x$property, ".")
    )
  }
  if (propname_to_scale(x$property) %in% c("x", "y") && is.null(x$expand)) {
    x$expand <- 0.05
  }

  x
}

#' @export
apply_scale_defaults.scale_datetime <- function(x) {
  x$type <- x$type %||% "time"
  x$clamp <- x$clamp %||% FALSE

  if (is.null(x$range)) {
    x$range <- switch(propname_to_scale(x$property),
      x = "width",
      y = "height",
      stroke = c("#132B43", "#56B1F7"),
      fill = c("#132B43", "#56B1F7"),
      size = c(20, 100),
      fontSize = c(10, 20),
      opacity = c(0, 1),
      angle = c(0, 2 * pi),
      radius = c(0, 50),
      stop("Don't know how to automatically set range for ", x$property, ".")
    )
  }
  if (propname_to_scale(x$property) %in% c("x", "y") && is.null(x$expand)) {
    x$expand <- 0.05
  }

  x
}

#' @export
apply_scale_defaults.scale_ordinal <- function(x) {
  x$points <- x$points %||% TRUE
  x$sort <- x$sort %||% FALSE

  if (is.null(x$range)) {
    x$range <- switch(propname_to_scale(x$property),
      x = "width",
      y = "height",
      stroke = "category10",
      strokeDash = list(c(100000, 1), c(8, 6), c(2, 2), c(3, 4, 10, 4),
                        c(15, 3), c(5, 2, 10, 2)),
      fill = "category10",
      size = c(10, 100),
      stop("Don't know how to automatically set range for ", x$property, ".")
    )
  }
  if (is.null(x$padding) && x$property %in% c("x", "y")) {
    if (isTRUE(x$points)) x$padding <- 0.5
    else x$padding <- 0.1
  }
  x
}

#' @export
apply_scale_defaults.scale_nominal <- function(x) {
  x$points <- x$points %||% TRUE
  x$sort <- x$sort %||% FALSE

  if (is.null(x$range)) {
    x$range <- switch(propname_to_scale(x$property),
      x = "width",
      y = "height",
      stroke = "category10",
      strokeDash = list(c(100000, 1), c(8, 6), c(2, 2), c(3, 4, 10, 4),
                        c(15, 3), c(5, 2, 10, 2)),
      fill = "category10",
      shape = "shapes"
    )
  }
  if (is.null(x$padding) && x$property %in% c("x", "y")) {
    if (isTRUE(x$points)) x$padding <- 0.5
    else x$padding <- 0.1
  }
  x
}

# Callback helper functions ---------------------------------------------------

# Check that input is a funtion with the specified arguments
check_callback <- function(f, args) {
  if (!is.null(f)) return()
  fname <- deparse2(substitute(f))

  if (!is.function(f)) {
    stop(fname, " is not a function", call. = FALSE)
  }

  f_args <- names(formals(f))
  if (any(f_args == "...")) return()

  if (!identical(f_args, args)) {
    stop(fname, " needs arguments: ", paste(args, collapse = ", "),
      call. = FALSE)
  }
}

# Given callback, id and session, setup an observer
setup_callback <- function(f, id, session) {
  if (is.null(f)) return()

  shiny::observe({
    value <- session$input[[id]]
    f(value, session)
  })
}

#' Generate sequence of fixed size intervals covering range.
#'
#' @param range range
#' @param size interval size
#' @param ... other arguments passed on to methods
#' @keywords internal
#' @export
fullseq <- function(range, size, ...) UseMethod("fullseq")

#' @export
fullseq.numeric <- function(range, size, ..., pad = FALSE) {
  # if (zero_range(range)) return(range + size * c(-1, 1) / 2)

  x <- seq(
    round_any(range[1], size, floor),
    round_any(range[2], size, ceiling),
    by = size
  )

  if (pad) {
    # Add extra bin on bottom and on top, to guarantee that we cover complete
    # range of data, whether right = T or F
    c(min(x) - size, x, max(x) + size)
  } else {
    x
  }
}

#' @export
fullseq.POSIXt <- function(range, size, ...) {
  seq(range[1], range[2], by = size)
}
#' @export
fullseq.Date <- fullseq.POSIXt

round_any <- function(x, accuracy, f = round) {
  f(x / accuracy) * accuracy
}

# Modifying scales
library(ggvis)

set.seed(1014)
df <- data.frame(x = 1:5, y = 5:1, z = runif(5))

# Override default scale property
df %>%
  ggvis(x = ~x, y = ~x) %>%
  layer_points() %>%
  scale_numeric("x", trans = "log") %>%
  scale_numeric("y", trans = "log") %>%
  save_spec("scales/log.json")

# Override default scale name
df %>%
  ggvis(x = ~x, y = ~x, prop("fill", ~z, scale = "blah")) %>%
  layer_points() %>%
  add_legend("blah") %>%
  save_spec("scales/custom.json")

# Dual scale
df %>%
  ggvis(x = ~x) %>%
  layer_points(prop("y", ~y, scale = "y-y"), fill := "red") %>%
  layer_points(prop("y", ~z, scale = "y-z")) %>%
  save_spec("scales/dual.json")

# Numeric domains
df %>%
  ggvis(x = ~x, y = ~x, fill = ~z) %>%
  layer_points() %>%
  scale_numeric("x", domain = c(0, 10)) %>%
  scale_numeric("fill", domain = c(0, 2)) %>%
  save_spec("scales/domain_numeric.json")

# Datetimes
dat <- data.frame(
  time = as.POSIXct("2013-07-01", tz = "GMT") + rnorm(40) * 60 * 60 * 24 * 7,
  value = rnorm(40)
)

dat %>% ggvis(~time, ~value) %>% layer_points() %>%
  save_spec("scales/datetime.json")

dat %>% ggvis(~time) %>% layer_histograms() %>%
  save_spec("scales/datetime_hist.json")

# Two properties in one legend, one on a custom scale
df %>%
  ggvis(x = ~x, y = ~x, fill = ~factor(y), prop("shape", ~factor(y), "shape2")) %>%
  layer_points() %>%
  add_legend(c("fill", "shape2")) %>%
  save_spec("scales/combined_legend.json")

# Nominal x with bars
df %>%
  ggvis(x = ~factor(x), y = ~y) %>%
  layer_bars() %>%
  save_spec("scales/bars.json")

# Hide axes and legends
df %>%
  ggvis(x = ~x, y = ~y, fill = ~z, shape = ~factor(x)) %>%
  layer_points() %>%
  hide_legend("fill") %>%
  hide_legend("shape") %>%
  hide_axis("x") %>%
  hide_axis("y") %>%
  save_spec("scales/hide_guides.json")

#' Create a property.
#'
#' Properties are used to describe the visual properties of \link{marks}.
#' You create a single property defintion with \code{prop}, and manage
#' sets of named properties with \code{\link{props}} (which also provides
#' shortcuts for creating the most common kind of properties)
#'
#' @param property A property, like "x", "x2", "y", "fill", and so on.
#' @param x The value of the property. This can be an atomic vector
#'   (a constant), a name or quoted call (a variable), a single-sided
#'   formula (a constant or variable depending on its contents), or a delayed
#'   reactive (which can be either variable or constant).
#' @param scale If \code{NULL}, automatically determine behavior by the kind of
#'   value (constant, variable, or reactive).
#'   If \code{TRUE} use the default scale associated with property.
#'   If \code{FALSE}, do not scale the value.
#'   Otherwise supply a string to select a custom scale.
#'   If \code{x} is an interactive input, then this defaults to the scale
#'   parameter of the input.
#' @param offset,mult Additive and multiplicate pixel offset used to adjust
#'   scaled values. These are useful if you want to place labels offset from
#'   points.
#' @param env If \code{x} is a quoted call this provides the environment in
#'   which to look for variables not in the data. You should not need this in
#'   ordinary operation.
#' @param event An event to which this property applies. One of "update",
#'   "enter", "exit", "hover", "brush".
#' @param label A label for this prop to use for reporting errors.
#' @seealso \code{\link{props}} to manage multiple properties and to
#'   succintly create the most common types.
#' @export
#' @examples
#' prop("x", 1)
#' prop("x", ~1)
#' prop("fill", quote(cyl))
#' prop("fill", ~cyl)
#' prop("x", input_slider(0, 100))
#'
#' # If you have a variable name as a string
#' var <- "cyl"
#' prop("x", as.name(var))
#'
#' # Use a custom scale
#' prop("y", quote(cyl), scale = "y-2")
#'
#' # Don't scale variable (i.e. it already makes sense in the visual space)
#' prop("fill", ~colour, scale = FALSE)
#'
#' # Use a constant, but scaled
#' prop("x", 5, scale = TRUE)
#'
#' # Use other events
#' prop("y", quote(cyl), scale = "y-2")
#'
prop <- function(property, x, scale = NULL, offset = NULL, mult = NULL,
                 env = parent.frame(), event = NULL, label = NULL) {

  if (missing(property)) stop("Property required for prop().")
  if (!is.character(property)) {
    stop("The interface to the prop() function has changed. The first argument ",
      "must be the name of a property, like 'x', 'y', or 'fill'. Instead of ",
      "props(x = prop(1)), use props(prop(\"x\", 1)). See ?prop and ?props.")
  }
  if (missing(x)) stop("Value required for prop().")
  if (property != "key" && is.null(event)) event <- "update"

  p <- new_prop(x, property, scale, offset, mult, env, event, label)

  if (p$property == "key") {
    if (!is.null(p$event)) stop("key prop cannot have an event.")
    if (!is.null(p$scale)) stop("key prop cannot have a scale.")
    if (is.prop_constant(p)) stop("key prop cannot be constant.")
  }

  p
}

#' Create new prop object
#'
#' The resulting object has the following fields:
#'
#' \itemize{
#'   \item property The name of a visual property, like "x", "x2", "width", "y",
#'     "fill".
#'   \item value A value. Can be a constant, reactive, or quoted expression.
#'   \item scale A string with name of a scale. Typically something like "x",
#'     "y", "fill", but can also be a custom name like "foo".
#'   \item offset Additive pixel offset used to adjust scaled values.
#'   \item mult Multiplicative pixel offset used to adjust scaled values.
#'   \item event A event like "update", "enter", "exit", "hover", "brush".
#'   \item env An environment in which to evaluate a variable or reactive value.
#' }
#' @keywords internal
new_prop <- function(x, property, scale, offset, mult, env, event, label) {
  UseMethod("new_prop")
}

#' @export
new_prop.prop <- function(x, ...) x

#' @export
new_prop.default <- function(x, property, scale, offset, mult, env, event,
                             label) {
  if (!is.atomic(x)) stop("Unknown input to prop: ", label)
  # If we got here, it's constant

  # strokeDash is the one property that can take an array
  if (property != "strokeDash") assert_that(length(x) == 1)

  structure(
    list(
      property = property,
      value = x,
      scale = decide_scale(scale %||% FALSE, property),
      offset = offset,
      mult = mult,
      event = event,
      env = NULL
    ),
    class = c("prop_constant", "prop")
  )
}

#' @export
new_prop.reactive <- function(x, property, scale, offset, mult, env, event,
                              label) {

  if (is.null(reactive_id(x))) {
    reactive_id(x) <- rand_id("reactive_")
  }

  structure(
    list(
      property = property,
      value = x,
      scale =  decide_scale(scale %||% FALSE, property),
      offset = offset,
      mult = mult,
      event = event,
      env = NULL
    ),
    class = c("prop_reactive", "prop")
  )
}

#' @export
new_prop.call <- function(x, property, scale, offset, mult, env, event,
                          label) {
  structure(
    list(
      property = property,
      value = x,
      scale = decide_scale(scale %||% TRUE, property),
      offset = offset,
      mult = mult,
      event = event,
      env = env
    ),
    class = c("prop_variable", "prop")
  )
}

#' @export
new_prop.name <- new_prop.call

#' @export
new_prop.formula <- function(x, property, scale, offset, mult, env, event,
                             label) {
  if (length(x) != 2) stop("Formulas must be single sided")
  new_prop.call(x[[2]], property, scale, offset, mult, environment(x),
                event, label)
}

# Given a value for scale and a property, return a string with the name of the
# scale. scale can be NULL, TRUE, FALSE, or a string.
decide_scale <- function(scale, property) {
  if (isTRUE(scale)) {
    propname_to_scale(trim_prop_event(property))
  } else if (identical(scale, FALSE)) {
    NULL
  } else {
    scale
  }
}


#' @export
#' @rdname prop
is.prop <- function(x) inherits(x, "prop")
#' @export
#' @rdname prop
is.prop_constant <- function(x) inherits(x, "prop_constant")
#' @export
#' @rdname prop
is.prop_variable <- function(x) inherits(x, "prop_variable")
#' @export
#' @rdname prop
is.prop_reactive <- function(x) inherits(x, "prop_reactive")



# Given a property and a dataset, get the value of the property.
prop_value <- function(x, data) UseMethod("prop_value")
#' @export
prop_value.prop_constant <- function(x, data) {
  rep(x$value, nrow(data))
}
#' @export
prop_value.prop_variable <- function(x, data) {
  # Calculate a "column"
  col <- eval(value(x$value), envir = data, enclos = x$env)

  if (!(length(col) == 1 || length(col) == nrow(data))) {
    stop("Length of calculated column '", prop_label(x), "' (", length(col),
      ") is not equal to 1 or the number of rows in data (", nrow(data), ").",
      call. = FALSE)
  }

  if (x$property == "key" && any(duplicated(col))) {
    warning("All values in column used for 'key' property should be unique,",
      " but some values are duplicated.")
  }

  rep(col, length.out = nrow(data))
}
#' @export
prop_value.prop_reactive <- prop_value.prop_variable


# The name of the property: used for naming the variable it produces in the
# vega data frame
prop_label <- function(x) UseMethod("prop_label")
#' @export
prop_label.prop_constant <- function(x) ""
#' @export
prop_label.prop_variable <- function(x) as_char(x$value)
#' @export
prop_label.prop_reactive <- function(x) as_char(reactive_id(x$value))

# Reports whether this is a scaled prop
prop_is_scaled <- function(prop) !is.null(prop$scale)

# Generate a vega object for the individual mark.
prop_vega <- function(x, default_scale) UseMethod("prop_vega")
#' @export
prop_vega.prop_constant <- function(x, default_scale) {
  compact(list(
    scale = if (prop_is_scaled(x)) x$scale,
    value = x$value,
    mult = x$mult,
    offset = x$offset
  ))
}
#' @export
prop_vega.prop_variable <- function(x, default_scale) {
  compact(list(
    scale = if (prop_is_scaled(x)) x$scale,
    field = paste0("data.", safe_vega_var(prop_label(x))),
    mult = x$mult,
    offset = x$offset
  ))
}
#' @export
prop_vega.prop_reactive <- prop_vega.prop_variable

#' Property domain.
#'
#' @param x property to dispatch on
#' @param data name of data set
prop_domain <- function(x, data) UseMethod("prop_domain")
#' @export
prop_domain.prop_constant <- function(x, data) {
  # FIXME: for scaled constants, this should really insert a literal value in
  #   to the domain, but it's not obvious how to do that in vega currently.
  NULL
}
#' @export
prop_domain.prop_variable <- function(x, data) {
  list(
    data = data,
    field = paste0("data.", safe_vega_var(prop_label(x)))
  )
}
#' @export
prop_domain.prop_reactive <- prop_domain.prop_variable


# Given a prop object, return a string representation of the value
# @examples
# p <- props(x = ~mpg, y = 10)
# as.character(p$x)
#
# p <- props(x := input_select(c("red", "blue")), y = 10)
# as.character.prop(p$x)
#' @export
as.character.prop_constant <- function(x, ...) as.character(x$value)
#' @export
as.character.prop_variable <- function(x, ...) deparse2(x$value)
#' @export
as.character.prop_reactive <- function(x, ...) reactive_id(x$value)

#' @export
format.prop <- function(x, ...) {
  if (!is.null(x$offset)) {
    offset <- paste0(" ", if (x$offset > 0) "+" else "-", " ", abs(x$offset))
  } else {
    offset <- ""
  }

  if (!is.null(x$mult)) {
    mult <- paste0(" * ", x$mult)
  } else {
    mult <- ""
  }
  scale <- if (prop_is_scaled(x)) x$scale else "<none>"
  event <- x$event %||% "<none>"
  type <- sub("^prop_", "", class(x)[1])

  paste0("<", type, "> ", as.character(x), offset, mult,
    " (property: ", x$property, ", scale: ", scale, ", event: ", event, ")")
}

#' @export
print.prop <- function(x, ...) cat(format(x, ...), "\n", sep = "")

# Determine the variable type given a data frame and property.
#
# @param data The data object.
# @param prop The property object.
# @keywords internal
prop_type <- function(data, prop) {
  UseMethod("prop_type")
}
#' @export
prop_type.data.frame <- function(data, prop) {
  vector_type(prop_value(prop, data))
}

# Continuous variables are not countable; categorical variables are.
prop_countable <- function(data, prop) {
  countable_prop_type(prop_type(data, prop))
}

# Report whether a prop type is countable
countable_prop_type <- function(type) {
  switch(type,
    NULL = NULL,
    "numeric" = FALSE,
    "datetime" = FALSE,
    "ordinal" = TRUE,
    "nominal" = TRUE,
    "logical" = TRUE,
    stop("Don't know whether prop type '", type, "' is countable")
  )
}

#' @export
formula.prop <- function(x, ...) {
  eval(substitute(~x, list(x = x$value)), x$env)
}

context("props")
library(shiny)

# Recursively crawl a list and replace any environments with a special
# environment. This is a workaround for a change in behavior in R 3.2.0
# for all.equal when given two environments.
blank_envs <- function(x) {
  replacement_env <- new.env(parent = emptyenv())

  # Use `x[]<-` to preserve any attributes on x
  x[] <- lapply(x, function(val) {
    if (is.environment(val)) replacement_env
    else if (is.list(val)) blank_envs(val)
    else val
  })

  x
}

test_prop <- function(p, property, value, scale, event = "update") {
  expect_identical(p$property, property)
  expect_identical(p$value, value)
  expect_identical(p$scale, scale)
  expect_identical(p$event, event)
}

test_that("creating prop objects with prop()", {
  expect_error(prop("y")) # Need value
  expect_error(prop(x = 1)) # Need scale

  # Unscaled, constant
  test_prop(prop("x", 1, scale = FALSE), "x", 1, NULL)
  test_prop(prop("x", 1), "x", 1, NULL)   # Use default scale
  test_prop(prop("x", ~1), "x", 1, "x")
  test_prop(prop("x2", 1), "x2", 1, NULL)

  # Unscaled, variable, with quote() and ~
  test_prop(prop("x", quote(cyl), scale = FALSE), "x", quote(cyl), NULL)
  test_prop(prop("x2", ~cyl, scale = FALSE), "x2", quote(cyl), NULL)

  # Scaled, constant
  test_prop(prop("x", 1, scale = "x"), "x", 1, "x")
  test_prop(prop("x2", 1, scale = "x"), "x2", 1, "x")
  test_prop(prop("fill", 1, scale = "foo"), "fill", 1, "foo")

  # Scaled, variable
  test_prop(prop("x", quote(cyl)), "x", quote(cyl), "x")   # Use default scale
  test_prop(prop("x2", ~cyl), "x2", quote(cyl), "x")
  test_prop(prop("x", quote(cyl), scale = "x"), "x", quote(cyl), "x")
  test_prop(prop("x", ~cyl, scale = "foo"), "x", quote(cyl), "foo")

  # Event is used
  test_prop(prop("x", 1, event = "update"), "x", 1, NULL, "update")
  test_prop(prop("x", 1, event = "enter"), "x", 1, NULL, "enter")

  # key doesn't get an event
  test_prop(prop("key", ~id, scale = FALSE), "key", quote(id), NULL, NULL)
  expect_error(prop("key", ~id, event = "update"))
  # scale must be FALSE
  expect_error(prop("key", ~id))
  expect_error(prop("key", ~id, scale = TRUE))
  # can't be constant
  expect_error(prop("key", 1:10))
})

test_that("property names for variables", {
  # pname(wt) is equivalent to prop_label(prop(quote(wt)))
  pname <- function(x) prop_label(prop("x", substitute(x)))

  # Regular variable names are simply converted to string
  expect_identical(pname(wt), "wt")

  # Variable names with special characters are made vega-safe
  expect_identical(pname(`wt/mpg`), "wt/mpg")
  expect_identical(pname(`wt.mpg`), "wt.mpg")

  # safe_vega_var escapes "."
  expect_identical(safe_vega_var("wt.mpg"), "wt\\.mpg")

  # Expressions are kept as is
  expect_identical(pname(wt/mpg), "wt/mpg")

  # Expressions and weird column names will produce same name
  # (but this is very unlikely)
  expect_true(pname(wt/mpg) == pname(`wt/mpg`))

  # Constants have no variable name
  expect_identical(pname(10), "")
  expect_identical(pname("wt"), "")
})

test_that("property values for variables", {
  # Create a data frame with columns 'a', 'b', and 'b-a'
  dat <- data.frame(a = 1:3, b = 4:6, c = 7:9)
  names(dat)[3] <- "b-a"

  # Column named `b-a`
  expect_identical(prop_value(prop("x", quote(`b-a`)), dat), 7:9)

  # Expression b-a
  expect_equal(prop_value(prop("x", quote(b-a)), dat), c(3, 3, 3))

  # Constant prop
  expect_equal(prop_value(prop("x", 10), dat), rep(10, 3))

  # Expression which returns a single value; value should be replicated
  expect_equal(prop_value(prop("x", quote(min(a))), dat), c(1, 1, 1))

  # Expression which returns # of values that doesn't divide into # of rows
  expect_error(prop_value(prop("x", quote(range(a))), dat))
})

test_that("prop captures environment for evaluation", {
  dat <- data.frame(a = 1:2, b = 3:4, c = 5:6)
  b <- 11:12
  d <- 13:14

  # Column from data
  expect_identical(prop_value(prop("x", quote(a)), dat), 1:2)

  # Column from data, even though a var in the calling env has same name
  expect_identical(prop_value(prop("x", quote(b)), dat), 3:4)

  # Variable in calling environment
  expect_identical(prop_value(prop("x", quote(d)), dat), 13:14)

  # Column from data with variable in calling environment
  expect_identical(prop_value(prop("x", quote(b+d)), dat), 3:4 + 13:14)

  # Column from data with variable in calling environment
  expect_identical(prop_value(prop("x", quote(b+d)), dat), 3:4 + 13:14)


  # Create new environment for storing variables
  env <- new.env()
  env$d <- 23:24

  # Variable in env
  expect_identical(prop_value(prop("x", quote(d), env = env), dat), 23:24)
  # Column from data, even though a var in env has same name
  expect_identical(prop_value(prop("x", quote(b), env = env), dat), 3:4)
})

test_that("props() creates correct prop objects", {
  # 6 combinations of := and constant/variable/reactive
  p <- props(x := 1, y := ~mpg, x2 := reactive(cyl),
             stroke = "foo", fill = ~wt, size = reactive(am))
  expect_equal(sort(names(p)),
    sort(c("x.update", "y.update", "x2.update",
           "stroke.update", "fill.update", "size.update"))
  )

  test_prop(p$x.update, "x", 1, NULL)
  test_prop(p$y.update, "y", quote(mpg), NULL)
  # Reactives are hard to test with identical, so test the components
  expect_identical(p$x2.update$property, "x2")
  expect_true(is.reactive(p$x2.update$value))
  expect_identical(p$x2.update$scale, NULL)

  test_prop(p$stroke.update, "stroke", "foo", "stroke")
  test_prop(p$fill.update, "fill", quote(wt), "fill")
  expect_identical(p$size.update$property, "size")
  expect_true(is.reactive(p$size.update$value))
  expect_identical(p$size.update$scale, "size")


  # Properties get default scales (e.g. x2 gets scale x)
  p <- props(x2 = ~cyl)
  test_prop(p$x2.update, "x2", quote(cyl), "x")

  # Explicitly-created prop objects
  p <- props(prop("fill", "foo"), prop("x2", ~cyl))
  test_prop(p$fill.update, "fill", "foo", NULL)
  test_prop(p$x2.update, "x2", quote(cyl), "x")

  # Unnamed arguments
  expect_error(props(~wt, ~mpg, 1))  # Too many
  p <- props(~wt, 1)
  test_prop(p$x.update, "x", quote(wt), "x")
  test_prop(p$y.update, "y", 1, "y")

  # Unnamed arguments, out of order
  p <- props(~mpg, prop("fill", ~cyl), x = ~wt)
  test_prop(p$x.update, "x", quote(wt), "x")
  test_prop(p$y.update, "y", quote(mpg), "y")
  test_prop(p$fill.update, "fill", quote(cyl), "fill")

  # Prop objects, stored in variables
  p1 <- prop("fill", ~cyl)
  p2 <- 1
  p <- props(p1, p2)
  test_prop(p$x.update, "x", 1, "x")
  test_prop(p$fill.update, "fill", quote(cyl), "fill")

  # Prop objects with event (.enter, .exit, .update, .hover)
  p <- props(x.exit := 1, prop("fill", ~cyl, event = "enter"))
  test_prop(p$x.exit, "x", 1, NULL, "exit")
  test_prop(p$fill.enter, "fill", quote(cyl), "fill", "enter")

  # x.enter doesn't block automatic naming for x.update
  p <- props(x.enter := 1, ~mpg)
  test_prop(p$x.enter, "x", 1, NULL, "enter")
  test_prop(p$x.update, "x", quote(mpg), "x", "update")

  # key is special
  p <- props(key := ~id)
  test_prop(p$key, "key", quote(id), NULL, NULL)
  expect_error(props(key = ~id)) # Must be unscaled
  expect_error(props(key := 1:10)) # Can't be constant
})

test_that("props evaluates arguments in correct environment", {
  p <- 1
  f <- function() {
    p <- 2
    props(p)
  }
  expect_identical(f()$x.update$value, 2)
})

test_that("props uses environment in formulas", {
  dat <- data.frame(a = 1:2, b = 3:4)
  val <- 11:12

  # Create a formula that captures the function environment
  gen_formula <- function() {
    val <- 21:22
    ~ val
  }

  p <- props(w = ~val, x := gen_formula(), y := 5, z := 6)

  # Should get val from this environment, where w~val was defined
  expect_identical(prop_value(p$w, dat), 11:12)

  # Should get val from gen_formula environment, where x~val was defined
  expect_identical(prop_value(p$x, dat), 21:22)
})

test_that("merging props", {
  # Utility function: sort props by name
  sortp <- function(p) p[sort(names(p))]

  p_i  <- blank_envs(props(x=~a, z:="red"))
  q_i  <- blank_envs(props(y=~b, z:="blue"))
  q_ni <- blank_envs(props(y=~b, z:="blue", inherit=FALSE))

  expect_equal(sortp(merge_props(p_i, q_i)), blank_envs(props(x=~a, y=~b, z:="blue")))
  expect_equal(sortp(merge_props(p_i, q_ni)), q_ni)
  expect_equal(
    sortp(merge_props(q_ni, p_i)),
    blank_envs(props(x=~a, y=~b, z:="red", inherit = FALSE))
  )
})

test_that("prop_event_sets splits up props properly", {
  p <- props(x.update = ~wt, x.enter = 1, y = ~mpg, fill.hover := "red")
  ps <- prop_event_sets(p)

  expect_equal(names(ps), c("enter", "update", "hover"))
  expect_equal(names(ps$enter), "x")
  expect_equal(names(ps$update), c("x", "y"))
  expect_equal(names(ps$hover), "fill")

  expect_identical(ps$enter$x, prop("x", 1, scale = TRUE, event = "enter"))
  expect_identical(ps$update$x, prop("x", quote(wt), event = "update"))
  expect_identical(ps$update$y, prop("y", quote(mpg), event = "update"))
  expect_identical(ps$hover$fill, prop("fill", "red", event = "hover"))

  expect_true(attr(ps$enter, "inherit"))
  expect_true(attr(ps$update, "inherit"))
  expect_true(attr(ps$hover, "inherit"))

  # value of inherit is passed along
  p <- props(x = ~wt, x.enter = 1, inherit = FALSE)
  ps <- prop_event_sets(p)
  expect_false(attr(ps$enter, "inherit"))
  expect_false(attr(ps$update, "inherit"))
})

test_that("drop_props", {
  p <- props(x = ~wt, x.enter = 0, stroke.enter := "black", stroke.hover := "red")

  expect_identical(
    drop_props(p, c("stroke", "strokeOpacity")),
    props(x = ~wt, x.enter = 0)
  )

  # Use unname() because it drop_props returns an named empty list as opposed
  # to an unnamed empty list.
  expect_identical(
    unname(drop_props(p, c("x", "stroke"))),
    props()
  )
})

test_that("band() is created properly", {
  # Automatic setting of scale, event
  test_prop(prop("width", band()), "width", NULL, "x", "update")
  test_prop(prop("height", band()), "height", NULL, "y", "update")

  # Explicit settings of scale, event
  test_prop(prop("width", band(), scale = "x"), "width", NULL, "x")
  test_prop(prop("width", band(), event = "enter"), "width", NULL, "x", "enter")
  test_prop(prop("width", band(), scale = "foo"), "width", NULL, "foo")

  # Create with props()
  test_prop(props(width = band())$width.update, "width", NULL, "x", "update")
  test_prop(props(height.enter = band())$height.enter, "height", NULL, "y", "enter")

  # Error if property isn't width or height
  expect_error(prop("x", band()))
  expect_error(props(x = band()))
})

#' Create a new "mark" object.
#'
#' A mark object is a close mapping to a vega mark object. Vega marks
#' are documented in \url{https://vega.github.io/vega/docs/marks/}.
#'
#' This function is designed to be used by authors of new types of mark.
#'
#' @param type A string with the vega type.
#' @param props A list of properties, created by \code{\link{props}}.
#' @param data A reactive data object.
#' @keywords internal
#' @export
mark <- function(type, props, data) {
  if (!is.ggvis_props(props)) stop("props must be a ggvis_props object")
  if (is.null(data)) stop("No data supplied to mark.", call. = FALSE)
  if (!is.function(data)) stop("data object must be a reactive or a function.")

  # Check that names are correct
  check_mark_props(type, names(props))

  # Some marks need more detailed validity checks of their props
  check_valid_props <- mark_props_validity_checks[[type]]
  if (!is.null(check_valid_props)) {
    check_valid_props(props)
  }

  # Merge in defaults
  props <- merge_props(default_props(type), props, inherit = TRUE)

  # FIXME: check that the variables in the prop can be found in data
  structure(list(type = type, data = data, props = props), class = "mark")
}

mark_group <- function(props, data, marks = list(), scales = list(),
                       axes = list(), legends = list()) {

  check_mark_props("rect", names(props))
  structure(
    list(
      type = "group",
      data = data,
      props = props,
      marks = marks,
      scales = scales,
      axes = axes,
      legends = legends
    ),
    class = c("mark_group", "mark")
  )
}

#' @rdname mark
#' @export
is.mark <- function(x) inherits(x, "mark")

is.mark_group <- function(x) inherits(x, "mark_group")

check_mark_props <- function(type, props) {
  props <- trim_prop_event(props)
  valid <- valid_props[[type]]

  invalid <- setdiff(props, valid)
  if (length(invalid) == 0) return(invisible(TRUE))

  ldist <- utils::adist(invalid, valid, ignore.case = TRUE, partial = FALSE,
    costs = c(ins = 0.5, sub = 1, del = 2))

  closest <- apply(ldist, 1, min)
  possible_match <- closest < 5
  if (any(possible_match)) {
    best <- apply(ldist, 1, which.min)

    matches <- valid[best][possible_match]
    suggest <- paste0("Did you mean: ", paste0(matches, collapse = ", "), "?")
  } else {
    suggest <- ""
  }

  stop("Unknown properties: ", paste0(invalid, collapse = ", "), ".\n", suggest,
    call. = FALSE)
}

#' @export
format.mark <- function(x, ...) {
  paste0("<", class(x)[1], ">",
    " (Data: ", data_id(x$data), ")",
    "\n",
    indent(format(x$props), 2)
  )
}

#' @export
print.mark <- function(x, ...) cat(format(x), "\n")

context("Mark: properties")

test_that("check_mark_props returns true if all ok", {
  expect_true(check_mark_props("symbol", c("x", "y", "stroke")))
  expect_true(check_mark_props("rect", c("x", "x2", "y", "y2")))
})

test_that("check_mark_props returns helpful suggestion for single incorrect", {
  expect_error(check_mark_props("symbol", "Stroke"), "stroke")
  expect_error(check_mark_props("symbol", "strke"), "stroke")
})

test_that("check_mark_props doesn't give suggestion if really wrong", {
  expect_error(check_mark_props("symbol", "asdfasdfdsa"), "Unknown properties")
})


test_that("check_mark_props returns helpful suggestion for mixture", {
  expect_error(check_mark_props("symbol", c("x", "Stroke")), "stroke")
  expect_error(check_mark_props("symbol", c("x", "strke")), "stroke")
})

#!/bin/bash

set -e

cd www/lib

cd vega
curl -O https://raw.github.com/trifacta/vega/master/vega.js
curl -O https://raw.github.com/trifacta/vega/master/vega.min.js

cd ../d3
curl -O https://raw.github.com/mbostock/d3/master/d3.js
curl -O https://raw.github.com/mbostock/d3/master/d3.min.js

cd ../jquery
curl http://code.jquery.com/jquery-1.11.0.min.js > jquery.min.js
curl http://code.jquery.com/jquery-1.11.0.js > jquery.js

cd ../../..

---
title: "Using ggvis with knitr and rmarkdown"
output:
  html_document:
    fig_width: 4
    fig_height: 2
---

```{r echo=FALSE}
# Set up default dimensions. Width and height are multiplied by dpi to get
# pixel dimensions.
knitr::opts_chunk$set(fig.width = 4, fig.height = 3)
```

To embed a ggvis plot just call `ggvis`:

```{r, message = FALSE}
library(ggvis)
mtcars %>% ggvis(x = ~wt, y = ~mpg) %>% layer_points()
```


```{r, fig.width = 8, fig.height = 6}
mtcars %>% ggvis(x = ~wt, y = ~mpg) %>%
  layer_points() %>%
  layer_smooths()
```

---
title: "Brushing"
output: html_document
runtime: shiny
---
```{r, message = FALSE, echo = FALSE}
library(ggvis)
library(shiny)

set.seed(1233)
cocaine <- cocaine[sample(1:nrow(cocaine), 500), ]
```

Two linked plots. Brushing in the scatter plot will update the histogram.

```{r fig.width = 4, fig.height = 3, echo = FALSE, results = "hold"}
cocaine$id <- seq_len(nrow(cocaine))

lb <- linked_brush(keys = cocaine$id, "red")

cocaine %>%
  ggvis(~weight, ~price, key := ~id) %>%
  layer_points(fill := lb$fill, fill.brush := "red", opacity := 0.3) %>%
  lb$input()

# A subset of cocaine, of only the selected points
selected <- lb$selected
cocaine_selected <- reactive({
  cocaine[selected(), ]
})

cocaine %>%
  ggvis(~potency) %>%
  layer_histograms(width = 5, boundary = 0) %>%
  add_data(cocaine_selected) %>%
  layer_histograms(width = 5, boundary = 0, fill := "#dd3333")
```

A summary of the selected points:

```{r, echo = FALSE}
renderPrint(
  summary(cocaine_selected())
)
```

---
title: "Zooming"
output: html_document
runtime: shiny
---

First we load ggvis and shiny:

```{r, message = FALSE}
options(width = 200)
library(ggvis)
library(shiny)
```

Then we create a linked brush object. This needs to know the possible linking keys. Here we're going to link together some plots of the `mtcars` dataset, so we'll add an `id` column, and use it as the keys:

```{r}
# Add ID column to mtcars so that we can select from it
mtcars2 <- cbind(mtcars, id = seq_len(nrow(mtcars)))

lb <- linked_brush(keys = mtcars2$id, "red")
```

Next we create two plots. Points that are brushed on the first plot will be displayed on the second plot, and the scales will automatically adjust so that those points fill the plot.

```{r fig.width = 3, fig.height = 3, results = "hold"}
mtcars2 %>%
  ggvis(~disp, ~mpg, key := ~id) %>%
  layer_points(fill := lb$fill, fill.brush := "red") %>%
  function(vis) lb$input(vis)

# A subset of mtcars2, of only the selected points, or the whole data if nothing
# is selected.
selected <- lb$selected
mtcars2_selected <- reactive({
  if (!any(selected())) return(mtcars2)
  mtcars2[selected(), ]
})

mtcars2_selected %>%
  ggvis(~disp, ~mpg) %>%
  layer_points(key := ~id) %>%
  set_options(duration = 100)
```

Also, we can show a table of the data, including a column indicating which ones are selected:

```{r}
renderTable(
  cbind(mtcars2, selected = selected())
)
```

---
title: "Using ggvis with knitr and rmarkdown"
author: Winston Chang
output:
  ioslides_presentation:
    widescreen: true
    fig_width: 6
    fig_height: 4
---

## First Plot

```{r, message = FALSE}
library(ggvis)
mtcars %>% ggvis(x = ~wt, y = ~mpg) %>% layer_points()
```

## Second Plot

```{r}
mtcars %>% ggvis(x = ~wt, y = ~mpg) %>%
  layer_points() %>%
  layer_smooths()
```

---
title: "ggvis in an interactive document"
output: html_document
runtime: shiny
---

```{r echo=FALSE, message = FALSE}
library(knitr)
library(ggvis)
library(shiny)
library(dplyr)
```

An interactive plot:

```{r, message = FALSE, fig.width = 6, fig.height = 4}
cocaine %>%
  ggvis(x = ~potency) %>%
  layer_histograms(width = input_slider(1, 20, value = 5))
```


An interactive table:

```{r, fig.width = 8, fig.height = 6}
cocaine %>%
  group_by(state) %>%
  summarise(potency = mean(potency), weight = mean(weight)) %>%
  renderDataTable()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{set_options}
\alias{set_options}
\title{Set options for a ggvis plot}
\usage{
set_options(
  vis,
  width = NULL,
  height = NULL,
  keep_aspect = NULL,
  resizable = NULL,
  padding = NULL,
  duration = NULL,
  renderer = NULL,
  hover_duration = NULL
)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{width, height}{Width and height of plot, in pixels. Default is 600x400.
\code{width} or \code{height} can also be \code{"auto"}, in which case the
plot will size to fit in the containing div. This is useful only in a Shiny
app or custom HTML output. Note that \code{height="auto"} should only be
used when the plot is placed within a div that has a fixed height; if not,
automatic height will not work, due to the way that web browsers do
vertical layout.}

\item{keep_aspect}{Should the aspect ratio be preserved? The default value is
\code{FALSE}, or the value of \code{getOption("ggvis.keep_aspect")}, if it
is set.}

\item{resizable}{If TRUE, allow the user to resize the plot. The default
value is \code{TRUE}, or the value of \code{getOption("ggvis.resizable")},
if it is set. Not compatible when \code{width} or \code{height} is
\code{"auto"}.}

\item{padding}{A padding object specifying padding on the top, right, left,
and bottom. See \code{\link{padding}}.}

\item{duration}{Duration of transitions, in milliseconds.}

\item{renderer}{The renderer to use in the browser. Can be \code{"canvas"} or
\code{"svg"} (the default).}

\item{hover_duration}{The amount of time for hover transitions, in
milliseconds.}
}
\description{
Set options for a ggvis plot
}
\examples{
mtcars \%>\%
  ggvis(~wt, ~mpg) \%>\%
  layer_points() \%>\%
  set_options(width = 300, height = 200, padding = padding(10, 10, 10, 10))

# Display the default options
str(default_options())

}
\seealso{
\code{\link{getOption}} and \code{\link{options}}, for getting and
  setting global options.

\code{\link{default_options}} to see the default options.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_seq.R
\name{fullseq}
\alias{fullseq}
\title{Generate sequence of fixed size intervals covering range.}
\usage{
fullseq(range, size, ...)
}
\arguments{
\item{range}{range}

\item{size}{interval size}

\item{...}{other arguments passed on to methods}
}
\description{
Generate sequence of fixed size intervals covering range.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr.R
\name{group_by}
\alias{group_by}
\title{Divide data into groups.}
\arguments{
\item{x}{a visualisation}

\item{...}{variables to group by.}

\item{add}{By default, when \code{add = FALSE}, \code{group_by} will
override existing groups. To instead add to the existing groups,
use \code{add = TRUE}}
}
\description{
Divide data into groups.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explain.R
\name{explain}
\alias{explain}
\title{Explain details of an object}
\description{
This is a generic function which gives more details about an object than
print, and is more focussed on human readable output than str.
}
\examples{
p <- mtcars \%>\% ggvis(x = ~cyl) \%>\% layer_bars()
explain(p)
}
\seealso{
\code{dplyr::\link[dplyr]{explain}} for more information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr.R
\name{dplyr-ggvis}
\alias{dplyr-ggvis}
\alias{groups.ggvis}
\alias{group_by.ggvis}
\alias{ungroup.ggvis}
\alias{summarise.ggvis}
\alias{mutate.ggvis}
\alias{arrange.ggvis}
\alias{select.ggvis}
\alias{filter.ggvis}
\alias{distinct.ggvis}
\alias{slice.ggvis}
\alias{rename.ggvis}
\alias{transmute.ggvis}
\alias{groups.reactive}
\alias{ungroup.reactive}
\alias{group_by.reactive}
\alias{summarise.reactive}
\alias{mutate.reactive}
\alias{arrange.reactive}
\alias{select.reactive}
\alias{filter.reactive}
\alias{distinct.reactive}
\alias{slice.reactive}
\alias{rename.reactive}
\alias{transmute.reactive}
\title{Dplyr verbs for ggvis.}
\usage{
\method{groups}{ggvis}(x)

\method{group_by}{ggvis}(.data, ..., .add = FALSE)

\method{ungroup}{ggvis}(x, ...)

\method{summarise}{ggvis}(.data, ...)

\method{mutate}{ggvis}(.data, ...)

\method{arrange}{ggvis}(.data, ...)

\method{select}{ggvis}(.data, ...)

filter.ggvis(.data, ...)

\method{distinct}{ggvis}(.data, ...)

\method{slice}{ggvis}(.data, ...)

\method{rename}{ggvis}(.data, ...)

\method{transmute}{ggvis}(.data, ...)

\method{groups}{reactive}(x)

\method{ungroup}{reactive}(x, ...)

\method{group_by}{reactive}(.data, ..., add = FALSE)

\method{summarise}{reactive}(.data, ...)

\method{mutate}{reactive}(.data, ...)

\method{arrange}{reactive}(.data, ...)

\method{select}{reactive}(.data, ...)

filter.reactive(.data, ...)

\method{distinct}{reactive}(.data, ...)

\method{slice}{reactive}(.data, ...)

\method{rename}{reactive}(.data, ...)

\method{transmute}{reactive}(.data, ...)
}
\description{
Applying a dplyr verb to a ggvis object creates a reactive transformation:
whenever the underlying data changes the transformation will be recomputed.
}
\section{Non-standard evaluation}{

Both dplyr and shiny do non-standard evaluation, so to help each package
figure out when it should evaluate its code, reactive components in
these functions must be wrapped in \code{eval()}.
}

\examples{
library(dplyr)
base <- mtcars \%>\% ggvis(~mpg, ~cyl) \%>\% layer_points()
base \%>\% group_by(cyl) \%>\% summarise(mpg = mean(mpg)) \%>\%
  layer_points(fill := "red", size := 100)

base \%>\% filter(mpg > 25) \%>\% layer_points(fill := "red")

base \%>\% mutate(cyl = jitter(cyl)) \%>\% layer_points(fill := "red")

\dontrun{
# Dynamically restrict range using filter
mtcars \%>\% ggvis(~disp, ~mpg) \%>\%
   filter(cyl > eval(input_slider(0, 10))) \%>\%
   layer_points()

# Dynamically compute box-cox transformation with mutate
bc <- function(x, lambda) {
  if (abs(lambda) < 1e-6) log(x) else (x ^ lambda - 1) / lambda
}
bc_slider <- input_slider(-2, 2, 1, step = 0.1)
mtcars \%>\%
 ggvis(~disp, ~mpg) \%>\%
 mutate(disp = bc(disp, eval(bc_slider))) \%>\%
 layer_points()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.R
\name{prop_domain}
\alias{prop_domain}
\title{Property domain.}
\usage{
prop_domain(x, data)
}
\arguments{
\item{x}{property to dispatch on}

\item{data}{name of data set}
}
\description{
Property domain.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_axis.R
\name{add_axis}
\alias{add_axis}
\alias{hide_axis}
\title{Add a vega axis specification to a ggvis plot}
\usage{
add_axis(
  vis,
  type,
  scale = NULL,
  orient = NULL,
  title = NULL,
  title_offset = NULL,
  format = NULL,
  ticks = NULL,
  values = NULL,
  subdivide = NULL,
  tick_padding = NULL,
  tick_size_major = NULL,
  tick_size_minor = tick_size_major,
  tick_size_end = tick_size_major,
  offset = NULL,
  layer = "back",
  grid = TRUE,
  properties = NULL
)

hide_axis(vis, scale)
}
\arguments{
\item{vis}{A ggvis object.}

\item{type}{The type of axis. Either x or y.}

\item{scale}{The name of the scale backing the axis component. Defaults to
the scale type - you will need to specify if you want (e.g.) a scale
for a secondary y-axis.}

\item{orient}{The orientation of the axis. One of top, bottom, left or right.
The orientation can be used to further specialize the axis type (e.g., a y
axis oriented for the right edge of the chart) - defaults to bottom for
x axes, and left for y axes.}

\item{title}{A title for the axis. By default, it uses the name of the field
in the first data set used by the scale. Use \code{""} to suppress the
title.}

\item{title_offset}{The offset (in pixels) from the axis at which to place
the title.}

\item{format}{The formatting pattern for axis labels. Vega uses D3's format
pattern.}

\item{ticks}{A desired number of ticks. The resulting number may be different
so that values are "nice" (multiples of 2, 5, 10) and lie within the
underlying scale's range.}

\item{values}{Explicitly set the visible axis tick values.}

\item{subdivide}{If provided, sets the number of minor ticks between major
ticks (the value 9 results in decimal subdivision).}

\item{tick_padding}{The padding, in pixels, between ticks and text labels.}

\item{tick_size_major, tick_size_minor, tick_size_end}{The size, in pixels, of major, minor and end ticks.}

\item{offset}{The offset, in pixels, by which to displace the axis from the
edge of the enclosing group or data rectangle.}

\item{layer}{A string indicating if the axis (and any gridlines) should be
placed above or below the data marks. One of "front" or "back" (default).}

\item{grid}{A flag indicating if gridlines should be created in addition to
ticks.}

\item{properties}{Optional mark property definitions for custom axis styling.
Should be an object created by \code{\link{axis_props}}, with properties
for ticks, majorTicks, minorTicks, grid, labels, title, and axis.}
}
\description{
Axis specifications allow you to either override the default axes,
or additional axes.
}
\details{
More information about axes can be found in the "axes and legends" vignettes.
}
\section{Compared to ggplot2}{


In ggplot2, axis (and legend) properties are part of the scales
specification. In vega, they are separate, which allows the specification
of multiple axes, and more flexible linkage between scales and axes.
}

\examples{
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  add_axis("x", title = "Weight", orient = "top")

# Suppress axis with hide_axis
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  hide_axis("x") \%>\% hide_axis("y")

mtcars \%>\% ggvis(x = ~wt, y = ~mpg) \%>\% layer_points() \%>\%
  add_axis("x", title = "Weight", ticks = 40,
    properties = axis_props(
      ticks = list(stroke = "red"),
      majorTicks = list(strokeWidth = 2),
      grid = list(stroke = "red"),
      labels = list(
        fill = "steelblue",
        angle = 50,
        fontSize = 14,
        align = "left",
        baseline = "middle",
        dx = 3
      ),
      title = list(fontSize = 16),
      axis = list(stroke = "#333", strokeWidth = 1.5)
    )
  )
}
\seealso{
Vega axis documentation:
  \url{https://vega.github.io/vega/docs/axes/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe graphics}
\arguments{
\item{lhs, rhs}{A visualisation and a function to apply to it}
}
\description{
Like dplyr, ggvis also uses the pipe function, \code{\%>\%} to turn
function composition into a series of imperative statements.
}
\examples{
# Instead of
layer_points(ggvis(mtcars, ~mpg, ~wt))
# you can write
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer.R
\name{layer_f}
\alias{layer_f}
\title{Create a new layering function.}
\usage{
layer_f(vis, fun)
}
\arguments{
\item{vis}{The ggvis visualisation to modify.}

\item{fun}{A function that takes a single argument, the current
visualisation as input, and returns a modified visualisation.}
}
\description{
The layer function is run, and then the state before the code was run
is restored - this allows layers to be effectively isolated from
the rest of the plot.
}
\examples{
mtcars \%>\% ggvis(~mpg) \%>\%
  layer_f(function(v) {
     v \%>\% compute_bin(~mpg) \%>\% layer_points(x = ~x_, y = ~count_)
  }) \%>\%
  layer_points(y = ~wt)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_tabulate.R
\name{compute_tabulate}
\alias{compute_tabulate}
\title{Count data at each location of a categorical variable}
\usage{
compute_tabulate(x, x_var, w_var = NULL)
}
\arguments{
\item{x}{Dataset-like object to count. Built-in methods for data frames,
grouped data frames and ggvis visualisations.}

\item{x_var, w_var}{Names of x and weight variables.}
}
\value{
A data frame with columns:
 \item{count_}{the number of points}
 \item{x_}{value of bin}
}
\description{
Count data at each location of a categorical variable
}
\examples{
library(dplyr)
# The tabulated column must be countable (not numeric)
\dontrun{mtcars \%>\% compute_tabulate(~cyl)}
mtcars \%>\% mutate(cyl = factor(cyl)) \%>\% compute_tabulate(~cyl)

# Or equivalently:
mtcars \%>\% compute_tabulate(~factor(cyl))

# If there's one weight value at each x, it effectively just renames columns.
pressure \%>\% compute_tabulate(~factor(temperature), ~pressure)

# It doesn't matter whether you transform inside or outside of a vis
mtcars \%>\% compute_tabulate(~factor(cyl)) \%>\%
  ggvis(x = ~x_, y = ~count_, y2 = 0) \%>\%
  layer_rects(width = band())

mtcars \%>\%
  ggvis(x = ~x_, y = ~count_, y2 = 0) \%>\%
  compute_tabulate(~factor(cyl)) \%>\%
  layer_rects(width = band())

# compute_tabulate is used automatically in layer_bars when no y prop
# is supplied.
mtcars \%>\% ggvis(x = ~factor(cyl)) \%>\% layer_bars()
}
\seealso{
\code{\link{compute_bin}} For counting cases within ranges of
  a continuous variable.

\code{\link{compute_count}} For counting cases at specific locations
  of a continuous variable. This is useful when the variable is continuous
  but the data is granular.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{add_data}
\alias{add_data}
\title{Add dataset to a visualisation}
\usage{
add_data(vis, data, name = deparse2(substitute(data)), add_suffix = TRUE)
}
\arguments{
\item{vis}{Visualisation to modify.}

\item{data}{Data set to add.}

\item{name}{Data of data - optional, but helps produce informative
error messages.}

\item{add_suffix}{Should a unique suffix be added to the data object's ID?
This should only be FALSE when the spec requires a data set with a
 specific name.}
}
\description{
Add dataset to a visualisation
}
\examples{
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points()
NULL \%>\% ggvis(~mpg, ~wt) \%>\% add_data(mtcars) \%>\% layer_points()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_legend.R
\name{add_guide_legend}
\alias{add_guide_legend}
\title{Defunct function for adding a legend}
\usage{
add_guide_legend(...)
}
\arguments{
\item{...}{Other arguments.}
}
\description{
This function has been replaced with \code{\link{add_legend}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handle_click.R
\name{handle_click}
\alias{handle_click}
\alias{handle_hover}
\title{Handle mouse actions on marks.}
\usage{
handle_click(vis, on_click = NULL)

handle_hover(vis, on_mouse_over = NULL, on_mouse_out = NULL)
}
\arguments{
\item{vis}{Visualisation to listen to.}

\item{on_click, on_mouse_over}{Callback function with arguments:
\describe{
 \item{data}{A data frame with one row}
 \item{location}{A named list with components x and y}
 \item{session}{The session, used to communicate with the browser}
}}

\item{on_mouse_out}{Callback function with argument:
\describe{
 \item{session}{The session, used to communicate with the browser}
}}
}
\description{
Handle mouse actions on marks.
}
\examples{
location <- function(location, ...) cat(location$x, "x", location$y, "\n")
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points() \%>\%
  handle_click(location)
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points() \%>\%
  handle_hover(function(...) cat("over\n"), function(...) cat("off\n"))
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points() \%>\%
  handle_hover(function(data, ...) str(data))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_model_predictions.R
\name{layer_model_predictions}
\alias{layer_model_predictions}
\alias{layer_smooths}
\title{Overlay model predictions or a smooth curve.}
\usage{
layer_model_predictions(
  vis,
  ...,
  model,
  formula = NULL,
  model_args = NULL,
  se = FALSE,
  domain = NULL
)

layer_smooths(vis, ..., span = 0.75, se = FALSE)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{...}{Visual properties. Stroke properties control only affect line,
fill properties only affect standard error band.}

\item{model}{Name of the model as a string, e.g. \code{"loess"}, \code{"lm"},
or \code{"MASS::rlm"}. Must be the name of a function that produces a
standard model object with a \code{\link{predict}} method. For
\code{layer_smooth} this is always "loess".}

\item{formula}{Model formula. If not supplied, guessed from the visual
properties, constructing \code{y ~ x}.}

\item{model_args}{A list of additional arguments passed on to the
\code{model} function.}

\item{se}{Also display a point-wise standard error band? Defaults to
\code{FALSE} because interpretation is non-trivial.}

\item{domain}{If \code{NULL} (the default), the domain of the predicted
values will be the same as the domain of the prediction variable in the
data. It can also be a two-element numeric vector specifying the min and
max.}

\item{span}{For \code{layer_smooth}, the span of the loess smoother.}
}
\description{
\code{layer_model_predictions} fits a model to the data and draw it with
\code{layer_paths} and, optionally, \code{layer_ribbons}.
\code{layer_smooths} is a special case of layering model predictions where
the model is a smooth loess curve whose smoothness is controlled by the
\code{span} parameter.
}
\examples{
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_smooths()
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_smooths(se = TRUE)

# Use group by to display multiple smoothes
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% group_by(cyl) \%>\% layer_smooths()

# Control appearance with props
mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  layer_smooths(se = TRUE, stroke := "red", fill := "red", strokeWidth := 5)

# Control the wiggliness with span. Default is 0.75
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points() \%>\%
  layer_smooths(span = 0.2)
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points() \%>\%
  layer_smooths(span = 1)
# Map to an input to modify interactively
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points() \%>\%
  layer_smooths(span = input_slider(0.2, 1))

# Use other modelling functions with layer_model_predictions
mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  layer_points() \%>\%
  layer_model_predictions(model = "lm") \%>\%
  layer_model_predictions(model = "MASS::rlm", stroke := "red")

# Custom domain for predictions
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points() \%>\%
  layer_model_predictions(model = "lm", domain = c(0, 8))
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points() \%>\%
  layer_model_predictions(model = "lm",
    domain = input_slider(0, 10, value = c(1, 4)))

# layer_smooths() is just compute_smooth() + layer_paths()
# Run loess or other model outside of a visualisation to see what variables
# you get.
mtcars \%>\% compute_smooth(mpg ~ wt)
mtcars \%>\% compute_model_prediction(mpg ~ wt, model = "lm")

mtcars \%>\%
  ggvis(~wt, ~mpg) \%>\%
  layer_points() \%>\%
  compute_smooth(mpg ~ wt) \%>\%
  layer_paths(~pred_, ~resp_, strokeWidth := 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_type.R
\name{vega_data_parser}
\alias{vega_data_parser}
\title{Determine the vega data type for a vector}
\usage{
vega_data_parser(x)
}
\arguments{
\item{x}{A vector.}
}
\description{
This is used to specify the data type so that the appropriate parser is used
when Vega receives the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{knit_print.ggvis}
\alias{knit_print.ggvis}
\title{Knit print method for ggvis plots.}
\usage{
knit_print.ggvis(x, options = list(), inline = FALSE, ...)
}
\description{
Knit print method for ggvis plots.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broker.R
\name{is.broker}
\alias{is.broker}
\title{Determine if an object is a broker object}
\usage{
is.broker(x)
}
\arguments{
\item{x}{An object to test.}
}
\description{
Determine if an object is a broker object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{add_relative_scales}
\alias{add_relative_scales}
\title{Add x_rel and y_rel scales}
\usage{
add_relative_scales(vis)
}
\arguments{
\item{vis}{A ggvis object.}
}
\description{
This function adds scales named \code{x_rel} and \code{y_rel}, each of which
has a domain of 0 to 1, and the range is the plot's width or height.
These scales are useful for positioning visual elements relative to the
plotting area. For example, with legends.
}
\seealso{
\code{\link{add_legend}} for a usage example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.ggvis}
\alias{print.ggvis}
\alias{view_static}
\alias{view_dynamic}
\title{View in a ggvis plot in the browser.}
\usage{
\method{print}{ggvis}(x, dynamic = NA, launch = interactive(), ...)

view_static(x, plot_id = rand_id("plot_"), dest = NULL)

view_dynamic(x, plot_id = rand_id("plot_"), port = NULL, quiet = FALSE)
}
\arguments{
\item{x}{A ggvis object.}

\item{dynamic}{Uses \code{view_dynamic} if \code{TRUE}, \code{view_static} if
\code{FALSE}. The default, \code{NA}, chooses automatically based on the
presence of reactives or interactive inputs in \code{x}.}

\item{launch}{If \code{TRUE}, will launch plot in a viewer/browser. If
\code{FALSE} returns an object that you can \code{print()} to launch.}

\item{...}{Other arguments passed on to \code{view_dynamic} and
\code{view_static} ?from \code{print}.}

\item{plot_id}{Unique identifier used to identify the plot on the page.}

\item{dest}{Deprecated (this no longer works).}

\item{port}{the port on which to start the shiny app. If NULL (the default),
Shiny will select a random port.}

\item{quiet}{If \code{TRUE} show status messages from Shiny. (Default is
\code{FALSE}.)}
}
\description{
\code{view_static} creates a static web page in a temporary directory;
\code{view_dynamic} generate a dynamic shiny app and launches it.
\code{print} automatically picks between the two.
}
\details{
If \code{view_static} is used on a ggvis object that has dynamic components,
it will output a static plot.
}
\examples{
## Run these examples only in interactive R sessions
if (interactive()) {
# In most cases view_static is unnecessary; these will do the same thing:
mtcars \%>\% ggvis(~wt, ~mpg)
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% view_static()

# Can find the output file with view_static() and html_print()
outfile <- mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  view_static() \%>\% htmltools::html_print(viewer = NULL)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_bars.R
\name{layer_bars}
\alias{layer_bars}
\title{Display data with bars (a barchart).}
\usage{
layer_bars(vis, ..., stack = TRUE, width = NULL)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{...}{Visual properties used to override defaults.}

\item{stack}{If there are multiple bars to be drawn at an x location, should
the bars be stacked? If FALSE, the bars will be overplotted on each other.}

\item{width}{Width of each bar. When x is continuous, this controls the width
in the same units as x. When x is categorical, this controls the width as a
proportion of the spacing between items (default is 0.9).}
}
\description{
This will add bars to a plot. The exact behavior is complicated because
the term bar chart is used to describe four important variations on a theme.
The action of \code{layer_bars} depends on two factors: whether or not a
\code{y} prop has been specified, and whether the \code{x} props is
continuous or categorical.
}
\section{Visualisations}{


If no y prop has been specified, then this will count the number of entries
at each unique x value. There will be one bar at each unique x value, and
the y value (or height) of each bar will represent the count at that x value.

If a y prop has been specified, then those y values will be used as weights
for a weighted count at each unique x value. If no x values appear more than
once in the data, then the end result is a plot where the height of the bar
at each x value is simply the y value. However, if an x value appear more
than once in the data, then this will sum up the y values at each x.

If the x variable is continuous, then a continuous x axis will be used, and
the width of each bar is by default equal to the resolution of the data --
that is, the smallest difference between any two x values.

If the x variable is categorical, then a categorical x axis will be used. By
default, the width of each bar is 0.9 times the space between the items.
}

\examples{
# Discrete x: bar graph of counts at each x value
cocaine \%>\% ggvis(~state) \%>\% layer_bars()
# Continuous x: bar graph of counts at unique locations
cocaine \%>\% ggvis(~month) \%>\% layer_bars()

# Use y prop to weight by additional variable. This is also useful
# if you have pretabulated data
cocaine \%>\% ggvis(~state, ~weight) \%>\% layer_bars()
cocaine \%>\% ggvis(~month, ~weight) \%>\% layer_bars()

# For continuous x, layer_bars is useful when the variable has a few
# unique values that you want to preserve. If you have many unique
# values and you want to bin, use layer_histogram
cocaine \%>\% ggvis(~price) \%>\% layer_bars()
cocaine \%>\% ggvis(~price) \%>\% layer_histograms(width = 100)

# If you have unique x values, you can use layer_bars() as an alternative
# to layer_points()
pressure \%>\% ggvis(~temperature, ~pressure) \%>\% layer_points()
pressure \%>\% ggvis(~temperature, ~pressure) \%>\% layer_bars()

# When x is continuous, width controls the width in x units
pressure \%>\% ggvis(~temperature, ~pressure) \%>\% layer_bars(width = 10)
# When x is categorical, width is proportional to spacing between bars
pressure \%>\% ggvis(~factor(temperature), ~pressure) \%>\%
  layer_bars(width = 0.5)

# Stacked bars
# If grouping var is continuous, you need to manually specify grouping
ToothGrowth \%>\% group_by(dose) \%>\%
  ggvis(x = ~supp, y = ~len, fill = ~dose) \%>\% layer_bars()
# If grouping var is categorical, grouping is done automatically
cocaine \%>\% ggvis(x = ~state, fill = ~as.factor(month)) \%>\%
  layer_bars()
}
\seealso{
\code{\link{layer_histograms}} For bar graphs of counts at each unique
  x value, in contrast to a histogram's bins along x ranges.

\code{\link{compute_count}} and \code{\link{compute_tabulate}} for
  more information on how data is transformed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/singular.R
\name{singular}
\alias{singular}
\alias{scale_singular}
\title{singular.}
\usage{
singular()

scale_singular(
  vis,
  property,
  name = property,
  label = name,
  points = TRUE,
  domain = NULL,
  override = NULL
)
}
\arguments{
\item{vis}{A ggvis object.}

\item{property}{The name of a property, such as "x", "y", "fill", "stroke", etc.}

\item{name}{Name of the scale, such as "x", "y", "fill", etc. Can also be an
arbitrary name like "foo".}

\item{label}{Label for the scale. Used for axis or legend titles.}

\item{points}{If \code{TRUE} (default), distributes the ordinal values over a
quantitative range at uniformly spaced points. The spacing of the points
can be adjusted using the padding property. If \code{FALSE}, the ordinal
scale will construct evenly-spaced bands, rather than points. Note that
if any mark is added with a \code{\link{band}()} prop, then the scale for
that prop will automatically have \code{points} set to \code{FALSE}.}

\item{domain}{The domain of the scale, representing the set of data values.
For ordinal scales, a character vector; for quantitative scales, a numeric
vector of length two. Either value (but not both) may be NA, in which
case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
can also be a reactive which returns the appropriate type of vector.}

\item{override}{Should the domain specified by this ggvis_scale object
override other ggvis_scale objects for the same scale? Useful when domain is
manually specified. For example, by default, the domain of the scale
will contain the range of the data, but when this is TRUE, the specified
domain will override, and the domain can be smaller than the range of the
data. If \code{FALSE}, the \code{domain} will not behave this way. If
left \code{NULL}, then it will be treated as \code{TRUE} whenever
\code{domain} is non-NULL.}
}
\description{
Use singular when you want constant x or y position.
}
\examples{
mtcars \%>\% ggvis("", ~mpg) \%>\%
  layer_points() \%>\%
  scale_nominal("x") \%>\%
  add_axis("x", title = "", tick_size_major = 0)

# OR
mtcars \%>\% ggvis("", ~mpg) \%>\%
  layer_points() \%>\%
  scale_singular("x")

# OR, even simpler
mtcars \%>\% ggvis(singular(), ~mpg) \%>\% layer_points()

# In the other direction:
mtcars \%>\% ggvis(~mpg, singular()) \%>\% layer_points()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_densities.R
\name{layer_densities}
\alias{layer_densities}
\title{Transformation: density estimate}
\usage{
layer_densities(
  vis,
  ...,
  kernel = "gaussian",
  adjust = 1,
  density_args = list(),
  area = TRUE
)
}
\arguments{
\item{vis}{The visualisation to modify}

\item{...}{Visual properties, passed on to \code{\link{props}}.}

\item{kernel}{Smoothing kernel. See \code{\link{density}} for details.}

\item{adjust}{Multiple the default bandwidth by this amount. Useful for
controlling wiggliness of density.}

\item{density_args}{Other arguments passed on to
\code{\link{compute_density}} and thence to \code{\link{density}}.}

\item{area}{Should there be a shaded region drawn under the curve?}
}
\description{
\code{transform_density} is a data transformation that computes a kernel
density estimate from a dataset. \code{layer_density} combines
\code{transform_density} with \code{mark_path} and \code{mark_area}
to display a smooth line and its standard errror.
}
\examples{
# Basic density estimate
faithful \%>\% ggvis(~waiting) \%>\% layer_densities()
faithful \%>\% ggvis(~waiting) \%>\% layer_densities(area = FALSE)

# Control bandwidth with adjust
faithful \%>\% ggvis(~waiting) \%>\% layer_densities(adjust = .25)
faithful \%>\% ggvis(~waiting) \%>\%
  layer_densities(adjust = input_slider(0.1, 5))

# Control stroke and fill
faithful \%>\% ggvis(~waiting) \%>\%
  layer_densities(stroke := "red", fill := "red")

# With groups
PlantGrowth \%>\% ggvis(~weight, fill = ~group) \%>\% group_by(group) \%>\%
  layer_densities()
PlantGrowth \%>\% ggvis(~weight, stroke = ~group) \%>\% group_by(group) \%>\%
  layer_densities(strokeWidth := 3, area = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/waggle.R
\name{waggle}
\alias{waggle}
\title{Waggle back and forth between two numbers}
\usage{
waggle(min, max, value = (min + max)/2, step = (max - min)/50, fps = 10)
}
\arguments{
\item{min}{A minimum value.}

\item{max}{A maximum value.}

\item{value}{Starting value. Defaults to half-way between \code{min} and
\code{max}.}

\item{step}{How much value changes at each frame. Defaults to 50 steps
between min and max so it takes 5 seconds to waggle once.}

\item{fps}{number of frames per second.}
}
\description{
Waggle back and forth between two numbers
}
\examples{
span <- waggle(0.2, 1)
mtcars \%>\% ggvis(~mpg, ~wt) \%>\%
 layer_points() \%>\%
 layer_smooths(span = span)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{add_props}
\alias{add_props}
\title{Add visual properties to a visualisation}
\usage{
add_props(vis, ..., .props = NULL, inherit = NULL, env = parent.frame())
}
\arguments{
\item{vis}{Visualisation to modify.}

\item{...}{A set of name-value pairs. The name should be a valid vega
  property.

  The first two unnamed components are taken to be \code{x} and \code{y}.
  Any additional unnamed components will raise an error.}

\item{.props}{When calling \code{props} from other functions, you'll
often have a list of quoted function functions. You can pass that function
to the \code{.props} argument instead of messing around with
substitute. In other words, \code{.props} lets you opt out of the
non-standard evaluation that \code{props} does.}

\item{inherit}{If \code{TRUE}, the defaults, will inherit from properties
from the parent layer If \code{FALSE}, it will start from nothing.}

\item{env}{The environment in which to evaluate variable properties.}
}
\description{
Add visual properties to a visualisation
}
\examples{
mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points()
mtcars \%>\% ggvis() \%>\% add_props(~wt, ~mpg) \%>\% layer_points()
mtcars \%>\% ggvis(~wt) \%>\% add_props(y = ~mpg) \%>\% layer_points()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_ordinal}
\alias{scale_ordinal}
\alias{scale_nominal}
\alias{scale_logical}
\title{Add a ordinal, nominal, or logical scale to a ggvis object.}
\usage{
scale_ordinal(
  vis,
  property,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  points = NULL,
  padding = NULL,
  sort = NULL,
  name = property,
  label = NULL,
  override = NULL
)

scale_nominal(
  vis,
  property,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  points = NULL,
  padding = NULL,
  sort = NULL,
  name = property,
  label = NULL,
  override = NULL
)

scale_logical(
  vis,
  property,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  points = NULL,
  padding = NULL,
  sort = NULL,
  name = property,
  label = NULL,
  override = NULL
)
}
\arguments{
\item{vis}{A ggvis object.}

\item{property}{The name of a property, such as "x", "y", "fill", "stroke", etc.}

\item{domain}{The domain of the scale, representing the set of data values.
For ordinal scales, a character vector; for quantitative scales, a numeric
vector of length two. Either value (but not both) may be NA, in which
case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
can also be a reactive which returns the appropriate type of vector.}

\item{range}{The range of the scale, representing the set of visual values.
For numeric values, the range can take the form of a two-element array with
minimum and maximum values. For ordinal data, the range may by an array of
desired output values, which are mapped to elements in the specified
domain. The following range literals are also available: "width", "height",
"shapes", "category10", "category20".}

\item{reverse}{If true, flips the scale range.}

\item{round}{If true, rounds numeric output values to integers. This can be
helpful for snapping to the pixel grid.}

\item{points}{If \code{TRUE} (default), distributes the ordinal values over a
quantitative range at uniformly spaced points. The spacing of the points
can be adjusted using the padding property. If \code{FALSE}, the ordinal
scale will construct evenly-spaced bands, rather than points. Note that
if any mark is added with a \code{\link{band}()} prop, then the scale for
that prop will automatically have \code{points} set to \code{FALSE}.}

\item{padding}{Applies spacing among ordinal elements in the scale range.
The actual effect depends on how the scale is configured. If the points
parameter is true, the padding value is interpreted as a multiple of the
spacing between points. A reasonable value is 1.0, such that the first and
last point will be offset from the minimum and maximum value by half the
distance between points. Otherwise, padding is typically in the range
[0, 1] and corresponds to the fraction of space in the range interval to
allocate to padding. A value of 0.5 means that the range band width will
be equal to the padding width. For positional (x and y) scales, the default
padding is 0.1. For other scales, the default padding is 0.5.}

\item{sort}{If \code{TRUE}, the values in the scale domain will be sorted
according to their natural order. Default is \code{FALSE}.}

\item{name}{Name of the scale, such as "x", "y", "fill", etc. Can also be an
arbitrary name like "foo".}

\item{label}{Label for the scale. Used for axis or legend titles.}

\item{override}{Should the domain specified by this ggvis_scale object
override other ggvis_scale objects for the same scale? Useful when domain is
manually specified. For example, by default, the domain of the scale
will contain the range of the data, but when this is TRUE, the specified
domain will override, and the domain can be smaller than the range of the
data. If \code{FALSE}, the \code{domain} will not behave this way. If
left \code{NULL}, then it will be treated as \code{TRUE} whenever
\code{domain} is non-NULL.}
}
\description{
Ordinal, nominal, and logical scales are all categorical, and are treated
similarly by ggvis.
}
\examples{
p <- PlantGrowth \%>\% ggvis(~group, ~weight) \%>\% layer_points()

p
p \%>\% scale_nominal("x", padding = 0)
p \%>\% scale_nominal("x", padding = 1)

p \%>\% scale_nominal("x", reverse = TRUE)

p <- ToothGrowth \%>\% group_by(supp) \%>\%
  ggvis(~len, fill = ~supp) \%>\%
  layer_histograms(width = 4, stack = TRUE)

# Control range of fill scale
p \%>\% scale_nominal("fill", range = c("pink", "lightblue"))

# There's no default range when the data is categorical but the output range
# is continuous, as in the case of opacity. In these cases, you can
# manually specify the range for the scale.
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, opacity = ~factor(cyl)) \%>\%
  layer_points() \%>\%
  scale_nominal("opacity", range = c(0.2, 1))
}
\seealso{
\code{\link{scales}}, \code{\link{scale_numeric}},
  \url{https://vega.github.io/vega/docs/scales/#ordinal}.

Other scales: 
\code{\link{scale_datetime}()},
\code{\link{scale_numeric}()}
}
\concept{scales}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_layout.R
\name{ggvisControlOutput}
\alias{ggvisControlOutput}
\title{Create a ggvis control output element in UI}
\usage{
ggvisControlOutput(outputId, plotId = NULL)
}
\arguments{
\item{outputId}{The output variable to read the value from.}

\item{plotId}{An optional plot ID or vector of plot IDs. The plots will
have their .onControlOutput functions called after the controls are drawn.}
}
\description{
This is effectively the same as \code{\link[shiny]{uiOutput}}, except that
on the client side it may call some plot resizing functions after new
controls are drawn.
}
\details{
\code{ggvisControlOutput} is intended to be used with
\code{\link{bind_shiny}} on the server side.
}
\examples{
ggvisControlOutput("plot1")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_bin.R
\name{bin_vector}
\alias{bin_vector}
\alias{bin_vector.numeric}
\title{Bin vectors}
\usage{
bin_vector(x, weight = NULL, ...)

\method{bin_vector}{numeric}(
  x,
  weight = NULL,
  ...,
  width = 1,
  origin = 0,
  closed = c("right", "left"),
  pad = FALSE
)
}
\arguments{
\item{x}{A vector to bin}

\item{weight}{If specified, an integer vector of the same length as \code{x}
representing the number of occurances of each value in \code{x}}

\item{...}{additional arguments passed through to methods.}

\item{width}{The width of a bin}

\item{origin}{The left-most value for bins.}

\item{closed}{One of \code{"right"} or \code{"left"} indicating whether
right or left edges of bins are included in the bin.}

\item{pad}{A logical indicating whether the bins should be padded to include
an empty bin on each side.}
}
\description{
A generic and several implementations for binning vectors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.R
\name{new_prop}
\alias{new_prop}
\title{Create new prop object}
\usage{
new_prop(x, property, scale, offset, mult, env, event, label)
}
\description{
The resulting object has the following fields:
}
\details{
\itemize{
  \item property The name of a visual property, like "x", "x2", "width", "y",
    "fill".
  \item value A value. Can be a constant, reactive, or quoted expression.
  \item scale A string with name of a scale. Typically something like "x",
    "y", "fill", but can also be a custom name like "foo".
  \item offset Additive pixel offset used to adjust scaled values.
  \item mult Multiplicative pixel offset used to adjust scaled values.
  \item event A event like "update", "enter", "exit", "hover", "brush".
  \item env An environment in which to evaluate a variable or reactive value.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_boxplots.R
\name{layer_boxplots}
\alias{layer_boxplots}
\title{Display data with a boxplot.}
\usage{
layer_boxplots(vis, ..., coef = 1.5, width = NULL)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{...}{Visual properties used to override defaults.}

\item{coef}{The maximum length of the whiskers as multiple of the
inter-quartile range. Default value is 1.5.}

\item{width}{Width of each bar. When x is continuous, this controls the width
in the same units as x. When x is categorical, this controls the width as a
proportion of the spacing between items (default is 0.9).}
}
\description{
This will add boxplots to a plot. The action of \code{layer_boxplots} depends
on whether the \code{x} prop is continuous or categorical.
}
\details{
The upper and lower "hinges" correspond to the first and third quartiles (the
25th and 75th percentiles). This differs slightly from the method used by the
\code{boxplot} function, and may be apparent with small samples. See
\code{\link{boxplot.stats}} for more information on how hinge positions are
calculated for \code{boxplot}.

The upper whisker extends from the hinge to the highest value that is within
1.5 * IQR of the hinge, where IQR is the inter-quartile range, or distance
between the first and third quartiles. The lower whisker extends from the
hinge to the lowest value within 1.5 * IQR of the hinge. Data beyond the end
of the whiskers are outliers and plotted as points (as specified by Tukey).
}
\examples{
library(dplyr)

mtcars \%>\% ggvis(~factor(cyl), ~mpg) \%>\% layer_boxplots()
# Set the width of the boxes to half the space between tick marks
mtcars \%>\% ggvis(~factor(cyl), ~mpg) \%>\% layer_boxplots(width = 0.5)

# Continuous x: boxes fill width between data values
mtcars \%>\% ggvis(~cyl, ~mpg) \%>\% layer_boxplots()
# Setting width=0.5 makes it 0.5 wide in the data space, which is 1/4 of the
# distance between data values in this particular case.
mtcars \%>\% ggvis(~cyl, ~mpg) \%>\% layer_boxplots(width = 0.5)

# Smaller outlier points
mtcars \%>\% ggvis(~factor(cyl), ~mpg) \%>\% layer_boxplots(size := 20)
}
\seealso{
\code{\link{compute_boxplot}} for more information on how data is
  transformed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{is.ggvis}
\alias{is.ggvis}
\title{Is an object a ggvis object?}
\usage{
is.ggvis(x)
}
\arguments{
\item{x}{an object to test}
}
\description{
Is an object a ggvis object?
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_type.R
\name{vector_type}
\alias{vector_type}
\title{Determine the "type" of a vector}
\usage{
vector_type(x)
}
\arguments{
\item{x}{a vector}
}
\description{
The \code{vector_type} collapses down the class of base vectors into
something useful more for visualisation, yielding one of "datetime",
"numeric", "ordinal", "nominal" or "logical".
}
\seealso{
\code{default_scale}, which uses this when picking the default
  scale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_count.R
\name{compute_count}
\alias{compute_count}
\title{Count data at each location}
\usage{
compute_count(x, x_var, w_var = NULL)
}
\arguments{
\item{x}{Dataset-like object to count. Built-in methods for data frames,
grouped data frames and ggvis visualisations.}

\item{x_var, w_var}{Names of x and weight variables.}
}
\value{
A data frame with columns:
 \item{count_}{the number of points}
 \item{x_}{the x value where the count was made}

The width of each "bin" is set to the resolution of the data -- that is, the
smallest difference between two x values.
}
\description{
Count data at each location
}
\examples{
mtcars \%>\% compute_count(~cyl)

# Weight the counts by car weight value
mtcars \%>\% compute_count(~cyl, ~wt)

# If there's one weight value at each x, it effectively just renames columns.
pressure \%>\% compute_count(~temperature, ~pressure)
# Also get the width of each bin
pressure \%>\% compute_count(~temperature, ~pressure) \%>\% compute_align(~x_)

# It doesn't matter whether you transform inside or outside of a vis
mtcars \%>\% compute_count(~cyl, ~wt) \%>\%
  compute_align(~x_) \%>\%
  ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) \%>\%
  layer_rects()

mtcars \%>\%
  ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) \%>\%
  compute_count(~cyl, ~wt) \%>\%
  compute_align(~x_) \%>\%
  layer_rects()
}
\seealso{
\code{\link{compute_bin}} For counting cases within ranges of
  a continuous variable.

\code{\link{compute_align}} For calculating the "width" of data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_data.R
\name{get_data}
\alias{get_data}
\title{Get data from a ggvis object}
\usage{
get_data(vis)
}
\arguments{
\item{vis}{A ggvis object.}
}
\description{
This function is useful for inspecting the data in a ggvis object.
}
\examples{
p <- cocaine \%>\% ggvis(~price) \%>\% layer_bars()
get_data(p)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_bins.R
\name{layer_histograms}
\alias{layer_histograms}
\alias{layer_freqpolys}
\title{Display binned data}
\usage{
layer_histograms(
  vis,
  ...,
  width = NULL,
  center = NULL,
  boundary = NULL,
  closed = c("right", "left"),
  stack = TRUE,
  binwidth
)

layer_freqpolys(
  vis,
  ...,
  width = NULL,
  center = NULL,
  boundary = NULL,
  closed = c("right", "left"),
  binwidth
)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{...}{Visual properties used to override defaults.}

\item{width}{The width of the bins. The default is \code{NULL}, which yields
30 bins that cover the range of the data. You should always override this
value, exploring multiple widths to find the best to illustrate the stories
in your data.}

\item{center}{The center of one of the bins.  Note that if center is above or
below the range of the data, things will be shifted by an appropriate
number of \code{width}s. To center on integers, for example, use
\code{width=1} and \code{center=0}, even if \code{0} is outside the range
of the data.  At most one of \code{center} and \code{boundary} may be
specified.}

\item{boundary}{A boundary between two bins. As with \code{center}, things
are shifted when \code{boundary} is outside the range of the data. For
example, to center on integers, use \code{width = 1} and \code{boundary =
0.5}, even if \code{1} is outside the range of the data.  At most one of
\code{center} and \code{boundary} may be specified.}

\item{closed}{One of \code{"right"} or \code{"left"} indicating whether right
or left edges of bins are included in the bin.}

\item{stack}{If \code{TRUE}, will automatically stack overlapping bars.}

\item{binwidth}{Deprecated; use \code{width} instead.}
}
\description{
Display binned data
}
\examples{
# Create histograms and frequency polygons with layers
mtcars \%>\% ggvis(~mpg) \%>\% layer_histograms()
mtcars \%>\% ggvis(~mpg) \%>\% layer_histograms(width = 2)
mtcars \%>\% ggvis(~mpg) \%>\% layer_freqpolys(width = 2)

# These are equivalent to combining compute_bin with the corresponding
# mark
mtcars \%>\% compute_bin(~mpg) \%>\% ggvis(~x_, ~count_) \%>\% layer_paths()

# With grouping
mtcars \%>\% ggvis(~mpg, fill = ~factor(cyl)) \%>\% group_by(cyl) \%>\%
  layer_histograms(width = 2)
mtcars \%>\% ggvis(~mpg, stroke = ~factor(cyl)) \%>\% group_by(cyl) \%>\%
  layer_freqpolys(width = 2)
}
\seealso{
\code{\link{layer_bars}} For bar graphs of counts at each unique
  x value, in contrast to a histogram's bins along x ranges.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_props.R
\name{axis_props}
\alias{axis_props}
\title{Create an axis_props object for controlling axis properties.}
\usage{
axis_props(
  ticks = NULL,
  majorTicks = NULL,
  minorTicks = NULL,
  grid = NULL,
  labels = NULL,
  title = NULL,
  axis = NULL
)
}
\arguments{
\item{ticks}{A named list of line properties for ticks.}

\item{majorTicks}{A named list of line properties for major ticks.}

\item{minorTicks}{A named list of line properties for minor ticks.}

\item{grid}{A named list of line properties for grid lines.}

\item{labels}{A named list of text properties for axis labels.}

\item{title}{A named list of text properties for the axis title.}

\item{axis}{A named list of line properties for the axis line.}
}
\description{
The items in each of the lists can be a literal value, like \code{5} or
"blue", or they can be a \code{\link{scaled_value}} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_model_prediction.R
\name{compute_model_prediction}
\alias{compute_model_prediction}
\alias{compute_smooth}
\title{Create a model of a data set and compute predictions.}
\usage{
compute_model_prediction(
  x,
  formula,
  ...,
  model = NULL,
  se = FALSE,
  level = 0.95,
  n = 80L,
  domain = NULL,
  method
)

compute_smooth(x, formula, ..., span = 0.75, se = FALSE)
}
\arguments{
\item{x}{Dataset-like object to model and predict. Built-in methods for data
frames, grouped data frames and ggvis visualisations.}

\item{formula}{Formula passed to modelling function. Can use any variables
from data.}

\item{...}{arguments passed on to \code{model} function}

\item{model}{Model fitting function to use - it must support R's standard
modelling interface, taking a formula and data frame as input, and
returning predictions with \code{\link{predict}}. If not supplied, will use
\code{\link{loess}} for <= 1000 points, otherwise it will use
\code{\link[mgcv]{gam}}. Other modelling functions that will work include
\code{\link{lm}}, \code{\link{glm}} and \code{\link[MASS]{rlm}}.}

\item{se}{include standard errors in output? Requires appropriate method of
\code{predict_grid}, since the interface for returning predictions with
standard errors is not consistent acrossing modelling frameworks.}

\item{level}{the confidence level of the standard errors.}

\item{n}{the number of grid points to use in the prediction}

\item{domain}{If \code{NULL} (the default), the domain of the predicted
values will be the same as the domain of the prediction variable in the
data. It can also be a two-element numeric vector specifying the min and
max.}

\item{method}{Deprecated. Please use \code{model} instead.}

\item{span}{Smoothing span used for loess model.}
}
\value{
A data frame with columns: \item{\code{resp_}}{regularly spaced grid
  of \code{n} locations} \item{\code{pred_}}{predicted value from model}
  \item{\code{pred_lwr_} and \code{pred_upr_}}{upper and lower bounds of
  confidence interval (if \code{se = TRUE})} \item{\code{pred_se_}}{the
  standard error (width of the confidence interval) (if \code{se = TRUE})}
}
\description{
Fit a 1d model, then compute predictions and (optionally) standard errors
over an evenly spaced grid.
}
\details{
\code{compute_model_prediction} fits a model to the data and makes
predictions with it. \code{compute_smooth} is a special case of model
predictions where the model is a smooth loess curve whose smoothness is
controlled by the \code{span} parameter.
}
\examples{
# Use a small value of n for these examples
mtcars \%>\% compute_model_prediction(mpg ~ wt, n = 10)
mtcars \%>\% compute_model_prediction(mpg ~ wt, n = 10, se = TRUE)
mtcars \%>\% group_by(cyl) \%>\% compute_model_prediction(mpg ~ wt, n = 10)

# compute_smooth defaults to loess
mtcars \%>\% compute_smooth(mpg ~ wt)

# Override model to suppress message or change approach
mtcars \%>\% compute_model_prediction(mpg ~ wt, n = 10, model = "loess")
mtcars \%>\% compute_model_prediction(mpg ~ wt, n = 10, model = "lm")

# Set the domain manually
mtcars \%>\%
  compute_model_prediction(mpg ~ wt, n = 20, model = "lm", domain = c(0, 8))

# Plot the results
mtcars \%>\% compute_model_prediction(mpg ~ wt) \%>\%
  ggvis(~pred_, ~resp_) \%>\%
  layer_paths()
mtcars \%>\% ggvis() \%>\%
  compute_model_prediction(mpg ~ wt) \%>\%
  layer_paths(~pred_, ~resp_)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_band.R
\name{band}
\alias{band}
\alias{is.prop_band}
\title{A band}
\usage{
band(offset = NULL, mult = NULL)

is.prop_band(x)
}
\arguments{
\item{offset, mult}{Additive and multiplicate offsets used to adjust the
band size. For example, use \code{mult = 0.9} to make a bar take up
90\% of the space allocated for its category.}

\item{x}{object to test for band-ness}
}
\description{
Bands are used to set the width or height on categorical scales - a band
represent the height or width allocated for one level of a factor.
}
\examples{
df <- data.frame(label = c("a", "b", "c"), n = c(10, 9, 4))

base <- df \%>\% ggvis(~label, y2 = 0, y = ~n)
base \%>\% layer_rects(width = band())
base \%>\% layer_rects(width = band(offset = -1))
base \%>\% layer_rects(width = band(mult = 0.9))

# A nominal scale with padding is more symmetrical than band with a mult
base \%>\% layer_rects(width = band(mult = 0.75))
base \%>\% layer_rects(width = band()) \%>\%
  scale_nominal("x", padding = 0.25, points = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{save_spec}
\alias{save_spec}
\alias{view_spec}
\title{Tools to save and view static specs.}
\usage{
save_spec(x, path, ...)

view_spec(path, ...)
}
\arguments{
\item{x}{a ggvis object}

\item{path}{location to save spec to, or load spec from}

\item{...}{other arguments passed to \code{as.vega}}
}
\description{
These functions are mainly useful for testing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputs.R
\name{input_select}
\alias{input_select}
\alias{input_radiobuttons}
\alias{input_checkboxgroup}
\title{Create interactive control to select one (or more options) from a list.}
\usage{
input_select(
  choices,
  selected = NULL,
  multiple = FALSE,
  label = "",
  id = rand_id("select_"),
  map = identity,
  selectize = FALSE
)

input_radiobuttons(
  choices,
  selected = NULL,
  label = "",
  id = rand_id("radio_"),
  map = identity
)

input_checkboxgroup(
  choices,
  selected = NULL,
  label = "",
  id = rand_id("radio_"),
  map = identity
)
}
\arguments{
\item{choices}{List of values to select from. If elements of the list are
named, then that name --- rather than the value --- is displayed to the
user. It's also possible to group related inputs by providing a named list
whose elements are (either named or unnamed) lists, vectors, or factors. In
this case, the outermost names will be used as the group labels (leveraging
the \verb{<optgroup>} HTML tag) for the elements in the respective sublist. See
the example section for a small demo of this feature.}

\item{selected}{The initially selected value (or multiple values if \code{multiple = TRUE}). If not specified then defaults to the first value for
single-select lists and no values for multiple select lists.}

\item{multiple}{Is selection of multiple items allowed?}

\item{label}{Display label for the control, or \code{NULL} for no label.}

\item{id}{A unique identifier for this input. Usually generated
automatically.}

\item{map}{A function with single argument \code{x}, the value of the
control on the client. Returns a modified value.}

\item{selectize}{Whether to use \pkg{selectize.js} or not.}
}
\description{
\itemize{
 \item \code{input_radiobuttons} only ever selects one value
 \item \code{input_checkboxgroup} can alway select multiple values
 \item \code{input_select} can select only one if \code{multiple = FALSE},
   otherwise the user can select multiple by using modifier keys
}
}
\examples{
# Dropdown
input_select(c("a", "b", "c"))
input_select(c("a", "b", "c"), multiple = TRUE)
input_select(c("a", "b", "c"), selected = "c")

# If you want to select variable names, you need to convert
# the output of the input to a name with map so that they get
# computed correctly
input_select(names(mtcars), map = as.name)

# Radio buttons
input_radiobuttons(choices = c("Linear" = "lm", "LOESS" = "loess"),
                   label = "Model type")
input_radiobuttons(choices = c("Linear" = "lm", "LOESS" = "loess"),
                   selected = "loess",
                   label = "Model type")

# Used in layer_model_predictions
mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  layer_model_predictions(model = input_radiobuttons(
    choices = c("Linear" = "lm", "LOESS" = "loess"),
    selected = "loess",
    label = "Model type"))

# Checkbox group
mtcars \%>\% ggvis(x = ~wt, y = ~mpg) \%>\%
  layer_points(
    fill := input_checkboxgroup(
      choices = c("Red" = "r", "Green" = "g", "Blue" = "b"),
      label = "Point color components",
      map = function(val) {
        rgb(0.8 * "r" \%in\% val, 0.8 * "g" \%in\% val, 0.8 * "b" \%in\% val)
      }
    )
  )
}
\seealso{
Other interactive input: 
\code{\link{input_checkbox}()},
\code{\link{input_slider}()},
\code{\link{input_text}()}
}
\concept{interactive input}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scales}
\alias{scales}
\alias{set_default_scale}
\alias{set_dscale}
\title{Add a scale to a ggvis plot}
\arguments{
\item{vis}{A ggvis object.}

\item{scale}{The name of a scale, such as "x", "y", "fill", "stroke", etc.}

\item{type}{A variable type. One of "numeric", "nominal", "ordinal",
"logical", "datetime".}

\item{...}{other arguments passed to the scale function. See the help for
\code{\link{scale_numeric}}, \code{\link{scale_ordinal}} and
\code{\link{scale_datetime}} for more details. For example, you might
supply \code{trans = "log"} to create a log scale.}

\item{name}{If \code{NULL}, the default, the scale name is the same as
\code{scale}. Set this to a custom name to create multiple scales for
stroke or fill, or (god forbid) a secondary y scale.}
}
\description{
This creates a scale object for a given scale and variable type, and adds it
to a ggvis plot. The scale object is populated with default settings, which
depend on the scale (e.g. fill, x, opacity) and the type of variable (e.g.
numeric, nominal, ordinal). Any settings that are passed in as arguments
will override the defaults.
}
\section{Scale selection}{


ggvis supports the following types of scales. Typical uses for each scale
type are listed below:
\itemize{
  \item numeric For continuous numeric values.
  \item nominal For character vectors and factors.
  \item ordinal For ordered factors (these presently behave the same as
    nominal).
  \item logical For logical (TRUE/FALSE) values.
  \item datetime For dates and date-times.
}

Each type has a corresponding function: \code{scale_numeric},
\code{scale_nominal}, and so on.

The scale types for ggvis are mapped to scale types for Vega, which include
"ordinal", "quantitative", and "time". See \code{\link{ggvis_scale}} for more
details.

Given a scale and type, the range is selected based on the combination of the
\code{scale} and \code{type}. For example, you get a different range of
colours depending on whether the data is numeric, ordinal, or nominal. Some
scales also set other properties. For example, nominal/ordinal position
scales also add some padding so that points are spaced away from plot edges.

Not all combinations have an existing default scale. If you use a
combination that does not have an existing combination, it may suggest
you're displaying the data in a suboptimal way. For example, there is
no default for a numeric shape scale, because there's no obvious way to
map continuous values to discrete shapes.
}

\examples{
p <- mtcars \%>\%
  ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl), stroke = ~hp) \%>\%
  layer_points()

p \%>\% scale_numeric("x")
p \%>\% scale_numeric("stroke")
p \%>\% scale_nominal("fill")

# You can also supply additional arguments or override the defaults
p \%>\% scale_numeric("x", trans = "log")
p \%>\% scale_numeric("stroke", range = c("red", "blue"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_align.R
\name{compute_align}
\alias{compute_align}
\title{Align positions using length.}
\usage{
compute_align(x, var, length = NULL, align = 0.5, dir = "x")
}
\arguments{
\item{x}{Dataset-like object to align. Built-in methods for data frames,
grouped data frames and ggvis visualisations.}

\item{var}{Name of variable to compute width of.}

\item{length}{An absolute length to use. If \code{NULL} (the default), the
width will be equivalent to the resolution of the data.}

\item{align}{Where does the existing variable fall on the new bins?
0 = left edge, 0.5 = center, 1 = right edge.}

\item{dir}{Direction, i.e. \code{"x"} or \code{"y"}. Used to generate
variable names in output.}
}
\value{
The original data frame, with additional columns:
 \item{'dir'min_}{left boundary of bin}
 \item{'dir'max_}{right boundary of bin}
 \item{'dir'len_}{width of bin}
}
\description{
This compute function is often used in conjunction with
\code{\link{compute_count}}, when used on data with a continuous x variable.
By default, the computed width will be equal to the resolution of the data,
or, in other words the smallest difference between two values in the data.
}
\details{
An absolute width for each x can be specified by using the \code{width}
argument. If \code{width} is NULL (the default), it will use the resolution
of the data as the width.
}
\examples{
mtcars \%>\% compute_count(~disp) \%>\% compute_align(~x_)
mtcars \%>\% compute_count(~mpg) \%>\% compute_align(~x_)

# Use a specific width
pressure \%>\% compute_count(~temperature) \%>\% compute_align(~x_)
pressure \%>\% compute_count(~temperature) \%>\% compute_align(~x_, length = 5)

# It doesn't matter whether you transform inside or outside of a vis
mtcars \%>\% compute_count(~cyl, ~wt) \%>\%
  compute_align(~x_, length = .5) \%>\%
  ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) \%>\%
  layer_rects()

mtcars \%>\%
  ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) \%>\%
  compute_count(~cyl, ~wt) \%>\%
  compute_align(~x_) \%>\%
  layer_rects()

# Varying align
mtcars \%>\%
  ggvis(x = ~xmin_, x2 = ~xmax_, y = ~count_, y2 = 0) \%>\%
  compute_count(~cyl, ~wt) \%>\%
  compute_align(~x_, length = 0.5, align = input_slider(0, 1)) \%>\%
  layer_rects()
}
\seealso{
\code{\link{compute_bin}} For counting cases within ranges of
  a continuous variable.

\code{\link{compute_count}} For counting cases at specific values
  of a variable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handle_brush.R
\name{handle_brush}
\alias{handle_brush}
\title{Handle brush events on a visualisation.}
\usage{
handle_brush(vis, on_move = NULL, fill = "black")
}
\arguments{
\item{vis}{Visualisation to listen to.}

\item{on_move}{Callback function with arguments:
\describe{
 \item{items}{A data frame containing information about the items
   under the plot. An empty data.frame if no points under the brush.}
 \item{page_loc}{Location of the brush with repsect to the page}
 \item{plot_loc}{Location of the brush with respect to the plot}
 \item{session}{The session, used to communicate with the browser}
}}

\item{fill}{Colour of the brush.}
}
\description{
Currently for brush events to be triggered on a visualisation, you must
use a \code{.brush} property. This limitation will be lifted in the future.
}
\examples{
# Display tooltip when objects are brushed
mtcars \%>\%
  ggvis(x = ~wt, y = ~mpg, size.brush := 400) \%>\%
  layer_points() \%>\%
  handle_brush(function(items, page_loc, session, ...) {
    show_tooltip(session, page_loc$r + 5, page_loc$t, html = nrow(items))
  })
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.R
\name{prop}
\alias{prop}
\alias{is.prop}
\alias{is.prop_constant}
\alias{is.prop_variable}
\alias{is.prop_reactive}
\title{Create a property.}
\usage{
prop(
  property,
  x,
  scale = NULL,
  offset = NULL,
  mult = NULL,
  env = parent.frame(),
  event = NULL,
  label = NULL
)

is.prop(x)

is.prop_constant(x)

is.prop_variable(x)

is.prop_reactive(x)
}
\arguments{
\item{property}{A property, like "x", "x2", "y", "fill", and so on.}

\item{x}{The value of the property. This can be an atomic vector
(a constant), a name or quoted call (a variable), a single-sided
formula (a constant or variable depending on its contents), or a delayed
reactive (which can be either variable or constant).}

\item{scale}{If \code{NULL}, automatically determine behavior by the kind of
value (constant, variable, or reactive).
If \code{TRUE} use the default scale associated with property.
If \code{FALSE}, do not scale the value.
Otherwise supply a string to select a custom scale.
If \code{x} is an interactive input, then this defaults to the scale
parameter of the input.}

\item{offset, mult}{Additive and multiplicate pixel offset used to adjust
scaled values. These are useful if you want to place labels offset from
points.}

\item{env}{If \code{x} is a quoted call this provides the environment in
which to look for variables not in the data. You should not need this in
ordinary operation.}

\item{event}{An event to which this property applies. One of "update",
"enter", "exit", "hover", "brush".}

\item{label}{A label for this prop to use for reporting errors.}
}
\description{
Properties are used to describe the visual properties of \link{marks}.
You create a single property defintion with \code{prop}, and manage
sets of named properties with \code{\link{props}} (which also provides
shortcuts for creating the most common kind of properties)
}
\examples{
prop("x", 1)
prop("x", ~1)
prop("fill", quote(cyl))
prop("fill", ~cyl)
prop("x", input_slider(0, 100))

# If you have a variable name as a string
var <- "cyl"
prop("x", as.name(var))

# Use a custom scale
prop("y", quote(cyl), scale = "y-2")

# Don't scale variable (i.e. it already makes sense in the visual space)
prop("fill", ~colour, scale = FALSE)

# Use a constant, but scaled
prop("x", 5, scale = TRUE)

# Use other events
prop("y", quote(cyl), scale = "y-2")

}
\seealso{
\code{\link{props}} to manage multiple properties and to
  succintly create the most common types.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputs.R
\name{input_checkbox}
\alias{input_checkbox}
\title{Create an interactive checkbox.}
\usage{
input_checkbox(
  value = FALSE,
  label = "",
  id = rand_id("checkbox_"),
  map = identity
)
}
\arguments{
\item{value}{Initial value (\code{TRUE} or \code{FALSE}).}

\item{label}{Display label for the control, or \code{NULL} for no label.}

\item{id}{A unique identifier for this input. Usually generated
automatically.}

\item{map}{A function with single argument \code{x}, the value of the
control on the client. Returns a modified value.}
}
\description{
Create an interactive checkbox.
}
\examples{

input_checkbox(label = "Confidence interval")
input_checkbox(label = "Confidence interval", value = TRUE)

# Used in layer_smooths
mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  layer_smooths(se = input_checkbox(label = "Confidence interval"))

# Used with a map function, to convert the boolean to another type of value
model_type <- input_checkbox(label = "Use flexible curve",
  map = function(val) if(val) "loess" else "lm")
mtcars \%>\% ggvis(~wt, ~mpg) \%>\%
  layer_model_predictions(model = model_type)
}
\seealso{
Other interactive input: 
\code{\link{input_select}()},
\code{\link{input_slider}()},
\code{\link{input_text}()}
}
\concept{interactive input}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interact_tooltip.R
\name{show_tooltip}
\alias{show_tooltip}
\alias{hide_tooltip}
\title{Send a message to the client to show or hide a tooltip}
\usage{
show_tooltip(session, l = 0, t = 0, html = "")

hide_tooltip(session)
}
\arguments{
\item{session}{A Shiny session object.}

\item{l}{Pixel location of left edge of tooltip (relative to page)}

\item{t}{Pixel location of top edge of tooltip (relative to page)}

\item{html}{HTML to display in the tooltip box.}
}
\description{
Send a message to the client to show or hide a tooltip
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{create_input}
\alias{create_input}
\title{Create a new interactive "input" object.}
\usage{
create_input(
  id = rand_id("input_"),
  default = NULL,
  map = identity,
  controls = NULL
)
}
\arguments{
\item{id}{The name of the input object in the Shiny app, such as
"slider_1338869".}

\item{default}{The default (starting) value for the input.}

\item{map}{A mapping function. Defaults to \code{identity}, which simply
returns the value unchanged.}

\item{controls}{A Shiny HTML tag object representing the UI for the controls.}
}
\description{
An interactive input object is a reactive expression which wraps a reactive
value. When the plot is rendered, an observer is created which pushes values
into the reactive value in response to changes of an input object. Those
changes invalidate the reactive expression, which will return the value,
optionally passed through a mapping function.
}
\details{
This function is designed to be used by authors of new types of interactive
inputs. If you are a ggvis user, please use one of the more specific input
functions starting with the \code{input_}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{padding}
\alias{padding}
\title{Define padding.}
\usage{
padding(top = NULL, right = NULL, bottom = NULL, left = NULL)
}
\arguments{
\item{top, right, bottom, left}{Amount of padding on each border. Can either
be a single number, "auto", or "strict"}
}
\description{
Define padding.
}
\examples{
p <- mtcars \%>\% ggvis(~wt, ~mpg) \%>\% layer_points()
p \%>\% set_options(padding = padding())
p \%>\% set_options(padding = padding(10, 10, 10, 10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broker.R
\name{create_broker}
\alias{create_broker}
\title{Create a broker object}
\usage{
create_broker(r, controls = NULL, connect = NULL, spec = NULL)
}
\arguments{
\item{r}{A reactive expression.}

\item{controls}{An HTML control, or a list of HTML controls.}

\item{connect}{A function to run at render time. This function takes the
Shiny \code{session} object as its only argument, and is used to connect
the session with the broker object.}

\item{spec}{Object to put in the Vega spec.}
}
\description{
A broker is a subclass of reactive. It can hold extra information to
facilitate (or broker) communication between the client and the server.
For example, an input broker may contain HTML controls to be emitted on the
client web page, as well as a function to connect the inputs from the client
to the reactive expression.
}
\details{
Other types of brokers are possible. Another broker may create reactive
observers and add information to the Vega spec, instead of having HTML
controls. In this case, a reactive expression is still needed, although
it can be a dummy value, like \code{reactive(NULL)}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{scaletype_to_vega_scaletype}
\alias{scaletype_to_vega_scaletype}
\title{Given the type of a ggvis scale, get the name of its corresponding vega scale}
\usage{
scaletype_to_vega_scaletype(type)
}
\arguments{
\item{type}{property type: numeric, ordinal, nominal, logical or datetime.}
}
\description{
Given the type of a ggvis scale, get the name of its corresponding vega scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_layout.R
\name{sidebarBottomPage}
\alias{sidebarBottomPage}
\alias{sidebarBottomPanel}
\alias{mainTopPanel}
\title{Create a page with a sidebar}
\usage{
sidebarBottomPage(sidebarPanel, mainPanel, shiny_headers = TRUE)

sidebarBottomPanel(...)

mainTopPanel(...)
}
\arguments{
\item{sidebarPanel}{The \code{\link{sidebarBottomPanel}} containing input
controls.}

\item{mainPanel}{The \code{\link{mainTopPanel}} containing the main content.}

\item{shiny_headers}{Should Shiny headers be embedded in the page? This
should be TRUE for interactive/dynamic pages, FALSE for static pages.}

\item{...}{Additional tags.}
}
\description{
This creates a page with a sidebar, where the sidebar moves to the bottom
when the width goes below a particular value.
}
\examples{
sidebarBottomPage(sidebarBottomPanel(), mainTopPanel())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputs.R
\name{input_slider}
\alias{input_slider}
\title{Create an interactive slider.}
\usage{
input_slider(
  min,
  max,
  value = (min + max)/2,
  step = NULL,
  round = FALSE,
  format = NULL,
  locale = "us",
  ticks = TRUE,
  animate = FALSE,
  sep = ",",
  pre = NULL,
  post = NULL,
  label = "",
  id = rand_id("slider_"),
  map = identity
)
}
\arguments{
\item{min, max}{The minimum and maximum values (inclusive) that can be
selected.}

\item{value}{The initial value of the slider, either a number, a date
(class Date), or a date-time (class POSIXt). A length one vector will
create a regular slider; a length two vector will create a double-ended
range slider. Must lie between \code{min} and \code{max}.}

\item{step}{Specifies the interval between each selectable value on the
slider. Either \code{NULL}, the default, which uses a heuristic to determine the
step size or a single number. If the values are dates, \code{step} is in days;
if the values are date-times, \code{step} is in seconds.}

\item{round}{\code{TRUE} to round all values to the nearest integer;
\code{FALSE} if no rounding is desired; or an integer to round to that
number of digits (for example, 1 will round to the nearest 10, and -2 will
round to the nearest .01). Any rounding will be applied after snapping to
the nearest step.}

\item{format}{A string specifying how to format the value.}

\item{locale}{A string specifying the locale to use for formatting.}

\item{ticks}{\code{FALSE} to hide tick marks, \code{TRUE} to show them
according to some simple heuristics.}

\item{animate}{\code{TRUE} to show simple animation controls with default
settings; \code{FALSE} not to; or a custom settings list, such as those
created using \code{\link[shiny:animationOptions]{animationOptions()}}.}

\item{sep}{Separator between thousands places in numbers.}

\item{pre}{A prefix string to put in front of the value.}

\item{post}{A suffix string to put after the value.}

\item{label}{Display label for the control, or \code{NULL} for no label.}

\item{id}{A unique identifier for this input. Usually generated
automatically.}

\item{map}{A function with single argument \code{x}, the value of the
control on the client. Returns a modified value.}
}
\description{
Create an interactive slider.
}
\examples{
input_slider(0, 100)
input_slider(0, 100, label = "binwidth")
input_slider(0, 100, value = 50)

# Supply two values to value to make a double-ended sliders
input_slider(0, 100, c(25, 75))

# You can use map to transform the outputs
input_slider(-5, 5, label = "Log scale", map = function(x) 10 ^ x)
}
\seealso{
Other interactive input: 
\code{\link{input_checkbox}()},
\code{\link{input_select}()},
\code{\link{input_text}()}
}
\concept{interactive input}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R, R/shiny_layout.R
\name{shiny-ggvis}
\alias{shiny-ggvis}
\alias{bind_shiny}
\alias{bind_shiny_ui}
\alias{ggvisOutput}
\title{Connect a ggvis graphic to a shiny app.}
\usage{
bind_shiny(
  vis,
  plot_id,
  controls_id = NULL,
  ...,
  session = shiny::getDefaultReactiveDomain()
)

bind_shiny_ui(vis, controls_id, session = shiny::getDefaultReactiveDomain())

ggvisOutput(plot_id = rand_id("plot_id"))
}
\arguments{
\item{vis}{A ggvis object, or a reactive expression that returns a ggvis
object.}

\item{plot_id}{unique identifier to use for the div containing the ggvis plot.}

\item{controls_id}{Unique identifier for controls div.}

\item{...}{Other arguments passed to \code{as.vega}.}

\item{session}{A Shiny session object.}
}
\description{
Embedding ggvis in a shiny app is easy. You need to make a place for it in
your \code{ui.r} with \code{ggvisOutput}, and tell your \code{server.r}
where to draw it with \code{bind_shiny}. It's easiest to learn by example:
there are many shiny apps in \code{demo/apps/} that you can learn from.
}
\section{Client-side}{

In your UI, use \code{ggvisOutput()} in \code{ui.r} to insert an html
placeholder for the plot.

If you're going to be using interactive controls generated by ggvis,
use \code{\link[shiny]{renderUI}()} to add a place holder. By convention,
if the id of plot placehold is called "plot", call the controls placeholder
"plot_ui".
}

\section{Server-side}{

When you run ggvis plot interactively, it is automatically plotted because
it triggers the default print method. In shiny apps, you need to
explicitly render the plot to a specific placeholder with
\code{bind_shiny}:

\code{p \%>\% bind_shiny("plot")}

If the plot has controls, and you've reserved space for them in the UI,
supply the name of the placeholder as the third argument:

\code{p \%>\% bind_shiny("plot", "plot_ui")}
}

\examples{
## Run these examples only in interactive R sessions
if (interactive()) {

# Simplest possible app:
library(shiny)
runApp(list(
  ui = bootstrapPage(
    ggvisOutput("p"),
    uiOutput("p_ui")
  ),
  server = function(..., session) {
    mtcars \%>\%
      ggvis(~wt, ~mpg) \%>\%
      layer_points() \%>\%
      layer_smooths(span = input_slider(0, 1)) \%>\%
      bind_shiny("p", "p_ui")
  }
))

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_props.R
\name{is.scaled_value}
\alias{is.scaled_value}
\title{Tests whether an object is a scaled_value object}
\usage{
is.scaled_value(x)
}
\arguments{
\item{x}{an object to test}
}
\description{
Tests whether an object is a scaled_value object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explain.R
\name{explain.ggvis}
\alias{explain.ggvis}
\title{Print out the structure of a ggvis object in a friendly format}
\usage{
\method{explain}{ggvis}(x, ...)
}
\arguments{
\item{x}{Visualisation to explain}

\item{...}{Needed for compatibility with generic. Ignored by this method.}
}
\description{
Print out the structure of a ggvis object in a friendly format
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_props.R
\name{scaled_value}
\alias{scaled_value}
\title{Create a scaled_value object}
\usage{
scaled_value(scale, value)
}
\arguments{
\item{scale}{The name of a scale, e.g., "x", "fill".}

\item{value}{A value which will be transformed using the scale.}
}
\description{
These are for use with legends and axes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{show_spec}
\alias{show_spec}
\title{Print out the vega plot specification}
\usage{
show_spec(vis, pieces = NULL)
}
\arguments{
\item{vis}{Visualisation to print}

\item{pieces}{Optional, a character or numeric vector used to
pull out selected pieces of the spec}
}
\description{
Print out the vega plot specification
}
\examples{
base <- mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points()
base \%>\% show_spec()
base \%>\% show_spec("scales")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{export_png}
\alias{export_png}
\alias{export_svg}
\title{Export a PNG or SVG from a ggvis object}
\usage{
export_png(vis, file = NULL)

export_svg(vis, file = NULL)
}
\arguments{
\item{vis}{A ggvis object.}

\item{file}{Output file name. If NULL, defaults to "plot.svg" or "plot.png".}
}
\description{
This requires that the external program \code{vg2png} is installed. This is
part of the \code{vega} node.js module.
}
\examples{
\dontrun{
mtcars \%>\% ggvis(x = ~wt) \%>\% export_png()
}
}
\seealso{
\url{https://github.com/trifacta/vega} for information on installing
  \code{vg2png} and \code{vg2svg}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_legend.R
\name{add_legend}
\alias{add_legend}
\alias{hide_legend}
\title{Add a vega legend specification to a ggvis plot}
\usage{
add_legend(
  vis,
  scales = NULL,
  orient = "right",
  title = NULL,
  format = NULL,
  values = NULL,
  properties = NULL
)

hide_legend(vis, scales)
}
\arguments{
\item{vis}{A ggvis object.}

\item{scales}{The name of one or more scales for which to add a legend.
Typically one of "size", "shape", "fill", "stroke", although custom scale
names may also be used. Multiple names can also be used, like
\code{c("fill", "shape")}.}

\item{orient}{The orientation of the legend. One of "left" or "right". This
determines how the legend is positioned within the scene. The default is
"right".}

\item{title}{A title for the legend. By default, it uses the name the fields
used in the legend. Use \code{""} to suppress the title.}

\item{format}{The formatting pattern for axis labels. Vega uses D3's format
pattern.}

\item{values}{Explicitly set the visible legend values.}

\item{properties}{Optional mark property definitions for custom legend
styling. Should be an object created by \code{\link{legend_props}}, with
properties for title, label, symbols, gradient, legend.}
}
\description{
Axis specifications allow you to either override the default legends,
or supply additional legends.
}
\details{
More information about axes can be found in the "axes and legends" vignettes.
}
\section{Compared to ggplot2}{


In ggplot2, legend (and axis) properties are part of the scales
specification. In vega, they are separate, which allows the specification
of multiple legends, and more flexible linkage between scales and legends.
}

\examples{
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  add_legend("fill", title = "Cylinders")

# Suppress legend with hide_legend
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  hide_legend("fill")

# Combining two properties in one legend
mtcars \%>\%
  ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl), shape = ~factor(cyl)) \%>\%
  layer_points() \%>\%
  add_legend(c("fill", "shape"))

# Control legend properties with a continuous legend, with x and y position
# in pixels.
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  add_legend("fill", title = "Cylinders",
    properties = legend_props(
      title = list(fontSize = 16),
      labels = list(fontSize = 12, fill = "#00F"),
      gradient = list(stroke = "red", strokeWidth = 2),
      legend = list(x = 500, y = 50)
    )
  )

# Control legend properties with a categorical legend, with x and y position
# in the scaled data space.
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~factor(cyl)) \%>\%
  layer_points() \%>\%
  add_legend("fill", title = "Cylinders",
    properties = legend_props(
      title = list(fontSize = 16),
      labels = list(fontSize = 14, dx = 5),
      symbol = list(stroke = "black", strokeWidth = 2,
        shape = "square", size = 200),
      legend = list(
        x = scaled_value("x", 4.5),
        y = scaled_value("y", 30)
      )
    )
  )

# Control legend position using x_rel and y_rel which specify relative
# position, going from 0 to 1. (0, 0) is the bottom-left corner, and
# (1, 1) is the upper-right corner. The values control the position of
# the upper-left corner of the legend.
mtcars \%>\% ggvis(x = ~wt, y = ~mpg, fill = ~cyl) \%>\%
  layer_points() \%>\%
  add_relative_scales() \%>\%
  add_legend("fill", title = "Cylinders",
    properties = legend_props(
      legend = list(
        x = scaled_value("x_rel", 0.8),
        y = scaled_value("y_rel", 1)
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_stack.R
\name{compute_stack}
\alias{compute_stack}
\title{Stack overlapping data.}
\usage{
compute_stack(x, stack_var = NULL, group_var = NULL)
}
\arguments{
\item{x}{A data object}

\item{stack_var}{A string specifying the stacking variable.}

\item{group_var}{A string specifying the grouping variable.}
}
\value{
A data frame with columns:
 \item{stack_upr_}{the lower y coordinate for a stack bar}
 \item{stack_lwr_}{the upper y coordinate for a stack bar}
}
\description{
Stack overlapping data.
}
\examples{
mtcars \%>\% cbind(count = 1) \%>\% compute_stack(~count, ~cyl)

# Shouldn't use or affect existing grouping
mtcars \%>\% cbind(count = 1) \%>\% group_by(am) \%>\% compute_stack(~count, ~cyl)

# If given a ggvis object, will use x variable for stacking by default
mtcars \%>\% ggvis(x = ~cyl, y = ~wt) \%>\%
  compute_stack(stack_var = ~wt, group_var = ~cyl) \%>\%
  layer_rects(x = ~cyl - 0.5, x2 = ~cyl + 0.5, y = ~stack_upr_,
    y2 = ~stack_lwr_)

# Collapse across hair & eye colour data across sex
hec <- as.data.frame(xtabs(Freq ~ Hair + Eye, HairEyeColor))
hec \%>\% compute_stack(~Freq, ~Hair)

# Without stacking - bars overlap
hec \%>\% ggvis(~Hair, ~Freq, fill = ~Eye, fillOpacity := 0.5) \%>\%
  layer_rects(y2 = 0, width = band())

# With stacking
hec \%>\% ggvis(x = ~Hair, y = ~Freq, fill = ~Eye, fillOpacity := 0.5) \%>\%
  compute_stack(~Freq, ~Hair) \%>\%
  layer_rects(y = ~stack_lwr_, y2 = ~stack_upr_, width = band())

# layer_bars stacks automatically:
hec \%>\% ggvis(~Hair, ~Freq, fill = ~Eye, fillOpacity := 0.5) \%>\%
  group_by(Eye) \%>\%
  layer_bars(width = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_boxplot.R
\name{compute_boxplot}
\alias{compute_boxplot}
\title{Calculate boxplot values}
\usage{
compute_boxplot(x, var = NULL, coef = 1.5)
}
\arguments{
\item{x}{Dataset-like object to compute boxplot values. There are built-in
methods for data frames, grouped data frames, and ggvis visualisations.}

\item{var}{Name of variable for which to compute boxplot values. The variable
must be continuous.}

\item{coef}{The maximum length of the whiskers as multiple of the
inter-quartile range. Default value is 1.5.}
}
\value{
A data frame with columns:
  \item{min_}{Lower whisker = smallest observation greater than or equal to lower hinge - 1.5 * IQR}
  \item{lower_}{Lower hinge (25th percentile)}
  \item{median_}{Median (50th percentile)}
  \item{upper_}{Upper hinge (75th percentile)}
  \item{max_}{Upper whisker = largest observation less than or equal to upper hinge + 1.5 * IQR}
  \item{outliers_}{A vector of values that are outside of the min and max}
}
\description{
Calculate boxplot values
}
\examples{
mtcars \%>\% compute_boxplot(~mpg)
mtcars \%>\% group_by(cyl) \%>\% compute_boxplot(~mpg)
}
\seealso{
\code{\link{layer_boxplots}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_guess.R
\name{layer_guess}
\alias{layer_guess}
\title{Guess the right type of layer based on current properties.}
\usage{
layer_guess(vis, ...)
}
\arguments{
\item{vis}{The visualisation to add the new layer to.}

\item{...}{Other arguments passed on individual layers.}
}
\description{
\code{layer_guess} provides the magic behind the default behaviour of
\code{\link{ggvis}}.
}
\section{Defaults}{


\itemize{
  \item Continuous x, \code{\link{layer_histograms}}
  \item Categorical x, \code{\link{layer_bars}}
  \item Continuous x and y, \code{\link{layer_points}}
}
}

\examples{
# A scatterplot:
mtcars \%>\% ggvis(~mpg, ~wt)
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_guess()

# A histogram:
mtcars \%>\% ggvis(~mpg)
mtcars \%>\% ggvis(~mpg) \%>\% layer_guess()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handle_keyboard.R
\name{left_right}
\alias{left_right}
\alias{up_down}
\title{Interactive inputs bound to arrow keys.}
\usage{
left_right(min, max, value = (min + max)/2, step = (max - min)/40)

up_down(min, max, value = (min + max)/2, step = (max - min)/40)
}
\arguments{
\item{min}{A minimum value.}

\item{max}{A maximum value.}

\item{value}{The initial value before any keys are pressed. Defaults to
half-way between \code{min} and \code{max}.}

\item{step}{How much each key press changes \code{value}. Defaults to
40 steps along range}
}
\description{
Interactive inputs bound to arrow keys.
}
\examples{
size <- left_right(1, 801, value = 51, step = 50)
opacity <- up_down(0, 1, value = 0.9, step = 0.05)

mtcars \%>\% ggvis(~mpg, ~wt, size := size, opacity := opacity) \%>\%
  layer_points()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{is.dynamic}
\alias{is.dynamic}
\title{Determine if an ggvis is dynamic (i.e. needs to be run in a shiny app)}
\usage{
is.dynamic(x)
}
\description{
Determine if an ggvis is dynamic (i.e. needs to be run in a shiny app)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputs.R
\name{input_text}
\alias{input_text}
\alias{input_numeric}
\title{Create an interactive text or numeric input box.}
\usage{
input_text(value, label = "", id = rand_id("text_"), map = identity)

input_numeric(value, label = "", id = rand_id("numeric_"), map = identity)
}
\arguments{
\item{value}{Initial value.}

\item{label}{Display label for the control, or \code{NULL} for no label.}

\item{id}{A unique identifier for this input. Usually generated
automatically.}

\item{map}{A function with single argument \code{x}, the value of the
control on the client. Returns a modified value.}
}
\description{
\code{input_numeric} only allows numbers and comes with a spin box control.
\code{input_text} allows any type of input.
}
\examples{
fill_text <- input_text(label = "Point color", value = "red")
mtcars \%>\% ggvis(~wt, ~mpg, fill := fill_text) \%>\% layer_bars()

size_num <- input_numeric(label = "Point size", value = 25)
mtcars \%>\% ggvis(~wt, ~mpg, size := size_num) \%>\% layer_points()
}
\seealso{
Other interactive input: 
\code{\link{input_checkbox}()},
\code{\link{input_select}()},
\code{\link{input_slider}()}
}
\concept{interactive input}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vega.R
\name{as.vega}
\alias{as.vega}
\alias{as.vega.ggvis}
\title{Coerce an ggvis object to a vega list.}
\usage{
as.vega(x, ...)

\method{as.vega}{ggvis}(x, session = NULL, dynamic = FALSE, ...)
}
\arguments{
\item{x}{an object to convert to vega}

\item{session}{a session object from shiny}

\item{dynamic}{whether to generate dynamic or static spec}
}
\value{
a list. When converted to JSON, will be the type of structure
  that vega expects.
}
\description{
This generic function powers the coercion of ggvis objects to vega
compatible data structures.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide_props.R
\name{legend_props}
\alias{legend_props}
\title{Create an axis_props object for controlling legend properties.}
\usage{
legend_props(
  title = NULL,
  labels = NULL,
  symbols = NULL,
  gradient = NULL,
  legend = NULL
)
}
\arguments{
\item{title}{A named list of text properties for the legend title.}

\item{labels}{A named list of text properties for legend labels.}

\item{symbols}{A named list of line properties for symbols (for discrete
legend items).}

\item{gradient}{A named list of line properties a continuous color gradient.}

\item{legend}{A named list of line properties for the overall legend. The
x and y position can be set here, which will override automatic
positioning.}
}
\description{
The items in each of the lists can be a literal value, like \code{5} or
"blue", or they can be a \code{\link{scaled_value}} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subvis.R
\name{subvis}
\alias{subvis}
\title{Create a subvisualisation.}
\usage{
subvis(vis, ..., data = NULL, width = NULL, height = NULL)
}
\description{
Create a subvisualisation.
}
\examples{
# Examples don't work yet
\dontrun{
library(dplyr, warn.conflicts = FALSE)

small <- nasaweather::atmos \%>\%
  filter(lat <= -11.217391, long <= -106.287, year == 1995) \%>\%
  group_by(long, lat)
small \%>\%
  ggvis(~long, ~lat) \%>\%
  layer_points()

small \%>\%
  ggvis(~long, ~lat) \%>\%
  subvis(width := 100, height := 100, stroke := "red") \%>\%
    layer_points(~month, ~ozone)

small \%>\%
  ggvis(~long, ~lat) \%>\%
  subvis(width := 100, height := 100, stroke := "red") \%>\%
    layer_points(~month, ~ozone) \%>\%
    add_axis("x", ticks = 3) \%>\%
    add_axis("y", ticks = 3)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{add_scale}
\alias{add_scale}
\title{Add arbitrary scales to ggvis.}
\usage{
add_scale(vis, scale, data_domain = TRUE)
}
\arguments{
\item{vis}{Visualisation to modify.}

\item{scale}{Scale object}

\item{data_domain}{Should the domain be controlled by a data set which is
added to the spec? Should only be set to FALSE in special cases.}
}
\description{
Add arbitrary scales to ggvis.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/props.R
\name{props}
\alias{props}
\alias{is.ggvis_props}
\title{Manage a list of properties.}
\usage{
props(..., .props = NULL, inherit = TRUE, env = parent.frame())

is.ggvis_props(x)
}
\arguments{
\item{...}{A set of name-value pairs. The name should be a valid vega
  property.

  The first two unnamed components are taken to be \code{x} and \code{y}.
  Any additional unnamed components will raise an error.}

\item{.props}{When calling \code{props} from other functions, you'll
often have a list of quoted function functions. You can pass that function
to the \code{.props} argument instead of messing around with
substitute. In other words, \code{.props} lets you opt out of the
non-standard evaluation that \code{props} does.}

\item{inherit}{If \code{TRUE}, the defaults, will inherit from properties
from the parent layer If \code{FALSE}, it will start from nothing.}

\item{env}{The environment in which to evaluate variable properties.}

\item{x}{an object to test for props-ness.}
}
\description{
\code{props()} provides a tool for concise creation of \code{prop} objects
using a set of conventions designed to capture the most common use cases.
If you need something less common, you'll need to use \code{\link{prop}} to
access all possible options.
}
\section{Heuristics}{


If the values are not already objects of class \code{prop}, \code{props}
uses the following heuristics to when creating the prop:

\itemize{
 \item atomic vectors, e.g. \code{x = 1}: scaled = FALSE
 \item an interative input, e.g. \code{x = input_slider}:
    scaled = FALSE
 \item a formula containing a single value, e.g. \code{x ~ 1}:
    scaled = TRUE
 \item a formula containing a name or expression, \code{x ~ mpg}:
    scaled = TRUE
}
}

\section{Non-standard evaluation}{


\code{props} uses non-standard evaluation in a slightly unusual way:
if you provide a formula input, the LHS of the formula will provide the
name of the component. In otherwise, \code{props(x = y ~ 1)} is the
same as \code{props(y ~ 1)}.

You can combine variables from the dataset and variables defined in the
local environment: expressions will be evaluated in the environment which
the formula was defined.

If you have the name of a variable in a string, see the
props vignette for how to create the needed property mapping.
}

\section{Enter, exit, hover, and update events}{


There are four different property events that the marks
can use. These can, for example, be used to change the appearance of a mark
when the mouse cursor is hovering over it: when the mark is hovered over, it
uses the hover event, and when the mark isn't hovered over, it uses the
update event

\itemize{
  \item enter: This event is used by marks when they are added to a plot.
  \item update: This event is used by marks after they have entered, and
    also after they have been hovered over.
  \item exit: This event is used by marks as they are removed from a plot.
  \item hover: This event is used when the mouse cursor is over the mark.
}

You can specify the event for a property, by putting a period and the
event after the property name. For example,
\code{props(fill.update := "black", fill.hover := "red")} will make a mark
have a black fill normally, and red fill when it is hovered over.

The default event is update, so if you run \code{props(fill := "red")},
this is equivalent to \code{props(fill.update := "red")}.

In practice, the enter and exit events are useful only when the update has
a duration (and is therefore not instantaneous). The update event can be
thought of as the "default" state.
}

\section{Key property}{


In addition to the standard properties, there is a special optional property
called \code{key}. This is useful for plots with dynamic data and smooth
transitions: as the data changes, the key is used to tell the plot how the
new data rows should be matched to the old data rows. Note that the key must
be an unscaled value. Additionally, the key property doesn't have a event,
since it is independent of enter, update, exit, and hover events.
}

\section{Properties}{


You can set the following mark properties:

\itemize{
\item  x  The first (typically left-most) x-coordinate.
\item  x2 The second (typically right-most) x-coordinate.
\item  width The width of the mark (if supported).
\item  y The first (typically top-most) y-coordinate.
\item  y2 The second (typically bottom-most) y-coordinate.
\item  height The height of the mark (if supported).
\item  opacity The overall opacity.
\item  fill The fill color.
\item  fillOpacity The fill opacity
\item  stroke The stroke color.
\item  strokeWidth The stroke width, in pixels.
\item  strokeOpacity The stroke opacity.
\item  size [symbol] The pixel area of the symbol. For example in the case
  of circles, the radius is determined in part by the square root of the size
  value.
\item  shape [symbol] The symbol shape to use. One of circle (default),
  square, cross, diamond, triangle-up, or triangle-down (symbol only)
\item  innerRadius [arc] The inner radius of the arc, in pixels.
\item  outerRadius [arc] The outer radius of the arc, in pixels.
\item  startAngle [arc] The start angle of the arc, in radians.
\item  endAngle [arc] The end angle of the arc, in radians.
\item  interpolate [area, line] The line interpolation method to use. One
  of linear, step-before, step-after, basis, basis-open, cardinal,
  cardinal-open, monotone.
\item  tension [area, line] Depending on the interpolation type, sets the
  tension parameter.
\item  url [image] The URL from which to retrieve the image.
\item  align [image, text] The horizontal alignment of the object. One of
  left, right, center.
\item  baseline [image, text] The vertical alignment of the object. One of
  top, middle, bottom.
\item  text [text] The text to display.
\item  dx [text] The horizontal margin, in pixels, between the text label
  and its anchor point. The value is ignored if the align property is center.
\item  dy [text] The vertical margin, in pixels, between the text label
  and its anchor point. The value is ignored if the baseline property is
  middle.
\item  angle [text] The rotation angle of the text, in degrees.
\item  font [text] The typeface to set the text in (e.g., Helvetica Neue).
\item  fontSize [text] The font size, in pixels.
\item  fontWeight [text] The font weight (e.g., bold).
\item  fontStyle [text] The font style (e.g., italic).
}

To each property, you can assign any property object (\code{\link{prop}})
either locally (i.e. in the mark), or in a parent \code{layer}.
}

\examples{
# Set to constant values
props(x := 1, y := 2)
# Map to variables in the dataset
props(x = ~mpg, y = ~cyl)
# Set to a constant value in the data space
props(x = 1, y = 1)
# Use an interactive slider
props(opacity := input_slider(0, 1))

# To control other settings (like custom scales, mult and offset)
# use a prop object
props(prop("x", "old", scale = "x", offset = -1))

# Red when hovered over, black otherwise (these are equivalent)
props(fill := "black", fill.hover := "red")
props(fill.update := "black", fill.hover := "red")

# Use a column called id as the key (for dynamic data)
props(key := ~id)

# Explicitly create prop objects. The following are equivalent:
props(fill = ~cyl)
props(fill.update = ~cyl)
props(prop("fill", ~cyl))
props(prop("fill", ~cyl, scale = "fill", event = "update"))

# Prop objects can be programmatically created and added:
property <- "fill"
expr <- parse(text = "wt/mpg")[[1]]
p <- prop(property, expr)
props(p)

# Using .props
props(.props = list(x = 1, y = 2))
props(.props = list(x = ~mpg, y = ~cyl))
props(.props = list(quote(x := ~mpg)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_ggvis.R
\name{ggvis_scale}
\alias{ggvis_scale}
\alias{is.ggvis_scale}
\title{Create a new ggvis_scale object.}
\usage{
ggvis_scale(
  property,
  name = property,
  label = name,
  type = NULL,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  ...,
  subclass = NULL,
  override = NULL
)

is.ggvis_scale(x)
}
\arguments{
\item{property}{The property to which the scale applies, such as "x", "y",
"fill", etc.}

\item{name}{Name of the scale, such as "x", "y", "fill", etc. Can also be an
arbitrary name like "foo".}

\item{label}{Label for the scale. Used for axis or legend titles.}

\item{type}{Type of scale. Should be one of "linear", "ordinal", "time",
"utc", "linear", "log", "pow", "sqrt", "quantile", "quantize", "threshold".}

\item{domain}{The domain of the scale, representing the set of data values.
For ordinal scales, a character vector; for quantitative scales, a numeric
vector of length two. Either value (but not both) may be NA, in which
case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
can also be a reactive which returns the appropriate type of vector.}

\item{range}{The range of the scale, representing the set of visual values.
For numeric values, the range can take the form of a two-element array with
minimum and maximum values. For ordinal data, the range may by an array of
desired output values, which are mapped to elements in the specified
domain. The following range literals are also available: "width", "height",
"shapes", "category10", "category20".}

\item{reverse}{If true, flips the scale range.}

\item{round}{If true, rounds numeric output values to integers. This can be
helpful for snapping to the pixel grid.}

\item{...}{other named arguments.}

\item{subclass}{Class name for subclass.  Will have \code{scale_} prepended.}

\item{override}{Should the domain specified by this ggvis_scale object
override other ggvis_scale objects for the same scale? Useful when domain is
manually specified. For example, by default, the domain of the scale
will contain the range of the data, but when this is TRUE, the specified
domain will override, and the domain can be smaller than the range of the
data. If \code{FALSE}, the \code{domain} will not behave this way. If
left \code{NULL}, then it will be treated as \code{TRUE} whenever
\code{domain} is non-NULL.}

\item{x}{object to test for scale-ness}
}
\description{
A scale object is a close mapping to a vega mark object. Vega scales
are documented in \url{https://vega.github.io/vega/docs/scales/}.
}
\details{
This function is designed to be used by authors of new types of scale.
If you are a ggvis user, please use one of the more specific scale
functions starting with the \code{scale_}.

This is very close, but not exactly a vega scale object. Instead of being a
named list with a set of values, the domain can be  a vector of values, or a
reactive that returns such values.
}
\examples{
ggvis_scale("x", type = "linear")
ggvis_scale("x", "ord")
}
\seealso{
\url{https://vega.github.io/vega/docs/scales/}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linked_brush.R
\name{linked_brush}
\alias{linked_brush}
\title{Create a linked brush object.}
\usage{
linked_brush(keys, fill = "red")
}
\arguments{
\item{keys}{vector of all possible keys, if known.}

\item{fill}{brush colour}
}
\value{
A list with components:
  \item{input}{A function that takes a visualisation as an argument and
     adds an input brush to that plot}
  \item{selected}{A reactive providing a logical vector that describes
    which points are under the brush}
  \item{fill}{A reactive that gives the fill colour of points under the
    brush}
}
\description{
A linked brush has two sides: input and output
}
\note{
\code{linked_brush} is very new and is likely to change substantially
  in the future
}
\examples{
lb <- linked_brush(keys = 1:nrow(mtcars), "red")

# Change the colour of the points
mtcars \%>\%
 ggvis(~disp, ~mpg) \%>\%
 layer_points(fill := lb$fill, size.brush := 400) \%>\%
 lb$input()

# Display one layer with all points and another layer with selected points
library(shiny)
mtcars \%>\%
 ggvis(~disp, ~mpg) \%>\%
 layer_points(size.brush := 400) \%>\%
 lb$input() \%>\%
 layer_points(fill := "red", data = reactive(mtcars[lb$selected(), ]))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{default_options}
\alias{default_options}
\title{Default options}
\usage{
default_options()
}
\description{
This returns an object containing the default options for ggvis.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark.R
\name{mark}
\alias{mark}
\alias{is.mark}
\title{Create a new "mark" object.}
\usage{
mark(type, props, data)

is.mark(x)
}
\arguments{
\item{type}{A string with the vega type.}

\item{props}{A list of properties, created by \code{\link{props}}.}

\item{data}{A reactive data object.}
}
\description{
A mark object is a close mapping to a vega mark object. Vega marks
are documented in \url{https://vega.github.io/vega/docs/marks/}.
}
\details{
This function is designed to be used by authors of new types of mark.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{ggvis}
\alias{ggvis}
\title{Visualise a data set with a ggvis graphic.}
\usage{
ggvis(data = NULL, ..., env = parent.frame())
}
\arguments{
\item{data}{A data object.}

\item{...}{Property mappings. If not named, the first two mappings are
taken to be \code{x} and \code{y}. Common properties are \code{x},
\code{y}, \code{stroke}, \code{fill}, \code{opacity}, \code{shape}}

\item{env}{Environment in which to evaluate properties.}
}
\description{
\code{ggvis} is used to turn a dataset into a visualisation, setting up
default mappings between variables in the dataset and visual properties.
Nothing will be displayed until you add additional layers.
}
\examples{
# If you don't supply a layer, ggvis uses layer_guess() to guess at
# an appropriate type:
mtcars \%>\% ggvis(~mpg, ~wt)
mtcars \%>\% ggvis(~mpg, ~wt, fill = ~cyl)
mtcars \%>\% ggvis(~mpg, ~wt, fill := "red")
mtcars \%>\% ggvis(~mpg)

# ggvis has a functional interface: every ggvis function takes a ggvis
# an input and returns a modified ggvis as output.
layer_points(ggvis(mtcars, ~mpg, ~wt))

# To make working with this interface more natural, ggvis imports the
# pipe operator from magrittr. x \%>\% f(y) is equivalent to f(x, y) so
# we can rewrite the previous command as
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_points()

# For more complicated plots, add a line break after \%>\%
mtcars \%>\%
  ggvis(~mpg, ~wt) \%>\%
  layer_points() \%>\%
  layer_smooths()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_datetime}
\alias{scale_datetime}
\title{Add a date-time scale to a ggvis object.}
\usage{
scale_datetime(
  vis,
  property,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  utc = NULL,
  clamp = NULL,
  nice = NULL,
  expand = NULL,
  name = property,
  label = NULL,
  override = NULL
)
}
\arguments{
\item{vis}{A ggvis object.}

\item{property}{The name of a property, such as "x", "y", "fill", "stroke", etc.}

\item{domain}{The domain of the scale, representing the set of data values.
For ordinal scales, a character vector; for quantitative scales, a numeric
vector of length two. Either value (but not both) may be NA, in which
case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
can also be a reactive which returns the appropriate type of vector.}

\item{range}{The range of the scale, representing the set of visual values.
For numeric values, the range can take the form of a two-element array with
minimum and maximum values. For ordinal data, the range may by an array of
desired output values, which are mapped to elements in the specified
domain. The following range literals are also available: "width", "height",
"shapes", "category10", "category20".}

\item{reverse}{If true, flips the scale range.}

\item{round}{If true, rounds numeric output values to integers. This can be
helpful for snapping to the pixel grid.}

\item{utc}{if \code{TRUE}, uses UTC times. Default is \code{FALSE}.}

\item{clamp}{If \code{TRUE}, values that exceed the data domain are clamped
to either the minimum or maximum range value. Default is \code{FALSE}.}

\item{nice}{If specified, modifies the scale domain to use a more
human-friendly value range. Should be a string indicating the desired time
interval; legal values are "second", "minute", "hour", "day", "week",
"month", or "year".}

\item{expand}{A multiplier for how much the scale should be expanded beyond
the domain of the data. For example, if the data goes from 10 to 110, and
\code{expand} is 0.05, then the resulting domain of the scale is 5 to 115.
Set to 0 and use \code{nice=FALSE} if you want exact control over the
domain.}

\item{name}{Name of the scale, such as "x", "y", "fill", etc. Can also be an
arbitrary name like "foo".}

\item{label}{Label for the scale. Used for axis or legend titles.}

\item{override}{Should the domain specified by this ggvis_scale object
override other ggvis_scale objects for the same scale? Useful when domain is
manually specified. For example, by default, the domain of the scale
will contain the range of the data, but when this is TRUE, the specified
domain will override, and the domain can be smaller than the range of the
data. If \code{FALSE}, the \code{domain} will not behave this way. If
left \code{NULL}, then it will be treated as \code{TRUE} whenever
\code{domain} is non-NULL.}
}
\description{
A date/time scale controls the mapping of date and time variables to
visual properties.
}
\examples{
set.seed(2934)
dat <- data.frame(
  time = as.Date("2013-07-01") + 1:100,
  value = seq(1, 10, length.out = 100) + rnorm(100)
)
p <- dat \%>\% ggvis(~time, ~value) \%>\% layer_points()

# Start and end on month boundaries
p \%>\% scale_datetime("x", nice = "month")


dist <- data.frame(times = as.POSIXct("2013-07-01", tz = "GMT") +
                           rnorm(200) * 60 * 60 * 24 * 7)
p <- dist \%>\% ggvis(x = ~times) \%>\% layer_histograms()
p

# Start and end on month boundaries
p \%>\% scale_datetime("x", nice = "month")

p \%>\% scale_datetime("x", utc = TRUE)
}
\seealso{
\code{\link{scales}}, \code{\link{scale_numeric}},
  \url{https://vega.github.io/vega/docs/scales/#time}

Other scales: 
\code{\link{scale_numeric}()},
\code{\link{scale_ordinal}()}
}
\concept{scales}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_density.R
\name{compute_density}
\alias{compute_density}
\title{Compute density of data.}
\usage{
compute_density(
  x,
  x_var,
  w_var = NULL,
  kernel = "gaussian",
  trim = FALSE,
  n = 256L,
  na.rm = FALSE,
  ...
)
}
\arguments{
\item{x}{Dataset (data frame, \code{grouped_df} or ggvis) object to work
with.}

\item{x_var, w_var}{Names of variables to use for x position, and for
weights.}

\item{kernel}{Smoothing kernel. See \code{\link{density}} for details.}

\item{trim}{If \code{TRUE}, the default, density estimates are trimmed to the
actual range of the data.  If \code{FALSE}, they are extended by the
default 3 bandwidths (as specified by the \code{cut} parameter to
\code{\link{density}}).}

\item{n}{Number of points (along x) to use in the density estimate.}

\item{na.rm}{If \code{TRUE} missing values will be silently removed,
otherwise they will be removed with a warning.}

\item{...}{Additional arguments passed on to \code{\link{density}}.}
}
\value{
A data frame with columns:
 \item{pred_}{regularly spaced grid of \code{n} locations}
 \item{resp_}{density estimate}
}
\description{
Compute density of data.
}
\examples{
mtcars \%>\% compute_density(~mpg, n = 5)
mtcars \%>\% group_by(cyl) \%>\% compute_density(~mpg, n = 5)
mtcars \%>\% ggvis(~mpg) \%>\% compute_density(~mpg, n = 5) \%>\%
  layer_points(~pred_, ~resp_)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/message.R
\name{ggvis_message}
\alias{ggvis_message}
\title{Send a message to ggvis running on client}
\usage{
ggvis_message(session, type, data = NULL, id = NULL)
}
\arguments{
\item{session}{A session object.}

\item{type}{A string representing the type of the message.}

\item{data}{An object (typically a list) containing information for the client.}

\item{id}{A unique identifier for ggvis message handler (optional).}
}
\description{
This will be sent to the client and passed to a handler in ggvis.messages on
the client side. The handler is specified by \code{type}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interact_tooltip.R
\name{add_tooltip}
\alias{add_tooltip}
\title{Add tooltips to a plot.}
\usage{
add_tooltip(vis, html, on = c("hover", "click"))
}
\arguments{
\item{vis}{Visualisation to add tooltips to.}

\item{html}{A function that takes a single argument as input. This argument
will be a list containing the data in the mark currently under the
mouse. It should return a string containing HTML or \code{NULL} to
hide tooltip for the current element.}

\item{on}{Should tooltips appear on hover, or on click?}
}
\description{
Add tooltips to a plot.
}
\examples{
## Run these examples only in interactive R sessions
if (interactive()) {

all_values <- function(x) {
  if(is.null(x)) return(NULL)
  paste0(names(x), ": ", format(x), collapse = "<br />")
}

base <- mtcars \%>\% ggvis(x = ~wt, y = ~mpg) \%>\%
  layer_points()
base \%>\% add_tooltip(all_values, "hover")
base \%>\% add_tooltip(all_values, "click")

# The data sent from client to the server contains only the data columns that
# are used in the plot. If you want to get other columns of data, you should
# to use a key to line up the item from the plot with a row in the data.
mtc <- mtcars
mtc$id <- 1:nrow(mtc)  # Add an id column to use ask the key

all_values <- function(x) {
  if(is.null(x)) return(NULL)
  row <- mtc[mtc$id == x$id, ]
  paste0(names(row), ": ", format(row), collapse = "<br />")
}

mtc \%>\% ggvis(x = ~wt, y = ~mpg, key := ~id) \%>\%
  layer_points() \%>\%
  add_tooltip(all_values, "hover")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layer_lines.R
\name{layer_lines}
\alias{layer_lines}
\title{Layer lines on a plot.}
\usage{
layer_lines(vis, ...)
}
\arguments{
\item{vis}{Visualisation to modify.}

\item{...}{Visual properties.}
}
\description{
\code{layer_lines} differs from \code{layer_paths} in that \code{layer_lines}
sorts the data on the x variable, so the line will always proceed from left
to right, whereas \code{layer_paths} will draw a line in whatever order
appears in the data.
}
\examples{
mtcars \%>\% ggvis(~wt, ~mpg, stroke = ~factor(cyl)) \%>\% layer_lines()

# Equivalent to
mtcars \%>\% ggvis(~wt, ~mpg, stroke = ~factor(cyl)) \%>\%
  group_by(cyl) \%>\% dplyr::arrange(wt) \%>\% layer_paths()
}
\seealso{
\code{\link{layer_paths}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_bin.R
\name{compute_bin}
\alias{compute_bin}
\title{Bin data along a continuous variable}
\usage{
compute_bin(
  x,
  x_var,
  w_var = NULL,
  width = NULL,
  center = NULL,
  boundary = NULL,
  closed = c("right", "left"),
  pad = FALSE,
  binwidth
)
}
\arguments{
\item{x}{Dataset-like object to bin. Built-in methods for data frames,
grouped data frames and ggvis visualisations.}

\item{x_var, w_var}{Names of x and weight variables. The x variable must be
continuous.}

\item{width}{The width of the bins. The default is \code{NULL}, which yields
30 bins that cover the range of the data. You should always override this
value, exploring multiple widths to find the best to illustrate the stories
in your data.}

\item{center}{The center of one of the bins.  Note that if center is above or
below the range of the data, things will be shifted by an appropriate
number of \code{width}s. To center on integers, for example, use
\code{width=1} and \code{center=0}, even if \code{0} is outside the range
of the data.  At most one of \code{center} and \code{boundary} may be
specified.}

\item{boundary}{A boundary between two bins. As with \code{center}, things
are shifted when \code{boundary} is outside the range of the data. For
example, to center on integers, use \code{width = 1} and \code{boundary =
0.5}, even if \code{1} is outside the range of the data.  At most one of
\code{center} and \code{boundary} may be specified.}

\item{closed}{One of \code{"right"} or \code{"left"} indicating whether right
or left edges of bins are included in the bin.}

\item{pad}{If \code{TRUE}, adds empty bins at either end of x. This ensures
frequency polygons touch 0. Defaults to \code{FALSE}.}

\item{binwidth}{Deprecated; use \code{width} instead.}
}
\value{
A data frame with columns:
 \item{count_}{the number of points}
 \item{x_}{mid-point of bin}
 \item{xmin_}{left boundary of bin}
 \item{xmax_}{right boundary of bin}
 \item{width_}{width of bin}
}
\description{
Bin data along a continuous variable
}
\examples{
mtcars \%>\% compute_bin(~mpg)
mtcars \%>\% compute_bin(~mpg, width = 10)
mtcars \%>\% group_by(cyl) \%>\% compute_bin(~mpg, width = 10)

# It doesn't matter whether you transform inside or outside of a vis
mtcars \%>\% compute_bin(~mpg) \%>\% ggvis(~x_, ~count_) \%>\% layer_paths()
mtcars \%>\% ggvis(~ x_, ~ count_) \%>\% compute_bin(~mpg) \%>\% layer_paths()

# Missing values get own bin
mtcars2 <- mtcars
mtcars2$mpg[sample(32, 5)] <- NA
mtcars2 \%>\% compute_bin(~mpg, width = 10)

# But are currently silently dropped in histograms
mtcars2 \%>\% ggvis() \%>\% layer_histograms(~mpg)
}
\seealso{
\code{\link{compute_count}} For counting cases at specific locations
  of a continuous variable. This is useful when the variable is continuous
  but the data is granular.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_auto_group.R
\name{auto_group}
\alias{auto_group}
\title{Automatically group data by grouping variables}
\usage{
auto_group(vis, exclude = NULL)
}
\arguments{
\item{vis}{The ggvis visualisation to modify.}

\item{exclude}{A vector containing names of props to exclude from auto grouping.
It is often useful to exclude \code{c("x", "y")}, when one of those variables
is categorical.}
}
\description{
Use \code{auto_group} to group up a dataset on all categorical variables
specified by props, and have each piece rendered by the same mark.
}
\examples{
# One line
mtcars \%>\% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) \%>\% layer_paths()
# One line for each level of cyl
mtcars \%>\% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) \%>\% group_by(cyl) \%>\%
  layer_paths()
mtcars \%>\% ggvis(~disp, ~mpg, stroke = ~factor(cyl)) \%>\% auto_group() \%>\%
  layer_paths()

# The grouping column can already be stored as a factor
mtcars2 <- mtcars
mtcars2$cyl <- factor(mtcars2$cyl)
mtcars2 \%>\% ggvis(~disp, ~mpg, stroke = ~cyl) \%>\% auto_group() \%>\%
  layer_paths()
}
\seealso{
To manually specify grouping variables, see \code{\link{group_by}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marks.R
\name{marks}
\alias{marks}
\alias{emit_points}
\alias{layer_points}
\alias{emit_images}
\alias{layer_images}
\alias{emit_arcs}
\alias{layer_arcs}
\alias{emit_ribbons}
\alias{layer_ribbons}
\alias{emit_paths}
\alias{layer_paths}
\alias{emit_rects}
\alias{layer_rects}
\alias{emit_text}
\alias{layer_text}
\title{Vega marks.}
\usage{
emit_points(vis, props)

layer_points(vis, ..., data = NULL)

emit_images(vis, props)

layer_images(vis, ..., data = NULL)

emit_arcs(vis, props)

layer_arcs(vis, ..., data = NULL)

emit_ribbons(vis, props)

layer_ribbons(vis, ..., data = NULL)

emit_paths(vis, props)

layer_paths(vis, ..., data = NULL)

emit_rects(vis, props)

layer_rects(vis, ..., data = NULL)

emit_text(vis, props)

layer_text(vis, ..., data = NULL)
}
\arguments{
\item{vis}{Visualisation to modify}

\item{props, ...}{A \code{\link{props}} object, named according to the
properties listed below.}

\item{data}{An optional dataset, if you want to override the usual data
inheritance for this mark.}
}
\description{
These functions create mark objects, corresponding to vega marks. Marks
are leaves in the plot tree, and control the details of the final rendering.
Marks are equivalent to the basic geoms in ggplot2 (e.g. point, line,
polygon), where ggvis layers correspond to combinations of geoms and
statistical transforms.
}
\details{
Note that by supplying a fill property to \code{mark_path} will produce
a filled property. \code{mark_point} is an alias to \code{mark_symbol}.
}
\section{Properties}{


You can set the following mark properties:

\itemize{
\item  x  The first (typically left-most) x-coordinate.
\item  x2 The second (typically right-most) x-coordinate.
\item  width The width of the mark (if supported).
\item  y The first (typically top-most) y-coordinate.
\item  y2 The second (typically bottom-most) y-coordinate.
\item  height The height of the mark (if supported).
\item  opacity The overall opacity.
\item  fill The fill color.
\item  fillOpacity The fill opacity
\item  stroke The stroke color.
\item  strokeWidth The stroke width, in pixels.
\item  strokeOpacity The stroke opacity.
\item  size [symbol] The pixel area of the symbol. For example in the case
  of circles, the radius is determined in part by the square root of the size
  value.
\item  shape [symbol] The symbol shape to use. One of circle (default),
  square, cross, diamond, triangle-up, or triangle-down (symbol only)
\item  innerRadius [arc] The inner radius of the arc, in pixels.
\item  outerRadius [arc] The outer radius of the arc, in pixels.
\item  startAngle [arc] The start angle of the arc, in radians.
\item  endAngle [arc] The end angle of the arc, in radians.
\item  interpolate [area, line] The line interpolation method to use. One
  of linear, step-before, step-after, basis, basis-open, cardinal,
  cardinal-open, monotone.
\item  tension [area, line] Depending on the interpolation type, sets the
  tension parameter.
\item  url [image] The URL from which to retrieve the image.
\item  align [image, text] The horizontal alignment of the object. One of
  left, right, center.
\item  baseline [image, text] The vertical alignment of the object. One of
  top, middle, bottom.
\item  text [text] The text to display.
\item  dx [text] The horizontal margin, in pixels, between the text label
  and its anchor point. The value is ignored if the align property is center.
\item  dy [text] The vertical margin, in pixels, between the text label
  and its anchor point. The value is ignored if the baseline property is
  middle.
\item  angle [text] The rotation angle of the text, in degrees.
\item  font [text] The typeface to set the text in (e.g., Helvetica Neue).
\item  fontSize [text] The font size, in pixels.
\item  fontWeight [text] The font weight (e.g., bold).
\item  fontStyle [text] The font style (e.g., italic).
}

To each property, you can assign any property object (\code{\link{prop}})
either locally (i.e. in the mark), or in a parent \code{layer}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{set_scale_label}
\alias{set_scale_label}
\title{Set the label for a scale}
\usage{
set_scale_label(vis, scale, label)
}
\arguments{
\item{vis}{A ggvis object.}

\item{scale}{The name of a scale, like "x".}

\item{label}{Text to use for the label.}
}
\description{
Set the label for a scale
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_layout.R
\name{ggvisOutputElements}
\alias{ggvisOutputElements}
\title{Create HTML elements for ggvis output}
\usage{
ggvisOutputElements(plot_id = rand_id("plot_id"), spec = NULL, shiny = TRUE)
}
\arguments{
\item{plot_id}{Unique identifier to use for the div containing the ggvis plot.}

\item{spec}{Plot specification, used internally.}

\item{shiny}{Should this include headers for Shiny? For dynamic and
interactive plots, this should be TRUE; otherwise FALSE.}
}
\description{
This is an internal-facing function similar to ggvisOutput, but with more
options.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{propname_to_scale}
\alias{propname_to_scale}
\title{Convert the name of a property to the name of its default scale.}
\usage{
propname_to_scale(prop)
}
\arguments{
\item{prop}{character vector of property names. Any unrecognised names
are left unchanged.}
}
\value{
character vector of default scale names.
}
\description{
This is mainly used to ensure that similar properties share the same
scale by default - e.g. \code{x} and \code{x2} should use the same
scale.
}
\examples{
propname_to_scale(c("x", "x2"))
propname_to_scale(c("foo", "bar"))
propname_to_scale(c("opacity", "fillOpacity", "strokeOpacity"))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_resolution.R
\name{zero_range}
\alias{zero_range}
\title{Determine if range of vector is close to zero, with a specified tolerance}
\usage{
zero_range(x, tol = .Machine$double.eps * 100)
}
\arguments{
\item{x}{numeric range: vector of length 2}

\item{tol}{A value specifying the tolerance. Defaults to
\code{.Machine$double.eps * 100}.}
}
\value{
logical \code{TRUE} if the relative difference of the endpoints of
the range are not distinguishable from 0.
}
\description{
The machine epsilon is the difference between 1.0 and the next number
that can be represented by the machine. By default, this function
uses epsilon * 100 as the tolerance. First it scales the values so that
they have a mean of 1, and then it checks if the difference between
them is larger than the tolerance.
}
\examples{
eps <- .Machine$double.eps
zero_range(c(1, 1 + eps))       # TRUE
zero_range(c(1, 1 + 99 * eps))  # TRUE
zero_range(c(1, 1 + 101 * eps)) # FALSE - Crossed the tol threshold
zero_range(c(1, 1 + 2 * eps), tol = eps) # FALSE - Changed tol

# Scaling up or down all the values has no effect since the values
# are rescaled to 1 before checking against tol
zero_range(100000 * c(1, 1 + eps))        # TRUE
zero_range(100000 * c(1, 1 + 200 * eps))  # FALSE
zero_range(.00001 * c(1, 1 + eps))        # TRUE
zero_range(.00001 * c(1, 1 + 200 * eps))  # FALSE

# NA values
zero_range(c(1, NA))   # NA
zero_range(c(1, NaN))  # NA

# Infinite values
zero_range(c(1, Inf))     # FALSE
zero_range(c(-Inf, Inf))  # FALSE
zero_range(c(Inf, Inf))   # TRUE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_numeric}
\alias{scale_numeric}
\title{Add a numeric scale to a ggvis object.}
\usage{
scale_numeric(
  vis,
  property,
  domain = NULL,
  range = NULL,
  reverse = NULL,
  round = NULL,
  trans = NULL,
  clamp = NULL,
  exponent = NULL,
  nice = NULL,
  zero = NULL,
  expand = NULL,
  name = property,
  label = NULL,
  override = NULL
)
}
\arguments{
\item{vis}{A ggvis object.}

\item{property}{The name of a visual property, such as "x", "y", "fill",
"stroke". Note both x and x2 use the "x" scale (similarly for y and y2).
fillOpacity, opacity and strokeOpacity use the "opacity" scale.}

\item{domain}{The domain of the scale, representing the set of data values.
For ordinal scales, a character vector; for quantitative scales, a numeric
vector of length two. Either value (but not both) may be NA, in which
case \code{domainMin} or \code{domainMax} is set. For dynamic scales, this
can also be a reactive which returns the appropriate type of vector.}

\item{range}{The range of the scale, representing the set of visual values.
For numeric values, the range can take the form of a two-element array with
minimum and maximum values. For ordinal data, the range may by an array of
desired output values, which are mapped to elements in the specified
domain. The following range literals are also available: "width", "height",
"shapes", "category10", "category20".}

\item{reverse}{If true, flips the scale range.}

\item{round}{If true, rounds numeric output values to integers. This can be
helpful for snapping to the pixel grid.}

\item{trans}{A scale transformation: one of "linear", "log", "pow", "sqrt",
"quantile", "quantize", "threshold". Default is "linear".}

\item{clamp}{If \code{TRUE}, values that exceed the data domain are clamped
to either the minimum or maximum range value. Default is \code{FALSE}.}

\item{exponent}{Sets the exponent of the scale transformation. For pow
transform only.}

\item{nice}{If \code{TRUE}, modifies the scale domain to use a more
human-friendly number range (e.g., 7 instead of 6.96). Default is
\code{FALSE}.}

\item{zero}{If \code{TRUE}, ensures that a zero baseline value is included
in the scale domain. This option is ignored for non-quantitative scales.
Default is \code{FALSE}.}

\item{expand}{A multiplier for how much the scale should be expanded beyond
the domain of the data. For example, if the data goes from 10 to 110, and
\code{expand} is 0.05, then the resulting domain of the scale is 5 to 115.
Set to 0 and use \code{nice=FALSE} if you want exact control over the
domain. If left \code{NULL}, behavior will depend on the scale type. For
positional scales (x and y), \code{expand} will default to 0.05. For other
scales, it will default to 0.}

\item{name}{Name of the scale, such as "x", "y", "fill", etc. Can also be an
arbitrary name like "foo".}

\item{label}{Label for the scale. Used for axis or legend titles.}

\item{override}{Should the domain specified by this ggvis_scale object
override other ggvis_scale objects for the same scale? Useful when domain is
manually specified. For example, by default, the domain of the scale
will contain the range of the data, but when this is TRUE, the specified
domain will override, and the domain can be smaller than the range of the
data. If \code{FALSE}, the \code{domain} will not behave this way. If
left \code{NULL}, then it will be treated as \code{TRUE} whenever
\code{domain} is non-NULL.}
}
\description{
A numeric (quantitative) scale controls the mapping of continuous variables
to visual properties.
}
\details{
The default values for most of the arguments is NULL. When the plot is
created, these NULL values will be replaced with default values, as indicated
below.
}
\examples{
p <- mtcars \%>\% ggvis(~wt, ~mpg, fill = ~hp) \%>\% layer_points()

p \%>\% scale_numeric("y")

p \%>\% scale_numeric("y", trans = "pow", exponent = 0.5)

p \%>\% scale_numeric("y", trans = "log")

# Can control other properties other than x and y
p \%>\% scale_numeric("fill", domain = c(0, 120), clamp = TRUE)

# Set range of data from 0 to 3
p \%>\% scale_numeric("x", domain = c(0, 3), clamp = TRUE, expand = 0,
                     nice = FALSE)

# Lower bound is set to lower limit of data, upper bound set to 3.
p \%>\% scale_numeric("x", domain = c(NA, 3), clamp = TRUE, nice = FALSE)
}
\seealso{
\code{\link{scales}}, \code{\link{scale_ordinal}},
  \url{https://vega.github.io/vega/docs/scales/#quantitative}

Other scales: 
\code{\link{scale_datetime}()},
\code{\link{scale_ordinal}()}
}
\concept{scales}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cocaine.R
\docType{data}
\name{cocaine}
\alias{cocaine}
\title{Cocaine seizures in the US.}
\format{
Data frame with 3380 observations of 5 variables.
}
\usage{
cocaine
}
\description{
This dataset comes from STRIDE, the System to Retrieve Information from Drug
Evidence. It contains all concaine seizures in the US from 2007 that have
a known weight.
}
\section{Variables}{

\describe{
  \item{state}{State where seizure occured.}
  \item{potency}{Purity of cocaine, as percentage (100\% = pure cocaine,
    0\% = all filler)}
  \item{weight}{Weight, in grams.}
  \item{month}{Month in which seizure occured.}
  \item{price}{Estimated value in USD.}
}
}

\section{Use}{

Use of this data requires your agreement to refer to your analyses as
"unvalidated DEA data and to claim authorship and responsibility for any
inferences and/or conclusions you may draw from this information."
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handle_resize.R
\name{handle_resize}
\alias{handle_resize}
\alias{plot_width}
\alias{plot_height}
\title{Handlers and interactive inputs for plot sizing.}
\usage{
handle_resize(vis, on_resize)

plot_width(vis)

plot_height(vis)
}
\arguments{
\item{vis}{Visualisation to listen to.}

\item{on_resize}{Callback function with arguments:
\describe{
 \item{width,height}{Width and height in pixels}
 \item{padding}{A named list of four components giving the padding in
   each direction}
 \item{session}{The session, used to communicate with the browser}
}}
}
\description{
Handlers and interactive inputs for plot sizing.
}
\examples{
# This example just prints out the current dimensions to the console
mtcars \%>\% ggvis(~mpg, ~wt) \%>\%
  layer_points() \%>\%
  handle_resize(function(width, height, ...) cat(width, "x", height, "\n"))

# Use plot_width() and plot_height() to dynamically get the plot size
# inside the plot.
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_text(text := plot_width())
mtcars \%>\% ggvis(~mpg, ~wt) \%>\% layer_text(text := plot_height())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_resolution.R
\name{resolution}
\alias{resolution}
\title{Compute the "resolution" of a data vector.}
\usage{
resolution(x, zero = TRUE)
}
\arguments{
\item{x}{numeric vector}

\item{zero}{should a zero value be automatically included in the
computation of resolution}
}
\description{
The resolution is is the smallest non-zero distance between adjacent
values.  If there is only one unique value, then the resolution is defined
to be one.
}
\details{
If x is an integer vector, then it is assumed to represent a discrete
variable, and the resolution is 1.
}
\examples{
resolution(1:10)
resolution((1:10) - 0.5)
resolution((1:10) - 0.5, FALSE)
resolution(c(1,2, 10, 20, 50))
resolution(as.integer(c(1, 10, 20, 50)))  # Returns 1
}
